@@ -14,14 +14,14 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #include "config.h" // For GLOM_ENABLE_CLIENT_ONLY
 
 #include <glom/base_db.h>
-#include "application.h" //Application.
+#include "appwindow.h" //AppWindow.
 #include <libglom/appstate.h>
 #include <libglom/standard_table_prefs_fields.h>
 #include <libglom/document/document.h>
@@ -39,7 +39,6 @@
 #include <glom/utils_ui.h>
 #include <glom/glade_utils.h>
 #include <libglom/data_structure/glomconversions.h>
-#include <libglom/data_structure/parameternamegenerator.h>
 #include <libglom/data_structure/layout/report_parts/layoutitem_summary.h>
 #include <libglom/data_structure/layout/report_parts/layoutitem_fieldsummary.h>
 #include <libglom/data_structure/layout/report_parts/layoutitem_verticalgroup.h>
@@ -49,15 +48,6 @@
 #include <libglom/privs.h>
 #include <libglom/db_utils.h>
 #include <glibmm/i18n.h>
-//#include <libgnomeui/gnome-app-helper.h>
-
-//#include <libgdamm/metastore.h> //For MetaStoreError
-//#undef GDA_DISABLE_DEPRECATED
-//#include <sql-parser/gda-statement-struct-util.h> //For gda_sql_identifier_remove_quotes().
-
-#ifdef GLOM_ENABLE_MAEMO
-#include <hildonmm/note.h>
-#endif
 
 #include <sstream> //For stringstream
 
@@ -131,7 +121,7 @@ void Base_DB::handle_error(const Glib::Exception& ex)
 
   Gtk::MessageDialog dialog(Utils::bold_message(_("Internal error")), true, Gtk::MESSAGE_WARNING );
   dialog.set_secondary_text(ex.what());
-  //TODO: dialog.set_transient_for(*get_application());
+  //TODO: dialog.set_transient_for(*get_appwindow());
   dialog.run();
 }
 
@@ -139,13 +129,9 @@ void Base_DB::handle_error(const std::exception& ex)
 {
   std::cerr << "Internal Error (Base_DB::handle_error()): exception type=" << typeid(ex).name() << ", ex.what()=" << ex.what() << std::endl;
 
-#ifdef GLOM_ENABLE_MAEMO
-  Hildon::Note dialog(Hildon::NOTE_TYPE_INFORMATION, ex.what());
-#else
   Gtk::MessageDialog dialog(Utils::bold_message(_("Internal error")), true, Gtk::MESSAGE_WARNING );
   dialog.set_secondary_text(ex.what());
-  //TODO: dialog.set_transient_for(*get_application());
-#endif
+  //TODO: dialog.set_transient_for(*get_appwindow());
 
   dialog.run();
 }
@@ -177,7 +163,7 @@ AppState::userlevels Base_DB::get_userlevel() const
   }
   else
   {
-    g_warning("Base_DB::get_userlevel(): document not found.");
+    std::cerr << G_STRFUNC << ": document not found." << std::endl;
     return AppState::USERLEVEL_OPERATOR;
   }
 }
@@ -229,69 +215,6 @@ Base_DB::type_vec_strings Base_DB::util_vecStrings_from_Fields(const type_vec_fi
   return vecNames;
 }
 
-Base_DB::type_vec_fields Base_DB::get_fields_for_table(const Glib::ustring& table_name, bool including_system_fields) const
-{
-  //Get field definitions from the database:
-  type_vec_fields fieldsDatabase = DbUtils::get_fields_for_table_from_database(table_name, including_system_fields);
-
-  const Document* pDoc = dynamic_cast<const Document*>(get_document());
-  if(!pDoc)
-    return fieldsDatabase; //This should never happen.
-  else
-  {
-    type_vec_fields result;
-
-    type_vec_fields fieldsDocument = pDoc->get_table_fields(table_name);
-
-    //Look at each field in the database:
-    for(type_vec_fields::iterator iter = fieldsDocument.begin(); iter != fieldsDocument.end(); ++iter)
-    {
-      sharedptr<Field> field = *iter;
-      const Glib::ustring field_name = field->get_name();
-
-      //Get the field info from the database:
-      //This is in the document as well, but it _might_ have changed.
-      type_vec_fields::const_iterator iterFindDatabase = std::find_if(fieldsDatabase.begin(), fieldsDatabase.end(), predicate_FieldHasName<Field>(field_name));
-
-      if(iterFindDatabase != fieldsDatabase.end() ) //Ignore fields that don't exist in the database anymore.
-      {
-        Glib::RefPtr<Gnome::Gda::Column> field_info_document = field->get_field_info();
-
-        //Update the Field information that _might_ have changed in the database.
-        Glib::RefPtr<Gnome::Gda::Column> field_info = (*iterFindDatabase)->get_field_info();
-
-        //libgda does not tell us whether the field is auto_incremented, so we need to get that from the document.
-        field_info->set_auto_increment( field_info_document->get_auto_increment() );
-
-        //libgda does not tell us whether the field is auto_incremented, so we need to get that from the document.
-        //TODO_gda:field_info->set_primary_key( field_info_document->get_primary_key() );
-
-        //libgda does yet tell us correct default_value information so we need to get that from the document.
-        field_info->set_default_value( field_info_document->get_default_value() );
-
-        field->set_field_info(field_info);
-
-        result.push_back(*iter);
-      }
-    }
-
-    //Add any fields that are in the database, but not in the document:
-    for(type_vec_fields::iterator iter = fieldsDatabase.begin(); iter != fieldsDatabase.end(); ++iter)
-    {
-      Glib::ustring field_name = (*iter)->get_name();
-
-       //Look in the result so far:
-       type_vec_fields::const_iterator iterFind = std::find_if(result.begin(), result.end(), predicate_FieldHasName<Field>(field_name));
-
-       //Add it if it is not there:
-       if(iterFind == result.end() )
-         result.push_back(*iter);
-    }
-
-    return result;
-  }
-}
-
 #ifndef GLOM_ENABLE_CLIENT_ONLY
 
 namespace
@@ -608,24 +531,6 @@ sharedptr<LayoutItem_Notebook> Base_DB::offer_notebook(const sharedptr<LayoutIte
 }
 #endif // !GLOM_ENABLE_CLIENT_ONLY
 
-sharedptr<Field> Base_DB::get_fields_for_table_one_field(const Glib::ustring& table_name, const Glib::ustring& field_name) const
-{
-  //Initialize output parameter:
-  sharedptr<Field> result;
-
-  if(field_name.empty() || table_name.empty())
-    return result;
-
-  type_vec_fields fields = get_fields_for_table(table_name);
-  type_vec_fields::iterator iter = std::find_if(fields.begin(), fields.end(), predicate_FieldHasName<Field>(field_name));
-  if(iter != fields.end()) //TODO: Handle error?
-  {
-    return *iter;
-  }
-
-  return sharedptr<Field>();
-}
-
 //static:
 bool Base_DB::get_field_primary_key_index_for_fields(const type_vec_fields& fields, guint& field_column)
 {
@@ -702,6 +607,8 @@ sharedptr<Field> Base_DB::get_field_primary_key_for_table(const Glib::ustring& t
 
 void Base_DB::get_table_fields_to_show_for_sequence_add_group(const Glib::ustring& table_name, const Privileges& table_privs, const type_vec_fields& all_db_fields, const sharedptr<LayoutGroup>& group, Base_DB::type_vecConstLayoutFields& vecFields) const
 {
+  const Document* document = dynamic_cast<const Document*>(get_document());
+
   //g_warning("Box_Data::get_table_fields_to_show_for_sequence_add_group(): table_name=%s, all_db_fields.size()=%d, group->name=%s", table_name.c_str(), all_db_fields.size(), group->get_name().c_str());
 
   LayoutGroup::type_list_items items = group->get_items();
@@ -718,7 +625,7 @@ void Base_DB::get_table_fields_to_show_for_sequence_add_group(const Glib::ustrin
       if(item_field->get_has_relationship_name()) //If it's a field in a related table.
       {
         //TODO_Performance: get_fields_for_table_one_field() is probably very inefficient
-        sharedptr<Field> field = get_fields_for_table_one_field(item_field->get_table_used(table_name), item->get_name());
+        sharedptr<Field> field = DbUtils::get_fields_for_table_one_field(document, item_field->get_table_used(table_name), item->get_name());
         if(field)
         {
           sharedptr<LayoutItem_Field> layout_item = item_field;
@@ -774,20 +681,21 @@ void Base_DB::get_table_fields_to_show_for_sequence_add_group(const Glib::ustrin
 
   if(vecFields.empty())
   {
-    //g_warning("Box_Data::get_table_fields_to_show_for_sequence_add_group(): Returning empty list.");
+    //std::cerr << G_STRFUNC << ": Returning empty list." << std::endl;
   }
 }
 
 Base_DB::type_vecConstLayoutFields Base_DB::get_table_fields_to_show_for_sequence(const Glib::ustring& table_name, const Document::type_list_layout_groups& mapGroupSequence) const
 {
+  const Document* pDoc = dynamic_cast<const Document*>(get_document());
+
   //Get field definitions from the database, with corrections from the document:
-  type_vec_fields all_fields = get_fields_for_table(table_name);
+  type_vec_fields all_fields = DbUtils::get_fields_for_table(pDoc, table_name);
 
   const Privileges table_privs = Privs::get_current_privs(table_name);
 
   //Get fields that the document says we should show:
   type_vecConstLayoutFields result;
-  const Document* pDoc = dynamic_cast<const Document*>(get_document());
   if(pDoc)
   {
     if(mapGroupSequence.empty())
@@ -844,7 +752,7 @@ Base_DB::type_vecConstLayoutFields Base_DB::get_table_fields_to_show_for_sequenc
       {
         sharedptr<LayoutGroup> group = *iter;
 
-        if(true) //!group->m_hidden)
+        if(true) //!group->get_hidden())
         {
           //Get the fields:
           get_table_fields_to_show_for_sequence_add_group(table_name, table_privs, all_fields, group, result);
@@ -855,7 +763,7 @@ Base_DB::type_vecConstLayoutFields Base_DB::get_table_fields_to_show_for_sequenc
 
   if(result.empty())
   {
-    //g_warning("Box_Data::get_table_fields_to_show_for_sequence_add_group(): Returning empty list.");
+    //std::cerr << G_STRFUNC << ": Returning empty list." << std::endl;
   }
 
   return result;
@@ -909,7 +817,7 @@ void Base_DB::calculate_field_in_all_records(const Glib::ustring& table_name, co
 void Base_DB::calculate_field(const LayoutFieldInRecord& field_in_record)
 {
   const Glib::ustring field_name = field_in_record.m_field->get_name();
-  //g_warning("Box_Data::calculate_field(): field_name=%s", field_name.c_str());
+  //std::cerr << G_STRFUNC << ": field_name=" << field_name << std::endl;
 
   //Do we already have this in our list?
   type_field_calcs::iterator iterFind = m_FieldsCalculationInProgress.find(field_name);
@@ -928,18 +836,18 @@ void Base_DB::calculate_field(const LayoutFieldInRecord& field_in_record)
   //Use the previously-calculated value if possible:
   if(refCalcProgress.m_calc_in_progress)
   {
-    //g_warning("  Box_Data::calculate_field(): Circular calculation detected. field_name=%s", field_name.c_str());
+    //std::cerr << G_STRFUNC << ": Circular calculation detected. field_name=" << field_name << std::endl;
     //refCalcProgress.m_value = Conversions::get_empty_value(field->get_glom_type()); //Give up.
   }
   else if(refCalcProgress.m_calc_finished)
   {
-    //g_warning("  Box_Data::calculate_field(): Already calculated.");
+    //std::cerr << G_STRFUNC << ": Already calculated." << std::endl;
 
     //Don't bother calculating it again. The correct value is already in the database and layout.
   }
   else
   {
-    //g_warning("  Box_Data::calculate_field(): setting calc_in_progress: field_name=%s", field_name.c_str());
+    //std::cerr << G_STRFUNC << ": setting calc_in_progress: field_name=" << field_name << std::endl;
 
     refCalcProgress.m_calc_in_progress = true; //Let the recursive calls to calculate_field() check this.
 
@@ -1155,13 +1063,12 @@ bool Base_DB::set_field_value_in_database(const LayoutFieldInRecord& layoutfield
   const Glib::ustring field_name = field_in_record.m_field->get_name();
   if(!field_name.empty()) //This should not happen.
   {
-    Glib::RefPtr<Gnome::Gda::SqlBuilder> builder = Gnome::Gda::SqlBuilder::create(Gnome::Gda::SQL_STATEMENT_UPDATE);
-    builder->set_table(field_in_record.m_table_name);
-    builder->add_field_value_as_value(field_in_record.m_field->get_name(), field_value);
-    builder->set_where(
-      builder->add_cond(Gnome::Gda::SQL_OPERATOR_TYPE_EQ,
-        builder->add_field_id(field_in_record.m_key->get_name(), field_in_record.m_table_name),
-        builder->add_expr_as_value(field_in_record.m_key_value)));
+    const Gnome::Gda::SqlExpr where_clause = 
+      Utils::build_simple_where_expression(field_in_record.m_table_name,
+        field_in_record.m_key, field_in_record.m_key_value);
+    const Glib::RefPtr<const Gnome::Gda::SqlBuilder> builder = 
+      Utils::build_sql_update_with_where_clause(field_in_record.m_table_name,
+        field_in_record.m_field, field_value, where_clause);
 
     try //TODO: The exceptions are probably already handled by query_execute(
     {
@@ -1194,7 +1101,7 @@ bool Base_DB::set_field_value_in_database(const LayoutFieldInRecord& layoutfield
     //Prevent circular calculations during the recursive do_calculations:
     {
       //Recalculate any calculated fields that depend on this calculated field.
-      //g_warning("Box_Data::set_field_value_in_database(): calling do_calculations");
+      //std::cerr << G_STRFUNC << ": calling do_calculations" << std::endl;
 
       do_calculations(layoutfield_in_record, !use_current_calculations);
     }
@@ -1214,11 +1121,24 @@ Gnome::Gda::Value Base_DB::get_field_value_in_database(const LayoutFieldInRecord
     return result;
   }
 
+  //Check that there is a key value, if there should be one:
+  //System Preferences, for instance, should not need a key to identify the record:
   if(!(field_in_record.m_key))
   {
-    std::cerr << G_STRFUNC << ": field_in_record.m_key is empty." << std::endl;
-    return result;
+    Glib::ustring to_field;
+    if(field_in_record.m_field &&
+      field_in_record.m_field->get_relationship())
+    {
+      to_field = field_in_record.m_field->get_relationship()->get_to_field();
+    }
+      
+    if(!to_field.empty())
+    {
+      std::cerr << G_STRFUNC << ": field_in_record.m_key is empty." << std::endl;
+      return result;
+    }
   }
+  
 
   type_vecConstLayoutFields list_fields;
   sharedptr<const LayoutItem_Field> layout_item = field_in_record.m_field;
@@ -1438,12 +1358,16 @@ Base_DB::type_list_const_field_items Base_DB::get_calculation_fields(const Glib:
 
 void Base_DB::do_lookups(const LayoutFieldInRecord& field_in_record, const Gtk::TreeModel::iterator& row, const Gnome::Gda::Value& field_value)
 {
+  Document* document = get_document();
+  if(!document)
+    return;
+
    //Get values for lookup fields, if this field triggers those relationships:
    //TODO_performance: There is a LOT of iterating and copying here.
    const Glib::ustring strFieldName = field_in_record.m_field->get_name();
-   const type_list_lookups lookups = get_lookup_fields(field_in_record.m_table_name, strFieldName);
+   const Document::type_list_lookups lookups = document->get_lookup_fields(field_in_record.m_table_name, strFieldName);
    //std::cout << "debug: " << G_STRFUNC << ": lookups size=" << lookups.size() << std::endl;
-   for(type_list_lookups::const_iterator iter = lookups.begin(); iter != lookups.end(); ++iter)
+   for(Document::type_list_lookups::const_iterator iter = lookups.begin(); iter != lookups.end(); ++iter)
    {
      sharedptr<const LayoutItem_Field> layout_item = iter->first;
 
@@ -1453,10 +1377,10 @@ void Base_DB::do_lookups(const LayoutFieldInRecord& field_in_record, const Gtk::
      const sharedptr<const Field> field_lookup = layout_item->get_full_field_details();
      if(field_lookup)
      {
-      sharedptr<const Field> field_source = get_fields_for_table_one_field(relationship->get_to_table(), field_lookup->get_lookup_field());
+      sharedptr<const Field> field_source = DbUtils::get_fields_for_table_one_field(document, relationship->get_to_table(), field_lookup->get_lookup_field());
       if(field_source)
       {
-        const Gnome::Gda::Value value = get_lookup_value(field_in_record.m_table_name, iter->second /* relationship */,  field_source /* the field to look in to get the value */, field_value /* Value of to and from fields */);
+        const Gnome::Gda::Value value = DbUtils::get_lookup_value(document, field_in_record.m_table_name, iter->second /* relationship */,  field_source /* the field to look in to get the value */, field_value /* Value of to and from fields */);
 
         const Gnome::Gda::Value value_converted = Conversions::convert_value(value, layout_item->get_glom_type());
 
@@ -1475,87 +1399,11 @@ void Base_DB::do_lookups(const LayoutFieldInRecord& field_in_record, const Gtk::
   }
 }
 
-
-/** Get the fields whose values should be looked up when @a field_name changes, with
- * the relationship used to lookup the value.
- */
-Base_DB::type_list_lookups Base_DB::get_lookup_fields(const Glib::ustring& table_name, const Glib::ustring& field_name) const
-{
-  type_list_lookups result;
-
-  const Document* document = dynamic_cast<const Document*>(get_document());
-  if(document)
-  {
-    //Examine all fields, not just the the shown fields (m_Fields):
-    const type_vec_fields fields = document->get_table_fields(table_name); //TODO_Performance: Cache this?
-    //Examine all fields, not just the the shown fields.
-    for(type_vec_fields::const_iterator iter = fields.begin(); iter != fields.end();  ++iter)
-    {
-      sharedptr<Field> field = *iter;
-
-      //Examine each field that looks up its data from a relationship:
-      if(field && field->get_is_lookup())
-      {
-        //Get the relationship information:
-        sharedptr<Relationship> relationship = field->get_lookup_relationship();
-        if(relationship)
-        {
-          //If the relationship is triggererd by the specified field:
-          if(relationship->get_from_field() == field_name)
-          {
-            //Add it:
-            sharedptr<LayoutItem_Field> item = sharedptr<LayoutItem_Field>::create();
-            item->set_full_field_details(field);
-            result.push_back( type_pairFieldTrigger(item, relationship) );
-          }
-        }
-      }
-    }
-  }
-
-  return result;
-}
-
 void Base_DB::refresh_related_fields(const LayoutFieldInRecord& /* field_in_record_changed */, const Gtk::TreeModel::iterator& /* row */, const Gnome::Gda::Value& /* field_value */)
 {
   //overridden in Box_Data.
 }
 
-Gnome::Gda::Value Base_DB::get_lookup_value(const Glib::ustring& /* table_name */, const sharedptr<const Relationship>& relationship, const sharedptr<const Field>& source_field, const Gnome::Gda::Value& key_value)
-{
-  Gnome::Gda::Value result;
-
-  sharedptr<Field> to_key_field = get_fields_for_table_one_field(relationship->get_to_table(), relationship->get_to_field());
-  if(to_key_field)
-  {
-    //Convert the value, in case the from and to fields have different types:
-    const Gnome::Gda::Value value_to_key_field = Conversions::convert_value(key_value, to_key_field->get_glom_type());
-
-    const Glib::ustring target_table = relationship->get_to_table();
-    Glib::RefPtr<Gnome::Gda::SqlBuilder> builder =
-      Gnome::Gda::SqlBuilder::create(Gnome::Gda::SQL_STATEMENT_SELECT);
-    builder->select_add_field(source_field->get_name(), target_table );
-    builder->select_add_target(target_table );
-    builder->set_where(
-      builder->add_cond(Gnome::Gda::SQL_OPERATOR_TYPE_EQ,
-        builder->add_field_id(to_key_field->get_name(), target_table),
-        builder->add_expr(value_to_key_field)));
-
-    Glib::RefPtr<Gnome::Gda::DataModel> data_model = DbUtils::query_execute_select(builder);
-    if(data_model && data_model->get_n_rows())
-    {
-      //There should be only 1 row. Well, there could be more but we will ignore them.
-      result = data_model->get_value_at(0, 0);
-    }
-    else
-    {
-      handle_error();
-    }
-  }
-
-  return result;
-}
-
 bool Base_DB::get_field_value_is_unique(const Glib::ustring& table_name, const sharedptr<const LayoutItem_Field>& field, const Gnome::Gda::Value& value)
 {
   bool result = true;  //Arbitrarily default to saying it's unique if we can't get any result.
@@ -1628,126 +1476,6 @@ bool Base_DB::get_relationship_exists(const Glib::ustring& table_name, const Gli
   return false;
 }
 
-sharedptr<const LayoutItem_Field> Base_DB::get_field_is_from_non_hidden_related_record(const sharedptr<const LayoutItem_Portal>& portal) const
-{
-  //Find the first field that is from a non-hidden related table.
-  sharedptr<LayoutItem_Field> result;
-
-  if(!portal)
-    return result;
-
-  const Document* document = get_document();
-  if(!document)
-    return result;
-
-  const Glib::ustring parent_table_name = portal->get_table_used(Glib::ustring() /* parent table - not relevant */);
-
-  LayoutItem_Portal::type_list_const_items items = portal->get_items();
-  for(LayoutItem_Portal::type_list_const_items::const_iterator iter = items.begin(); iter != items.end(); ++iter)
-  {
-    sharedptr<const LayoutItem_Field> field = sharedptr<const LayoutItem_Field>::cast_dynamic(*iter);
-    if(field)
-    {
-      if(field->get_has_relationship_name())
-      {
-        const Glib::ustring table_name = field->get_table_used(parent_table_name);
-        if(!(document->get_table_is_hidden(table_name)))
-          return field;
-      }
-
-    }
-  }
-
-  return result;
-}
-
-sharedptr<const LayoutItem_Field> Base_DB::get_field_identifies_non_hidden_related_record(const sharedptr<const LayoutItem_Portal>& portal, sharedptr<const Relationship>& used_in_relationship) const
-{
-  //Find the first field that is from a non-hidden related table.
-  sharedptr<LayoutItem_Field> result;
-
-  const Document* document = get_document();
-  if(!document)
-    return result;
-
-  const Glib::ustring parent_table_name = portal->get_table_used(Glib::ustring() /* parent table - not relevant */);
-
-  LayoutItem_Portal::type_list_const_items items = portal->get_items();
-  for(LayoutItem_Portal::type_list_const_items::const_iterator iter = items.begin(); iter != items.end(); ++iter)
-  {
-    sharedptr<const LayoutItem_Field> field = sharedptr<const LayoutItem_Field>::cast_dynamic(*iter);
-    if(field && !(field->get_has_relationship_name()))
-    {
-      sharedptr<const Relationship> relationship = document->get_field_used_in_relationship_to_one(parent_table_name, field);
-      if(relationship)
-      {
-        const Glib::ustring table_name = relationship->get_to_table();
-        if(!(table_name.empty()))
-        {
-          if(!(document->get_table_is_hidden(table_name)))
-          {
-            used_in_relationship = relationship;
-            return field;
-          }
-        }
-      }
-    }
-  }
-
-  return result;
-}
-
-sharedptr<const UsesRelationship> Base_DB::get_portal_navigation_relationship_automatic(const sharedptr<LayoutItem_Portal>& portal, bool& navigation_main) const
-{
-  //Initialize output parameters:
-  navigation_main = false;
-
-  const Document* document = get_document();
-
-  //If the related table is not hidden then we can just navigate to that:
-  const Glib::ustring direct_related_table_name = portal->get_table_used(Glib::ustring() /* parent table - not relevant */);
-  if(!(document->get_table_is_hidden(direct_related_table_name)))
-  {
-    //Non-hidden tables can just be shown directly. Navigate to it:
-    navigation_main = true;
-    return sharedptr<UsesRelationship>();
-  }
-  else
-  {
-    //If the related table is hidden,
-    //then find a suitable related non-hidden table by finding the first layout field that mentions one:
-    sharedptr<const LayoutItem_Field> field = get_field_is_from_non_hidden_related_record(portal);
-    if(field)
-    {
-      return field; //Returns the relationship part. (A relationship belonging to the portal's related table.)
-      //sharedptr<UsesRelationship> result = sharedptr<UsesRelationship>::create();
-      //result->set_relationship( portal->get_relationship() );
-      //result->set_related_relationship( field->get_relationship() );
-
-      //return result;
-    }
-    else
-    {
-      //Instead, find a key field that's used in a relationship,
-      //and pretend that we are showing the to field as a related field:
-      sharedptr<const Relationship> used_in_relationship;
-      sharedptr<const LayoutItem_Field> field_identifies = get_field_identifies_non_hidden_related_record(portal, used_in_relationship);
-      if(field_identifies)
-      {
-        sharedptr<UsesRelationship> result = sharedptr<UsesRelationship>::create();
-
-        sharedptr<Relationship> rel_nonconst = sharedptr<Relationship>::cast_const(used_in_relationship);
-        result->set_relationship(rel_nonconst);
-
-        return result;
-      }
-    }
-  }
-
-  //There was no suitable related table to show:
-  return sharedptr<UsesRelationship>();
-}
-
 bool Base_DB::get_primary_key_is_in_foundset(const FoundSet& found_set, const Gnome::Gda::Value& primary_key_value)
 {
   //TODO_Performance: This is probably called too often, when we should know that the key is in the found set.
@@ -1799,46 +1527,6 @@ bool Base_DB::get_primary_key_is_in_foundset(const FoundSet& found_set, const Gn
     return false;
 }
 
-int Base_DB::count_rows_returned_by(const Glib::RefPtr<Gnome::Gda::SqlBuilder>& sql_query)
-{
-  if(!sql_query)
-  {
-    std::cerr << G_STRFUNC << ": sql_query was null." << std::endl;
-    return 0;
-  }
-
-  Glib::RefPtr<Gnome::Gda::SqlBuilder> builder =
-    Gnome::Gda::SqlBuilder::create(Gnome::Gda::SQL_STATEMENT_SELECT);
-
-  //Note that the alias is just because the SQL syntax requires it - we get an error if we don't use it.
-  //Be careful not to include ORDER BY clauses in this, because that would make it unnecessarily slow:
-  const guint target_id = builder->add_sub_select( sql_query->get_sql_statement() );
-  builder->select_add_target_id(target_id, "glomarbitraryalias");
-
-  const Gnome::Gda::SqlBuilder::Id id_function = builder->add_function("COUNT", builder->add_id("*"));
-  builder->add_field_value_id(id_function);
-
-
-  int result = 0;
-
-  //TODO: Is this inefficient?
-    Glib::RefPtr<Gnome::Gda::DataModel> datamodel = DbUtils::query_execute_select(builder);
-    if(datamodel && datamodel->get_n_rows() && datamodel->get_n_columns())
-    {
-      Gnome::Gda::Value value = datamodel->get_value_at(0, 0);
-      //This showed me that this contains a gint64: std::cerr << "DEBUG: value type=" << G_VALUE_TYPE_NAME(value.gobj()) << std::endl;
-      //For sqlite, this is an integer
-      if(value.get_value_type() == G_TYPE_INT64)
-        result = (int)value.get_int64();
-      else
-        result = value.get_int();
-    }
-
-  //std::cout << "debug: " << G_STRFUNC << ": Returning " << result << std::endl;
-  return result;
-}
-
-
 void Base_DB::set_found_set_where_clause_for_portal(FoundSet& found_set, const sharedptr<LayoutItem_Portal>& portal, const Gnome::Gda::Value& foreign_key_value)
 {
   found_set.m_table_name = Glib::ustring();
@@ -1858,8 +1546,10 @@ void Base_DB::set_found_set_where_clause_for_portal(FoundSet& found_set, const s
   // The WHERE clause mentions the first-related table (though by the alias defined in extra_join)
   // and we add an extra JOIN to mention the second-related table.
 
+  Document* document = get_document();
+
   Glib::ustring where_clause_to_table_name = relationship->get_to_table();
-  sharedptr<Field> where_clause_to_key_field = get_fields_for_table_one_field(relationship->get_to_table(), relationship->get_to_field());
+  sharedptr<Field> where_clause_to_key_field = DbUtils::get_fields_for_table_one_field(document, relationship->get_to_table(), relationship->get_to_field());
 
   found_set.m_table_name = portal->get_table_used(Glib::ustring() /* parent table - not relevant */);
 
@@ -1875,7 +1565,7 @@ void Base_DB::set_found_set_where_clause_for_portal(FoundSet& found_set, const s
     where_clause_to_table_name = uses_rel_temp->get_sql_join_alias_name();
 
     const Glib::ustring to_field_name = uses_rel_temp->get_to_field_used();
-    where_clause_to_key_field = get_fields_for_table_one_field(relationship->get_to_table(), to_field_name);
+    where_clause_to_key_field = DbUtils::get_fields_for_table_one_field(document, relationship->get_to_table(), to_field_name);
     //std::cout << "extra_join=" << found_set.m_extra_join << std::endl;
 
     //std::cout << "extra_join where_clause_to_key_field=" << where_clause_to_key_field->get_name() << std::endl;
@@ -1888,91 +1578,6 @@ void Base_DB::set_found_set_where_clause_for_portal(FoundSet& found_set, const s
   }
 }
 
-bool Base_DB::add_user(const Glib::ustring& user, const Glib::ustring& password, const Glib::ustring& group)
-{
-  if(user.empty() || password.empty() || group.empty())
-    return false;
-
-  //Create the user:
-  //Note that ' around the user fails, so we use ".
-  Glib::ustring strQuery = "CREATE USER \"" + user + "\" PASSWORD '" + password + "'" ; //TODO: Escape the password.
-  if(group == GLOM_STANDARD_GROUP_NAME_DEVELOPER)
-    strQuery += " SUPERUSER CREATEDB CREATEROLE"; //Because SUPERUSER is not "inherited" from groups to members.
-
-
-  //Glib::ustring strQuery = "CREATE USER \"" + user + "\"";
-  //if(group == GLOM_STANDARD_GROUP_NAME_DEVELOPER)
-  //  strQuery += " WITH SUPERUSER"; //Because SUPERUSER is not "inherited" from groups to members.
-  //strQuery +=  " PASSWORD '" + password + "'" ; //TODO: Escape the password.
-
-
-  bool test = DbUtils::query_execute_string(strQuery);
-  if(!test)
-  {
-    std::cerr << G_STRFUNC << ": CREATE USER failed." << std::endl;
-    return false;
-  }
-
-  //Add it to the group:
-  strQuery = "ALTER GROUP \"" + group + "\" ADD USER \"" + user + "\"";
-  test = DbUtils::query_execute_string(strQuery);
-  if(!test)
-  {
-    std::cerr << G_STRFUNC << ": ALTER GROUP failed." << std::endl;
-    return false;
-  }
-
-  //Remove any user rights, so that all rights come from the user's presence in the group:
-  Document* document = get_document();
-  if(!document)
-    return true;
-
-  Document::type_listTableInfo table_list = document->get_tables();
-
-  for(Document::type_listTableInfo::const_iterator iter = table_list.begin(); iter != table_list.end(); ++iter)
-  {
-    const Glib::ustring strQuery = "REVOKE ALL PRIVILEGES ON \"" + (*iter)->get_name() + "\" FROM \"" + user + "\"";
-    const bool test = DbUtils::query_execute_string(strQuery);
-    if(!test)
-      std::cerr << G_STRFUNC << ": REVOKE failed." << std::endl;
-  }
-
-  return true;
-}
-
-
-bool Base_DB::remove_user(const Glib::ustring& user)
-{
-  if(user.empty())
-    return false;
-
-  const Glib::ustring strQuery = "DROP USER \"" + user + "\"";
-  const bool test = DbUtils::query_execute_string(strQuery);
-  if(!test)
-  {
-    std::cerr << G_STRFUNC << ": DROP USER failed" << std::endl;
-    return false;
-  }
-
-  return true;
-}
-
-bool Base_DB::remove_user_from_group(const Glib::ustring& user, const Glib::ustring& group)
-{
-  if(user.empty() || group.empty())
-    return false;
-
-  const Glib::ustring strQuery = "ALTER GROUP \"" + group + "\" DROP USER \"" + user + "\"";
-  const bool test = DbUtils::query_execute_string(strQuery);
-  if(!test)
-  {
-    std::cerr << G_STRFUNC << ": ALTER GROUP failed." << std::endl;
-    return false;
-  }
-
-  return true;
-}
-
 bool Base_DB::set_database_owner_user(const Glib::ustring& user)
 {
   if(user.empty())
@@ -1983,7 +1588,7 @@ bool Base_DB::set_database_owner_user(const Glib::ustring& user)
   if(database_name.empty())
     return false;
 
-  const Glib::ustring strQuery = "ALTER DATABASE \"" + database_name + "\" OWNER TO \"" + user + "\"";
+  const Glib::ustring strQuery = "ALTER DATABASE " + DbUtils::escape_sql_id(database_name) + " OWNER TO " + DbUtils::escape_sql_id(user);
   const bool test = DbUtils::query_execute_string(strQuery);
   if(!test)
   {
@@ -2004,10 +1609,10 @@ bool Base_DB::disable_user(const Glib::ustring& user)
   for(type_vec_strings::const_iterator iter = vecGroups.begin(); iter != vecGroups.end(); ++iter)
   {
     const Glib::ustring group = *iter;
-    remove_user_from_group(user, group);
+    DbUtils::remove_user_from_group(user, group);
   }
 
-  const Glib::ustring strQuery = "ALTER ROLE \"" + user + "\" NOLOGIN NOSUPERUSER NOCREATEDB NOCREATEROLE";
+  const Glib::ustring strQuery = "ALTER ROLE " + DbUtils::escape_sql_id(user) + " NOLOGIN NOSUPERUSER NOCREATEDB NOCREATEROLE";
   const bool test = DbUtils::query_execute_string(strQuery);
   if(!test)
   {
@@ -2024,14 +1629,6 @@ Glib::ustring Base_DB::get_active_layout_platform(Document* document)
   if(document)
     result = document->get_active_layout_platform();
 
-  if(result.empty())
-  {
-    //Make Glom use the special "maemo" layouts if they exist.
-    #ifdef GLOM_ENABLE_MAEMO
-    return "maemo";
-    #endif
-  }
-
   return result;
 }
 
