@@ -14,14 +14,14 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #include "config.h"
 #include "base_db_table_data.h"
 #include <libglom/data_structure/glomconversions.h>
-#include <glom/application.h>
+#include <glom/appwindow.h>
 #include <glom/python_embed/glom_python.h>
 #include <glom/utils_ui.h>
 #include <gtkmm/stock.h>
@@ -67,16 +67,20 @@ Gtk::TreeModel::iterator Base_DB_Table_Data::get_row_selected()
 
 bool Base_DB_Table_Data::record_new(bool use_entered_data, const Gnome::Gda::Value& primary_key_value)
 {
+  //std::cout << G_STRFUNC << ": debug: use_entered_data=" << use_entered_data << std::endl;
+
+  Document* document = get_document();
+
   sharedptr<const Field> fieldPrimaryKey = get_field_primary_key();
 
   const Glib::ustring primary_key_name = fieldPrimaryKey->get_name();
 
   type_vecConstLayoutFields fieldsToAdd = m_FieldsShown;
   if(m_TableFields.empty())
-    m_TableFields = get_fields_for_table(m_table_name);
+    m_TableFields = DbUtils::get_fields_for_table(document, m_table_name);
 
   //Add values for all fields, not just the shown ones:
-  //For instance, we must always add the primary key, and fields with default/calculated/lookup values:
+  //For instance, we must always add the primary key, and fields with default/calculated/lookup/auto-incremented values:
   for(type_vec_fields::const_iterator iter = m_TableFields.begin(); iter != m_TableFields.end(); ++iter)
   {
     //TODO: Search for the non-related field with the name, not just the field with the name:
@@ -90,60 +94,6 @@ bool Base_DB_Table_Data::record_new(bool use_entered_data, const Gnome::Gda::Val
     }
   }
 
-  Document* document = get_document();
-
-  //Calculate any necessary field values and enter them:
-  for(type_vecConstLayoutFields::const_iterator iter = fieldsToAdd.begin(); iter != fieldsToAdd.end(); ++iter)
-  {
-    sharedptr<const LayoutItem_Field> layout_item = *iter;
-
-    //If the user did not enter something in this field:
-    Gnome::Gda::Value value = get_entered_field_data(layout_item);
-
-    if(Conversions::value_is_empty(value)) //This deals with empty strings too.
-    {
-      const sharedptr<const Field>& field = layout_item->get_full_field_details();
-      if(field)
-      {
-        //If the default value should be calculated, then calculate it:
-        if(field->get_has_calculation())
-        {
-          const Glib::ustring calculation = field->get_calculation();
-          const type_map_fields field_values = get_record_field_values_for_calculation(m_table_name, fieldPrimaryKey, primary_key_value);
-
-          //We need the connection when we run the script, so that the script may use it.
-          // TODO: Is this function supposed to throw an exception?
-          sharedptr<SharedConnection> sharedconnection = connect_to_server(Application::get_application());
-
-            Glib::ustring error_message; //TODO: Check this.
-            const Gnome::Gda::Value value =
-              glom_evaluate_python_function_implementation(
-                field->get_glom_type(),
-                calculation,
-                field_values,
-                document,
-                m_table_name,
-                fieldPrimaryKey, primary_key_value,
-                sharedconnection->get_gda_connection(),
-                error_message);
-            set_entered_field_data(layout_item, value);
-        }
-
-        //Use default values (These are also specified in postgres as part of the field definition,
-        //so we could theoretically just not specify it here.)
-        //TODO_Performance: Add has_default_value()?
-        if(Conversions::value_is_empty(value))
-        {
-          const Gnome::Gda::Value default_value = field->get_default_value();
-          if(!Conversions::value_is_empty(default_value))
-          {
-            set_entered_field_data(layout_item, default_value);
-          }
-        }
-      }
-    }
-  }
-
   //Get all entered field name/value pairs:
   Glib::RefPtr<Gnome::Gda::SqlBuilder> builder = Gnome::Gda::SqlBuilder::create(Gnome::Gda::SQL_STATEMENT_INSERT);
   builder->set_table(m_table_name);
@@ -165,35 +115,80 @@ bool Base_DB_Table_Data::record_new(bool use_entered_data, const Gnome::Gda::Val
         Gnome::Gda::Value value;
 
         const sharedptr<const Field>& field = layout_item->get_full_field_details();
-        if(field)
+        if(!field)
+          continue;
+        
+        //Use the specified (generated) primary key value, if there is one:
+        if(primary_key_name == field_name && !Conversions::value_is_empty(primary_key_value))
         {
-          //Use the specified (generated) primary key value, if there is one:
-          if(primary_key_name == field_name && !Conversions::value_is_empty(primary_key_value))
-          {
-            value = primary_key_value;
-          }
-          else
-          {
-            if(use_entered_data)
-              value = get_entered_field_data(layout_item);
-          }
+          value = primary_key_value;
+        }
+        else
+        {
+          if(use_entered_data)
+            value = get_entered_field_data(layout_item);
+        }
 
-          /* //TODO: This would be too many small queries when adding one record.
-          //Check whether the value meets uniqueness constraints:
-          if(field->get_primary_key() || field->get_unique_key())
+        if(Conversions::value_is_empty(value)) //This deals with empty strings too.
+        {
+          //If the default value should be calculated, then calculate it:
+          if(field->get_has_calculation())
           {
-            if(!get_field_value_is_unique(m_table_name, layout_item, value))
-            {
-              //Ignore this field value. TODO: Warn the user about it.
-            }
+            const Glib::ustring calculation = field->get_calculation();
+            const type_map_fields field_values = get_record_field_values_for_calculation(m_table_name, fieldPrimaryKey, primary_key_value);
+
+            //We need the connection when we run the script, so that the script may use it.
+            // TODO: Is this function supposed to throw an exception?
+            sharedptr<SharedConnection> sharedconnection = connect_to_server(AppWindow::get_appwindow());
+
+            Glib::ustring error_message; //TODO: Check this.
+            value =
+              glom_evaluate_python_function_implementation(
+                field->get_glom_type(),
+                calculation,
+                field_values,
+                document,
+                m_table_name,
+                fieldPrimaryKey, primary_key_value,
+                sharedconnection->get_gda_connection(),
+                error_message);
           }
-          */
-          if(!value.is_null())
+        }
+
+        //Use default values (These are also specified in postgres as part of the field definition,
+        //so we could theoretically just not specify it here.)
+        //TODO_Performance: Add has_default_value()?
+        if(Conversions::value_is_empty(value))
+        {
+          value = field->get_default_value();
+        }
+
+        //Use auto-increment values:
+        if(field->get_auto_increment() && Conversions::value_is_empty(value))
+        {
+          value = 
+            DbUtils::get_next_auto_increment_value(m_table_name, field->get_name());
+        }
+
+        /* //TODO: This would be too many small queries when adding one record.
+        //Check whether the value meets uniqueness constraints:
+        if(field->get_primary_key() || field->get_unique_key())
+        {
+          if(!get_field_value_is_unique(m_table_name, layout_item, value))
           {
-            builder->add_field_value(field_name, value);
-            map_added[field_name] = true;
+            //Ignore this field value. TODO: Warn the user about it.
           }
         }
+        */
+        if(!value.is_null())
+        {
+          builder->add_field_value(field_name, value);
+          map_added[field_name] = true;
+        }
+        else
+        {
+          std::cerr << G_STRFUNC << ": value is null for field: " << field_name << std::endl;
+        }
       }
     }
   }
@@ -262,7 +257,7 @@ bool Base_DB_Table_Data::add_related_record_for_field(const sharedptr<const Layo
     const Glib::ustring message = _("Data may not be entered into this related field, because the related record does not yet exist, and the relationship does not allow automatic creation of new related records.");
     Gtk::MessageDialog dialog(Utils::bold_message(_("Related Record Does Not Exist")), true);
     dialog.set_secondary_text(message);
-    dialog.set_transient_for(*Application::get_application());
+    dialog.set_transient_for(*AppWindow::get_appwindow());
     dialog.run();
 
     //Clear the field again, discarding the entered data.
@@ -284,7 +279,7 @@ bool Base_DB_Table_Data::add_related_record_for_field(const sharedptr<const Layo
       Gtk::MessageDialog dialog(Utils::bold_message(_("Related Record Cannot Be Created")), true);
       //TODO: This is a very complex error message:
       dialog.set_secondary_text(message);
-      dialog.set_transient_for(*Application::get_application());
+      dialog.set_transient_for(*AppWindow::get_appwindow());
       dialog.run();
 
       //Clear the field again, discarding the entered data.
@@ -324,7 +319,9 @@ bool Base_DB_Table_Data::add_related_record_for_field(const sharedptr<const Layo
         set_entered_field_data(item_from_key, primary_key_value);
 
         //Set it in the database too:
-        sharedptr<Field> field_from_key = get_fields_for_table_one_field(relationship->get_from_table(), relationship->get_from_field()); //TODO_Performance.
+        Document* document = get_document();
+        sharedptr<Field> field_from_key = DbUtils::get_fields_for_table_one_field(document,
+          relationship->get_from_table(), relationship->get_from_field()); //TODO_Performance.
         if(!field_from_key)
         {
           std::cerr << G_STRFUNC << ": get_fields_for_table_one_field() failed." << std::endl;
@@ -334,7 +331,7 @@ bool Base_DB_Table_Data::add_related_record_for_field(const sharedptr<const Layo
         sharedptr<Field> parent_primary_key_field = get_field_primary_key();
         if(!parent_primary_key_field)
         {
-          g_warning("Base_DB_Table_Data::add_related_record_for_field(): get_field_primary_key() failed. table = %s", get_table_name().c_str());
+          std::cerr << G_STRFUNC << ": get_field_primary_key() failed. table = " << get_table_name() << std::endl;
           return false;
         }
         else
@@ -342,7 +339,7 @@ bool Base_DB_Table_Data::add_related_record_for_field(const sharedptr<const Layo
           const Gnome::Gda::Value parent_primary_key_value = get_primary_key_value_selected();
           if(parent_primary_key_value.is_null())
           {
-            g_warning("Base_DB_Table_Data::add_related_record_for_field(): get_primary_key_value_selected() failed. table = %s", get_table_name().c_str());
+            std::cerr << G_STRFUNC << ": get_primary_key_value_selected() failed. table = " << get_table_name() << std::endl;
             return false;
           }
           else
@@ -393,7 +390,7 @@ bool Base_DB_Table_Data::confirm_delete_record()
   const Glib::ustring message = _("Are you sure that you would like to delete this record? The data in this record will then be permanently lost.");
   Gtk::MessageDialog dialog(Utils::bold_message(_("Delete record")), true, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_NONE);
   dialog.set_secondary_text(message);
-  dialog.set_transient_for(*Application::get_application());
+  dialog.set_transient_for(*AppWindow::get_appwindow());
   dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
   dialog.add_button(Gtk::Stock::DELETE, Gtk::RESPONSE_OK);
 
@@ -429,7 +426,7 @@ Base_DB_Table_Data::type_signal_record_changed Base_DB_Table_Data::signal_record
 
 bool Base_DB_Table_Data::get_related_record_exists(const sharedptr<const Relationship>& relationship, const Gnome::Gda::Value& key_value)
 {
-  BusyCursor cursor(Application::get_application());
+  BusyCursor cursor(AppWindow::get_appwindow());
 
   bool result = false;
 
