@@ -14,19 +14,18 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 
-#ifndef BOX_DB_TABLE_H
-#define BOX_DB_TABLE_H
+#ifndef GLOM_BOX_DB_TABLE_H
+#define GLOM_BOX_DB_TABLE_H
 
 #include <glom/box_withbuttons.h>
 #include <glom/base_db_table.h>
-#include <glom/libglom/data_structure/field.h>
-#include <libglademm.h>
-#include <algorithm> //find_if used in various places.
+#include <libglom/data_structure/field.h>
+#include <gtkmm/builder.h>
 
 namespace Glom
 {
@@ -39,7 +38,7 @@ class Box_DB_Table
 {
 public: 
   Box_DB_Table();
-  Box_DB_Table(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade);
+  Box_DB_Table(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& builder);
   virtual ~Box_DB_Table();
     
   Gtk::Window* get_app_window();
@@ -48,4 +47,4 @@ public:
 
 } //namespace Glom
 
-#endif //BOX_DB_TABLE_H
+#endif // GLOM_BOX_DB_TABLE_H
