@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #include "config.h" //For GLOM_ENABLE_POSTGRESQL
@@ -29,10 +29,12 @@
 #include <libglom/connectionpool_backends/postgres_self.h>
 #endif //#ifdef GLOM_ENABLE_POSTGRESQL
 
+#include <iostream>
+
 namespace Glom
 {
 
-const char* Dialog_Connection::glade_id("dialog_connection");
+const char Dialog_Connection::glade_id[] = "dialog_connection";
 const bool Dialog_Connection::glade_developer(false);
 
 Dialog_Connection::Dialog_Connection(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& builder)
@@ -49,23 +51,13 @@ Dialog_Connection::Dialog_Connection(BaseObjectType* cobject, const Glib::RefPtr
   builder->get_widget("entry_password", m_entry_password);
   builder->get_widget("label_database", m_label_database);
   builder->get_widget("connection_note", m_label_note);
-
-#ifdef GLOM_ENABLE_MAEMO
-  // Without size request, this label enlarges the dialog significantly,
-  // and the text is still truncated.
-  m_label_note->set_size_request(400, -1);
-#endif
 }
 
 Dialog_Connection::~Dialog_Connection()
 {
 }
 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
 sharedptr<SharedConnection> Dialog_Connection::connect_to_server_with_connection_settings() const
-#else
-sharedptr<SharedConnection> Dialog_Connection::connect_to_server_with_connection_settings(std::auto_ptr<ExceptionConnection>& error) const
-#endif
 {
   //std::cout << "debug: Dialog_Connection::connect_to_server_with_connection_settings()" << std::endl;
 
@@ -83,7 +75,7 @@ sharedptr<SharedConnection> Dialog_Connection::connect_to_server_with_connection
   if(!document)
     return result;
 
-  //std::cout << "debug: Dialog_Connection::connect_to_server_with_connection_settings(): m_database_name=" << m_database_name << std::endl;
+  //std::cout << "debug: " << G_STRFUNC << ": m_database_name=" << m_database_name << std::endl;
   connection_pool->set_database(m_database_name);
 
 #ifdef GLOM_ENABLE_POSTGRESQL
@@ -101,11 +93,7 @@ sharedptr<SharedConnection> Dialog_Connection::connect_to_server_with_connection
   connection_pool->set_password(m_entry_password->get_text());
 
 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
   result = Base_DB::connect_to_server(const_cast<Dialog_Connection*>(this));
-#else
-  result = Base_DB::connect_to_server(const_cast<Dialog_Connection*>(this), error);
-#endif
 
 #ifdef GLOM_ENABLE_POSTGRESQL
   //Remember the port, 
@@ -184,7 +172,7 @@ void Dialog_Connection::load_from_document()
     set_database_name(document->get_connection_database());
   }
   else
-    g_warning("Dialog_Connection::load_from_document(): no document");
+    std::cerr << G_STRFUNC << ": no document" << std::endl;
 
 }
 
