@@ -14,16 +14,19 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
-#ifndef FILECHOOSER_EXPORT_GLOM_H
-#define FILECHOOSER_EXPORT_GLOM_H
+#ifndef GLOM_FILECHOOSER_EXPORT_H
+#define GLOM_FILECHOOSER_EXPORT_H
 
-#include <glom/libglom/document/document_glom.h>
+#include <libglom/document/document.h>
 #include <gtkmm/filechooserdialog.h>
 
+namespace Glom
+{
+
 class Dialog_Layout_Export;
 
 class FileChooser_Export :
@@ -33,23 +36,31 @@ public:
   FileChooser_Export();
   virtual ~FileChooser_Export();
 
-  void set_export_layout(const Document_Glom::type_mapLayoutGroupSequence& layout_groups, const Glib::ustring& table_name, Document_Glom* document);
+  void set_export_layout(const Document::type_list_layout_groups& layout_groups, const Glib::ustring& table_name, Document* document);
 
-  void get_layout_groups(Document_Glom::type_mapLayoutGroupSequence& layout_groups) const;
+  void get_layout_groups(Document::type_list_layout_groups& layout_groups) const;
 
-protected:
+private:
 
+#ifndef GLOM_ENABLE_CLIENT_ONLY
   void on_button_define_layout();
   void on_dialog_layout_hide();
+#endif
 
   //Member widgets:
-  Gtk::HBox m_extra_widget;
-  Gtk::Button m_button_format;
-  Glib::ustring m_table_name;
-  Dialog_Layout_Export* m_pDialogLayout;
+  Gtk::Box m_extra_widget;
 
-  Document_Glom::type_mapLayoutGroupSequence m_layout_groups;
-  Document_Glom* m_document;
+#ifndef GLOM_ENABLE_CLIENT_ONLY
+  Gtk::Button m_button_format;
+  Dialog_Layout_Export* m_pDialogLayout;
+#endif //GLOM_ENABLE_CLIENT_ONLY
+
+  Glib::ustring m_table_name;
+
+  Document::type_list_layout_groups m_layout_groups;
+  Document* m_document;
 };
 
-#endif
+} //namespace Glom
+
+#endif // GLOM_FILESCHOOSER_EXPORT_H
