@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #ifndef GLOM_GLADE_UTILS_H
@@ -23,8 +23,11 @@
 
 #include <iostream> // For std::cerr
 #include <gtkmm/builder.h>
-#include <glom/dialog_progress_creating.h>
+#include <gtkmm/window.h>
 #include <giomm/file.h>
+#include <glibmm/miscutils.h>
+#include <glibmm/markup.h>
+#include <glibmm/fileutils.h>
 
 namespace Glom
 {
@@ -34,7 +37,6 @@ namespace Utils
 
 inline std::string get_glade_file_path(const std::string& filename)
 {
-  // Check the path to the installed .glade file:
 #ifdef G_OS_WIN32
   gchar* directory = g_win32_get_package_installation_directory_of_module(0);
   const std::string result = Glib::build_filename(directory, Glib::build_filename("share/glom/glade", filename));
@@ -57,36 +59,39 @@ inline std::string get_glade_file_path(const std::string& filename)
   return Glib::build_filename(GLOM_PKGDATADIR_NOTINSTALLED, filename);
 }
 
+/** This assumes that there are no other top-level windows in the glade file.
+ * This allows us to get all object, including any necessary secondary objects,
+ * such as a GtkSpinButton's GtkAdjustment.
+ */
 template<class T_Widget>
-void helper_get_glade_widget_derived_with_warning(const std::string& filename, const Glib::ustring& id, T_Widget*& widget)
+void helper_get_glade_widget_derived_with_warning(const std::string& filename, const Glib::ustring& id, T_Widget*& widget, bool load_all)
 {
   Glib::RefPtr<Gtk::Builder> refXml;
 
-  #ifdef GLIBMM_EXCEPTIONS_ENABLED
   try
   {
-    refXml = Gtk::Builder::create_from_file(Utils::get_glade_file_path(filename), id);
+    const std::string filepath = Utils::get_glade_file_path(filename);
+    if(load_all)
+      refXml = Gtk::Builder::create_from_file(filepath);
+     else
+      refXml = Gtk::Builder::create_from_file(filepath, id);
   }
   catch(const Gtk::BuilderError& ex)
   {
-    std::cerr << ex.what() << std::endl;
+    std::cerr << G_STRFUNC << ": BuilderError Exception: " << ex.what() << std::endl;
   }
   catch(const Glib::MarkupError& ex)
   {
-    std::cerr << ex.what() << std::endl;
+    std::cerr << G_STRFUNC << ": MarkupError exception:" << ex.what() << std::endl;
   }
   catch(const Glib::FileError& ex)
   {
-    std::cerr << ex.what() << std::endl;
+    std::cerr << G_STRFUNC << ": FileError: exception" << ex.what() << std::endl;
   }
-#else
-  std::auto_ptr<Glib::Error> error;
-  refXml = Gtk::Builder::create_from_file(Utils::get_glade_file_path(filename), id, error);
-  if (error.get())
+  catch(const Glib::Error& ex)
   {
-    std::cerr << error->what() << std::endl;
+    std::cerr << G_STRFUNC << ": Exception of unexpected type: " << ex.what() << std::endl;
   }
-#endif
 
   if(refXml)
   {
@@ -94,32 +99,47 @@ void helper_get_glade_widget_derived_with_warning(const std::string& filename, c
   }
 }
 
+/** This should be used with classes that have a static glade_id member.
+ * This loads only the widget's own object, as specified by its ID.
+ */
 template<class T_Widget>
-void helper_get_glade_widget_derived_with_warning(const Glib::ustring& id, T_Widget*& widget)
+void get_glade_child_widget_derived_with_warning(T_Widget*& widget)
 {
-  helper_get_glade_widget_derived_with_warning("glom.glade", id, widget);
+  widget = 0;
+
+  // Check the path to the installed .glade file:
+  // The id is the same as the filename, in a developer/operator sub-directory:
+  const std::string filename = Glib::build_filename(
+      (T_Widget::glade_developer ? "developer" : "operator"),
+      std::string(T_Widget::glade_id) + ".glade"); 
+  
+  helper_get_glade_widget_derived_with_warning(filename, T_Widget::glade_id, widget, false /* load just this */);
 }
 
 /** This should be used with classes that have a static glade_id member.
+ * This assumes that there are no other top-level windows in the glade file.
+ * This allows us to get all object, including any necessary secondary objects,
+ * such as a GtkSpinButton's GtkAdjustment.
  */
 template<class T_Widget>
 void get_glade_widget_derived_with_warning(T_Widget*& widget)
 {
   widget = 0;
 
-  if(T_Widget::glade_developer)
-    helper_get_glade_widget_derived_with_warning("glom_developer.glade", T_Widget::glade_id, widget);
-  else
-    helper_get_glade_widget_derived_with_warning("glom.glade", T_Widget::glade_id, widget);
+  // Check the path to the installed .glade file:
+  // The id is the same as the filename, in a developer/operator sub-directory:
+  const std::string filename = Glib::build_filename(
+      (T_Widget::glade_developer ? "developer" : "operator"),
+      std::string(T_Widget::glade_id) + ".glade"); 
+  
+  helper_get_glade_widget_derived_with_warning(filename, T_Widget::glade_id, widget, true /* load_all */);
 }
 
-
 template<class T_Widget>
 void get_glade_widget_with_warning(const std::string& filename, const Glib::ustring& id, T_Widget*& widget)
 {
   Glib::RefPtr<Gtk::Builder> refXml;
 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
   try
   {
     refXml = Gtk::Builder::create_from_file(Utils::get_glade_file_path(filename), id);
@@ -136,14 +156,6 @@ void get_glade_widget_with_warning(const std::string& filename, const Glib::ustr
   {
     std::cerr << ex.what() << std::endl;
   }
-#else
-  std::auto_ptr<Glib::Error> error;
-  refXml = Gtk::Builder::create_from_file(Utils::get_glade_file_path(filename), id, error);
-  if (error.get())
-  {
-    std::cerr << error->what() << std::endl;
-  }
-#endif
 
   if(refXml)
   {
@@ -158,8 +170,6 @@ void get_glade_widget_with_warning(const std::string& filename, const Glib::ustr
     window->set_icon_name("glom");
 }
 
-Dialog_ProgressCreating* get_and_show_pulse_dialog(const Glib::ustring& message, Gtk::Window* parent_window);
-
 } //namespace Utils
 
 } //namespace Glom
