@@ -15,11 +15,13 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #include "csv_parser.h"
+#include <glibmm/convert.h>
+#include <glibmm/main.h>
 
 #include <cerrno>
 #include <iostream>
@@ -56,6 +58,7 @@ CsvParser::CsvParser(const std::string& encoding_charset)
 : m_raw(0),
   m_encoding(encoding_charset),
   m_input_position(0),
+  m_in_quotes(false),
   m_idle_connection(),
   m_line_number(0),
   m_state(STATE_NONE),
@@ -73,6 +76,10 @@ void CsvParser::set_file_and_start_parsing(const std::string& file_uri)
   Glib::RefPtr<Gio::File> file = Gio::File::create_for_uri(file_uri);
 
   set_state(CsvParser::STATE_PARSING);
+  
+  //TODO Get a rough estimate of the number of rows (for showing progress),
+  //by counting the number of lines:
+  //guint lines_count = 0;
 
   // Query the display name of the file to set in the title:
   file->query_info_async(sigc::bind(sigc::mem_fun(*this, &CsvParser::on_file_query_info), file),
@@ -190,7 +197,7 @@ void CsvParser::set_encoding(const Glib::ustring& encoding_charset)
     return;
 
   m_encoding = encoding_charset;
-  
+
   //Stop parsing if the encoding changes.
   //The caller should restart the parsing when wanted.
   clear();
@@ -247,8 +254,6 @@ Glib::ustring::const_iterator CsvParser::advance_field(const Glib::ustring::cons
       {
         break;
       }
-
-      continue;
     }
 
     field += c; // Just so that we don't need to iterate through the field again, since there is no Glib::ustring::substr(iter, iter)
@@ -266,6 +271,8 @@ void CsvParser::clear()
   //m_stream.reset();
   //m_raw.clear();
   m_rows.clear();
+  m_in_quotes = false;
+
   // Set to current encoding I guess ...
   //m_conv("UTF-8", encoding),
   m_input_position= 0;
@@ -332,9 +339,8 @@ bool CsvParser::on_idle_parse()
   const char* prev = prev_line_end;
 
   // Identify the record rows in the .csv file.
-  // We can't just search for newlines because they may be inside quotes too. 
+  // We can't just search for newlines because they may be inside quotes too.
   // TODO: Use a regex instead, to more easily handle quotes?
-  bool in_quotes = false;
   while(true)
   {
     //std::cout << "debug: checking start: " << std::string(prev, 10) << std::endl;
@@ -365,21 +371,21 @@ bool CsvParser::on_idle_parse()
     {
       // There is a null byte in the conversion. Because normal text files don't
       // contain null bytes this only occurs when converting, for example, a UTF-16
-      // file from ISO-8859-1 to UTF-8 (note that the UTF-16 file is valid ISO-8859-1 - 
+      // file from ISO-8859-1 to UTF-8 (note that the UTF-16 file is valid ISO-8859-1 -
       // it just contains lots of nullbytes). We therefore produce an error here.
       //std::cerr << G_STRFUNC << ": on_idle_parse(): Encoding error" << std::endl;
       set_state(STATE_ENCODING_ERROR);
       signal_encoding_error().emit();
       return false;  //Stop calling the idle handler.
     }
-    else if(in_quotes)
+    else if(m_in_quotes)
     {
       // Ignore newlines inside quotes.
 
       // End quote:
       if(ch == (char)QUOTE)
       {
-        in_quotes = false;
+        m_in_quotes = false;
 
         /*
         const size_t len = pos - prev;
@@ -400,7 +406,7 @@ bool CsvParser::on_idle_parse()
       // Start quote:
       if(ch == (char)QUOTE)
       {
-        in_quotes = true;
+        m_in_quotes = true;
         prev = pos + 1;
         continue;
       }
@@ -421,7 +427,7 @@ bool CsvParser::on_idle_parse()
       prev = pos + 1;
 
       // Skip DOS-style linebreak (\r\n)
-      if(ch == '\r' 
+      if(ch == '\r'
          && prev != outbuf && *prev == '\n')
       {
          ++prev;
@@ -492,14 +498,19 @@ void CsvParser::do_line_scanned(const Glib::ustring& line, guint line_number)
   signal_line_scanned().emit(row, line_number);
 }
 
-void CsvParser::on_file_read(const Glib::RefPtr<Gio::AsyncResult>& result, const Glib::RefPtr<Gio::File>& source)
+void CsvParser::ensure_idle_handler_connection()
 {
-  // TODO: Introduce CsvParser::is_idle_handler_connected() instead?
   if(!m_idle_connection.connected())
   {
     m_idle_connection = Glib::signal_idle().connect(
-      sigc::mem_fun(*this, &CsvParser::on_idle_parse));
+      sigc::mem_fun(*this, &CsvParser::on_idle_parse),
+      Glib::PRIORITY_DEFAULT_IDLE - 20);
   }
+}
+
+void CsvParser::on_file_read(const Glib::RefPtr<Gio::AsyncResult>& result, const Glib::RefPtr<Gio::File>& source)
+{
+  ensure_idle_handler_connection();
 
   try
   {
@@ -530,6 +541,9 @@ void CsvParser::copy_buffer_and_continue_reading(gssize size)
     m_buffer.reset(0);
     m_stream.reset();
   }
+
+  //In case it has been disconnected too early due to timing problems:
+  ensure_idle_handler_connection();
 }
 
 void CsvParser::on_buffer_read(const Glib::RefPtr<Gio::AsyncResult>& result)
