@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #include "dialog_import_csv.h"
@@ -27,8 +27,12 @@
 #include <gtkmm/messagedialog.h>
 #include <gtkmm/cellrenderercombo.h>
 #include <glom/utils_ui.h>
+#include <gtkmm/main.h>
+#include <glibmm/convert.h>
+#include <glibmm/markup.h>
 #include <glibmm/i18n.h>
 #include <cerrno>
+#include <iostream>
 
 namespace
 {
@@ -76,10 +80,17 @@ Dialog_Import_CSV::Dialog_Import_CSV(BaseObjectType* cobject, const Glib::RefPtr
   builder->get_widget("import_csv_sample_rows", m_sample_rows);
   builder->get_widget("import_csv_advice_label", m_advice_label);
   builder->get_widget("import_csv_error_label", m_error_label);
-#ifdef GLIBMM_EXCEPTIONS_ENABLED  
+
   if(!m_sample_view || !m_encoding_combo || !m_target_table || !m_encoding_info || !m_first_line_as_title || !m_sample_rows || !m_error_label)
     throw std::runtime_error("Missing widgets from glade file for Dialog_Import_CSV");
-#endif
+
+  //Set the adjustment details, to avoid a useless 0-to-0 range and a 0 incremenet.
+  //We don't do this the Glade file because GtkBuilder wouldn't find the
+  //associated adjustment object unless we specified it explictly:
+  //See http://bugzilla.gnome.org/show_bug.cgi?id=575714
+  m_sample_rows->set_range(0, 100);
+  m_sample_rows->set_increments(1, 10);
+  m_sample_rows->set_value(2); //A sensible default.
 
   //Fill the list of encodings:
   m_encoding_model = Gtk::ListStore::create(m_encoding_columns);
@@ -103,8 +114,6 @@ Dialog_Import_CSV::Dialog_Import_CSV(BaseObjectType* cobject, const Glib::RefPtr
     row[m_encoding_columns.m_col_charset] = encoding.get_charset();
   }
 
-  m_sample_rows->set_value(2); //A sensible default.
-
   Gtk::CellRendererText* renderer = Gtk::manage(new Gtk::CellRendererText);
   m_encoding_combo->set_model(m_encoding_model);
   m_encoding_combo->pack_start(*renderer);
@@ -207,7 +216,7 @@ void Dialog_Import_CSV::import(const Glib::ustring& uri, const Glib::ustring& in
       }
     }
 
-    // Create the sorted version of this model, 
+    // Create the sorted version of this model,
     // so the user sees the fields in alphabetical order:
     m_field_model_sorted = Gtk::TreeModelSort::create(m_field_model);
     m_field_model_sorted->set_sort_column(m_field_columns.m_col_field_name, Gtk::SORT_ASCENDING);
@@ -217,11 +226,6 @@ void Dialog_Import_CSV::import(const Glib::ustring& uri, const Glib::ustring& in
   }
 }
 
-guint Dialog_Import_CSV::get_column_count() const
-{
-  return m_cols_count;
-}
-
 sharedptr<const Field> Dialog_Import_CSV::get_field_for_column(guint col) const
 {
   if(col >= m_fields.size())
@@ -248,7 +252,7 @@ void Dialog_Import_CSV::clear()
   // TODO: Do we explicitely need to cancel async operations?
   // TODO: Disconnect idle handlers
   m_sample_model.reset();
-  m_sample_view->remove_all_columns();
+  Utils::treeview_delete_all_columns(m_sample_view);
   m_sample_view->set_model(m_sample_model);
   m_field_model.reset();
   m_field_model_sorted.reset();
@@ -429,7 +433,7 @@ void Dialog_Import_CSV::begin_parse()
   // Clear sample preview since we reparse everything, perhaps with
   // another encoding.
   m_sample_model.reset();
-  m_sample_view->remove_all_columns();
+  Utils::treeview_delete_all_columns(m_sample_view);
   m_sample_view->set_model(m_sample_model); // Empty model
   m_parser->clear();
 
@@ -446,7 +450,7 @@ void Dialog_Import_CSV::on_parser_encoding_error()
   m_parser->clear();
   // Clear sample preview (TODO: Let it visible, and only remove when reparsing?)
   m_sample_model.reset();
-  m_sample_view->remove_all_columns();
+  Utils::treeview_delete_all_columns(m_sample_view);
   m_sample_view->set_model(m_sample_model); // Empty model
 
   // Don't allow the import button to be pressed when an error occured. This
@@ -533,15 +537,9 @@ Gtk::TreeViewColumn* Dialog_Import_CSV::create_sample_column(const Glib::ustring
 Gtk::CellRendererCombo* Dialog_Import_CSV::create_sample_cell(guint index)
 {
   Gtk::CellRendererCombo* cell = new Gtk::CellRendererCombo;
-#ifdef GLIBMM_PROPERTIES_ENABLED
   cell->property_model() = m_field_model_sorted;
   cell->property_text_column() = 0;
   cell->property_has_entry() = false;
-#else
-  cell->set_property("model", m_field_model_sorted);
-  cell->set_property("text-column", 0);
-  cell->set_property("has_entry", false);
-#endif
   cell->signal_edited().connect(sigc::bind(sigc::mem_fun(*this, &Dialog_Import_CSV::on_field_edited), index));
 
   return cell;
@@ -551,10 +549,8 @@ void Dialog_Import_CSV::line_data_func(Gtk::CellRenderer* renderer, const Gtk::T
 {
   const int row = (*iter)[m_sample_columns.m_col_row];
   Gtk::CellRendererText* renderer_text = dynamic_cast<Gtk::CellRendererText*>(renderer);
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
   if(!renderer_text)
     throw std::logic_error("CellRenderer is not a CellRendererText in line_data_func");
-#endif
 
   if(row == -1)
     renderer_text->set_property("text", Glib::ustring(_("Target Field")));
@@ -566,9 +562,7 @@ void Dialog_Import_CSV::field_data_func(Gtk::CellRenderer* renderer, const Gtk::
 {
   const int row = (*iter)[m_sample_columns.m_col_row];
   Gtk::CellRendererCombo* renderer_combo = dynamic_cast<Gtk::CellRendererCombo*>(renderer);
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
   if(!renderer_combo) throw std::logic_error("CellRenderer is not a CellRendererCombo in field_data_func");
-#endif
 
   Glib::ustring text;
   bool editable = false;
@@ -633,13 +627,13 @@ void Dialog_Import_CSV::field_data_func(Gtk::CellRenderer* renderer, const Gtk::
       }
     }
   }
-  
+
   renderer_combo->set_property("text", text);
   renderer_combo->set_property("editable", editable);
 }
 
 
-/**  Parse a row from a .cvs file. Note that this "line" might have newline 
+/**  Parse a row from a .cvs file. Note that this "line" might have newline
   *  characters inside one field value, inside quotes.
   **/
 void Dialog_Import_CSV::on_field_edited(const Glib::ustring& path, const Glib::ustring& new_text, guint column_number)
@@ -733,7 +727,6 @@ void Dialog_Import_CSV::validate_primary_key()
 void Dialog_Import_CSV::on_parser_file_read_error(const Glib::ustring& error_message)
 {
   std::string filename;
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
   try
   {
     filename = Glib::filename_from_uri(m_file_uri);
@@ -741,19 +734,15 @@ void Dialog_Import_CSV::on_parser_file_read_error(const Glib::ustring& error_mes
   catch(const Glib::ConvertError& ex)
   {
     std::cerr << "Glib::filename_from_uri() failed: " << ex.what() << std::endl;
+
+    show_error_dialog(_("Could Not Open file"),
+      Glib::ustring::compose(_("The file at \"%1\" could not be opened: %2"), filename, error_message) );
   }
-#else
-  std::auto_ptr<Glib::Error> error;
-  filename = Glib::filename_from_uri(m_file_uri, error);
-#endif
-  
-  show_error_dialog(_("Could Not Open file"),
-    Glib::ustring::compose(_("The file at \"%1\" could not be opened: %2"), filename, error_message) );
 }
 
 void Dialog_Import_CSV::on_parser_have_display_name(const Glib::ustring& display_name)
 {
-  set_title(_("Import From CSV File: ") + display_name);
+  set_title( Glib::ustring::compose(_("Import From CSV File: %1"), display_name) );
 }
 
 void Dialog_Import_CSV::on_parser_state_changed()
@@ -768,4 +757,3 @@ Dialog_Import_CSV::type_signal_state_changed Dialog_Import_CSV::signal_state_cha
 }
 
 } //namespace Glom
-
