@@ -14,23 +14,19 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
-#include <libglom/libglom_config.h> // GLOM_ENABLE_CLIENT_ONLY
-#include "appstate.h"
+#include <libglom/libglom_config.h>
+#include <libglom/appstate.h>
 
 
 namespace Glom
 {
 
 AppState::AppState()
-#ifdef GLOM_ENABLE_CLIENT_ONLY
-: m_userlevel(USERLEVEL_OPERATOR)
-#else
 : m_userlevel(USERLEVEL_DEVELOPER)
-#endif
 {
 }
 
@@ -46,11 +42,6 @@ AppState::userlevels AppState::get_userlevel() const
 
 void AppState::set_userlevel(userlevels value)
 {
-  //Make sure that developer mode is never possible in client-only mode:
-  #ifdef GLOM_ENABLE_CLIENT_ONLY
-  value = USERLEVEL_OPERATOR;
-  #endif //GLOM_ENABLE_CLIENT_ONLY
-
   if(m_userlevel != value)
   {
     m_userlevel = value;
