@@ -14,14 +14,14 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #ifndef GLOM_MODE_DATA_CALCINPROGRESS_H
 #define GLOM_MODE_DATA_CALCINPROGRESS_H
 
-#include <glom/libglom/data_structure/field.h>
+#include <libglom/data_structure/field.h>
 
 namespace Glom
 {
