@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #include <libglom/libglom_config.h>
@@ -23,11 +23,14 @@
 #include <libglom/connectionpool.h>
 #include <libglom/document/document.h>
 #include <libglom/utils.h>
+#include <libglom/db_utils.h>
 //#include <libgdamm/connectionevent.h>
 
 #include <libglom/connectionpool_backends/postgres_central.h>
 #include <libglom/connectionpool_backends/postgres_self.h>
-# include <libglom/connectionpool_backends/sqlite.h>
+#include <libglom/connectionpool_backends/sqlite.h>
+
+#include <glibmm/main.h>
 
 #ifdef G_OS_WIN32
 # include <windows.h>
@@ -38,6 +41,8 @@
 
 #include <signal.h> //To catch segfaults
 
+#include <glibmm/convert.h>
+#include <glibmm/miscutils.h>
 #include <glibmm/i18n.h>
 
 #include <iostream>
@@ -86,8 +91,9 @@ SharedConnection::type_signal_finished SharedConnection::signal_finished()
 void SharedConnection::close()
 {
   if(m_gda_connection)
+  {
     m_gda_connection.reset();
-
+  }
 
   //Tell the connection pool that we have finished with this connection.
   //It might want to close it, or keep it open if somebody else is using it.
@@ -106,17 +112,16 @@ ConnectionPool::ConnectionPool()
   m_sharedconnection_refcount(0),
   m_ready_to_connect(false),
   m_pFieldTypes(0),
-  m_show_debug_output(false)
+  m_show_debug_output(false),
+  m_auto_server_shutdown(true),
+  m_fake_connection(false)
 {
 }
 
 ConnectionPool::~ConnectionPool()
 {
-  if(m_pFieldTypes)
-  {
-    delete m_pFieldTypes;
-    m_pFieldTypes = 0;
-  }
+  delete m_pFieldTypes;
+  m_pFieldTypes = 0;
 }
 
 //static
@@ -162,7 +167,7 @@ void ConnectionPool::setup_from_document(const Document* document)
 
   default:
     //on_document_load() should have checked for this already, informing the user.
-    std::cerr << "Glom: setup_connection_pool_from_document(): Unhandled hosting mode: " << document->get_hosting_mode() << std::endl;
+    std::cerr << G_STRFUNC << ": Unhandled hosting mode: " << document->get_hosting_mode() << std::endl;
     g_assert_not_reached();
     break;
   }
@@ -176,11 +181,8 @@ void ConnectionPool::setup_from_document(const Document* document)
 
 void ConnectionPool::delete_instance()
 {
-  if(m_instance)
-  {
-    delete m_instance;
-    m_instance = 0;
-  }
+  delete m_instance;
+  m_instance = 0;
 }
 
 bool ConnectionPool::get_ready_to_connect() const
@@ -208,14 +210,16 @@ const ConnectionPool::Backend* ConnectionPool::get_backend() const
   return m_backend.get();
 }
 
-
+bool ConnectionPool::get_backend_supports_cursor() const
+{
+  //TODO: Is there a generic libgda way to discover this?
+  const ConnectionPoolBackends::Sqlite* sqlite_backend =
+    dynamic_cast<const ConnectionPoolBackends::Sqlite*>(get_backend());
+  return !sqlite_backend;
+}
 
 //static:
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
 sharedptr<SharedConnection> ConnectionPool::get_and_connect()
-#else
-sharedptr<SharedConnection> ConnectionPool::get_and_connect(std::auto_ptr<ExceptionConnection>& error)
-#endif // GLIBMM_EXCEPTIONS_ENABLED
 {
   sharedptr<SharedConnection> result(0);
 
@@ -225,19 +229,23 @@ sharedptr<SharedConnection> ConnectionPool::get_and_connect(std::auto_ptr<Except
 
   if(!(connection_pool->m_backend.get()))
   {
-    std::cerr << "ConnectionPool::get_and_connect(): m_backend is null." << std::endl;
+    std::cerr << G_STRFUNC << ": m_backend is null." << std::endl;
     return result; //TODO: Return a FAILURE_NO_BACKEND error?, though that would be tedious.
   }
 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
   result = connection_pool->connect();
-#else
-  result = connection_pool->connect(error);
-#endif // GLIBMM_EXCEPTIONS_ENABLED
 
   return result;
 }
 
+bool ConnectionPool::get_instance_is_ready()
+{
+  ConnectionPool* instance = get_instance();
+  if(!instance)
+    return false;
+
+  return instance->get_ready_to_connect();
+}
 
 
 // Store the connection for a few seconds in case it
@@ -259,16 +267,14 @@ static bool on_connection_pool_cache_timeout()
 }
 
 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
 sharedptr<SharedConnection> ConnectionPool::connect()
-#else
-sharedptr<SharedConnection> ConnectionPool::connect(std::auto_ptr<ExceptionConnection>& error)
-#endif
 {
+  //std::cout << G_STRFUNC << ": debug" << std::endl;
+
   //Don't try to connect if we don't have a backend to connect to.
   g_return_val_if_fail(m_backend.get(), sharedptr<SharedConnection>(0));
 
-  if(get_ready_to_connect())
+  if(get_ready_to_connect() || m_fake_connection)
   {
     if(connection_cached)
     {
@@ -302,57 +308,59 @@ sharedptr<SharedConnection> ConnectionPool::connect(std::auto_ptr<ExceptionConne
     }
     else
     {
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
-      std::auto_ptr<ExceptionConnection> error;
-#endif
-      m_refGdaConnection = m_backend->connect(m_database, get_user(), get_password(), error);
-
-      if(!m_refGdaConnection)
+      try
       {
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
-        throw *error;
-#endif
-        return sharedptr<SharedConnection>(0);
+        m_refGdaConnection = m_backend->connect(m_database, get_user(), get_password(), m_fake_connection);
       }
-      else
+      catch(const Glib::Error& ex)
+      {
+        throw ex;
+      }
+
       {
         //Allow get_meta_store_data() to succeed:
         //Hopefully this (and the update_meta_store_for_table() calls) is all we need.
         //std::cout << "DEBUG: Calling update_meta_store_data_types() ..." << std::endl;
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
         try
         {
           m_refGdaConnection->update_meta_store_data_types();
         }
         catch(const Glib::Error& ex)
         {
-          std::cerr << "ConnectionPool::connect(): update_meta_store_data_types() failed: " << ex.what() << std::endl;
+          //If the conneciton was not opened, because it is a fake connection,
+          //then we should not be surprised that this fails,
+          //and a warning will only be useful later when get_meta_store_data() fails when used in FieldTypes, from Field::set_*().
+          if(!m_fake_connection)
+          {
+            std::cerr << G_STRFUNC << ": update_meta_store_data_types() failed: " << ex.what() << std::endl;
+          }
         }
         //std::cout << "DEBUG: ... update_meta_store_data_types() has finished." << std::endl;
-#else
-        std::auto_ptr<Glib::Error> ex;
-        m_refGdaConnection->update_meta_store_data_types(ex);
-        if (ex.get())
-          std::cerr << "ConnectionPool::connect(): update_meta_store_data_types() failed: " << ex->what() << std::endl;
-#endif
+
         //std::cout << "DEBUG: Calling update_meta_store_table_names() ..." << std::endl;
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
+
         try
         {
           //update_meta_store_table_names() has been known to throw an exception.
           //Glom is mostly unusable when it fails, but that's still better than a crash.
-          m_refGdaConnection->update_meta_store_table_names(m_backend->get_public_schema_name());
+          //std::cout << G_STRFUNC << ": Before update_meta_store_table_name()" << std::endl;
+          const bool test = m_refGdaConnection->update_meta_store_table_names(m_backend->get_public_schema_name());
+          if(!test && !m_fake_connection)
+          {
+            std::cerr << G_STRFUNC << ": update_meta_store_table_names() failed without an exception." << std::endl;
+          }
         }
         catch(const Glib::Error& ex)
         {
-          std::cerr << "ConnectionPool::connect(): update_meta_store_table_names() failed: " << ex.what() << std::endl;
+          //If the connection was not opened, because it is a fake connection,
+          //then we should not be surprised that this fails,
+          //and a warning will only be useful later when get_meta_store_data() fails when used in get_table_names_from_database().
+          if(!m_fake_connection)
+          {
+            std::cerr << G_STRFUNC << ": update_meta_store_table_names() failed: " << ex.what() << std::endl;
+          }
         }
         //std::cout << "DEBUG: ... update_meta_store_table_names() has finished." << std::endl;
-#else
-        m_refGdaConnection->update_meta_store_table_names(m_backend->get_public_schema_name(), ex);
-        if (ex.get())
-          std::cerr << "ConnectionPool::connect(): update_meta_store_data_types() failed: " << ex->what() << std::endl;
-#endif
 
         // Connection succeeded
         // Create the fieldtypes member if it has not already been done:
@@ -370,37 +378,22 @@ sharedptr<SharedConnection> ConnectionPool::connect(std::auto_ptr<ExceptionConne
           avahi_start_publishing(); //Stopped in the signal_finished handler.
 #endif // !G_OS_WIN32
 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
         return connect(); //Call this method recursively. This time m_refGdaConnection exists.
-#else
-        return connect(error); //Call this method recursively. This time m_refGdaConnection exists.
-#endif // GLIBMM_EXCEPTIONS_ENABLED
       }
     }
   }
   else
   {
-    //g_warning("ConnectionPool::connect(): not ready to connect.");
+    //std::cerr << G_STRFUNC << ": not ready to connect." << std::endl;
   }
 
   return sharedptr<SharedConnection>(0);
 }
 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
-void ConnectionPool::create_database(const Glib::ustring& database_name)
-#else
-void ConnectionPool::create_database(const Glib::ustring& database_name, std::auto_ptr<Glib::Error>& error)
-#endif
+void ConnectionPool::create_database(const SlotProgress& slot_progress, const Glib::ustring& database_name)
 {
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
-  std::auto_ptr<Glib::Error> error;
-#endif
   if(m_backend.get())
-    m_backend->create_database(database_name, get_user(), get_password(), error);
-
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
-  if(error.get()) throw *error;
-#endif
+    m_backend->create_database(slot_progress, database_name, get_user(), get_password());
 }
 
 void ConnectionPool::set_user(const Glib::ustring& value)
@@ -408,7 +401,7 @@ void ConnectionPool::set_user(const Glib::ustring& value)
   if(value.empty())
   {
 #ifdef GLOM_CONNECTION_DEBUG
-    std::cout << "debug: ConnectionPool::set_user(): user is empty." << std::endl;
+    std::cout << "debug: " << G_STRFUNC << ": user is empty." << std::endl;
 #endif
   }
 
@@ -466,7 +459,13 @@ bool ConnectionPool::convert_backup(const SlotProgress& slot_progress, const std
   }
   catch(const Glib::Error& ex)
   {
-    std::cerr << "ConnectionPool::connect(): update_meta_store_table_names() failed: " << ex.what() << std::endl;
+    //If the conneciton was not opened, because it is a fake connection,
+    //then we should not be surprised that this fails,
+    //and a warning will only be useful later when get_meta_store_data() fails when used in get_table_names_from_database().
+    if(!m_fake_connection)
+    {
+      std::cerr << "ConnectionPool::connect(): update_meta_store_table_names() failed: " << ex.what() << std::endl;
+    }
   }
 
   return result;
@@ -503,18 +502,16 @@ Glib::ustring ConnectionPool::get_database() const
   return m_database;
 }
 
-Field::sql_format ConnectionPool::get_sql_format() const
-{
-  g_assert(m_backend.get());
-  return m_backend->get_sql_format();
-}
-
 const FieldTypes* ConnectionPool::get_field_types() const
 {
+  //TODO: Investigate this:
+  //if(!m_pFieldTypes)
+  //  std::cerr << G_STRFUNC << ": m_pFieldTypes is null but this should never happen." << std::endl;
+
   return m_pFieldTypes;
 }
 
-Glib::ustring ConnectionPool::get_string_find_operator() const
+Gnome::Gda::SqlOperatorType ConnectionPool::get_string_find_operator() const
 {
   g_assert(m_backend.get());
   return m_backend->get_string_find_operator();
@@ -522,11 +519,19 @@ Glib::ustring ConnectionPool::get_string_find_operator() const
 
 void ConnectionPool::invalidate_connection()
 {
+  //std::cerr << G_STRFUNC << ": debug" << std::endl;
   connection_cached.clear();
   connection_cached_timeout_connection.disconnect();
   connection_cached_finished_connection.disconnect();
+
+  if(m_refGdaConnection)
+    m_refGdaConnection->close();
+    
   m_refGdaConnection.reset();
   m_sharedconnection_refcount = 0;
+
+  delete m_pFieldTypes;
+  m_pFieldTypes = 0;
 }
 
 void ConnectionPool::on_sharedconnection_finished()
@@ -541,7 +546,7 @@ void ConnectionPool::on_sharedconnection_finished()
   {
     //There should be no copies of the m_refConnection, so the Gnome::Gda::Connection destructor should
     //run when we clear this last RefPtr of it, but we will explicitly close it just in case.
-    //g_warning("ConnectionPool::on_sharedconnection_finished(): closing GdaConnection");
+    //std::cerr << G_STRFUNC << ": closing GdaConnection" << std::endl;
     m_refGdaConnection->close();
 
     m_refGdaConnection.reset();
@@ -558,18 +563,13 @@ void ConnectionPool::on_sharedconnection_finished()
 //static
 bool ConnectionPool::handle_error_cerr_only()
 {
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
   sharedptr<SharedConnection> sharedconnection = get_and_connect();
-#else
-  std::auto_ptr<ExceptionConnection> conn_error;
-  sharedptr<SharedConnection> sharedconnection = get_and_connect(conn_error);
-  // Ignore error, sharedconnection presence is checked below
-#endif
+
   if(sharedconnection)
   {
     Glib::RefPtr<Gnome::Gda::Connection> gda_connection = sharedconnection->get_gda_connection();
 
-    typedef std::list< Glib::RefPtr<Gnome::Gda::ConnectionEvent> > type_list_errors;
+    typedef std::vector< Glib::RefPtr<Gnome::Gda::ConnectionEvent> > type_list_errors;
     type_list_errors list_errors = gda_connection->get_events();
 
     if(!list_errors.empty())
@@ -584,7 +584,7 @@ bool ConnectionPool::handle_error_cerr_only()
             error_details += '\n'; //Add newline after each error.
 
           error_details += (*iter)->get_description();
-          std::cerr << "Internal error (Database): " << error_details << std::endl;
+          std::cerr << G_STRFUNC << ": Internal error (Database): " << error_details << std::endl;
         }
       }
 
@@ -610,7 +610,7 @@ static void on_linux_signal(int signum)
 {
   ConnectionPool* connection_pool = ConnectionPool::get_instance();
   if(!connection_pool)
-  return;
+    return;
 
   if(signum == SIGSEGV)
   {
@@ -641,7 +641,8 @@ ConnectionPool::StartupErrors ConnectionPool::startup(const SlotProgress& slot_p
 
   //If we crash while running (unlikely, hopefully), then try to cleanup.
   //Comment this out if you want to see the backtrace in a debugger.
-  previous_sig_handler = signal(SIGSEGV, &on_linux_signal);
+  if(m_auto_server_shutdown)
+    previous_sig_handler = signal(SIGSEGV, &on_linux_signal);
 
   return started;
 }
@@ -659,21 +660,25 @@ bool ConnectionPool::cleanup(const SlotProgress& slot_progress)
 
   bool result = false;
 
+  //Make sure that no connections are still open.
+  //Otherwise database shutdown could fail.
+  //And make sure that connect() tries to make a new connection:
+  invalidate_connection();
+      
   if(m_backend.get())
     result = m_backend->cleanup(slot_progress);
 
-  //Make sure that connect() tries to make a new connection:
-  invalidate_connection();
-
-
 #ifndef G_OS_WIN32
   /* Stop advertising the self-hosting database server via avahi: */
   //avahi_stop_publishing();
 #endif // !G_OS_WIN32
 
   //We don't need the segfault handler anymore:
-  signal(SIGSEGV, previous_sig_handler);
-  previous_sig_handler = SIG_DFL; /* Arbitrary default */
+  if(previous_sig_handler != SIG_DFL) /* Arbitrary default */
+  {
+    signal(SIGSEGV, previous_sig_handler);
+    previous_sig_handler = SIG_DFL; /* Arbitrary default */
+  }
 
   return result;
 }
@@ -687,134 +692,112 @@ bool ConnectionPool::set_network_shared(const SlotProgress& slot_progress, bool
     return false;
 }
 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
-bool ConnectionPool::add_column(const Glib::ustring& table_name, const sharedptr<const Field>& field)
-#else
-bool ConnectionPool::add_column(const Glib::ustring& table_name, const sharedptr<const Field>& field, std::auto_ptr<Glib::Error>& error)
-#endif
+bool ConnectionPool::add_column(const Glib::ustring& table_name, const sharedptr<const Field>& field) throw()
 {
   sharedptr<SharedConnection> conn;
   if(!m_refGdaConnection)
   {
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
     conn = connect();
-#else
-    std::auto_ptr<ExceptionConnection> local_error;
-    // TODO: We can't rethrow local_error here since ExceptionConnection does
-    // not derive from Glib::Error
-    conn = connect(local_error);
-#endif
-  }
-
-  if(!m_refGdaConnection)
-    return false;
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
-  std::auto_ptr<Glib::Error> error;
-#endif
-  const bool result = m_backend->add_column(m_refGdaConnection, table_name, field, error);
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
-  if(error.get()) throw *error;
-  m_refGdaConnection->update_meta_store_table(table_name, m_backend->get_public_schema_name());
-#else
-  if(error.get())
-    std::cerr << "Error: " << error->what() << std::endl;
-  m_refGdaConnection->update_meta_store_table(table_name, m_backend->get_public_schema_name(), error);
-  if(error.get())
-    std::cerr << "Error: " << error->what() << std::endl;
-#endif
-  return result;
-}
-
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
-bool ConnectionPool::drop_column(const Glib::ustring& table_name, const Glib::ustring& field_name)
-#else
-bool ConnectionPool::drop_column(const Glib::ustring& table_name, const Glib::ustring& field_name, std::auto_ptr<Glib::Error>& error)
-#endif
-{
-  sharedptr<SharedConnection> conn;
-  if(!m_refGdaConnection)
-  {
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
-    conn = connect();
-#else
-    std::auto_ptr<ExceptionConnection> local_error;
-    // TODO: We can't rethrow local_error here since ExceptionConnection does
-    // not derive from Glib::Error
-    conn = connect(local_error);
-#endif
   }
 
   if(!m_refGdaConnection)
     return false;
 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
-  std::auto_ptr<Glib::Error> error;
-#endif
-  const bool result = m_backend->drop_column(m_refGdaConnection, table_name, field_name, error);
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
-  if(error.get()) throw *error;
-  m_refGdaConnection->update_meta_store_table(table_name, m_backend->get_public_schema_name());
-#else
-  if(error.get())
-    std::cerr << "Error: " << error->what() << std::endl;
-  m_refGdaConnection->update_meta_store_table(table_name, m_backend->get_public_schema_name(), error);
-  if(error.get())
-    std::cerr << "Error: " << error->what() << std::endl;
-#endif
-  return result;
+  try
+  {
+    const bool result = m_backend->add_column(m_refGdaConnection, table_name, field);
+    m_refGdaConnection->update_meta_store_table(table_name, m_backend->get_public_schema_name());
+    return result;
+  }
+  catch(const Glib::Error& ex)
+  {
+    std::cerr << G_STRFUNC << ": exception:" << ex.what() << std::endl;
+  }
+
+  return false;
 }
 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
-bool ConnectionPool::change_column(const Glib::ustring& table_name, const sharedptr<const Field>& field_old, const sharedptr<const Field>& field)
-#else
-bool ConnectionPool::change_column(const Glib::ustring& table_name, const sharedptr<const Field>& field_old, const sharedptr<const Field>& field, std::auto_ptr<Glib::Error>& error)
-#endif
+bool ConnectionPool::drop_column(const Glib::ustring& table_name, const Glib::ustring& field_name) throw()
+{
+  sharedptr<SharedConnection> conn;
+  if(!m_refGdaConnection)
+  {
+    conn = connect();
+  }
+
+  if(!m_refGdaConnection)
+    return false;
+
+  try
+  {
+    const bool result = m_backend->drop_column(m_refGdaConnection, table_name, field_name);
+    m_refGdaConnection->update_meta_store_table(table_name, m_backend->get_public_schema_name());
+    return result;
+  }
+  catch(const Glib::Error& ex)
+  {
+    std::cerr << G_STRFUNC << ": exception:" << ex.what() << std::endl;
+  }
+
+  return false;
+}
+
+bool ConnectionPool::change_column(const Glib::ustring& table_name, const sharedptr<const Field>& field_old, const sharedptr<const Field>& field) throw()
 {
   type_vec_const_fields old_fields(1, field_old);
   type_vec_const_fields new_fields(1, field);
 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
   return change_columns(table_name, old_fields, new_fields);
-#else
-  return change_columns(table_name, old_fields, new_fields, error);
-#endif
 }
 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
-bool ConnectionPool::change_columns(const Glib::ustring& table_name, const type_vec_const_fields& old_fields, const type_vec_const_fields& new_fields)
-#else
-bool ConnectionPool::change_columns(const Glib::ustring& table_name, const type_vec_const_fields& old_fields, const type_vec_const_fields& new_fields, std::auto_ptr<Glib::Error>& error)
-#endif
+bool ConnectionPool::change_columns(const Glib::ustring& table_name, const type_vec_const_fields& old_fields, const type_vec_const_fields& new_fields) throw()
 {
   sharedptr<SharedConnection> conn;
   if(!m_refGdaConnection)
   {
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
     conn = connect();
-#else
-    std::auto_ptr<ExceptionConnection> local_error;
-    // TODO: We can't rethrow local_error here since ExceptionConnection does
-    // not derive from Glib::Error
-    conn = connect(local_error);
-#endif
   }
 
   if(!m_refGdaConnection)
     return false;
 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
-  std::auto_ptr<Glib::Error> error;
-#endif
-  const bool result = m_backend->change_columns(m_refGdaConnection, table_name, old_fields, new_fields, error);
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
-  if(error.get()) throw *error;
-  m_refGdaConnection->update_meta_store_table(table_name, m_backend->get_public_schema_name());
+  const bool result = m_backend->change_columns(m_refGdaConnection, table_name, old_fields, new_fields);
 
-#else
-  if(error.get())
-    std::cerr << "Error: " << error->what() << std::endl;
-  m_refGdaConnection->update_meta_store_table(table_name, m_backend->get_public_schema_name(), error);
-#endif
+  try
+  {
+    m_refGdaConnection->update_meta_store_table(table_name, m_backend->get_public_schema_name());
+  }
+  catch(const Glib::Error& ex)
+  {
+    std::cerr << G_STRFUNC << ": update_meta_store_table() failed: " << ex.what() << std::endl;
+    return false;
+  }
+
+  if(!result)
+    return false;
+
+  //Add or remove any auto-increment rows:
+  //The new auto-increment row would actually be added automatically,
+  //but this makes it available even before a record has been added. 
+  type_vec_const_fields::const_iterator iter_old = old_fields.begin();
+  type_vec_const_fields::const_iterator iter_new = new_fields.begin();
+  while( (iter_old != old_fields.end()) && (iter_new != new_fields.end()) )
+  {
+    const sharedptr<const Field> field_old = *iter_old;
+    const sharedptr<const Field> field_new = *iter_new;
+    if(field_old && field_new
+      && field_old->get_auto_increment() != field_new->get_auto_increment())
+    {
+      if(field_new->get_auto_increment())
+        DbUtils::auto_increment_insert_first_if_necessary(table_name, field_new->get_name());
+      else
+        DbUtils::remove_auto_increment(table_name, field_new->get_name());
+    }
+
+    ++iter_old;
+    ++iter_new;
+  }
+ 
   return result;
 }
 
@@ -830,7 +813,9 @@ Document* ConnectionPool::get_document()
 {
   if(!m_slot_get_document)
   {
-    std::cerr << "Glom ConnectionPool::get_document(): m_slot_get_document is null." << std::endl;
+    //Don't bother warning because all the code that calls get_document() checks 
+    //for 0 and responds reasonably.
+    //std::cerr << G_STRFUNC << ": m_slot_get_document is null." << std::endl;
     return 0;
   }
 
@@ -850,7 +835,7 @@ EpcContents* ConnectionPool::on_publisher_document_requested(EpcPublisher* /* pu
     return 0;
 
   const Glib::ustring contents = document->get_contents();
-  //std::cout << "DEBUG: ConnectionPool::on_publisher_document_requested(): returning: " << std::endl << "  " << contents << std::endl;
+  //std::cout << "debug: " << G_STRFUNC << ": returning: " << std::endl << "  " << contents << std::endl;
   return epc_contents_new_dup ("text/plain", (void*)contents.c_str(), -1);
 }
 
@@ -869,22 +854,22 @@ gboolean ConnectionPool::on_publisher_document_authentication(EpcAuthContext* co
   const gchar* password = epc_auth_context_get_password(context);
   g_return_val_if_fail(password, false); //TODO: This seems to happen once before this callback is called again properly.
 
-  //std::cout << "ConnectionPool::on_publisher_document_authentication(): username=" << user_name << ", password=" << password << std::endl;
+  //std::cout << "debug: " << G_STRFUNC << ": username=" << user_name << ", password=" << password << std::endl;
 
   g_return_val_if_fail(connection_pool->m_backend.get(), false);
 
   //Attempt a connection with this username/password:
   std::auto_ptr<ExceptionConnection> error;
-  Glib::RefPtr<Gnome::Gda::Connection> connection = connection_pool->m_backend->connect(connection_pool->get_database(), user_name, password, error);
+  Glib::RefPtr<Gnome::Gda::Connection> connection = connection_pool->m_backend->connect(connection_pool->get_database(), user_name, password);
 
   if(connection)
   {
-    //std::cout << "ConnectionPool::on_publisher_document_authentication(): succeeded." << std::endl;
+    //std::cout << "debug: " << G_STRFUNC << ": succeeded." << std::endl;
     return true; //Succeeded.
   }
   else
   {
-    //std::cout << "ConnectionPool::on_publisher_document_authentication(): failed." << std::endl;
+    //std::cout << "debug: " << G_STRFUNC << ": failed." << std::endl;
     return false; //Failed.
   }
 }
@@ -943,7 +928,7 @@ void ConnectionPool::avahi_start_publishing()
   if(!document)
     return;
 
-  m_epc_publisher = epc_publisher_new(document->get_database_title().c_str(), "glom", 0);
+  m_epc_publisher = epc_publisher_new(document->get_database_title_original().c_str(), "glom", 0);
   epc_publisher_set_protocol(m_epc_publisher, publish_protocol);
 
   epc_publisher_add_handler(m_epc_publisher, "document", on_publisher_document_requested, this /* user_data */, 0);
@@ -976,7 +961,7 @@ void ConnectionPool::avahi_start_publishing()
   if(error)
   {
 #ifdef GLOM_CONNECTION_DEBUG
-    std::cout << "Glom: ConnectionPool::avahi_start_publishing(): Error while running epc_publisher_run_async: " << error->message << std::endl;
+    std::cout << "debug: " << G_STRFUNC << ": Error while running epc_publisher_run_async: " << error->message << std::endl;
 #endif
     g_clear_error(&error);
   }
@@ -1010,10 +995,24 @@ void ConnectionPool::set_show_debug_output(bool val)
 {
   m_show_debug_output = val;
 }
- 
+
 bool ConnectionPool::get_show_debug_output() const
 {
   return m_show_debug_output;
 }
 
+void ConnectionPool::set_auto_server_shutdown(bool val)
+{
+  m_auto_server_shutdown = val;
+}
+
+void ConnectionPool::set_fake_connection()
+{
+  m_fake_connection = true;
+
+  //Set a fake username and password, to avoid a GError from gda_connection_new_from_string():
+  set_user("glom_fake_user");
+  set_password("glom_fake_password");
+}
+
 } //namespace Glom
