@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #ifndef GLOM_CONNECTIONPOOL_H
@@ -23,7 +23,7 @@
 
 #include <libglom/libglom_config.h>
 
-#include <libgdamm.h>
+#include <libgdamm/connection.h>
 #include <libglom/sharedptr.h>
 #include <libglom/data_structure/fieldtypes.h>
 #include <libglom/data_structure/field.h>
@@ -98,6 +98,10 @@ public:
    * Use delete_instance() when the program quits.
    */
   static ConnectionPool* get_instance();
+  
+  /** Whether the connection is ready to be used.
+   */ 
+  static bool get_instance_is_ready();
 
   /** Make the ConnectionPool use the correct backend, with the necessary details,
    * as required by the document.
@@ -120,14 +124,26 @@ public:
   void set_avahi_publish_callbacks(const type_void_slot& slot_begin, const type_void_slot& slot_progress, const type_void_slot& slot_done);
 #endif
 
-
   bool get_ready_to_connect() const;
   void set_ready_to_connect(bool val = true);
 
+  /** Do not actually connect to the database,
+   * but create a Gnome::Gda::Connection object
+   * just so that utility functions such as Utils::sqlbuilder_get_full_query()
+   * and DbUtils::escape_sql_id() can work.
+   */
+  void set_fake_connection();
+
   void set_backend(std::auto_ptr<Backend> backend);
 
   Backend* get_backend();
   const Backend* get_backend() const;
+  
+  /** Discover whether the backend can create GdaDataModels that can be iterated,
+   * by creating them with the GDA_STATEMENT_MODEL_CURSOR_FORWARD flag.
+   * If not (with sqlite, for instance), the GdaDataAccessWrapper model can provide that API, without the performance.
+   */
+  bool get_backend_supports_cursor() const;
 
   /** This method will return a SharedConnection, either by opening a new connection or returning an already-open connection.
    * When that SharedConnection is destroyed, or when SharedConnection::close() is called, then the ConnectionPool will be informed.
@@ -146,10 +162,9 @@ public:
    */
   typedef Backend::SlotProgress SlotProgress;
 
- //TODO: Add SlotProgress?
   /** Creates a new database.
    */
-  void create_database(const Glib::ustring& database_name);
+  void create_database(const SlotProgress& slot_progress, const Glib::ustring& database_name);
 
   /** Save a backup of the database in a tarball.
    * This backup can later be used to recreate the database,
@@ -177,9 +192,8 @@ public:
   Glib::ustring get_password() const;
   Glib::ustring get_database() const;
 
-  Field::sql_format get_sql_format() const;
   const FieldTypes* get_field_types() const;
-  Glib::ustring get_string_find_operator() const;
+  Gnome::Gda::SqlOperatorType get_string_find_operator() const;
 
   typedef Backend::InitErrors InitErrors;
 
@@ -227,16 +241,46 @@ public:
    */
   virtual bool set_network_shared(const SlotProgress& slot_progress, bool network_shared = true);
 
+  /** Add a field to the database.
+   * The caller should then update the document's list of fields,
+   * for instance by calling Document::set_table_fields(). 
+   *
+   * @param table_name The parent table of the fields to be changed.
+   * @param field The field to be added.
+   */
   bool add_column(const Glib::ustring& table_name, const sharedptr<const Field>& field) throw();
 
+  /** Remove a field from the database.
+   * The caller should then update the document's list of fields,
+   * for instance by calling Document::set_table_fields(). 
+   *
+   * @param table_name The parent table of the fields to be changed.
+   * @param field_name The name of the field to be removed.
+   */
   bool drop_column(const Glib::ustring& table_name, const Glib::ustring& field_name) throw();
 
+  /** Change some detail about a field in the database.
+   * The caller should then update the document's list of fields,
+   * for instance by calling Document::set_table_fields(). 
+   *
+   * @param table_name The parent table of the field to be changed.
+   * @param field_old The old field information.
+   * @param field The new field information. 
+   */
   bool change_column(const Glib::ustring& table_name, const sharedptr<const Field>& field_old, const sharedptr<const Field>& field) throw();
 
+  /** Change some detail about some fields in the database.
+   * The caller should then update the document's list of fields,
+   * for instance by calling Document::set_table_fields(). 
+   *
+   * @param table_name The parent table of the fields to be changed.
+   * @param old_fields The old field information.
+   * @param fields The new field information. 
+   */
   bool change_columns(const Glib::ustring& table_name, const type_vec_const_fields& old_fields, const type_vec_const_fields& fields) throw();
 
   /** Specify a callback that the ConnectionPool can call to get a pointer to the document.
-   * This callback avoids Connection having to link to Application,
+   * This callback avoids Connection having to link to AppWindow,
    * and avoids us worrying about whether a previously-set document (via a set_document() method) is still valid.
    */
   typedef sigc::slot<Document*> SlotGetDocument;
@@ -289,11 +333,13 @@ private:
   Glib::RefPtr<Gnome::Gda::Connection> m_refGdaConnection;
   guint m_sharedconnection_refcount;
   bool m_ready_to_connect;
-  Glib::ustring m_host, m_user, m_password, m_database;
+  Glib::ustring m_user, m_password, m_database;
 
   FieldTypes* m_pFieldTypes;
   bool m_show_debug_output, m_auto_server_shutdown;
 
+  bool m_fake_connection;
+
 private:
 
   static ConnectionPool* m_instance;
