@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #include "config.h" // For POSTGRES_UTILS_PATH
@@ -28,6 +28,7 @@
 #include <libgdamm/config.h>
 #include <giomm/file.h>
 #include <glibmm/convert.h>
+#include <glibmm/fileutils.h> //For Glib::file_test().
 #include <glibmm/miscutils.h>
 #include <glibmm/shell.h>
 #include <glib/gstdio.h> /* For g_rename(). TODO: Wrap this in glibmm? */
@@ -106,7 +107,7 @@ Glib::RefPtr<Gnome::Gda::Connection> Postgres::attempt_connect(const Glib::ustri
   catch(const Glib::Error& ex)
   {
 #ifdef GLOM_CONNECTION_DEBUG
-    std::cout << "debug: " << G_STRFUNC << ": Attempt to connect to database failed on port=" << port << ", database=" << database << ": " << ex.what() << std::endl;
+    std::cout << "debug: " << G_STRFUNC << ": Attempt to connect to database failed on port=" << port << ", database=" << database << ": " << "error code=" << ex.code() << ", error message: " <<  ex.what() << std::endl;
     std::cout << "debug: " << G_STRFUNC << ": Attempting to connect without specifying the database." << std::endl;
 #endif
 
@@ -119,8 +120,11 @@ Glib::RefPtr<Gnome::Gda::Connection> Postgres::attempt_connect(const Glib::ustri
         cnc_string, auth_string,
         Gnome::Gda::CONNECTION_OPTIONS_SQL_IDENTIFIERS_CASE_SENSITIVE);
     }
-    catch(const Glib::Error& ex)
-    {}
+    catch(const Glib::Error& /* ex */)
+    {
+      //Show this on stderr because it can contain useful clues such as a hostname that cannot be resolved.
+      //std::cerr << G_STRFUNC << ": Attempt to connect to default database failed on port=" << port << " : " << "error code=" << ex.code() << ", error message: " <<  ex.what() << std::endl;
+    }
 
 #ifdef GLOM_CONNECTION_DEBUG
     if(temp_conn)
@@ -384,10 +388,15 @@ bool Postgres::change_columns(const Glib::RefPtr<Gnome::Gda::Connection>& connec
   return false;
 }
 
-bool Postgres::attempt_create_database(const Glib::ustring& database_name, const Glib::ustring& host, const Glib::ustring& port, const Glib::ustring& username, const Glib::ustring& password)
+bool Postgres::attempt_create_database(const SlotProgress& slot_progress, const Glib::ustring& database_name, const Glib::ustring& host, const Glib::ustring& port, const Glib::ustring& username, const Glib::ustring& password)
 {
-  Glib::RefPtr<Gnome::Gda::ServerOperation> op = Gnome::Gda::ServerOperation::prepare_create_database("PostgreSQL",
-                                                              database_name);
+  slot_progress();
+
+  Glib::RefPtr<Gnome::Gda::ServerOperation> op = 
+    Gnome::Gda::ServerOperation::prepare_create_database("PostgreSQL", database_name);
+
+  slot_progress();
+
   g_assert(op);
   try
   {
@@ -403,6 +412,8 @@ bool Postgres::attempt_create_database(const Glib::ustring& database_name, const
     return false;
   }
 
+  slot_progress();
+
   return true;
 }
 
@@ -474,7 +485,7 @@ std::string Postgres::get_path_to_postgres_executable(const std::string& program
   if(Glib::file_test(test, Glib::FILE_TEST_IS_EXECUTABLE))
   {
     if(quoted)
-      test = Glib::shell_quote(path);
+      test = Glib::shell_quote(test);
     return test;
   }
 
@@ -928,11 +939,6 @@ bool Postgres::create_text_file(const std::string& file_uri, const std::string&
   return true; //Success.
 }
 
-Field::sql_format Postgres::get_sql_format() const
-{
-  return Field::SQL_FORMAT_POSTGRES;
-}
-
 bool Postgres::supports_remote_access() const
 {
   return true;
