@@ -14,14 +14,15 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #include <libglom/data_structure/field.h>
 #include <libglom/connectionpool.h>
 #include <libglom/data_structure/glomconversions.h>
 #include <libglom/utils.h>
+#include <libgda/gda-blob-op.h>
 #include <glibmm/i18n.h>
 
 #include <iostream>
@@ -134,24 +135,36 @@ Glib::RefPtr<const Gnome::Gda::Column> Field::get_field_info() const
   return m_field_info;
 }
 
+static const FieldTypes* get_field_types()
+{
+  ConnectionPool* connection_pool = ConnectionPool::get_instance();
+  if(!connection_pool)
+    return 0;
+
+  return connection_pool->get_field_types();
+}
+
 void Field::set_field_info(const Glib::RefPtr<Gnome::Gda::Column>& fieldinfo)
 {
   m_field_info = fieldinfo;
 
+  const glom_field_type glom_type = get_glom_type();
+  const GType new_type = fieldinfo->get_g_type();
+  if( (glom_type == TYPE_INVALID) &&
+    (new_type == GDA_TYPE_NULL)) //GDA_TYPE_NULL is the default for GdaColumn.
+  {
+    //Don't bother with any of the following checks.
+    return;
+  }
+
   // Also take fallback types into account as fieldinfo might originate from
   // the database system directly.
-  GType new_type = fieldinfo->get_g_type();
-
   GType cur_type = G_TYPE_NONE;
-  if(get_glom_type() != TYPE_INVALID)
+  if(glom_type != TYPE_INVALID)
   {
-    cur_type = get_gda_type_for_glom_type(get_glom_type());
+    cur_type = get_gda_type_for_glom_type(glom_type);
 
-    const FieldTypes* pFieldTypes = 0;
-
-    ConnectionPool* pConnectionPool = ConnectionPool::get_instance();
-    if(pConnectionPool)
-      pFieldTypes = pConnectionPool->get_field_types();
+    const FieldTypes* pFieldTypes = get_field_types();
 
     if(pFieldTypes)
     {
@@ -170,11 +183,7 @@ void Field::set_field_info(const Glib::RefPtr<Gnome::Gda::Column>& fieldinfo)
     // TODO: Basically copied from set_default_value(). Maybe this check should
     // be moved into an extra function.
     GType cur_type = get_gda_type_for_glom_type(get_glom_type());
-    const FieldTypes* pFieldTypes = 0;
-
-    ConnectionPool* pConnectionPool = ConnectionPool::get_instance();
-    if(pConnectionPool)
-      pFieldTypes = pConnectionPool->get_field_types();
+    const FieldTypes* pFieldTypes = get_field_types();
 
     // Take into account that value might be one of the fallback types
     if(pFieldTypes)
@@ -185,7 +194,7 @@ void Field::set_field_info(const Glib::RefPtr<Gnome::Gda::Column>& fieldinfo)
 
     if(!value.is_null() && value.get_value_type() != cur_type)
     {
-      g_warning("Field::set_field_info: New field's default value type (%s) does not match field type (%s). Resetting default value.", g_type_name(value.get_value_type()), g_type_name(get_gda_type_for_glom_type(get_glom_type())));
+      std::cerr << G_STRFUNC << ": New field's default value type (" << g_type_name(value.get_value_type()) << " does not match field type (" << g_type_name(get_gda_type_for_glom_type(get_glom_type())) << "). Resetting default value." << std::endl;
       m_field_info->set_default_value(Gnome::Gda::Value());
     }
   }
@@ -219,7 +228,7 @@ bool Field::get_is_lookup() const
 
 Glib::ustring Field::sql(const Gnome::Gda::Value& value, const Glib::RefPtr<Gnome::Gda::Connection>& connection) const
 {
-  //g_warning("Field::sql: glom_type=%d", get_glom_type());
+  //std::cout << ": glom_type=" << get_glom_type() << std::endl;
 
   if(value.is_null() && (get_glom_type() == TYPE_TEXT))
   {
@@ -231,7 +240,7 @@ Glib::ustring Field::sql(const Gnome::Gda::Value& value, const Glib::RefPtr<Gnom
   Glib::RefPtr<const Gnome::Gda::ServerProvider> provider = connection->get_provider();
   if(!provider)
   {
-    std::cerr << "Field::sql(): The ServerProvider was null." << std::endl;
+    std::cerr << G_STRFUNC << ": The ServerProvider was null." << std::endl;
     return Glib::ustring();
   } 
 
@@ -246,42 +255,13 @@ Glib::ustring Field::sql(const Gnome::Gda::Value& value, const Glib::RefPtr<Gnom
   }
   else
   {
-    std::cerr << "Field::sql(): The DataHandler was null." << std::endl;
+    std::cerr << G_STRFUNC << ": The DataHandler was null." << std::endl;
     return Glib::ustring();
   }
 
   return Glib::ustring();
 }
 
-Glib::ustring Field::sql(const Gnome::Gda::Value& value) const
-{
-  sharedptr<SharedConnection> connection;
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
-  try
-  {
-    connection = ConnectionPool::get_instance()->connect();
-  }
-  catch (Glom::ExceptionConnection& ex)
-  {
-    //TODO: Do something more useful here
-    std::cerr << "Field::sql, connection failed: " << ex.what() << std::endl;
-  }
-#else
-  std::auto_ptr<Glom::ExceptionConnection> ex;
-  connection = ConnectionPool::get_instance()->connect(ex);
-  if (ex.get())
-    std::cerr << "Field::sql, connection failed: " << ex->what() << std::endl;
-#endif
-  if(connection)
-  {
-    Glib::RefPtr<Gnome::Gda::Connection> gda_connection = connection->get_gda_connection();
-    if(gda_connection)
-      return sql(value, gda_connection);
-  }
-
-  return Glib::ustring();
-}
-
 #define GLOM_QUOTE_FOR_FILE_FORMAT "\""
 
 Glib::ustring Field::to_file_format(const Gnome::Gda::Value& value) const
@@ -291,32 +271,53 @@ Glib::ustring Field::to_file_format(const Gnome::Gda::Value& value) const
 
 Glib::ustring Field::to_file_format(const Gnome::Gda::Value& value, glom_field_type glom_type)
 {
+  //Handle TYPE_IMAGE specially:
   if(glom_type == TYPE_IMAGE)
   {
-    if(!value.gobj() || (value.get_value_type() != GDA_TYPE_BINARY))
+    if(!value.gobj())
       return Glib::ustring();
  
-    const GdaBinary* gdabinary = gda_value_get_binary(value.gobj());
-    if(!gdabinary)
-      return Glib::ustring();
-    else
+    gchar* str = 0;
+    const GType value_type = value.get_value_type();
+    if(value_type == GDA_TYPE_BINARY)
     {
-      gchar* str = gda_binary_to_string(gdabinary, 0);
-      Glib::ustring result = (str) ? 
-        Glib::ustring(Glib::ScopedPtr<char>(str).get()) : Glib::ustring();
-
-      //Avoid arbitrary newlines in this text.
-      //See libgda bug: https://bugzilla.gnome.org/show_bug.cgi?id=597390
-      result = Utils::string_replace(result, "\n", "\\012");
-
-      //Avoid arbitrary newlines in this text.
-      //See libgda bug: https://bugzilla.gnome.org/show_bug.cgi?id=597390
-      result = Utils::string_replace(result, "\r", "\\015");
-
-      //Escape any quotes in this text:
-      //See libgda bug: https://bugzilla.gnome.org/show_bug.cgi?id=597390
-      return Utils::string_replace(result, "\"", "\\042");
+      const GdaBinary* gdabinary = gda_value_get_binary(value.gobj());
+      if(!gdabinary)
+        return Glib::ustring();
+      else
+      {
+        str = gda_binary_to_string(gdabinary, 0);
+      }
     }
+    else if(value_type == GDA_TYPE_BLOB)
+    {
+      const GdaBlob* gdablob = gda_value_get_blob(value.gobj());
+      if(!gdablob)
+        return Glib::ustring();
+      else
+      {
+        str = gda_blob_to_string(const_cast<GdaBlob*>(gdablob), 0);
+      }
+    }
+
+    if(!str)
+      return Glib::ustring();
+
+    Glib::ustring result = (str) ? 
+      Glib::ustring(Glib::ScopedPtr<char>(str).get()) : Glib::ustring();
+
+    //Correction for text representations of image (binary) data:
+    //Avoid arbitrary newlines in this text.
+    //See libgda bug: https://bugzilla.gnome.org/show_bug.cgi?id=597390
+    result = Utils::string_replace(result, "\n", "\\012");
+
+    //Avoid arbitrary newlines in this text.
+    //See libgda bug: https://bugzilla.gnome.org/show_bug.cgi?id=597390
+    result = Utils::string_replace(result, "\r", "\\015");
+
+    //Escape any quotes in this text:
+    //See libgda bug: https://bugzilla.gnome.org/show_bug.cgi?id=597390
+    return Utils::string_replace(result, "\"", "\\042");
   }
   
   NumericFormat format_ignored; //Because we use ISO format.
@@ -347,13 +348,26 @@ Gnome::Gda::Value Field::from_file_format(const Glib::ustring& str, bool& succes
 Gnome::Gda::Value Field::from_file_format(const Glib::ustring& str, glom_field_type glom_type, bool& success)
 {
   success = true;
-  
+
+  //Unescape "" to ", because to_file_format() escaped ", as specified by the CSV RFC:
+  Glib::ustring string_unescaped;
   if(glom_type == TYPE_IMAGE)
   {
-    if(str.empty())
+    string_unescaped = str; //binary data does not have quote characters so we do not bother to escape or unescape it.
+  }
+  else
+  {
+    string_unescaped = 
+      Utils::string_replace(str, GLOM_QUOTE_FOR_FILE_FORMAT GLOM_QUOTE_FOR_FILE_FORMAT, GLOM_QUOTE_FOR_FILE_FORMAT);
+  }
+
+
+  if(glom_type == TYPE_IMAGE)
+  {
+    if(string_unescaped.empty())
       return  Gnome::Gda::Value();
 
-    GdaBinary* gdabinary = gda_string_to_binary(str.c_str());
+    GdaBinary* gdabinary = gda_string_to_binary(string_unescaped.c_str());
     if(!success || !gdabinary)
       return Gnome::Gda::Value();
     else
@@ -367,11 +381,11 @@ Gnome::Gda::Value Field::from_file_format(const Glib::ustring& str, glom_field_t
   else
   {
     NumericFormat format_ignored; //Because we use ISO format.
-    return Conversions::parse_value(glom_type, str, format_ignored, success, true);
+    return Conversions::parse_value(glom_type, string_unescaped, format_ignored, success, true);
   }
 }
 
-Glib::ustring Field::sql_find(const Gnome::Gda::Value& value) const
+Glib::ustring Field::sql_find(const Gnome::Gda::Value& value, const Glib::RefPtr<Gnome::Gda::Connection>& connection) const
 {
   switch(get_glom_type())
   {
@@ -382,7 +396,7 @@ Glib::ustring Field::sql_find(const Gnome::Gda::Value& value) const
       if(value.is_null())
         return "''"; //We want to ignore the concept of NULL strings, and deal only with empty strings.
       else
-        return ("'%" + value.to_string() + "%'"); //Add single-quotes. TODO: Actually escape it.
+        return ("%" + value.to_string() + "%"); //TODO: Escape it before adding "%", but then prevent SqlBuilder from re-escaping it.
         
       break;
     }
@@ -392,13 +406,13 @@ Glib::ustring Field::sql_find(const Gnome::Gda::Value& value) const
     case(TYPE_BOOLEAN):
     default:
     {
-      return sql(value);
+      return sql(value, connection);
       break;
     }
   }
 }
 
-Glib::ustring Field::sql_find_operator() const
+Gnome::Gda::SqlOperatorType Field::sql_find_operator() const
 {
   switch(get_glom_type())
   {
@@ -408,7 +422,7 @@ Glib::ustring Field::sql_find_operator() const
       if(connection_pool && connection_pool->get_backend())
         return connection_pool->get_string_find_operator();
       else
-        return "LIKE"; // Default
+        return Gnome::Gda::SQL_OPERATOR_TYPE_LIKE; // Default
       break;
     }
     case(TYPE_DATE):
@@ -417,7 +431,7 @@ Glib::ustring Field::sql_find_operator() const
     case(TYPE_BOOLEAN):
     default:
     {
-      return "=";
+      return Gnome::Gda::SQL_OPERATOR_TYPE_EQ;
       break;
     }
   }
@@ -495,7 +509,7 @@ void Field::set_default_value(const Gnome::Gda::Value& value)
   if(value.is_null() || value.get_value_type() == cur_type)
     m_field_info->set_default_value(value);
   else
-    g_warning("Field::set_default_value: Cannot set incompatible default value: Default value has type %s, but field has type %s", g_type_name(value.get_value_type()), g_type_name(get_gda_type_for_glom_type(get_glom_type())));
+    std::cerr << G_STRFUNC << ": Cannot set incompatible default value: Default value has type " << g_type_name(value.get_value_type()) << ", but field has type " << g_type_name(get_gda_type_for_glom_type(get_glom_type())) << std::endl;
 }
 
 Glib::ustring Field::get_sql_type() const
@@ -518,11 +532,15 @@ Glib::ustring Field::get_sql_type() const
         const GType fieldType = m_field_info->get_g_type();
         strType = pFieldTypes->get_string_name_for_gdavaluetype(fieldType);
       }
+      else
+      {
+        std::cerr << G_STRFUNC << ": get_field_types() returned null" << std::endl;
+      }
     }
 
     if(strType == "unknowntype")
     {
-      g_warning("Field::get_sql_type(): returning unknowntype for field name=%s , glom_type=%d, gda_type=%d", get_name().c_str(), get_glom_type(), (int)m_field_info->get_g_type());
+      std::cerr << G_STRFUNC << ": returning unknowntype for field name=" << get_name() << ", glom_type=" << get_glom_type() << ", gda_type=" << (int)m_field_info->get_g_type() << std::endl;
     }
 
     return strType;
@@ -535,42 +553,6 @@ Glib::ustring Field::get_gda_type_name() const
   return g_type_name( m_field_info->get_g_type());
 }
 
-Glib::RefPtr<Gnome::Gda::Holder> Field::get_holder(const Gnome::Gda::Value& value, const Glib::ustring& name) const
-{
-  const Glib::ustring real_name = name.empty() ? get_name() : name;
-
-  const GType gtype = value.get_value_type();
-
-  /*
-  const GType field_type = m_field_info->get_g_type();
-  if(m_field_info->get_g_type() != gtype)
-  {
-    // This currently happens with SQLite for pictures. The value type is
-    // GdaBinary, but the field type is GdaBlob. This is not an optimal
-    // solution this way. We should maybe check fallback types here, or
-    // investigate why the field type is not GdaBinary as well.
-    // Maybe get_gda_type_for_glom_type() should already return fallback
-    // types if necessary.
-    std::cout << "DEBUG: Field::get_holder(): Field type " << g_type_name(field_type) << " and value type " << g_type_name(gtype) << " don't match." << std::endl;
-  }
-  */
-
-  Glib::RefPtr<Gnome::Gda::Holder> holder = Gnome::Gda::Holder::create(gtype, real_name);
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
-  holder->set_value_as_value(value);
-#else
-  std::auto_ptr<Glib::Error> ex;
-  holder->set_value_as_value(value, ex);
-#endif
-  return holder;
-}
-
-Glib::ustring Field::get_gda_holder_string(const Glib::ustring& name) const
-{
-  const Glib::ustring real_name = name.empty() ? get_name() : name;
-  return "##" + real_name + "::" + get_gda_type_name();
-}
-
 /// Ignores any part of FieldAttributes that libgda does not properly fill.
 bool Field::field_info_from_database_is_equal(const Glib::RefPtr<const Gnome::Gda::Column>& field)
 {
@@ -599,7 +581,7 @@ Field::glom_field_type Field::get_glom_type_for_gda_type(GType gda_type)
       result = iterFind->second;
     else
     {
-      // g_warning("FieldType::FieldType(GType gda_type): Invalid gda type: %d",  gda_type);
+      std::cerr << G_STRFUNC << ": Unhandled GType: " << g_type_name(gda_type) << std::endl;
     }
   }
 
@@ -618,10 +600,10 @@ GType Field::get_gda_type_for_glom_type(Field::glom_field_type glom_type)
 
   if(ideal_gda_type == G_TYPE_NONE)
   {
-    g_warning("Field::get_gda_type_for_glom_type(): Returning G_TYPE_NONE for glom_type=%d", glom_type);
+    std::cerr << G_STRFUNC << ": Returning G_TYPE_NONE for glom_type=" << glom_type << std::endl;
   }
 
-  //std::cout << "Field::get_gda_type_for_glom_type(): returning: " << g_type_name(ideal_gda_type) << std::endl;
+  //std::cout << "debug: " << G_STRFUNC << ": returning: " << g_type_name(ideal_gda_type) << std::endl;
   
   return ideal_gda_type;
 }
@@ -642,6 +624,14 @@ void Field::init_map()
     m_map_gda_type_to_glom_type[G_TYPE_BOOLEAN] = TYPE_BOOLEAN;
     m_map_gda_type_to_glom_type[GDA_TYPE_BINARY] = TYPE_IMAGE;
 
+    //SQLite can return a GdaBlob though it can take a GdaBinary:
+    m_map_gda_type_to_glom_type[GDA_TYPE_BLOB] = TYPE_IMAGE;
+
+    //Extra conversions for GTypes that can be returned by glom_pygda_value_from_pyobject():
+    m_map_gda_type_to_glom_type[G_TYPE_DOUBLE] = TYPE_NUMERIC;
+    //TODO? m_map_gda_type_to_glom_type[GDA_TYPE_TIME] = ;
+    //TODO? m_map_gda_type_to_glom_type[GDA_TYPE_TIMESTAMP] = ;
+
     m_map_glom_type_to_gda_type[TYPE_NUMERIC] = GDA_TYPE_NUMERIC;
     m_map_glom_type_to_gda_type[TYPE_TEXT] = G_TYPE_STRING;
     m_map_glom_type_to_gda_type[TYPE_TIME] = GDA_TYPE_TIME;
@@ -649,12 +639,25 @@ void Field::init_map()
     m_map_glom_type_to_gda_type[TYPE_BOOLEAN] = G_TYPE_BOOLEAN;
     m_map_glom_type_to_gda_type[TYPE_IMAGE] = GDA_TYPE_BINARY;
 
+    // Translators: This means an unknown or unnacceptable value type in a database.
     m_map_type_names_ui[TYPE_INVALID] = _("Invalid");
+    
+    // Translators: This means a numeric value type in a database.
     m_map_type_names_ui[TYPE_NUMERIC] = _("Number");
+    
+    // Translators: This means a text/string value type in a database.
     m_map_type_names_ui[TYPE_TEXT] = _("Text");
+    
+    // Translators: This means a time value type in a database.
     m_map_type_names_ui[TYPE_TIME] = _("Time");
-    m_map_type_names_ui[TYPE_DATE] = _("Date");    
+    
+    // Translators: This means a time value type in a database.
+    m_map_type_names_ui[TYPE_DATE] = _("Date");
+    
+    // Translators: This means a true/false value type in a database.
     m_map_type_names_ui[TYPE_BOOLEAN] = _("Boolean");
+    
+    // Translators: This means a picture value type in a database.
     m_map_type_names_ui[TYPE_IMAGE] = _("Image");
 
     //Non-translated names used for the document:
