@@ -14,16 +14,18 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #ifndef GLOM_DATASTRUCTURE_FIELD_H
 #define GLOM_DATASTRUCTURE_FIELD_H
 
-#include <libgdamm.h>
+#include <libgdamm/column.h>
+#include <libgdamm/holder.h>
+#include <libgdamm/connection.h>
 #include <libglom/data_structure/translatable_item.h>
-#include <libglom/data_structure/layout/fieldformatting.h>
+#include <libglom/data_structure/layout/formatting.h>
 #include <libglom/sharedptr.h>
 
 //Predicate, for use with std::find_if():
@@ -157,31 +159,20 @@ public:
   Glib::ustring get_sql_type() const;
   Glib::ustring get_gda_type_name() const;
 
-  //TODO: Shouldn't this be called get_gda_holder_name()?
-  /// A convenience when using parameters with a libgda SQL query.
-  Glib::RefPtr<Gnome::Gda::Holder> get_holder(const Gnome::Gda::Value& value, const Glib::ustring& name = Glib::ustring()) const;
-
-  /** TODO: Documentation.
-   */
-  Glib::ustring get_gda_holder_string(const Glib::ustring& name = Glib::ustring()) const;
-
   /** Escape and quote the value so that it can be used in a SQL command.
    */
   Glib::ustring sql(const Gnome::Gda::Value& value, const Glib::RefPtr<Gnome::Gda::Connection>& connection) const;
 
-  //TODO: Remove this:
-  /** Escape and quote the value so that it can be used in a SQL command.
-   * Uses the sql_format of the current connectionpool backend.
-   */
-  Glib::ustring sql(const Gnome::Gda::Value& value) const;
-
   /** Get the canonical format for a file, for instance for 
    * a default value or for example data.
    * This does not add quotes for text fields so the caller may need to do that.
    * Note that this does not do any extra escaping such as an XML file might need.
+   * However, this will escape data as per the CSV RFC.
    */
   Glib::ustring to_file_format(const Gnome::Gda::Value& value) const;
 
+  /** See to_file_format(const Gnome::Gda::Value& value).
+   */
   static Glib::ustring to_file_format(const Gnome::Gda::Value& value, glom_field_type glom_type);
 
   /** Parse the value from the canonical file format. See to_file_format()
@@ -193,12 +184,12 @@ public:
 
   /** Escape the value so that it can be used in a SQL command for a find.
    */
-  Glib::ustring sql_find(const Gnome::Gda::Value& value) const;
+  Glib::ustring sql_find(const Gnome::Gda::Value& value, const Glib::RefPtr<Gnome::Gda::Connection>& connection) const;
 
   /** Get a suitable operator to use when finding records.
    * For instance, == for numbers, or LIKE for text.
    */
-  Glib::ustring sql_find_operator() const;
+  Gnome::Gda::SqlOperatorType sql_find_operator() const;
 
   Glib::ustring get_calculation() const;
   void set_calculation(const Glib::ustring& calculation);
@@ -207,7 +198,7 @@ public:
   /* Discover what fields are used in the calculation,
    * so the value can be recalculated when their values change.
    */
-  typedef std::list<Glib::ustring> type_list_strings;
+  typedef std::vector<Glib::ustring> type_list_strings;
   type_list_strings get_calculation_relationships() const;
 
   void set_visible(bool val = true);
@@ -235,7 +226,7 @@ public:
 
   static bool get_conversion_possible(glom_field_type field_type_src, glom_field_type field_type_dest);
 
-  FieldFormatting m_default_formatting;
+  Formatting m_default_formatting;
 
 private:
 
@@ -249,7 +240,7 @@ private:
   typedef std::map<glom_field_type, GType> type_map_glom_type_to_gda_type;
   static type_map_glom_type_to_gda_type m_map_glom_type_to_gda_type;
 
-  typedef std::list<glom_field_type> type_list_conversion_targets;
+  typedef std::vector<glom_field_type> type_list_conversion_targets;
   typedef std::map<glom_field_type, type_list_conversion_targets> type_map_conversions;
 
   static type_map_type_names m_map_type_names; //These are canonical, for internal use.
