@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 
@@ -25,7 +25,7 @@
 #include <libglom/connectionpool.h>
 #include <libglom/utils.h>
 
-#include <glibmm/i18n.h>
+#include <glibmm/convert.h>
 #include <sstream> //For stringstream
 
 #include <locale>     // for locale, time_put
@@ -33,38 +33,25 @@
 #include <iostream>   // for cout, endl
 #include <iomanip>
 #include <string.h> // for strlen, memset, strcmp
+#include <stdexcept>
 
-namespace
-{
-  char hextochar(guint8 hex)
-  {
-    if(hex < 10) return '0' + hex;
-    if(hex < 16) return 'a' - 10 + hex;
-    return '\0';
-  }
-
-  guint8 chartohex(char c)
-  {
-    if(c >= '0' && c <= '9') return c - '0';
-    if(c >= 'a' && c <= 'f') return c - 'a' + 10;
-    if(c >= 'A' && c <= 'F') return c - 'A' + 10;
-    return 0;
-  }
-}
+#include <glibmm/i18n.h>
 
 namespace Glom
 {
 
-
-unsigned int Conversions::get_stringstream_precision_default()
-{
-    static const unsigned int stringstream_precision_default = 15;
-    return stringstream_precision_default;
-}
-
 Glib::ustring Conversions::format_time(const tm& tm_data)
 {
-  return format_time( tm_data, std::locale("") /* the user's current locale */ ); //Get the current locale.
+  try
+  {
+    return format_time( tm_data, std::locale("") /* the user's current locale */ ); //Get the current locale.
+  }
+  catch(const std::runtime_error& ex)
+  {
+    std::cerr << G_STRFUNC << ": exception from std::locale(\"\")): " << ex.what() << std::endl;
+    std::cerr << "  This can happen if the locale is not properly installed or configured." << std::endl;
+    return Glib::ustring();
+  } 
 }
 
 
@@ -80,7 +67,16 @@ Glib::ustring Conversions::format_time(const tm& tm_data, const std::locale& loc
 
 Glib::ustring Conversions::format_date(const tm& tm_data)
 {
-  return format_date( tm_data, std::locale("") /* the user's current locale */ ); //Get the current locale.
+  try
+  {
+    return format_date( tm_data, std::locale("") /* the user's current locale */ ); //Get the current locale.
+  }
+  catch(const std::runtime_error& ex)
+  {
+    std::cerr << G_STRFUNC << ": exception from std::locale(\"\")): " << ex.what() << std::endl;
+    std::cerr << "  This can happen if the locale is not properly installed or configured." << std::endl;
+    return Glib::ustring();
+  }
 }
 
 #define GLOM_NON_TRANSLATED_LOCALE_DATE_FORMAT "%x"
@@ -103,6 +99,7 @@ static inline const char* glom_get_locale_date_format()
     c_locale_date_format = _("%x");
   }
 
+  //std::cout << G_STRFUNC << ": c_locale_date_format=" << c_locale_date_format << std::endl;
   return c_locale_date_format;
 }
 
@@ -147,7 +144,7 @@ bool Conversions::sanity_check_date_parsing()
      parsed_date.tm_mday != the_c_time.tm_mday)
   {
     //Note to translators: If you see this error in the terminal at startup then you need to translate the %x elsewhere.
-    std::cerr << _("ERROR: sanity_check_date_parsing(): Sanity check failed: Glom could not parse a date's text representation that it generated itself, in this locale.") << std::endl;
+    std::cerr << _("ERROR: sanity_check_date_parsing(): Sanity check failed: Glom could not parse a date's text representation that it generated itself, in this locale.") << " (" << std::locale("").name() << ")" << std::endl;
 
     //If translators cannot be relied upon to do this, maybe we should default to "%d/%m/%Y" when "%x" fails this test.
 
@@ -172,7 +169,9 @@ bool Conversions::sanity_check_date_text_representation_uses_4_digit_years(bool
   const Glib::ustring date_text = format_date(the_c_time);
 
   if(debug_output)
-    std::cout << "DEBUG: 22nd November 2008 in this locale has this text represention: " << date_text << std::endl;
+  {
+    std::cout << "DEBUG: 22nd November 2008 in this locale (" << std::locale("").name() << ") has this text represention: " << date_text << std::endl;
+  }
 
   //See if the year appears in full in that date.
   //There are probably some locales for which this fails.
@@ -183,6 +182,8 @@ bool Conversions::sanity_check_date_text_representation_uses_4_digit_years(bool
     //Note to translators: If you see this error in the terminal at startup then you need to translate the %x elsewhere.
     std::cerr << _("ERROR: sanity_check_date_text_representation_uses_4_digit_year(): Sanity check failed: Glom does not seem to use 4 digits to display years in a date's text representation, in this locale. Defaulting to dd/mm/yyyy though this might be incorrect for your locale. This needs attention from a translator. Please file a bug - see http://www.glom.org") << std::endl;
 
+    //std::cout << "  Unexpected date text: " << date_text << std::endl;
+
     //Do not depend on translators to do what we ask.
     //Default to a common format, though this would be incorrect in some 
     //locales, such as German.
@@ -207,7 +208,6 @@ Glib::ustring Conversions::format_tm(const tm& tm_data, const std::locale& local
 
   // Get a time_put face:
   typedef std::time_put<char> type_time_put;
-  typedef type_time_put::iter_type type_iterator;
   const type_time_put& tp = std::use_facet<type_time_put>(locale);
 
   //type_iterator begin(the_stream);
@@ -216,11 +216,19 @@ Glib::ustring Conversions::format_tm(const tm& tm_data, const std::locale& local
   Glib::ustring text = the_stream.str();
   //std::cout << "debug: " << G_STRFUNC << ": result from tp.put: " << text << std::endl;
 
-  if(locale == std::locale("") /* The user's current locale */)
+  try
   {
-    //Converts from the user's current locale to utf8. I would prefer a generic conversion from any locale,
-    // but there is no such function, and it's hard to do because I don't know how to get the encoding name from the std::locale()
-    text = Glib::locale_to_utf8(text);
+    if(locale == std::locale("") /* The user's current locale */)
+    {
+      // Converts from the user's current locale to utf8. I would prefer a generic conversion from any locale,
+      // but there is no such function, and it's hard to do because I don't know how to get the encoding name from the std::locale()
+      text = Glib::locale_to_utf8(text);
+    }
+  }
+  catch(const std::runtime_error& ex)
+  {
+    std::cerr << G_STRFUNC << ": exception from std::locale(\"\")): " << ex.what() << std::endl;
+    std::cerr << "  This can happen if the locale is not properly installed or configured." << std::endl;
   }
 
   //std::cout << "debug: " << G_STRFUNC << ": returning: " << text << std::endl;
@@ -250,15 +258,17 @@ Glib::ustring Conversions::format_tm(const tm& tm_data, const std::locale& local
   while((bufsize *= 2) <= 65536);
 
   // This error is quite unlikely (unless strftime is buggy).
-  g_warning("Conversions::format_time(): maximum size of strftime buffer exceeded, giving up");
+  std::cerr << G_STRFUNC << ": maximum size of strftime buffer exceeded. Giving up." << std::endl;
 
   return Glib::ustring();
   */
 }
 
+
 namespace //anonymous
 {
 
+/*
 class numpunct_thousands_separator: public std::numpunct<char>
 {
   //Override
@@ -267,6 +277,7 @@ class numpunct_thousands_separator: public std::numpunct<char>
     return "\3";
   };
 };
+*/
 
 class numpunct_no_thousands_separator: public std::numpunct<char>
 {
@@ -279,9 +290,19 @@ class numpunct_no_thousands_separator: public std::numpunct<char>
 
 } //anonymous namespace
 
+
 Glib::ustring Conversions::get_text_for_gda_value(Field::glom_field_type glom_type, const Gnome::Gda::Value& value, const NumericFormat& numeric_format)
 {
-  return get_text_for_gda_value(glom_type, value, std::locale("") /* the user's current locale */, numeric_format); //Get the current locale.
+  try
+  {
+    return get_text_for_gda_value(glom_type, value, std::locale("") /* the user's current locale */, numeric_format); //Get the current locale.
+  }
+  catch(const std::runtime_error& ex)
+  {
+    std::cerr << G_STRFUNC << ": exception from std::locale(\"\")): " << ex.what() << std::endl;
+    std::cerr << "  This can happen if the locale is not properly installed or configured." << std::endl;
+    return Glib::ustring();
+  }
 }
 
 double Conversions::get_double_for_gda_value_numeric(const Gnome::Gda::Value& value)
@@ -314,19 +335,8 @@ double Conversions::get_double_for_gda_value_numeric(const Gnome::Gda::Value& va
     }
   }
 
-  const GdaNumeric* gda_numeric = value.get_numeric();
-  std::string text_in_c_locale;
-  if(gda_numeric && gda_numeric->number) //A char* - I assume that it formatted as per the C locale. murrayc. TODO: Do we need to look at the other fields?
-    text_in_c_locale = gda_numeric->number; //What formatting does this use?
-
-  //Get an actual numeric value, so we can get a locale-specific text representation:
-  std::stringstream the_stream;
-  the_stream.imbue( std::locale::classic() ); //The C locale.
-  the_stream.str(text_in_c_locale); //Avoid using << because Glib::ustring does implicit character conversion with that.
-
-  double number = 0;
-  the_stream >> number;
-  return number;
+  const Gnome::Gda::Numeric numeric = value.get_numeric();
+  return numeric.get_double();
 }
 
 Glib::ustring Conversions::get_text_for_gda_value(Field::glom_field_type glom_type, const Gnome::Gda::Value& value, const std::locale& locale, const NumericFormat& numeric_format, bool iso_format)
@@ -411,7 +421,10 @@ Glib::ustring Conversions::get_text_for_gda_value(Field::glom_field_type glom_ty
   }
   else if(glom_type == Field::TYPE_NUMERIC)
   {
-    if(value.get_value_type() != GDA_TYPE_NUMERIC && value.get_value_type() != G_TYPE_DOUBLE)
+    const GType value_type = value.get_value_type();
+    if(value_type != GDA_TYPE_NUMERIC
+      && value_type != G_TYPE_DOUBLE
+      && value_type != G_TYPE_INT) //SQLite uses int for summary field results.
     {
       std::cerr << G_STRFUNC << ": glom field type is NUMERIC but GdaValue type is: " << g_type_name(value.get_value_type()) << std::endl;
       return value.to_string();
@@ -465,12 +478,20 @@ Glib::ustring Conversions::get_text_for_gda_value(Field::glom_field_type glom_ty
     another_stream << number;
     Glib::ustring text = another_stream.str();  //Avoid using << because Glib::ustring does implicit character conversion with that.
 
-    if(locale == std::locale("") /* The user's current locale */)
+    try
     {
-      // Converts from the user's current locale to utf8. I would prefer a generic conversion from any locale,
-      // but there is no such function, and it's hard to do because I don't know how to get the encoding name from the std::locale()
-      text = Glib::locale_to_utf8(text);
+      if(locale == std::locale("") /* The user's current locale */)
+      {
+        // Converts from the user's current locale to utf8. I would prefer a generic conversion from any locale,
+        // but there is no such function, and it's hard to do because I don't know how to get the encoding name from the std::locale()
+        text = Glib::locale_to_utf8(text);
+      }
     }
+    catch(const std::runtime_error& ex)
+    {
+      std::cerr << G_STRFUNC << ": exception from std::locale(\"\")): " << ex.what() << std::endl;
+      std::cerr << "  This can happen if the locale is not properly installed or configured." << std::endl;
+    }      
 
     //std::cout << "debug: " << G_STRFUNC << ": number=" << number << ", text=" << text << std::endl;
     return text; //Do something like Glib::locale_to_utf(), but with the specified locale instead of the current locale.
@@ -507,25 +528,9 @@ Glib::ustring Conversions::get_text_for_gda_value(Field::glom_field_type glom_ty
 Gnome::Gda::Value Conversions::parse_value(double number)
 {
   //This is just a way to get a NUMERIC Gnome::Gda::Value from a numeric type:
-  //Try to parse the inputted number, according to the current locale.
-
-  GdaNumeric gda_numeric = {0, 0, 0, 0};
-
-  //Then generate a canonical representation of the number:
-  std::stringstream clocale_stream;
-  clocale_stream.imbue( std::locale::classic() ); //The C locale.
-  clocale_stream << number;
-  Glib::ustring number_canonical_text = clocale_stream.str(); //Avoid << because it does implicit character conversion (though that might not be a problem here. Not sure). murrayc
-
-  //TODO: What about the precision and width values?
-  /* From the postgres docs:
-  *  The scale of a numeric is the count of decimal digits in the fractional part, to the right of the decimal point.
-  * The precision of a numeric is the total count of significant digits in the whole number, that is, the number of digits to both sides of the decimal point.
-  * So the number 23.5141 has a precision of 6 and a scale of 4. Integers can be considered to have a scale of zero.
-  */
-  gda_numeric.number = g_strdup(number_canonical_text.c_str());
-
-  return Gnome::Gda::Value(&gda_numeric);
+  Gnome::Gda::Numeric numeric;
+  numeric.set_double(number);
+  return Gnome::Gda::Value(numeric);
 }
 
 Gnome::Gda::Value Conversions::parse_value(Field::glom_field_type glom_type, const Glib::ustring& text, bool& success, bool iso_format)
@@ -536,8 +541,17 @@ Gnome::Gda::Value Conversions::parse_value(Field::glom_field_type glom_type, con
 
 Gnome::Gda::Value Conversions::parse_value(Field::glom_field_type glom_type, const Glib::ustring& text, const NumericFormat& numeric_format, bool& success, bool iso_format)
 {
-  const std::locale the_locale = (iso_format ? std::locale::classic() :  std::locale("") /* The user's current locale */);
-
+  std::locale the_locale;
+  try
+  {
+    the_locale = (iso_format ? std::locale::classic() :  std::locale("") /* The user's current locale */);
+  }
+  catch(const std::runtime_error& ex)
+  {
+    std::cerr << G_STRFUNC << ": exception from std::locale(\"\")): " << ex.what() << std::endl;
+    std::cerr << "  This can happen if the locale is not properly installed or configured." << std::endl;
+  } 
+  
   //Put a NULL in the database for empty dates, times, and numerics, because 0 would be an actual value.
   //But we use "" for strings, because the distinction between NULL and "" would not be clear to users.
   if(text.empty())
@@ -602,31 +616,13 @@ Gnome::Gda::Value Conversions::parse_value(Field::glom_field_type glom_type, con
     the_stream >> the_number;  //TODO: Does this throw any exception if the text is an invalid number?
 
     //std::cout << "debug: " << G_STRFUNC << ": text=" << text_to_parse << ", number=" << the_number << std::endl;
-    
+   
 
-    GdaNumeric gda_numeric = {0, 0, 0, 0};
-
-    //Then generate a canonical representation of the number:
-
-    std::stringstream clocale_stream;
-    clocale_stream.imbue( std::locale::classic() ); //The C locale.
-
-    //Avoid the e syntax. Normally this happens afer 7 digits, with loss of precision. TODO: Handle more.
-    clocale_stream << std::setprecision( NumericFormat::get_default_precision() );
-    clocale_stream << the_number;
-    const Glib::ustring number_canonical_text = clocale_stream.str(); //Avoid << because it does implicit character conversion (though that might not be a problem here. Not sure). murrayc
-    //std::cout << "  DEBUG: number_canonical_text=" << number_canonical_text << std::endl;  
-
-    //TODO: What about the precision and width values?
-    /* From the postgres docs:
-    *  The scale of a numeric is the count of decimal digits in the fractional part, to the right of the decimal point.
-    * The precision of a numeric is the total count of significant digits in the whole number, that is, the number of digits to both sides of the decimal point.
-    * So the number 23.5141 has a precision of 6 and a scale of 4. Integers can be considered to have a scale of zero.
-    */
-    gda_numeric.number = g_strdup(number_canonical_text.c_str());
+    Gnome::Gda::Numeric numeric;
+    numeric.set_double(the_number);
 
     success = true; //Can this ever fail?
-    return Gnome::Gda::Value(&gda_numeric);
+    return Gnome::Gda::Value(numeric);
   }
   else if(glom_type == Field::TYPE_BOOLEAN)
   {
@@ -648,7 +644,19 @@ Gnome::Gda::Value Conversions::parse_value(Field::glom_field_type glom_type, con
 
 tm Conversions::parse_date(const Glib::ustring& text, bool& success)
 {
-  return parse_date( text, std::locale("") /* the user's current locale */, success ); //Get the current locale.
+  try
+  {
+    return parse_date( text, std::locale("") /* the user's current locale */, success ); //Get the current locale.
+  }
+  catch(const std::runtime_error& ex)
+  {
+    std::cerr << G_STRFUNC << ": exception from std::locale(\"\")): " << ex.what() << std::endl;
+    std::cerr << "  This can happen if the locale is not properly installed or configured." << std::endl;
+    
+    tm the_c_time;
+    memset(&the_c_time, 0, sizeof(the_c_time));
+    return the_c_time;
+  } 
 }
 
 tm Conversions::parse_date(const Glib::ustring& text, const std::locale& locale, bool& success)
@@ -710,7 +718,7 @@ tm Conversions::parse_date(const Glib::ustring& text, const std::locale& locale,
   }
   else
   {
-    //time_get can fail just because you have entered "1/2/1903" instead "01/02/1903", 
+    //time_get can fail just because you have entered "1/2/1903" instead of "01/02/1903", 
     //or maybe we chose to skip it because it is not useful in this locale,
     //so let's try another, more liberal, way:
     Glib::Date date;
@@ -763,10 +771,24 @@ tm Conversions::parse_date(const Glib::ustring& text, const std::locale& locale,
 
 tm Conversions::parse_time(const Glib::ustring& text, bool& success)
 {
+  //Initialize output parameter:
+  success = false;
+
   //return parse_time( text, std::locale("") /* the user's current locale */ ); //Get the current locale.
 
   //time_get() does not seem to work with non-C locales.  TODO: Try again.
-  tm the_time = parse_time( text, std::locale("") /* the user's current locale */, success );
+  tm the_time;
+  
+  try
+  {
+    the_time = parse_time( text, std::locale("") /* the user's current locale */, success );
+  }
+  catch(const std::runtime_error& ex)
+  {
+    std::cerr << G_STRFUNC << ": exception from std::locale(\"\")): " << ex.what() << std::endl;
+    std::cerr << "  This can happen if the locale is not properly installed or configured." << std::endl;
+  } 
+  
   if(success)
   {
     return the_time;
@@ -926,10 +948,12 @@ bool Conversions::value_is_empty(const Gnome::Gda::Value& value)
 
   switch(value.get_value_type())
   {
+    case(0):
+      return true; //Empty and invalid. It has not been initalized with a type.
     case(G_TYPE_STRING):
       return value.get_string().empty();
     default:
-      return false; //None of the other types can be empty. (An empty numeric, date, or time type shows up as a NULL).
+      return false; //None of the other types can be empty. (An empty numeric, date, or time type shows up as a GDA_TYPE_NULL).
   }
 }
 
@@ -1010,7 +1034,7 @@ Gnome::Gda::Value Conversions::convert_value(const Gnome::Gda::Value& value, Fie
   }
 
   //Fallback for other conversions:
-  const Glib::ustring text = get_text_for_gda_value(source_glom_type, value);
+  const Glib::ustring text = get_text_for_gda_value(source_glom_type, value, std::locale::classic(), NumericFormat(), true /* iso_format */);
   bool test = false;
   return parse_value(target_glom_type, text, test, true /* iso_format */);
 }
