@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #ifndef GLOM_DATASTRUCTURE_LAYOUTITEM_H
@@ -62,7 +62,11 @@ public:
 
   /// This is used only for the print layouts.
   void set_print_layout_position(double x, double y, double width, double height);
+  
+  /// This is used only for the print layouts.
+  void set_print_layout_position_y(double y);
 
+  //TODO: Do we use this? Do we want to?
   /// This is used only for the print layouts.
   void set_print_layout_split_across_pages(bool split = true);
 
