@@ -14,11 +14,11 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
-#include "layoutitem_button.h"
+#include <libglom/data_structure/layout/layoutitem_button.h>
 #include <glibmm/i18n.h>
 
 namespace Glom
@@ -64,6 +64,7 @@ LayoutItem_Button& LayoutItem_Button::operator=(const LayoutItem_Button& src)
 
 Glib::ustring LayoutItem_Button::get_part_type_name() const
 {
+  //Translators: This is the name of a UI element (a layout part name).
   return _("Button");
 }
 
