@@ -14,11 +14,11 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
-#include "layoutitem_field.h"
+#include <libglom/data_structure/layout/layoutitem_field.h>
 #include <glibmm/i18n.h>
 
 namespace Glom
@@ -44,7 +44,7 @@ LayoutItem_Field::LayoutItem_Field(const LayoutItem_Field& src)
   m_formatting_use_default(src.m_formatting_use_default),
   m_title_custom(src.m_title_custom)
 {
-//g_warning("LayoutItem_Field::LayoutItem_Field: m_choices_related_relationship=%s, src.m_choices_related_relationship=%s", m_choices_related_relationship.c_str(), src.m_choices_related_relationship.c_str());
+//std::cerr << G_STRFUNC << ": m_choices_related_relationship=" << m_choices_related_relationship << ", src.m_choices_related_relationship=" << src.m_choices_related_relationship << std::endl;
 
   m_field = src.m_field;
 }
@@ -69,12 +69,12 @@ bool LayoutItem_Field::operator==(const LayoutItem_Field& src) const
     (m_field_cache_valid == src.m_field_cache_valid);
 
   if(m_field && src.m_field)
-    result == result && (*m_field == *(src.m_field));
+    result = result && (*m_field == *(src.m_field));
   else
     result = result && (m_field == src.m_field);
 
   if(m_title_custom && src.m_title_custom)
-    result == result && (*m_title_custom == *(src.m_title_custom));
+    result = result && (*m_title_custom == *(src.m_title_custom));
   else
     result = result && (m_title_custom == src.m_title_custom);
 
@@ -115,28 +115,63 @@ Glib::ustring LayoutItem_Field::get_name() const
   return LayoutItem_WithFormatting::get_name();
 }
 
-Glib::ustring LayoutItem_Field::get_title_or_name_no_custom() const
+Glib::ustring LayoutItem_Field::get_title_no_custom(const Glib::ustring& locale) const
 {
   //Use the field's default title:
   if(m_field_cache_valid && m_field)
   {
-    return m_field->get_title_or_name();
+    return m_field->get_title_or_name(locale);
   }
   else
     return get_name(); //We ignore TranslatableItem::get_title() for LayoutItem_Field.
 }
 
-Glib::ustring LayoutItem_Field::get_title_or_name() const
+Glib::ustring LayoutItem_Field::get_title_no_custom_translation(const Glib::ustring& locale, bool fallback) const
+{
+  //Use the field's default title:
+  if(m_field_cache_valid && m_field)
+  {
+    return m_field->get_title_translation(locale, fallback);
+  }
+  else
+    return Glib::ustring();
+}
+
+Glib::ustring LayoutItem_Field::get_title_or_name_no_custom(const Glib::ustring& locale) const
+{
+  //Use the field's default title:
+  if(m_field_cache_valid && m_field)
+  {
+    return m_field->get_title(locale);
+  }
+
+  return Glib::ustring();
+}
+
+Glib::ustring LayoutItem_Field::get_title(const Glib::ustring& locale) const
 {
   //Use the custom title (overriding the field's default title), if there is one:
   //This may even be empty if the developer specifies that.
   if(m_title_custom && m_title_custom->get_use_custom_title())
   {
-    return m_title_custom->get_title();
+    return m_title_custom->get_title(locale);
   }
 
   //Use the field's default title:
-  return get_title_or_name_no_custom();
+  return get_title_no_custom(locale);
+}
+
+Glib::ustring LayoutItem_Field::get_title_or_name(const Glib::ustring& locale) const
+{
+  //Use the custom title (overriding the field's default title), if there is one:
+  //This may even be empty if the developer specifies that.
+  if(m_title_custom && m_title_custom->get_use_custom_title())
+  {
+    return m_title_custom->get_title(locale);
+  }
+
+  //Use the field's default title:
+  return get_title_no_custom(locale);
 }
 
 bool LayoutItem_Field::get_editable_and_allowed() const
@@ -190,6 +225,7 @@ void LayoutItem_Field::set_hidden(bool val)
 
 Glib::ustring LayoutItem_Field::get_part_type_name() const
 {
+  //Translators: This is the name of a UI element (a layout part name).
   return _("Field");
 }
 
@@ -208,7 +244,7 @@ void LayoutItem_Field::set_formatting_use_default(bool use_default)
   m_formatting_use_default = use_default;
 }
 
-const FieldFormatting& LayoutItem_Field::get_formatting_used() const
+const Formatting& LayoutItem_Field::get_formatting_used() const
 {
   if(m_formatting_use_default && m_field_cache_valid && m_field)
     return m_field->m_default_formatting;
@@ -216,35 +252,49 @@ const FieldFormatting& LayoutItem_Field::get_formatting_used() const
     return m_formatting;
 }
 
-FieldFormatting::HorizontalAlignment LayoutItem_Field::get_formatting_used_horizontal_alignment() const
+Formatting::HorizontalAlignment LayoutItem_Field::get_formatting_used_horizontal_alignment(bool for_details_view) const
 {
-  const FieldFormatting& format = get_formatting_used();
-  FieldFormatting::HorizontalAlignment alignment = 
+  const Formatting& format = get_formatting_used();
+  Formatting::HorizontalAlignment alignment = 
     format.get_horizontal_alignment();
   
-  if(alignment == FieldFormatting::HORIZONTAL_ALIGNMENT_AUTO)
+  if(alignment == Formatting::HORIZONTAL_ALIGNMENT_AUTO)
   {
-    //By default, right-align numbers unless they are ID fields:
-    if(m_field && !m_field->get_primary_key()) //TODO: Also prevent this when it is a foreign key.
+    //By default, right-align numbers on list views, unless they are ID fields.
+    //And left-align them on details views, because that looks silly otherwise.
+    if(!for_details_view && (m_field && !m_field->get_primary_key())) //TODO: Also prevent this when it is a foreign key.
     {
       //Align numbers to the right by default:
-      alignment = (m_field->get_glom_type() == Field::TYPE_NUMERIC ? FieldFormatting::HORIZONTAL_ALIGNMENT_RIGHT : FieldFormatting::HORIZONTAL_ALIGNMENT_LEFT);
+      alignment = (m_field->get_glom_type() == Field::TYPE_NUMERIC ? Formatting::HORIZONTAL_ALIGNMENT_RIGHT : Formatting::HORIZONTAL_ALIGNMENT_LEFT);
     }
     else
-      alignment = FieldFormatting::HORIZONTAL_ALIGNMENT_LEFT;
+      alignment = Formatting::HORIZONTAL_ALIGNMENT_LEFT;
   }
   
   return alignment;
 }
 
+bool LayoutItem_Field::get_formatting_used_has_translatable_choices() const
+{
+  const Formatting& formatting = get_formatting_used();
+  if(!formatting.get_has_custom_choices())
+    return false;
+
+  bool as_radio_buttons = false; //Ignored.
+  if(!formatting.get_choices_restricted(as_radio_buttons))
+    return false;
+
+  return true;
+}
+
 
 void LayoutItem_Field::set_full_field_details(const sharedptr<const Field>& field)
 {
 
   if(field)
   {
-    //std::cout << "LayoutItem_Field::set_full_field_details(): name=" << field->get_name() << std::endl;
-    //std::cout << "LayoutItem_Field::set_full_field_details(): field->get_title_or_name()=" << field->get_title_or_name() << std::endl;
+    //std::cout << "debug: " << G_STRFUNC << ": name=" << field->get_name() << std::endl;
+    //std::cout << "debug: " << G_STRFUNC << ": field->get_title_or_name()=" << field->get_title_or_name() << std::endl;
     m_field = field;
     m_field_cache_valid = true;
 
@@ -298,12 +348,6 @@ bool LayoutItem_Field::is_same_field(const sharedptr<const LayoutItem_Field>& fi
          (*uses_a == *uses_b);
 }
 
-Glib::ustring LayoutItem_Field::get_sql_name(const Glib::ustring& parent_table) const
-{
-  return "\"" + get_sql_table_or_join_alias_name(parent_table) + "\".\"" + get_name() + "\"";
-}
-
-
 } //namespace Glom
 
 
