@@ -14,14 +14,14 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #ifndef GLOM_DATASTRUCTURE_LAYOUTITEM_LINE_H
 #define GLOM_DATASTRUCTURE_LAYOUTITEM_LINE_H
 
-#include "layoutitem.h"
+#include <libglom/data_structure/layout/layoutitem.h>
 #include <libgdamm/value.h>
 
 namespace Glom
@@ -51,10 +51,19 @@ public:
   /** Set the coordinates.
    */
   void set_coordinates(double start_x, double start_y, double end_x, double end_y);
+  
+  double get_line_width() const;
+  void set_line_width(double line_width);
+  
+  //TODO: Document the format:
+  Glib::ustring get_line_color() const;
+  void set_line_color(const Glib::ustring& color);
 
 private:
 
   double m_start_x, m_start_y, m_end_x, m_end_y;
+  double m_line_width;
+  Glib::ustring m_color;
 };
 
 } //namespace Glom
