@@ -14,16 +14,16 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #ifndef GLOM_DATASTRUCTURE_LAYOUTITEM_NOTEBOOK_H
 #define GLOM_DATASTRUCTURE_LAYOUTITEM_NOTEBOOK_H
 
-#include "layoutgroup.h"
-#include "../field.h"
-#include "../relationship.h"
+#include <libglom/data_structure/layout/layoutgroup.h>
+#include <libglom/data_structure/field.h>
+#include <libglom/data_structure/relationship.h>
 
 namespace Glom
 {
