@@ -14,11 +14,11 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
-#include "layoutitem_text.h"
+#include <libglom/data_structure/layout/layoutitem_text.h>
 #include <glibmm/i18n.h>
 
 namespace Glom
@@ -69,6 +69,7 @@ LayoutItem_Text& LayoutItem_Text::operator=(const LayoutItem_Text& src)
 
 Glib::ustring LayoutItem_Text::get_part_type_name() const
 {
+  //Translators: This is the name of a UI element (a layout part name).
   return _("Text");
 }
 
@@ -77,14 +78,19 @@ Glib::ustring LayoutItem_Text::get_report_part_id() const
   return "field"; //We reuse this for this node.
 }
 
-Glib::ustring LayoutItem_Text::get_text() const
+Glib::ustring LayoutItem_Text::get_text(const Glib::ustring& locale) const
 {
-  return m_text->get_title();
+  return m_text->get_title(locale);
 }
 
-void LayoutItem_Text::set_text(const Glib::ustring& text)
+void LayoutItem_Text::set_text(const Glib::ustring& text, const Glib::ustring& locale)
 {
-  m_text->set_title(text);
+  m_text->set_title(text, locale);
+}
+
+void LayoutItem_Text::set_text_original(const Glib::ustring& text)
+{
+  m_text->set_title_original(text);
 }
 
 } //namespace Glom
