@@ -14,11 +14,11 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
-#include "fieldformatting.h"
+#include <libglom/data_structure/layout/formatting.h>
 #include <glibmm/i18n.h>
 
 namespace Glom
@@ -40,7 +40,7 @@ UsesRelationship::~UsesRelationship()
 
 bool UsesRelationship::operator==(const UsesRelationship& src) const
 {
-  return (m_relationship == src.m_relationship) 
+  return (m_relationship == src.m_relationship)
          && (m_related_relationship == src.m_related_relationship);
 }
 
@@ -86,22 +86,22 @@ Glib::ustring UsesRelationship::get_related_relationship_name() const
     return Glib::ustring();
 }
 
-sharedptr<Relationship> UsesRelationship::get_relationship() const
+sharedptr<const Relationship> UsesRelationship::get_relationship() const
 {
   return m_relationship;
 }
 
-void UsesRelationship::set_relationship(const sharedptr<Relationship>& relationship)
+void UsesRelationship::set_relationship(const sharedptr<const Relationship>& relationship)
 {
   m_relationship = relationship;
 }
 
-sharedptr<Relationship> UsesRelationship::get_related_relationship() const
+sharedptr<const Relationship> UsesRelationship::get_related_relationship() const
 {
   return m_related_relationship;
 }
 
-void UsesRelationship::set_related_relationship(const sharedptr<Relationship>& relationship)
+void UsesRelationship::set_related_relationship(const sharedptr<const Relationship>& relationship)
 {
   m_related_relationship = relationship;
 }
@@ -125,7 +125,7 @@ Glib::ustring UsesRelationship::get_sql_table_or_join_alias_name(const Glib::ust
 
 Glib::ustring UsesRelationship::get_table_used(const Glib::ustring& parent_table) const
 {
-  //std::cout << "UsesRelationship::get_table_used(): relationship=" << glom_get_sharedptr_name(m_relationship) << "related_relationship=" << glom_get_sharedptr_name(m_related_relationship) << std::endl;
+  //std::cout << "debug: " << G_STRFUNC << ": relationship=" << glom_get_sharedptr_name(m_relationship) << "related_relationship=" << glom_get_sharedptr_name(m_related_relationship) << std::endl;
 
   if(m_related_relationship)
     return m_related_relationship->get_to_table();
@@ -135,16 +135,32 @@ Glib::ustring UsesRelationship::get_table_used(const Glib::ustring& parent_table
     return parent_table;
 }
 
-Glib::ustring UsesRelationship::get_title_used(const Glib::ustring& parent_table_title) const
+Glib::ustring UsesRelationship::get_title_used(const Glib::ustring& parent_table_title, const Glib::ustring& locale) const
 {
   if(m_related_relationship)
-    return m_related_relationship->get_title_or_name();
+    return m_related_relationship->get_title_or_name(locale);
   else if(m_relationship)
-    return m_relationship->get_title_or_name();
+    return m_relationship->get_title_or_name(locale);
   else
     return parent_table_title;
 }
 
+Glib::ustring UsesRelationship::get_title_singular_used(const Glib::ustring& parent_table_title, const Glib::ustring& locale) const
+{
+  sharedptr<const Relationship> used = m_related_relationship;
+  if(!used)
+    used = m_relationship;
+
+  if(!used)
+    return Glib::ustring();
+
+  const Glib::ustring result = used->get_title_singular(locale);
+  if(!result.empty())
+    return result;
+  else
+    return get_title_used(parent_table_title, locale);
+}
+
 Glib::ustring UsesRelationship::get_to_field_used() const
 {
   if(m_related_relationship)
@@ -189,42 +205,30 @@ Glib::ustring UsesRelationship::get_sql_join_alias_name() const
     const Glib::ustring field_table_name = relationship->get_to_table();
     if(field_table_name.empty())
     {
-      g_warning("build_sql_select_with_where_clause(): field_table_name is null. relationship name = %s", relationship->get_name().c_str());
+      std::cerr << G_STRFUNC << ": field_table_name is null. relationship name = " << relationship->get_name() << std::endl;
     }
     */
 
     if(get_has_related_relationship_name() && m_related_relationship->get_has_fields())
     {
-      result += ("_" + m_related_relationship->get_name());
+      result += ('_' + m_related_relationship->get_name());
     }
   }
 
   return result;
 }
 
-Glib::ustring UsesRelationship::get_sql_join_alias_definition() const
+Glib::ustring UsesRelationship::get_relationship_display_name() const
 {
   Glib::ustring result;
+  
+  if(get_has_relationship_name())
+    result = get_relationship_name();
 
-  if(!get_has_related_relationship_name())
-  {
-    result = " LEFT OUTER JOIN \"" + m_relationship->get_to_table() + "\""
-             + " AS \"" + get_sql_join_alias_name() + "\"" //Specify an alias, to avoid ambiguity when using 2 relationships to the same table.
-             + " ON (\"" + m_relationship->get_from_table() + "\".\"" + m_relationship->get_from_field() + "\" = \""
-             + get_sql_join_alias_name() + "\".\"" + m_relationship->get_to_field() + "\")";
-  }
-  else
-  {
-     UsesRelationship parent_relationship;
-     parent_relationship.set_relationship(m_relationship);
-     result = " LEFT OUTER JOIN \"" + m_related_relationship->get_to_table() + "\""
-             + " AS \"" + get_sql_join_alias_name() + "\"" //Specify an alias, to avoid ambiguity when using 2 relationships to the same table.
-             + " ON (\"" + parent_relationship.get_sql_join_alias_name() + "\".\"" + m_related_relationship->get_from_field() + "\" = \""
-             + get_sql_join_alias_name() + "\".\"" + m_related_relationship->get_to_field() + "\")";
-  }
+  if(get_has_related_relationship_name())
+    result += ("::" + get_related_relationship_name());
+  
   return result;
 }
 
 } //namespace Glom
-
-
