@@ -14,11 +14,11 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
-#include "report.h"
+#include <libglom/data_structure/report.h>
 
 namespace Glom
 {
@@ -57,5 +57,16 @@ void Report::set_show_table_title(bool show_table_title)
   m_show_table_title = show_table_title;
 }
 
+
+sharedptr<LayoutGroup> Report::get_layout_group()
+{
+  return m_layout_group;
+}
+
+sharedptr<const LayoutGroup> Report::get_layout_group() const
+{
+  return m_layout_group;
+}
+
 } //namespace Glom
 
