@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #ifndef GLOM_DATASTRUCTURE_REPORT_H
@@ -38,9 +38,11 @@ public:
   bool get_show_table_title() const;
   void set_show_table_title(bool show_table_title = true);
 
-  sharedptr<LayoutGroup> m_layout_group;
+  sharedptr<LayoutGroup> get_layout_group();
+  sharedptr<const LayoutGroup> get_layout_group() const;
 
 private:
+  sharedptr<LayoutGroup> m_layout_group;
   bool m_show_table_title;
 };
 
