@@ -14,27 +14,46 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #ifndef GLOM_DATASTRUCTURE_TABLEINFO_H
 #define GLOM_DATASTRUCTURE_TABLEINFO_H
 
-#include "translatable_item.h"
-#include <glibmm/ustring.h>
+#include <libglom/data_structure/translatable_item.h>
+#include <libglom/data_structure/has_title_singular.h>
 
 namespace Glom
 {
 
-class TableInfo : public TranslatableItem
+class TableInfo
+ : public TranslatableItem,
+   public HasTitleSingular
 {
 public:
   TableInfo();
   TableInfo(const TableInfo& src);
   TableInfo& operator=(const TableInfo& src);
 
-  guint m_sequence; //TODO: Use this?
+  /** Returns true if this table should not be shown in the list of tables when in operator mode.
+   */
+  bool get_hidden() const;
+
+  /** See get_default().
+   */
+  void set_hidden(bool val = true);
+
+  /** Returns true if this table should be shown when the system is opened.
+   * Only one table can be the default table.
+   */
+  bool get_default() const;
+
+  /** See get_default().
+   */
+  void set_default(bool val = true);
+
+private:
   bool m_hidden;
   bool m_default;
 };
