@@ -17,6 +17,7 @@
  */
 
 #include <libglom/document/bakery/document_xml.h>
+#include <libglom/xml_utils.h>
 #include <iostream>
 
 namespace GlomBakery
@@ -51,7 +52,7 @@ bool Document_XML::load_after(int& failure_code)
     //m_DOM_Parser.setDoValidation(true);
 
     if(m_strContents.empty())
-      g_warning("Document_XML::load_after(): parsing empty document.");
+      std::cerr << G_STRFUNC << ": parsing empty document." << std::endl;
 
     m_DOM_Parser.parse_memory(m_strContents);
     m_pDOM_Document = m_DOM_Parser.get_document();
@@ -78,43 +79,16 @@ bool Document_XML::save_before()
     m_strContents.erase();
 
     Util_DOM_Write(m_strContents);
+  }
 
-    //Save the XML string:
-    return type_base::save_before();
-  }
-  else
-  {
-    return true; //Success. (At doing nothing, because nothing needed to be done.)
-  }
+  //Write the file even if nothing was modified,
+  //to make sure that we write empty documents that have only default values,
+  //when save() is explicitly called.
+  //
+  //Save the XML string:
+  return type_base::save_before();
 }
 
-//TODO: Why is this parameter here if it is not used?
-bool Document_XML::set_xml(const Glib::ustring& /* strXML */)
-{ 
-#ifdef LIBXMLCPP_EXCEPTIONS_ENABLED
-  try
-#endif
-  {
-    if(m_strContents.empty())
-      g_warning("Document_XML::set_xml(): parsing empty document.");
-
-    m_DOM_Parser.parse_memory(m_strContents);
-    m_pDOM_Document = m_DOM_Parser.get_document();
-    if(!m_pDOM_Document)
-      return false;
-
-    return true;	
-  }
-#ifdef LIBXMLCPP_EXCEPTIONS_ENABLED
-  catch(const std::exception& ex)
-  {
-    std::cerr << "XML Parser error: \n" << ex.what() << std::endl;
-    return false; //Failed.
-  }
-#endif
-} 
-
-
 Glib::ustring Document_XML::get_xml() const
 {
   //Write XML to string:
@@ -137,73 +111,11 @@ void Document_XML::Util_DOM_Write(Glib::ustring& refstrXML) const
 #ifdef LIBXMLCPP_EXCEPTIONS_ENABLED
   catch(xmlpp::exception& ex)
   {
-    std::cerr << "Document_XML::Util_DOM_Write(): exception caught: " << ex.what() << std::endl;
+    std::cerr << G_STRFUNC << ": exception caught: " << ex.what() << std::endl;
   }
 #endif
 }
 
-Glib::ustring Document_XML::get_node_attribute_value(const xmlpp::Element* node, const Glib::ustring& strAttributeName)
-{
-  if(node)
-  {
-    const xmlpp::Attribute* attribute = node->get_attribute(strAttributeName);
-    if(attribute)
-    {
-      Glib::ustring value = attribute->get_value(); //Success.
-      return value;
-    }
-  }
-
-  return ""; //Failed.
-}
-
-void Document_XML::set_node_attribute_value(xmlpp::Element* node, const Glib::ustring& strAttributeName, const Glib::ustring& strValue)
-{
-  if(node)
-  {
-    xmlpp::Attribute* attribute = node->get_attribute(strAttributeName);
-    if(attribute)
-      attribute->set_value(strValue);
-    else
-    {
-      if(!strValue.empty()) //Don't add an attribute if the value is empty, to keep the document smaller.
-        node->set_attribute(strAttributeName, strValue);
-    }
-  }
-}
-
-xmlpp::Element* Document_XML::get_node_child_named(const xmlpp::Element* node, const Glib::ustring& strName)
-{
-  xmlpp::Element* nodeResult = 0;
-
-  if(node)
-  { 
-    xmlpp::Node::NodeList list = node->get_children(strName);
-
-    //We check all of them, instead of just the first, until we find one,
-    //because get_children() returns, for instance, TextNodes (which are not Elements) for "text", 
-    //as well as Elements with the name "text".
-    for(xmlpp::Node::NodeList::iterator iter = list.begin(); iter != list.end(); ++iter)
-    {
-      nodeResult = dynamic_cast<xmlpp::Element*>(*iter);  
-      if(nodeResult)
-        return nodeResult;
-    }                       
-  }
-
-  return nodeResult;
-}
-
-xmlpp::Element* Document_XML::get_node_child_named_with_add(xmlpp::Element* node, const Glib::ustring& strName)
-{
-  xmlpp::Element* nodeResult = get_node_child_named(node, strName);
-
-  if(!nodeResult)
-    nodeResult = node->add_child(strName);
-
-  return nodeResult;
-}
-
 void Document_XML::set_dtd_name(const std::string& strVal)
 {
   m_strDTD_Name = strVal;
@@ -257,12 +169,6 @@ xmlpp::Element* Document_XML::get_node_document()
   return nodeRoot;
 }
 
-void Document_XML::set_write_formatted(bool formatted)
-{
-  m_write_formatted = formatted;
-}
-
-
 void Document_XML::add_indenting_white_space_to_node(xmlpp::Node* node, const Glib::ustring& start_indent)
 {
   if(!node)
