@@ -14,12 +14,12 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
-#ifndef DOCUMENT_GLOM_H
-#define DOCUMENT_GLOM_H
+#ifndef GLOM_DOCUMENT_H
+#define GLOM_DOCUMENT_H
 
 #include <libglom/libglom_config.h>
 
@@ -37,6 +37,7 @@
 #include <libglom/data_structure/report.h>
 #include <libglom/data_structure/print_layout.h>
 #include <libglom/data_structure/foundset.h>
+#include <libglom/data_structure/database_title.h>
 #include <libglom/appstate.h>
 #include <vector>
 #include <map>
@@ -58,6 +59,12 @@ public:
 
   virtual void set_modified(bool value = true);
 
+  /** Set the file URI that will be used in future calls to load() and save().
+   * Note that the document will not be saved immediately to the new URI. It will
+   * be saved either after the next change (if using autosave) or when calling save() explicitly.
+   * Likewise, the document at the URI will not be loaded until load() is called explicitly.
+   * That is unlike in the base class's implementation.
+   */
   virtual void set_file_uri(const Glib::ustring& file_uri, bool bEnforceFileExtension = false);
 
   /* Loads data from disk, using the URI (set with set_file_uri()) then asks the View to update itself.
@@ -168,6 +175,11 @@ public:
    */
   Glib::ustring get_translation_original_locale() const;
 
+  /** Get a list of locales for which at least one string is translated.
+   * The result will include the original, from get_translation_original_locale().
+   */
+  std::vector<Glib::ustring> get_translation_available_locales() const;
+
   typedef std::vector< sharedptr<Relationship> > type_vec_relationships;
   type_vec_relationships get_relationships(const Glib::ustring& table_name, bool plus_system_prefs = false) const;
   void set_relationships(const Glib::ustring& table_name, const type_vec_relationships& vecRelationships);
@@ -193,10 +205,22 @@ public:
 
   sharedptr<Field> get_field(const Glib::ustring& table_name, const Glib::ustring& strFieldName) const;
 
+  sharedptr<Field> get_field_primary_key(const Glib::ustring& table_name) const;
+
   /** Use this after removing a field from a table,
    * so that it is not used anymore in relationships, layouts, reports, etc.
    */
   void remove_field(const Glib::ustring& table_name, const Glib::ustring& field_name);
+  
+  
+  typedef std::pair< sharedptr<LayoutItem_Field>, sharedptr<Relationship> > type_pairFieldTrigger;
+  typedef std::vector<type_pairFieldTrigger> type_list_lookups;
+  
+  /** Get the fields whose values should be looked up when @a field_name changes, with
+   * the relationship used to lookup the value.
+   */
+  type_list_lookups get_lookup_fields(const Glib::ustring& table_name, const Glib::ustring& field_name) const;
+
 
 
   typedef std::vector< sharedptr<LayoutGroup> > type_list_layout_groups;
@@ -234,15 +258,18 @@ public:
 
   type_list_layout_groups get_data_layout_groups_default(const Glib::ustring& layout_name, const Glib::ustring& parent_table_name, const Glib::ustring& layout_platform = Glib::ustring()) const;
 
-  typedef std::list< sharedptr<TranslatableItem> > type_list_translatables;
-  type_list_translatables get_translatable_layout_items(const Glib::ustring& table_name);
-  type_list_translatables get_translatable_report_items(const Glib::ustring& table_name, const Glib::ustring& report_title);
+  /// The translatable item and a hint about what it is.
+  typedef std::pair< sharedptr<TranslatableItem>, Glib::ustring> pair_translatable_item_and_hint;
+
+  typedef std::vector<pair_translatable_item_and_hint> type_list_translatables;
+  type_list_translatables get_translatable_items();
+
+  static void fill_translatable_custom_choices(Formatting& formatting, type_list_translatables& the_list, const Glib::ustring& hint);
+
 
   void fill_layout_field_details(const Glib::ustring& parent_table_name, const sharedptr<LayoutGroup>& layout_group) const;
   void fill_layout_field_details(const Glib::ustring& parent_table_name, type_list_layout_groups& groups) const;
 
-
-
   ///When a field name is changed, change it in the relationships, layouts, reports, and fields data:
   void change_field_name(const Glib::ustring& table_name, const Glib::ustring& strFieldNameOld, const Glib::ustring& strFieldNameNew);
 
@@ -252,14 +279,14 @@ public:
   ///When a relationship name is changed, change it in layouts and reports:
   void change_relationship_name(const Glib::ustring& table_name, const Glib::ustring& name, const Glib::ustring& name_new);
 
-  typedef std::list< sharedptr<TableInfo> > type_listTableInfo;
+  typedef std::vector< sharedptr<TableInfo> > type_listTableInfo;
   type_listTableInfo get_tables(bool plus_system_prefs = false) const;
   std::vector<Glib::ustring> get_table_names(bool plus_system_prefs = false) const;
 
   void set_tables(const type_listTableInfo& tables);
 
   sharedptr<TableInfo> get_table(const Glib::ustring& table_name) const;
-  void add_table(const  sharedptr<TableInfo>& table_name);
+  void add_table(const sharedptr<TableInfo>& table_name);
 
   /** Use this after DROPing the table.
    * It removes information about the table, including fields and layouts,
@@ -270,10 +297,12 @@ public:
   bool get_table_is_known(const Glib::ustring& table_name) const;
   bool get_table_is_hidden(const Glib::ustring& table_name) const;
 
-  Glib::ustring get_table_title(const Glib::ustring& table_name) const;
-  void set_table_title(const Glib::ustring& table_name, const Glib::ustring& value);
+  Glib::ustring get_table_title(const Glib::ustring& table_name, const Glib::ustring& locale) const;
+  Glib::ustring get_table_title_original(const Glib::ustring& table_name) const;
+  void set_table_title(const Glib::ustring& table_name, const Glib::ustring& value, const Glib::ustring& locale);
 
-  Glib::ustring get_table_title_singular(const Glib::ustring& table_name) const;
+  Glib::ustring get_table_title_singular(const Glib::ustring& table_name, const Glib::ustring& locale) const;
+  Glib::ustring get_table_title_singular_original(const Glib::ustring& table_name) const;
 
   typedef std::vector< Gnome::Gda::Value > type_row_data;
   typedef std::vector< type_row_data > type_example_rows;
@@ -292,8 +321,9 @@ public:
   Glib::ustring get_default_table() const;
   Glib::ustring get_first_table() const;
 
-  Glib::ustring get_database_title() const;
-  void set_database_title(const Glib::ustring& title);
+  Glib::ustring get_database_title_original() const;
+  Glib::ustring get_database_title(const Glib::ustring& locale) const;
+  void set_database_title_original(const Glib::ustring& title);
 
   std::vector<Glib::ustring> get_library_module_names() const;
   void set_library_module(const Glib::ustring& name, const Glib::ustring& script);
@@ -309,7 +339,7 @@ public:
   void set_startup_script(const Glib::ustring& script);
 
   /// These are only used when recreating a database from an example file. The actualy access-control is on the server, of course.
-  typedef std::list<GroupInfo> type_list_groups;
+  typedef std::vector<GroupInfo> type_list_groups;
   type_list_groups get_groups() const;
 
   /// This adds the group if necessary.
@@ -318,18 +348,14 @@ public:
   void remove_group(const Glib::ustring& group_name);
 
 
-  typedef std::list<Glib::ustring> type_listReports;
-  type_listReports get_report_names(const Glib::ustring& table_name) const;
-  void remove_all_reports(const Glib::ustring& table_name);
+  std::vector<Glib::ustring> get_report_names(const Glib::ustring& table_name) const;
 
   void set_report(const Glib::ustring& table_name, const sharedptr<Report>& report);
   sharedptr<Report> get_report(const Glib::ustring& table_name, const Glib::ustring& report_name) const;
   void remove_report(const Glib::ustring& table_name, const Glib::ustring& report_name);
 
   //Print Layouts are precisely positioned layouts for printing to a printer:
-  typedef std::list<Glib::ustring> type_listPrintLayouts;
-  type_listPrintLayouts get_print_layout_names(const Glib::ustring& table_name) const;
-  void remove_all_print_layouts(const Glib::ustring& table_name);
+  std::vector<Glib::ustring> get_print_layout_names(const Glib::ustring& table_name) const;
   void set_print_layout(const Glib::ustring& table_name, const sharedptr<PrintLayout>& print_layout);
   sharedptr<PrintLayout> get_print_layout(const Glib::ustring& table_name, const Glib::ustring& print_layout_name) const;
   void remove_print_layout(const Glib::ustring& table_name, const Glib::ustring& print_layout_name);
@@ -415,29 +441,32 @@ public:
    */
   void set_active_layout_platform(const Glib::ustring& layout_platform = Glib::ustring());
 
-  /** Change any non-maemo layouts so their groups never have multiple columns,
-   * because this could never be displayed on the Maemo screen.
-   * This allows layouts to be shown fairly sanely on Maemo even if no custom
-   * maemo layout was defined.
-   * This does not save the change to disk.
-   */
-  void maemo_restrict_layouts_to_single_column();
-
 #ifndef SWIG //Hide this API from swig.
+
   Glib::ustring build_and_get_contents() const;
 
+  /** This callback should show UI to indicate that work is still happening.
+   * For instance, a pulsing ProgressBar.
+   */
+  typedef sigc::slot<void> SlotProgress;
+  
+  /** Save a copy of the document as a backup.
+   * This document (and its URI) will not be changed.
+   * @param The location at which to save the backup Glom file.
+   * @result The URI of the .tar.gz tarball.
+   */
+  Glib::ustring save_backup_file(const Glib::ustring& uri, const SlotProgress& slot_progress);
+  
+  /**
+   * @param backup_uri: The URI of a .tar.gz backup file.
+   * @result The URI of the restored .glom file.
+   */
+  static Glib::ustring restore_backup_file(const Glib::ustring& backup_uri, const SlotProgress& slot_progress);
+  
+
 protected:
 
 
-  //TODO: Remove this:
-  friend class ReportBuilder;
-
-  ///If value is equal to the default then no attribute will be set, to save text space in the XML file.
-  static void set_node_attribute_value_as_decimal(xmlpp::Element* node, const Glib::ustring& strAttributeName, guint value, guint value_default = 0);
-
-  // This is required by the report builder, so it cannot be disabled
-  // in client only mode
-  static void set_node_attribute_value_as_decimal_double(xmlpp::Element* node, const Glib::ustring& strAttributeName, double value);
 #endif //SWIG
 
 #ifndef SWIG
@@ -462,11 +491,12 @@ private:
   void save_before_sort_by(xmlpp::Element* node, const LayoutItem_GroupBy::type_list_sort_fields& list_fields);
   void save_before_layout_item_usesrelationship(xmlpp::Element* nodeItem, const sharedptr<const UsesRelationship>& item);
   void save_before_layout_item_field(xmlpp::Element* nodeItem, const sharedptr<const LayoutItem_Field>& item);
-  void save_before_layout_item_formatting(xmlpp::Element* nodeItem, const FieldFormatting& format, Field::glom_field_type field_type = Field::TYPE_INVALID);
+  void save_before_layout_item_formatting(xmlpp::Element* nodeItem, const Formatting& format, Field::glom_field_type field_type = Field::TYPE_INVALID);
   void save_before_layout_item_formatting(xmlpp::Element* nodeItem, const sharedptr<const LayoutItem_WithFormatting>& layout_item);
 
-  void save_before_translations(xmlpp::Element* nodeItem, const TranslatableItem& item);
+  void save_before_translations(xmlpp::Element* nodeItem, const sharedptr<const TranslatableItem>& item);
   void save_before_print_layout_position(xmlpp::Element* nodeItem, const sharedptr<const LayoutItem>& item);
+  void save_before_choicevalue(xmlpp::Element* nodeItem, const sharedptr<const ChoiceValue>& item, Field::glom_field_type field_type);
 
   void save_changes();
 
@@ -475,36 +505,22 @@ private:
   void load_after_sort_by(const xmlpp::Element* node, const Glib::ustring& table_name, LayoutItem_GroupBy::type_list_sort_fields& list_fields);
   void load_after_layout_item_usesrelationship(const xmlpp::Element* element, const Glib::ustring& table_name, const sharedptr<UsesRelationship>& item);
   void load_after_layout_item_field(const xmlpp::Element* element, const Glib::ustring& table_name, const sharedptr<LayoutItem_Field>& item);
-  void load_after_layout_item_formatting(const xmlpp::Element* element, FieldFormatting& format, Field::glom_field_type field_type = Field::TYPE_INVALID, const Glib::ustring& table_name = Glib::ustring(), const Glib::ustring& field_name = Glib::ustring());
+  void load_after_layout_item_formatting(const xmlpp::Element* element, Formatting& format, Field::glom_field_type field_type = Field::TYPE_INVALID, const Glib::ustring& table_name = Glib::ustring(), const Glib::ustring& field_name = Glib::ustring());
  void load_after_layout_item_formatting(const xmlpp::Element* element, const sharedptr<LayoutItem_WithFormatting>& layout_item, const Glib::ustring& table_name = Glib::ustring());
 
-  void load_after_translations(const xmlpp::Element* element, TranslatableItem& item);
+  void load_after_translations(const xmlpp::Element* element, const sharedptr<TranslatableItem>& item);
   void load_after_print_layout_position(const xmlpp::Element* nodeItem, const sharedptr<LayoutItem>& item);
+  void load_after_choicevalue(const xmlpp::Element* element, const sharedptr<ChoiceValue>& item, Field::glom_field_type field_type);
 
   void on_app_state_userlevel_changed(AppState::userlevels userlevel);
 
-  void fill_translatable_layout_items(const sharedptr<LayoutGroup>& group, type_list_translatables& the_list);
+  static void fill_translatable_layout_items(const sharedptr<LayoutItem_Field>& layout_field, type_list_translatables& the_list, const Glib::ustring& hint);
+  static void fill_translatable_layout_items(const sharedptr<LayoutGroup>& group, type_list_translatables& the_list, const Glib::ustring& hint);
 
+  void fill_sort_field_details(const Glib::ustring& parent_table_name, Formatting::type_list_sort_fields& sort_fields) const;
 
-  /// If the attribute is not there, then the default will be returned.
-  static bool get_node_attribute_value_as_bool(const xmlpp::Element* node, const Glib::ustring& strAttributeName, bool value_default = false);
-  static guint get_node_attribute_value_as_decimal(const xmlpp::Element* node, const Glib::ustring& strAttributeName, guint value_default = 0);
-  static double get_node_attribute_value_as_decimal_double(const xmlpp::Element* node, const Glib::ustring& strAttributeName);
-  static float get_node_attribute_value_as_float(const xmlpp::Element* node, const Glib::ustring& strAttributeName);
-
-  static void set_node_attribute_value_as_bool(xmlpp::Element* node, const Glib::ustring& strAttributeName, bool value = true, bool value_default = false);
-  static void set_node_attribute_value_as_float( xmlpp::Element* node, const Glib::ustring& strAttributeName, float value );
-  static void set_node_attribute_value_as_value(xmlpp::Element* node, const Glib::ustring& strAttributeName, const Gnome::Gda::Value& value, Field::glom_field_type field_type);
-  static void set_node_text_child_as_value(xmlpp::Element* node, const Gnome::Gda::Value& value, Field::glom_field_type field_type);
-
-  static Gnome::Gda::Value get_node_attribute_value_as_value(const xmlpp::Element* node, const Glib::ustring& strAttributeName, Field::glom_field_type field_type);
-  static Gnome::Gda::Value get_node_text_child_as_value(const xmlpp::Element* node, Field::glom_field_type field_type);
-
-  Glib::ustring get_child_text_node(const xmlpp::Element* node, const Glib::ustring& child_node_name) const;
-  void set_child_text_node(xmlpp::Element* node, const Glib::ustring& child_node_name, const Glib::ustring& text);
-
-  ///A recursive helper function.
-  static void maemo_restrict_layouts_to_single_column_group(const sharedptr<LayoutGroup>& layout_group);
+  type_list_translatables get_translatable_layout_items(const Glib::ustring& table_name, const Glib::ustring& hint);
+  type_list_translatables get_translatable_report_items(const Glib::ustring& table_name, const Glib::ustring& report_title, const Glib::ustring& hint);
 
   AppState m_app_state;
   type_signal_userlevel_changed m_signal_userlevel_changed;
@@ -531,7 +547,7 @@ private:
   {
   public:
     DocumentTableInfo()
-      : m_overviewx ( std::numeric_limits<float>::infinity () ),
+      : m_overviewx ( ),
         m_overviewy ( std::numeric_limits<float>::infinity () )
     {
       m_info = sharedptr<TableInfo>(new TableInfo()); //Avoid a null sharedptr.
@@ -578,7 +594,7 @@ private:
     type_vec_fields m_fields;
     type_vec_relationships m_relationships;
 
-    typedef std::list< LayoutInfo > type_layouts;
+    typedef std::vector< LayoutInfo > type_layouts;
     type_layouts m_layouts;
 
     typedef std::map< Glib::ustring, sharedptr<Report> > type_reports; //map of report names to reports
@@ -609,8 +625,9 @@ private:
   typedef std::map<Glib::ustring, GroupInfo> type_map_groups;
   type_map_groups m_groups;
 
-  Glib::ustring m_database_title;
+  sharedptr<DatabaseTitle> m_database_title;
   Glib::ustring m_translation_original_locale;
+  std::vector<Glib::ustring> m_translation_available_locales; //Just a cache, based on other data.
 
   typedef std::map<Glib::ustring, Glib::ustring> type_map_library_scripts;
   type_map_library_scripts m_map_library_scripts;
@@ -631,4 +648,4 @@ private:
 
 } //namespace Glom
 
-#endif //DOCUMENT_GLOM_H
+#endif // GLOM_DOCUMENT_H
