@@ -14,15 +14,16 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #include "privs.h"
 #include <libglom/standard_table_prefs_fields.h>
 #include <libglom/db_utils.h>
 #include <libglom/utils.h>
-//#include <glom/application.h>
+//#include <glom/appwindow.h>
+#include <glibmm/main.h>
 
 #include <iostream>
 
@@ -97,7 +98,11 @@ Glib::ustring Privs::get_default_developer_user_name(Glib::ustring& password)
 
 Privs::type_vec_strings Privs::get_database_users(const Glib::ustring& group_name)
 {
-  //TODO_Moved: BusyCursor cursor(Application::get_application());
+  //TODO_Moved: BusyCursor cursor(AppWindow::get_appwindow());
+
+  //Note that these queries can fail with "permission denied for relation pg_shadow" if the user is not a superuser.
+  //So the caller should interpret an empty result for group_name="" as an error,
+  //because there should always be at least one user.
 
   type_vec_strings result;
 
@@ -132,6 +137,7 @@ Privs::type_vec_strings Privs::get_database_users(const Glib::ustring& group_nam
       builder->add_cond(Gnome::Gda::SQL_OPERATOR_TYPE_EQ,
         builder->add_field_id("groname", "pg_group"),
         builder->add_expr(group_name)));
+    //TODO: Show SQL.
     Glib::RefPtr<Gnome::Gda::DataModel> data_model = DbUtils::query_execute_select(builder);
     if(data_model && data_model->get_n_rows())
     {
@@ -161,6 +167,7 @@ Privs::type_vec_strings Privs::get_database_users(const Glib::ustring& group_nam
           if(data_model && data_model->get_n_rows() && data_model->get_n_columns())
           {
             const Gnome::Gda::Value value = data_model->get_value_at(0, 0);
+            //std::cout << G_STRFUNC << "DEBUG:  username=" << value.get_string() << std::endl; 
             result.push_back(value.get_string());
           }
           else
@@ -176,10 +183,12 @@ Privs::type_vec_strings Privs::get_database_users(const Glib::ustring& group_nam
   return result;
 }
 
-void Privs::set_table_privileges(const Glib::ustring& group_name, const Glib::ustring& table_name, const Privileges& privs, bool developer_privs)
+bool Privs::set_table_privileges(const Glib::ustring& group_name, const Glib::ustring& table_name, const Privileges& privs, bool developer_privs)
 {
   if(group_name.empty() || table_name.empty())
-    return;
+  {
+    return false;
+  }
 
   //Change the permission in the database:
 
@@ -225,28 +234,58 @@ void Privs::set_table_privileges(const Glib::ustring& group_name, const Glib::us
     }
   }
 
-  strQuery += " " + strPrivilege + " ON \"" + table_name + "\" ";
+  strQuery += " " + strPrivilege + " ON " + DbUtils::escape_sql_id(table_name) + " ";
 
   //This must match the Grant or Revoke:
   strQuery += "TO";
 
-  strQuery += " GROUP \"" + group_name + "\"";
+  strQuery += " GROUP " + DbUtils::escape_sql_id(group_name);
 
   const bool test = DbUtils::query_execute_string(strQuery);
   if(!test)
-    std::cerr << G_STRFUNC << ": GRANT failed." << std::endl;
-  else
   {
-    if( (table_name != GLOM_STANDARD_TABLE_AUTOINCREMENTS_TABLE_NAME) && privs.m_create )
+    std::cerr << G_STRFUNC << ": GRANT failed." << std::endl;
+    return false;
+  }
+
+  if( (table_name != GLOM_STANDARD_TABLE_AUTOINCREMENTS_TABLE_NAME) && privs.m_create )
+  {
+    //To create a record, you will usually need write access to the autoincrements table,
+    //so grant this too:
+    Privileges priv_autoincrements;
+    priv_autoincrements.m_view = true;
+    priv_autoincrements.m_edit = true;
+    if(!set_table_privileges(group_name, GLOM_STANDARD_TABLE_AUTOINCREMENTS_TABLE_NAME, priv_autoincrements))
     {
-      //To create a record, you will usually need write access to the autoincrements table,
-      //so grant this too:
-      Privileges priv_autoincrements;
-      priv_autoincrements.m_view = true;
-      priv_autoincrements.m_edit = true;
-      set_table_privileges(group_name, GLOM_STANDARD_TABLE_AUTOINCREMENTS_TABLE_NAME, priv_autoincrements);
+      std::cerr << G_STRFUNC << ": GRANT failed on autoincrements table." << std::endl;
+      return false;
     }
   }
+
+  return true;
+}
+
+static Glib::RefPtr<Gnome::Gda::Connection> get_connection()
+{
+  sharedptr<SharedConnection> sharedconnection;
+  try
+  {
+     sharedconnection = ConnectionPool::get_and_connect();
+  }
+  catch (const Glib::Error& error)
+  {
+    std::cerr << G_STRFUNC << ": " << error.what() << std::endl;
+  }
+
+  if(!sharedconnection)
+  {
+    std::cerr << G_STRFUNC << ": No connection yet." << std::endl;
+    return Glib::RefPtr<Gnome::Gda::Connection>(0);
+  }
+
+  Glib::RefPtr<Gnome::Gda::Connection> gda_connection = sharedconnection->get_gda_connection();
+
+  return gda_connection;
 }
 
 Privileges Privs::get_table_privileges(const Glib::ustring& group_name, const Glib::ustring& table_name)
@@ -266,95 +305,70 @@ Privileges Privs::get_table_privileges(const Glib::ustring& group_name, const Gl
   //Get the permissions:
   Glib::RefPtr<Gnome::Gda::SqlBuilder> builder =
     Gnome::Gda::SqlBuilder::create(Gnome::Gda::SQL_STATEMENT_SELECT);
-  builder->select_add_field("relacl", "pg_class");
-  builder->select_add_target("pg_class");
-  builder->set_where(
-    builder->add_cond(Gnome::Gda::SQL_OPERATOR_TYPE_EQ,
-      builder->add_field_id("relname", "pg_class"),
-      builder->add_expr(table_name)));
-  Glib::RefPtr<Gnome::Gda::DataModel> data_model = DbUtils::query_execute_select(builder);
-  if(data_model && data_model->get_n_rows())
+
+  const Glib::ustring function_name = "has_table_privilege";
+  std::vector<Gnome::Gda::SqlBuilder::Id> args_base;
+
+  Glib::RefPtr<Gnome::Gda::Connection> connection = get_connection();
+  if(!connection)
   {
-    const Gnome::Gda::Value value = data_model->get_value_at(0, 0);
-
-    Glib::ustring access_details;
-    if(!value.is_null())
-      access_details = value.get_string();
-
-    //std::cout << "DEBUG: access_details:" << access_details << std::endl;
-
-    //Parse the strange postgres permissions format:
-    //For instance, "{murrayc=arwdxt/murrayc,operators=r/murrayc}" (Postgres 8.x)
-    //For instance, "{murrayc=arwdxt/murrayc,group operators=r/murrayc}" (Postgres <8.x)
-    const type_vec_strings vecItems = pg_list_separate(access_details);
-    for(type_vec_strings::const_iterator iterItems = vecItems.begin(); iterItems != vecItems.end(); ++iterItems)
-    {
-      Glib::ustring item = *iterItems;
-      //std::cout << "DEBUG: item:" << item << std::endl;
-
-      item = Utils::string_trim(item, "\""); //Remove quotes from front and back.
-
-      //std::cout << "DEBUG: item without quotes:" << item << std::endl;
-
-      //Find group permissions, ignoring user permissions:
-      //We need to find the role by name.
-      // Previous versions of Postgres (8.1, or maybe 7.4) prefixed group names by "group ",
-      // but that doesn't work for recent versions of Postgres,
-      // probably because the user and group concepts have been combined into "roles".
-      //
-      //const Glib::ustring strgroup = "group ";
-      const Glib::ustring strgroup = group_name + "=";
-      Glib::ustring::size_type posFind = item.find(strgroup);
-      if(posFind != Glib::ustring::npos)
-      {
-        //It is the needed group permision:
-
-        //Remove the "group " prefix (not needed for Postgres 8.x):
-        //item = item.substr(strgroup.size());
-        item = item.substr(posFind);
-        //std::cout << "DEBUG: user permissions:" << item << std::endl;
-
-        //Get the parts before and after the =:
-        const type_vec_strings vecParts = Utils::string_separate(item, "=");
-        if(vecParts.size() == 2)
-        {
-          const Glib::ustring this_group_name = vecParts[0];
-          if(this_group_name == group_name) //Only look at permissions for the requested group->
-          {
-            Glib::ustring group_permissions = vecParts[1];
-
-            //Get the part before the /user_who_granted_the_privileges:
-            const type_vec_strings vecParts = Utils::string_separate(group_permissions, "/");
-            if(!vecParts.empty())
-              group_permissions = vecParts[0];
-
-            //g_warning("  group=%s", group_name.c_str());
-            //g_warning("  permisisons=%s", group_permissions.c_str());
-
-            //Iterate through the characters:
-            for(Glib::ustring::iterator iter = group_permissions.begin(); iter != group_permissions.end(); ++iter)
-            {
-              gunichar chperm = *iter;
-              Glib::ustring perm(1, chperm);
-
-              //See http://www.postgresql.org/docs/8.0/interactive/sql-grant.html
-              if(perm == "r")
-                result.m_view = true;
-              else if(perm == "w")
-                result.m_edit = true;
-              else if(perm == "a")
-                result.m_create = true;
-              else if(perm == "d")
-                result.m_delete = true;
-            }
-          }
-        }
-      }
-
-    }
+    std::cerr << ": Could not get a connection." << std::endl;
+    return result;
   }
 
-  //g_warning("get_table_privileges(group_name=%s, table_name=%s) returning: %d", group_name.c_str(), table_name.c_str(), result.m_create);
+  //TODO: Why doesn't this need to be quoted like table_name.
+  //The group names are quoted when we create them:
+  //For some reason, this is what PostgreSQL wants: SELECT has_table_privilege('Some Group', '"Some Table"', 'SELECT');
+  //However, note that libgda does seem to escape characters here (for instance, changing ' to '').
+  //const Glib::ustring group_name_for_arg = connection->quote_sql_identifier(group_name);
+  const Glib::ustring group_name_for_arg = group_name;
+  args_base.push_back(builder->add_expr(group_name_for_arg));
+
+  //The table name must be quoted if it needs to be quoted,
+  //but not quoted if it is does not need to be quoted,
+  //because it must match how it was created, and libgda probably did not quote it unless necessary.
+  const Glib::ustring table_name_for_arg = connection->quote_sql_identifier(table_name);
+  args_base.push_back(builder->add_expr(table_name_for_arg));
+
+  std::vector<Gnome::Gda::SqlBuilder::Id> args = args_base;
+  args.push_back(builder->add_expr("SELECT"));
+  builder->add_field_value_id(builder->add_function(function_name, args));
+  args = args_base;
+  args.push_back(builder->add_expr("UPDATE"));
+  builder->add_field_value_id(builder->add_function(function_name, args));
+  args = args_base;
+  args.push_back(builder->add_expr("INSERT"));
+  builder->add_field_value_id(builder->add_function(function_name, args));
+  args = args_base;
+  args.push_back(builder->add_expr("DELETE"));
+  builder->add_field_value_id(builder->add_function(function_name, args));
+
+  //const Glib::ustring sql_debug = Utils::sqlbuilder_get_full_query(builder);
+  //std::cout << "DEBUG: " << sql_debug << std::endl;
+
+  Glib::RefPtr<Gnome::Gda::DataModel> data_model = DbUtils::query_execute_select(builder);
+  if(!data_model || (data_model->get_n_rows() <= 0))
+  {
+    std::cerr << G_STRFUNC << ": The query returned no data." << std::endl;
+    return result;
+  }
+
+  if(data_model->get_n_columns() < 4)
+  {
+    std::cerr << G_STRFUNC << ": The query did not return enough columns." << std::endl;
+    return result;
+  }
+
+  Gnome::Gda::Value value = data_model->get_value_at(0, 0);
+  result.m_view = value.get_boolean();
+  value = data_model->get_value_at(1, 0);
+  result.m_edit = value.get_boolean();
+  value = data_model->get_value_at(2, 0);
+  result.m_create = value.get_boolean();
+  value = data_model->get_value_at(3, 0);
+  result.m_delete = value.get_boolean();
+  
+  //std::cout << G_STRFUNC << ": group_name=" << group_name << ", table_name=" << table_name << ", returning create=" << result.m_create << std::endl;
   return result;
 }
 
@@ -416,10 +430,16 @@ bool Privs::on_privs_privileges_cache_timeout(const Glib::ustring& table_name)
 
 Privileges Privs::get_current_privs(const Glib::ustring& table_name)
 {
+  if(table_name.empty())
+  {
+    std::cerr << G_STRFUNC << ": table_name is empty." << std::endl;
+    return Privileges();
+  }
+  
   //TODO_Performance: There's lots of database access here.
   //We could maybe replace some with the postgres has_table_* function().
 
-  //TODO_Moved: BusyCursor cursor(Application::get_application());
+  //TODO_Moved: BusyCursor cursor(AppWindow::get_appwindow());
 
   //Return a cached value if possible.
   //(If it is in the cache then it's fairly recent)
