@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #ifndef GLOM_PYTHON_GLOM_RECORD_H
@@ -52,14 +52,14 @@ public:
 
   boost::python::object get_related();
 
-  //[] notation:
-  long len() const;
-  boost::python::object getitem(const boost::python::object& item);
-  void setitem(const boost::python::object& /* key */, const boost::python::object& /* value */);
-
   //Available, for instance, in python via record["name_first"]
   typedef std::map<Glib::ustring, Gnome::Gda::Value> type_map_field_values;
 
+  //[] notation:
+  type_map_field_values::size_type len() const;
+  boost::python::object getitem(const boost::python::object& item);
+  void setitem(const boost::python::object& /* key */, const boost::python::object& /* value */);
+
   void set_fields(const PyGlomRecord::type_map_field_values& field_values,
     Document* document,
     const Glib::ustring& table_name,
