@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #include <libglom/python_embed/py_glom_related.h>
@@ -39,7 +39,7 @@ PyGlomRelated::~PyGlomRelated()
 }
 
 
-long PyGlomRelated::len() const
+PyGlomRelated::type_map_relationships::size_type PyGlomRelated::len() const
 {
   return m_map_relationships.size();
 }
@@ -82,21 +82,14 @@ boost::python::object PyGlomRelated::getitem(const boost::python::object& cppite
             PyGlomRecord::type_map_field_values::const_iterator iterFromKey = record->m_map_field_values.find(from_key);
             if(iterFromKey != record->m_map_field_values.end())
             {
-              const Gnome::Gda::Value from_key_value = iterFromKey->second;
+              const Gnome::Gda::Value& from_key_value = iterFromKey->second;
 
               //TODO_Performance:
               //Get the full field details so we can sqlize its value:
-              sharedptr<Field> from_key_field;
-              from_key_field = record->m_document->get_field(record->m_table_name, from_key);
+              sharedptr<const Field> from_key_field = record->m_document->get_field(record->m_table_name, from_key);
               if(from_key_field)
               {
-                Glib::ustring key_value_sqlized;
-                //std::cout << "from_key_field=" << from_key_field->get_name() << ", from_key_value=" << from_key_value.to_string() << std::endl;
-
-                if(!Conversions::value_is_empty(from_key_value)) //Do not link on null-values. That would cause us to link on 0, or "0".
-                  key_value_sqlized = from_key_field->sql(from_key_value);
-
-                PyGlomRelatedRecord_SetRelationship(pyRelatedRecord, iterFind->second, key_value_sqlized, record->m_document);
+                pyRelatedRecord->set_relationship(iterFind->second, from_key_value, record->m_document);
 
                 //Store it in the cache:
                 boost::python::object objectRelatedRecord(pyRelatedRecord);
@@ -126,9 +119,9 @@ static void Related_HandlePythonError()
 */
 
 
-void PyGlomRelated_SetRelationships(PyGlomRelated* self, const PyGlomRelated::type_map_relationships& relationships)
+void PyGlomRelated::set_relationships(const PyGlomRelated::type_map_relationships& relationships)
 {
-  self->m_map_relationships = relationships;
+  m_map_relationships = relationships;
 }
 
 } //namespace Glom
