@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  */
 
@@ -369,11 +369,8 @@ void sharedptr<T_obj>::unref()
     //Unalloc if this is the last user of the obj:
     if(*m_pRefCount == 0)
     {
-      if(m_pobj)
-      {
-        delete m_pobj;
-        m_pobj = 0;
-      }
+      delete m_pobj;
+      m_pobj = 0;
 
        //Clear ref count:
        delete m_pRefCount;
