@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #include <libgdamm/init.h>
@@ -64,14 +64,16 @@ int main()
 
     //Connect:
     Glom::sharedptr<Glom::SharedConnection> connection;
-    #ifdef GLIBMM_EXCEPTIONS_ENABLED
-    connection = Glom::ConnectionPool::get_and_connect();
-    #else
-    std::auto_ptr<Glom::ExceptionConnection> error;
-    connection = Glom::ConnectionPool::get_and_connect(error);
-    #endif
-
-
+    
+    try
+    {
+      connection = Glom::ConnectionPool::get_and_connect();
+    }
+    catch(const Glom::ExceptionConnection& ex)
+    {
+      std::cout << "Exception: " << ex.what() << std::endl;
+    }
+    
     if(connection)
       std::cout << "Connected" << std::endl;
     else
