@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #include <libglom/libglom_config.h>
@@ -25,9 +25,12 @@
 #include <libglom/data_structure/layout/report_parts/layoutitem_fieldsummary.h>
 #include <libglom/data_structure/glomconversions.h>
 
+#include <giomm/file.h>
+#include <glibmm/convert.h>
+#include <glibmm/fileutils.h>
+#include <glibmm/miscutils.h>
 #include <glibmm/i18n.h>
 
-#include <giomm.h>
 
 #include <string.h> // for strchr
 #include <sstream> //For stringstream
@@ -42,6 +45,8 @@
 
 #include <stack>
 
+#include <fcntl.h> //For close(). This include is needed in mingw on MS Windows.
+
 namespace Glom
 {
 
@@ -132,7 +137,7 @@ Glib::ustring Utils::trim_whitespace(const Glib::ustring& text)
   return result;
 }
 
-Glib::ustring Utils::string_replace(const Glib::ustring& src, const Glib::ustring search_for, const Glib::ustring& replace_with)
+Glib::ustring Utils::string_replace(const Glib::ustring& src, const Glib::ustring& search_for, const Glib::ustring& replace_with)
 {
   if(search_for.empty())
   {
@@ -194,6 +199,15 @@ Glib::ustring Utils::string_replace(const Glib::ustring& src, const Glib::ustrin
 */
 }
 
+Glib::ustring Utils::string_clean_for_xml(const Glib::ustring& src)
+{
+  // The form feed character may not be in XML, even if escaped.
+  // So lets just lose it.
+  // Other unusual characters, such as &, are escaped by libxml later.
+  // TODO_Performance: Find a quicker way to do this.
+  return string_replace(src, "\f", Glib::ustring());
+}
+
 
 Glib::RefPtr<Gnome::Gda::SqlBuilder> Utils::build_sql_select_with_where_clause(const Glib::ustring& table_name, const type_vecLayoutFields& fieldsToGet, const Gnome::Gda::SqlExpr& where_clause, const sharedptr<const Relationship>& extra_join, const type_sort_clause& sort_clause, guint limit)
 {
@@ -207,6 +221,36 @@ Glib::RefPtr<Gnome::Gda::SqlBuilder> Utils::build_sql_select_with_where_clause(c
   return build_sql_select_with_where_clause(table_name, constFieldsToGet, where_clause, extra_join, sort_clause, limit);
 }
 
+/** Build a SQL query to discover how many rows a SQL query would return if it was run.
+ *
+ * This uses a COUNT * on a the @a sql_query as a sub-statement.
+ * Be careful not to include ORDER BY clauses in the supplied SQL query, because that would make it unnecessarily slow.
+ *
+ * @sql_query A SQL query.
+ * @result The number of rows.
+ */
+Glib::RefPtr<Gnome::Gda::SqlBuilder> Utils::build_sql_select_count_rows(const Glib::RefPtr<const Gnome::Gda::SqlBuilder>& sql_query)
+{
+  Glib::RefPtr<Gnome::Gda::SqlBuilder> result;
+
+  if(!sql_query)
+  {
+    std::cerr << G_STRFUNC << ": sql_query was null." << std::endl;
+    return result;
+  }
+
+  result = Gnome::Gda::SqlBuilder::create(Gnome::Gda::SQL_STATEMENT_SELECT);
+
+  //Note that the alias is just because the SQL syntax requires it - we get an error if we don't use it.
+  //Be careful not to include ORDER BY clauses in this, because that would make it unnecessarily slow:
+  const guint target_id = result->add_sub_select( sql_query->get_sql_statement() );
+  result->select_add_target_id(target_id, "glomarbitraryalias");
+
+  const Gnome::Gda::SqlBuilder::Id id_function = result->add_function("COUNT", result->add_id("*"));
+  result->add_field_value_id(id_function);
+
+  return result;
+}
 
 typedef std::list< sharedptr<const UsesRelationship> > type_list_relationships;
 
@@ -337,7 +381,7 @@ void Utils::build_sql_select_add_fields_to_get(const Glib::RefPtr<Gnome::Gda::Sq
     {
       const Gnome::Gda::SqlBuilder::Id id_function = builder->add_function(
         fieldsummary->get_summary_type_sql(),
-        builder->add_id(layout_item->get_sql_name(table_name)) ); //TODO: Just use add_field_id()?
+        builder->add_field_id(layout_item->get_name(), table_name));
       builder->add_field_value_id(id_function);
     }
     else
@@ -441,6 +485,12 @@ Glib::RefPtr<Gnome::Gda::SqlBuilder> Utils::build_sql_select_with_key(const Glib
 
 Gnome::Gda::SqlExpr Utils::build_simple_where_expression(const Glib::ustring& table_name, const sharedptr<const Field>& key_field, const Gnome::Gda::Value& key_value)
 {
+  if(!key_field)
+  {
+    std::cerr << G_STRFUNC << ": key_field was empty" << std::endl;
+    return Gnome::Gda::SqlExpr();
+  }
+  
   Glib::RefPtr<Gnome::Gda::SqlBuilder> builder = Gnome::Gda::SqlBuilder::create(Gnome::Gda::SQL_STATEMENT_SELECT);
   builder->select_add_target(table_name);  //This might not be necessary.
   const Gnome::Gda::SqlBuilder::Id id = builder->add_cond(Gnome::Gda::SQL_OPERATOR_TYPE_EQ,
@@ -480,19 +530,14 @@ Glib::RefPtr<Gnome::Gda::SqlBuilder> Utils::build_sql_select_with_key(const Glib
     sharedptr<const Relationship>(), sort_clause, limit);
 }
 
-Utils::type_list_values_with_second Utils::get_choice_values_all(const Document* document, const sharedptr<const LayoutItem_Field>& field, sharedptr<const LayoutItem_Field>& layout_choice_first, sharedptr<const LayoutItem_Field>& layout_choice_second)
+Utils::type_list_values_with_second Utils::get_choice_values_all(const Document* document, const sharedptr<const LayoutItem_Field>& field)
 {
   return get_choice_values(document, field,
-    Gnome::Gda::Value() /* means get all with no WHERE clause */,
-    layout_choice_first, layout_choice_second);
+    Gnome::Gda::Value() /* means get all with no WHERE clause */);
 }
 
-Utils::type_list_values_with_second Utils::get_choice_values(const Document* document, const sharedptr<const LayoutItem_Field>& field, const Gnome::Gda::Value& foreign_key_value, sharedptr<const LayoutItem_Field>& layout_choice_first, sharedptr<const LayoutItem_Field>& layout_choice_second)
+Utils::type_list_values_with_second Utils::get_choice_values(const Document* document, const sharedptr<const LayoutItem_Field>& field, const Gnome::Gda::Value& foreign_key_value)
 {
-  //Initialize output parameters:
-  layout_choice_first = sharedptr<LayoutItem_Field>();
-  layout_choice_second = sharedptr<LayoutItem_Field>();
-
   //TODO: Reduce duplication between this and get_choice_values(field).
 
   type_list_values_with_second result;
@@ -506,11 +551,13 @@ Utils::type_list_values_with_second Utils::get_choice_values(const Document* doc
   }
   */
 
-  const FieldFormatting& format = field->get_formatting_used();
+  const Formatting& format = field->get_formatting_used();
   sharedptr<const Relationship> choice_relationship;
-  Glib::ustring choice_field, choice_second;
+  sharedptr<const LayoutItem_Field> layout_choice_first;
+  sharedptr<const LayoutGroup> layout_choice_extra;
+  Formatting::type_list_sort_fields choice_sort_fields;
   bool choice_show_all = false;
-  format.get_choices_related(document, choice_relationship, layout_choice_first, layout_choice_second, choice_show_all);
+  format.get_choices_related(choice_relationship, layout_choice_first, layout_choice_extra, choice_sort_fields, choice_show_all);
 
   if(!choice_relationship)
   {
@@ -520,19 +567,33 @@ Utils::type_list_values_with_second Utils::get_choice_values(const Document* doc
 
   Utils::type_vecConstLayoutFields fields;
   fields.push_back(layout_choice_first);
-  if(layout_choice_second)
-    fields.push_back(layout_choice_second);
+
+  if(layout_choice_extra)
+  {
+    const LayoutGroup::type_list_const_items extra_fields = layout_choice_extra->get_items_recursive();
+    for(LayoutGroup::type_list_const_items::const_iterator iter = extra_fields.begin();
+      iter != extra_fields.end(); ++iter)
+    {
+      const sharedptr<const LayoutItem> item = *iter;
+      const sharedptr<const LayoutItem_Field> item_field = sharedptr<const LayoutItem_Field>::cast_dynamic(item);
+      if(item_field)
+         fields.push_back(item_field); //TODO: Don't ignore other usable items such as static text.
+    }
+  }
 
   const Glib::ustring to_table = choice_relationship->get_to_table();
-  sharedptr<Field> to_field = document->get_field(to_table, choice_relationship->get_to_field());
+  const sharedptr<const Field> to_field = document->get_field(to_table, choice_relationship->get_to_field());
 
   if(!to_field)
   {
     std::cerr << G_STRFUNC << ": to_field is null." << std::endl;
   }
 
-  type_sort_clause sort_clause;
-  sort_clause.push_back( type_pair_sort_field(layout_choice_first, true /* ascending */));
+  //Default to some sort order rather than none:
+  if(choice_sort_fields.empty())
+  {
+    choice_sort_fields.push_back( type_pair_sort_field(layout_choice_first, true /* ascending */));
+  }
 
   //TODO: Support related relationships (in the UI too):
   Glib::RefPtr<Gnome::Gda::SqlBuilder> builder = Utils::build_sql_select_with_key(
@@ -540,7 +601,7 @@ Utils::type_list_values_with_second Utils::get_choice_values(const Document* doc
     fields,
     to_field,
     foreign_key_value,
-    sort_clause);
+    choice_sort_fields);
 
   if(!builder)
   {
@@ -571,11 +632,19 @@ Utils::type_list_values_with_second Utils::get_choice_values(const Document* doc
     for(guint row = 0; row < count; ++row)
     {
 
-      std::pair<Gnome::Gda::Value, Gnome::Gda::Value> itempair;
+      std::pair<Gnome::Gda::Value, type_list_values> itempair;
       itempair.first = datamodel->get_value_at(0, row);
 
-      if(layout_choice_second && (cols_count > 1))
-        itempair.second = datamodel->get_value_at(1, row);
+      if(layout_choice_extra && (cols_count > 1))
+      {
+        type_list_values list_values;
+        for(guint i = 1; i < cols_count; ++i)
+        {
+          list_values.push_back(datamodel->get_value_at(i, row));
+        }
+
+        itempair.second = list_values;
+      }
 
       result.push_back(itempair);
     }
@@ -617,34 +686,62 @@ Glib::ustring Utils::locale_simplify(const Glib::ustring& locale_id)
 {
   Glib::ustring result = locale_id;
 
+  //At least Ubuntu Natty provides a long string such as this: LC_CTYPE=en_US.UTF-8;LC_NUMERIC=en_US.UTF-8;LC_TIME=en_US.UTF-8;LC_COLLATE=en_US.UTF-8;LC_MONETARY=en_US.UTF-8;LC_MESSAGES=en_AG.utf8;LC_PAPER=en_US.UTF-8;LC_NAME=en_US.UTF-8;LC_ADDRESS=en_US.UTF-8;LC_TELEPHONE=en_US.UTF-8;LC_MEASUREMENT=en_US.UTF-8;LC_IDENTIFICATION=en_US.UTF-8
+  //In Ubuntu Maverick, and earlier, it was apparently a simple string such as en_US.UTF-8.
+
+  //Look for LC_ALL or LC_COLLATE
+  //(We use the locale name only to identify translations
+  //Otherwise just start with the whole string.
+  Glib::ustring::size_type posCategory = result.find("LC_ALL=");
+  if(posCategory != Glib::ustring::npos)
+  {
+    result = result.substr(posCategory);
+  }
+  else
+  {
+    posCategory = result.find("LC_COLLATE=");
+    if(posCategory != Glib::ustring::npos)
+    {
+      result = result.substr(posCategory);
+    }
+  }
+
   //Get everything before the .:
-  Glib::ustring::size_type posDot = locale_id.find('.');
+  const Glib::ustring::size_type posDot = result.find('.');
   if(posDot != Glib::ustring::npos)
   {
     result = result.substr(0, posDot);
   }
 
   //Get everything before the @:
-  const Glib::ustring::size_type posAt = locale_id.find('@');
+  const Glib::ustring::size_type posAt = result.find('@');
   if(posAt != Glib::ustring::npos)
   {
     result = result.substr(0, posAt);
   }
 
+  //Get everything after the =, if any:
+  const Glib::ustring::size_type posEquals = result.find('=');
+  if(posEquals != Glib::ustring::npos)
+  {
+    result = result.substr(posEquals + 1);
+  }
+
   return result;
 }
 
 Glib::ustring Utils::locale_language_id(const Glib::ustring& locale_id)
 {
-  Glib::ustring result;
-
   const Glib::ustring::size_type posUnderscore = locale_id.find('_');
   if(posUnderscore != Glib::ustring::npos)
   {
-    result = locale_id.substr(0, posUnderscore);
+    return locale_id.substr(0, posUnderscore);
+  }
+  else
+  {
+    //We assume that this locale ID specifies a language but no specific country.
+    return locale_id;
   }
-
-  return result;
 }
 
 Glib::ustring Utils::create_local_image_uri(const Gnome::Gda::Value& value)
@@ -911,65 +1008,75 @@ bool Utils::file_exists(const Glib::ustring& uri)
      return false;
 
   //Check whether file exists already:
-  {
-    // Try to examine the input file.
-    Glib::RefPtr<Gio::File> file = Gio::File::create_for_uri(uri);
+  // Try to examine the input file.
+  Glib::RefPtr<Gio::File> file = Gio::File::create_for_uri(uri);
+  return file_exists(file);
+}
 
-    try
-    {
-      return file->query_exists();
-    }
-    catch(const Gio::Error& /* ex */)
-    {
-      return false; //Something went wrong. It does not exist.
-    }
+bool Utils::file_exists(const Glib::RefPtr<Gio::File>& file)
+{
+  try
+  {
+    return file->query_exists();
+  }
+  catch(const Gio::Error& /* ex */)
+  {
+    return false; //Something went wrong. It does not exist.
   }
 }
 
-std::string Utils::sqlbuilder_get_full_query(
-    const Glib::RefPtr<Gnome::Gda::Connection>& connection,
-    const Glib::ustring& query,
-    const Glib::RefPtr<const Gnome::Gda::Set>& params)
+//TODO: This is a duplicate of the one in db_utils.cc:
+//Merge all db utilities into db_utils in glom 1.22:
+static Glib::RefPtr<Gnome::Gda::Connection> get_connection()
 {
-  Glib::ustring result = "glom_query_not_parsed";
-
+  sharedptr<SharedConnection> sharedconnection;
   try
   {
-    Glib::RefPtr<Gnome::Gda::SqlParser> parser = connection->create_parser();
-    if(parser)
-    {
-      Glib::RefPtr<Gnome::Gda::Statement> stmt = parser->parse_string(query);
-      if(stmt)
-        result = stmt->to_sql(params);
-    }
+     sharedconnection = ConnectionPool::get_and_connect();
   }
-  catch(const Glib::Exception& ex)
+  catch (const Glib::Error& error)
   {
-    std::cerr << G_STRFUNC << ": exception while parsing query: " << ex.what() << std::endl;
-  }
-  catch(const std::exception& ex)
-  {
-    std::cerr << G_STRFUNC << ": exception while parsing query: " << ex.what() << std::endl;
+    std::cerr << G_STRFUNC << ": " << error.what() << std::endl;
   }
 
-  //Convert to something that std::cout should be able to handle.
-  const Glib::ScopedPtr<char> buf(g_convert_with_fallback(
-    result.raw().data(), result.raw().size(),
-    "ISO-8859-1", "UTF-8",
-    (char*)"?",
-    0, 0, 0));
-  return std::string(buf.get());
+  if(!sharedconnection)
+  {
+    std::cerr << G_STRFUNC << ": No connection yet." << std::endl;
+    return Glib::RefPtr<Gnome::Gda::Connection>(0);
+  }
+
+  Glib::RefPtr<Gnome::Gda::Connection> gda_connection = sharedconnection->get_gda_connection();
+
+  return gda_connection;
 }
 
 std::string Utils::sqlbuilder_get_full_query(
   const Glib::RefPtr<const Gnome::Gda::SqlBuilder>& builder)
 {
+  Glib::RefPtr<Gnome::Gda::Connection> connection = get_connection();
+  if(!connection)
+  {
+    //TODO: Just use the correct provider, without an actual connection?
+    std::cerr << G_STRFUNC << ": There is no connection, so the SQL statement might not be created correctly." << std::endl;
+  }
+
   Glib::ustring result = "glom_query_not_parsed";
 
   try
   {
     Glib::RefPtr<Gnome::Gda::Statement> stmt = builder->get_statement();
-    if(stmt)
+    if(!stmt)
+    {
+      std::cerr << G_STRFUNC << ": builder->get_statement() failed." << std::endl;
+      return result;
+    }
+
+    if(connection)
+    {
+      result = connection->statement_to_sql(stmt,
+          Gnome::Gda::STATEMENT_SQL_PARAMS_AS_VALUES | Gnome::Gda::STATEMENT_SQL_PRETTY);
+    }
+    else
       result = stmt->to_sql();
   }
   catch(const Gnome::Gda::SqlError& ex)
@@ -991,25 +1098,53 @@ std::string Utils::sqlbuilder_get_full_query(
     "ISO-8859-1", "UTF-8",
     (char*)"?",
     0, 0, 0));
-  return std::string(buf.get());
+
+  const Glib::ustring str = std::string(buf.get());
+  if(str.empty())
+  {
+    std::cerr << G_STRFUNC << ": Returning an empty string." << std::endl;
+  }
+
+  return str;
 }
 
 Gnome::Gda::SqlExpr Utils::get_find_where_clause_quick(const Document* document, const Glib::ustring& table_name, const Gnome::Gda::Value& quick_search)
 {
+  if(table_name.empty())
+  {
+    std::cerr << G_STRFUNC << ": table_name is empty." << std::endl;
+    return Gnome::Gda::SqlExpr();
+  }
+
+  if(Conversions::value_is_empty(quick_search))
+    return Gnome::Gda::SqlExpr();
+  
   Glib::RefPtr<Gnome::Gda::SqlBuilder> builder =
     Gnome::Gda::SqlBuilder::create(Gnome::Gda::SQL_STATEMENT_SELECT);
   builder->select_add_target(table_name);
 
+  //We need to add some fields to select,
+  //because otherwise the SqlBuilder would not contain a valid query.
+  builder->select_add_field("*", table_name);
+
   if(!document)
   {
     std::cerr << G_STRFUNC << ": document was null." << std::endl;
     return Gnome::Gda::SqlExpr();
   }
 
+  //We need the connection to generate the correct SQL syntax:
+  Glib::RefPtr<Gnome::Gda::Connection> connection = get_connection();
+  if(!connection)
+  {
+    std::cerr << G_STRFUNC << ": connection was null." << std::endl;
+    return Gnome::Gda::SqlExpr();
+  }
+
   //TODO: Cache the list of all fields, as well as caching (m_Fields) the list of all visible fields:
   const Document::type_vec_fields fields = document->get_table_fields(table_name);
 
-  guint previous_and_id = 0;
+  guint previous_id = 0;
   typedef std::vector< sharedptr<LayoutItem_Field> > type_vecLayoutFields;
   type_vecLayoutFields fieldsToGet;
   for(Document::type_vec_fields::const_iterator iter = fields.begin(); iter != fields.end(); ++iter)
@@ -1026,32 +1161,77 @@ Gnome::Gda::SqlExpr Utils::get_find_where_clause_quick(const Document* document,
 
     if(use_this_field)
     {
-      const guint eq_id = builder->add_cond(Gnome::Gda::SQL_OPERATOR_TYPE_EQ, //TODO: Ue field->sql_find_operator().
+      //std::cout << "Using field: " << field->get_name() << std::endl;
+      const guint eq_id = builder->add_cond(field->sql_find_operator(),
         builder->add_field_id(field->get_name(), table_name),
-        builder->add_expr(quick_search)); //Use  field->sql_find(quick_search);
+        builder->add_expr( field->sql_find(quick_search, connection) )); //sql_find() modifies the value for the operator.
 
-      guint and_id = 0;
-      if(previous_and_id)
+      if(previous_id)
       {
-        and_id = builder->add_cond(Gnome::Gda::SQL_OPERATOR_TYPE_AND,
-        previous_and_id, eq_id);
+        const guint or_id = builder->add_cond(Gnome::Gda::SQL_OPERATOR_TYPE_OR,
+          previous_id, eq_id);
+        previous_id = or_id;
       }
-
-      previous_and_id = and_id;
+      else
+        previous_id = eq_id;
     }
   }
 
-  if(previous_and_id)
+  if(previous_id)
   {
-    builder->set_where(previous_and_id); //This might be unnecessary.
-    return builder->export_expression(previous_and_id);
+    builder->set_where(previous_id); //This might be unnecessary.
+    //std::cout << G_STRFUNC << ": builder: " << sqlbuilder_get_full_query(builder) << std::endl; 
+    return builder->export_expression(previous_id);
   }
   else
   {
+    std::cerr << G_STRFUNC << ": Returning null SqlExpr" << std::endl;
     return Gnome::Gda::SqlExpr();
   }
 }
 
+Glib::RefPtr<Gnome::Gda::SqlBuilder> Utils::build_sql_update_with_where_clause(
+  const Glib::ustring& table_name,
+  const sharedptr<const Field>& field, const Gnome::Gda::Value& value,
+  const Gnome::Gda::SqlExpr& where_clause)
+{
+  Glib::RefPtr<Gnome::Gda::SqlBuilder> builder;
+
+  if(!field || field->get_name().empty())
+  {
+    std::cerr << "field was null or its name was empty." << std::endl;
+    return builder;
+  }
+
+  if(table_name.empty())
+  {
+    std::cerr << "table_name was empty." << std::endl;
+    return builder;
+  }
+
+  //Build the whole SQL statement:
+  try
+  {
+    builder = Gnome::Gda::SqlBuilder::create(Gnome::Gda::SQL_STATEMENT_UPDATE);
+    builder->set_table(table_name);
+
+    builder->add_field_value_as_value(field->get_name(), value);
+
+    //Add the WHERE clause:
+    if(!where_clause.empty())
+    {
+      const int id = builder->import_expression(where_clause);
+      builder->set_where(id);
+    }
+  }
+  catch(const Glib::Error& ex)
+  {
+    std::cerr << G_STRFUNC << ": Exception: " << ex.what() << std::endl;
+  }
+
+  return builder;
+}
+
 bool Utils::delete_directory(const Glib::RefPtr<Gio::File>& directory)
 {
   if(!(directory->query_exists()))
@@ -1124,5 +1304,238 @@ Glib::ustring Utils::get_directory_child_with_suffix(const Glib::ustring& uri_di
   return Glib::ustring();
 }
 
+Glib::ustring Utils::get_file_uri_without_extension(const Glib::ustring& uri)
+{
+  if(uri.empty())
+    return uri;
+
+  Glib::RefPtr<Gio::File> file = Gio::File::create_for_uri(uri);
+  if(!file)
+    return uri; //Actually an error.
+
+  const Glib::ustring filename_part = file->get_basename();
+
+  const Glib::ustring::size_type pos_dot = filename_part.rfind(".");
+  if(pos_dot == Glib::ustring::npos)
+    return uri; //There was no extension, so just return the existing URI.
+  else
+  {
+    const Glib::ustring filename_part_without_ext = filename_part.substr(0, pos_dot);
+
+    //Use the Gio::File API to manipulate the URI:
+    Glib::RefPtr<Gio::File> parent = file->get_parent();
+    Glib::RefPtr<Gio::File> file_without_extension = parent->get_child(filename_part_without_ext);
+
+    return file_without_extension->get_uri();
+  }
+}
+
+std::string Utils::get_file_path_without_extension(const std::string& filepath)
+{
+  if(filepath.empty())
+    return filepath;
+
+  Glib::RefPtr<Gio::File> file = Gio::File::create_for_path(filepath);
+  if(!file)
+    return filepath; //Actually an error.
+
+  const Glib::ustring filename_part = file->get_basename();
+
+  const Glib::ustring::size_type pos_dot = filename_part.rfind(".");
+  if(pos_dot == Glib::ustring::npos)
+    return filepath; //There was no extension, so just return the existing URI.
+  else
+  {
+    const Glib::ustring filename_part_without_ext = filename_part.substr(0, pos_dot);
+
+    //Use the Gio::File API to manipulate the URI:
+    Glib::RefPtr<Gio::File> parent = file->get_parent();
+    Glib::RefPtr<Gio::File> file_without_extension = parent->get_child(filename_part_without_ext);
+
+    return file_without_extension->get_path();
+  }
+}
+
+Glib::ustring Utils::get_list_of_layout_items_for_display(const LayoutGroup::type_list_items& list_layout_fields)
+{
+  Glib::ustring result;
+  for(LayoutGroup::type_list_items::const_iterator iter = list_layout_fields.begin(); iter != list_layout_fields.end(); ++iter)
+  {
+    const sharedptr<LayoutItem> item = *iter;
+    if(item)
+    {
+      if(!result.empty())
+       result += ", ";
+
+      result += item->get_layout_display_name();
+    }
+  }
+
+  return result;
+}
+
+Glib::ustring Utils::get_list_of_layout_items_for_display(const sharedptr<const LayoutGroup>& layout_group)
+{
+  if(layout_group)
+    return get_list_of_layout_items_for_display(layout_group->m_list_items);
+  else
+    return Glib::ustring();
+}
+
+Glib::ustring Utils::get_list_of_sort_fields_for_display(const Formatting::type_list_sort_fields& sort_fields)
+{
+  Glib::ustring text;
+  for(Formatting::type_list_sort_fields::const_iterator iter = sort_fields.begin(); iter != sort_fields.end(); ++iter)
+  {
+    const sharedptr<const LayoutItem_Field> item = iter->first;
+    if(!item)
+      continue;
+    
+    if(!text.empty())
+      text += ", ";
+
+    text += item->get_layout_display_name();
+    //TODO: Show Ascending/Descending?
+  }
+
+  return text;
+}
+
+std::string Utils::get_temp_file_path(const std::string& prefix, const std::string& extension)
+{
+  //Get a temporary file path:
+  std::string filepath;
+  try
+  {
+    const std::string prefix_pattern = prefix + "XXXXXX" + extension;
+    const int filehandle = Glib::file_open_tmp(filepath, prefix);
+    ::close(filehandle);
+  }
+  catch(const Glib::Error& ex)
+  {
+    std::cerr << G_STRFUNC << ": Glib::file_open_tmp() failed" << std::endl;
+    return filepath;
+  }
+  
+  if(filepath.empty())
+  {
+    std::cerr << G_STRFUNC << ": Glib::file_open_tmp() returned an empty filepath" << std::endl;
+  }
+
+  return filepath;
+}
+
+Glib::ustring Utils::get_temp_file_uri(const std::string& prefix, const std::string& extension)
+{
+  try
+  {
+    const std::string filepath = get_temp_file_path(prefix, extension);
+    return Glib::filename_to_uri(filepath);
+  }
+  catch(const Glib::Error& ex)
+  {
+    std::cerr << G_STRFUNC << ": Exception from filename_to_uri(): " << ex.what() << std::endl;
+    return std::string();
+  }
+}
+
+std::string Utils::get_temp_directory_path(const std::string& prefix)
+{
+  std::string result;
+
+  const std::string pattern = Glib::build_filename(
+    Glib::get_tmp_dir(), prefix + "XXXXXX");
+
+  //We must copy the pattern, because mkdtemp() modifies it:
+  char* c_pattern = g_strdup(pattern.c_str());
+  
+  const char* filepath = g_mkdtemp(c_pattern);
+  if(filepath)
+    result = filepath;
+
+  return result;
+}
+
+Glib::ustring Utils::get_temp_directory_uri(const std::string& prefix)
+{
+  try
+  {
+    const std::string filepath = get_temp_directory_path(prefix);
+    return Glib::filename_to_uri(filepath);
+  }
+  catch(const Glib::Error& ex)
+  {
+    std::cerr << G_STRFUNC << ": Exception from filename_to_uri(): " << ex.what() << std::endl;
+    return Glib::ustring();
+  }
+}
+
+LayoutGroup::type_list_const_items Utils::get_layout_items_plus_primary_key(const LayoutGroup::type_list_const_items& items, const Document* document, const Glib::ustring& table_name)
+{
+  if(!document)
+  {
+    std::cerr << G_STRFUNC << ": document was null." << std::endl;
+    return items;
+  }
+
+  const sharedptr<Field> field_primary_key = document->get_field_primary_key(table_name);
+  if(!field_primary_key)
+  {
+    std::cerr << G_STRFUNC << ": Could not find the primary key." << std::endl;
+    return items;
+  }
+
+  sharedptr<LayoutItem_Field> pk_layout_item = sharedptr<LayoutItem_Field>::create();
+  pk_layout_item->set_hidden();
+  pk_layout_item->set_full_field_details(field_primary_key);
+  
+  const LayoutGroup::type_list_const_items::const_iterator iterFind = std::find_if(items.begin(), items.end(), predicate_LayoutItem_Field_IsSameField<LayoutItem_Field, LayoutItem>(pk_layout_item));
+  if(iterFind != items.end())
+    return items; //It is already in the list:
+
+  LayoutGroup::type_list_const_items items_plus_pk = items;
+  items_plus_pk.push_back(pk_layout_item);
+  return items_plus_pk;
+}
+
+//TODO: Avoid the horrible code duplication with the const version.
+LayoutGroup::type_list_items Utils::get_layout_items_plus_primary_key(const LayoutGroup::type_list_items& items, const Document* document, const Glib::ustring& table_name)
+{
+  if(!document)
+  {
+    std::cerr << G_STRFUNC << ": document was null." << std::endl;
+    return items;
+  }
+
+  const sharedptr<Field> field_primary_key = document->get_field_primary_key(table_name);
+  if(!field_primary_key)
+  {
+    std::cerr << G_STRFUNC << ": Could not find the primary key." << std::endl;
+    return items;
+  }
+
+  sharedptr<LayoutItem_Field> pk_layout_item = sharedptr<LayoutItem_Field>::create();
+  pk_layout_item->set_hidden();
+  pk_layout_item->set_full_field_details(field_primary_key);
+  
+  const LayoutGroup::type_list_items::const_iterator iterFind = std::find_if(items.begin(), items.end(), predicate_LayoutItem_Field_IsSameField<LayoutItem_Field, LayoutItem>(pk_layout_item));
+  if(iterFind != items.end())
+    return items; //It is already in the list:
+
+  LayoutGroup::type_list_items items_plus_pk = items;
+  items_plus_pk.push_back(pk_layout_item);
+  return items_plus_pk;
+}
+
+bool Utils::script_check_for_pygtk2(const Glib::ustring& script)
+{
+  //There are probably other code that this will not catch,
+  //but this is better than nothing.
+  //TODO: Instead override python's import mechanism somehow?
+  if(script.find("import pygtk") != std::string::npos)
+    return false;
+
+  return true;
+}
 
 } //namespace Glom
