@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #ifndef GLOM_UTILS_H
@@ -35,14 +35,18 @@ namespace Glom
 
 ///field, ascending
 typedef std::pair< sharedptr<const LayoutItem_Field>, bool> type_pair_sort_field;
-typedef std::list<type_pair_sort_field> type_sort_clause;
+typedef std::vector<type_pair_sort_field> type_sort_clause;
 
 namespace Utils
 {
 
 Glib::ustring trim_whitespace(const Glib::ustring& text);
 
-Glib::ustring string_replace(const Glib::ustring& src, const Glib::ustring search_for, const Glib::ustring& replace_with);
+Glib::ustring string_replace(const Glib::ustring& src, const Glib::ustring& search_for, const Glib::ustring& replace_with);
+
+/** Remove any characters that may not be in XML even when escaped.
+ */
+Glib::ustring string_clean_for_xml(const Glib::ustring& src);
 
 //typedef Base_DB::type_vecLayoutFields type_vecLayoutFields;
 typedef std::vector< sharedptr<LayoutItem_Field> > type_vecLayoutFields;
@@ -109,19 +113,31 @@ Glib::RefPtr<Gnome::Gda::SqlBuilder> build_sql_select_with_key(
   const type_sort_clause& sort_clause = type_sort_clause(),
   guint limit = 0);
 
+//Note: This is not used by glom itself, but it is used by java-libglom.
+/** Build a SQL query to discover how many rows a SQL query would return if it was run.
+ *
+ * This uses a COUNT * on a the @a sql_query as a sub-statement.
+ * Be careful not to include ORDER BY clauses in the supplied SQL query, because that would make it unnecessarily slow.
+ *
+ * @sql_query A SQL query.
+ * @result The number of rows.
+ */
+Glib::RefPtr<Gnome::Gda::SqlBuilder> build_sql_select_count_rows(const Glib::RefPtr<const Gnome::Gda::SqlBuilder>& sql_query);
+
 Gnome::Gda::SqlExpr get_find_where_clause_quick(const Document* document, const Glib::ustring& table_name, const Gnome::Gda::Value& quick_search);
 
+/** Generate a SQL statement to UPDATE field values,
+ */
+Glib::RefPtr<Gnome::Gda::SqlBuilder> build_sql_update_with_where_clause(
+  const Glib::ustring& table_name,
+  const sharedptr<const Field>& field, const Gnome::Gda::Value& value,
+  const Gnome::Gda::SqlExpr& where_clause);
 
-typedef std::list< std::pair<Gnome::Gda::Value, Gnome::Gda::Value> > type_list_values_with_second;
-type_list_values_with_second get_choice_values_all(const Document* document, const sharedptr<const LayoutItem_Field>& field, sharedptr<const LayoutItem_Field>& layout_choice_first, sharedptr<const LayoutItem_Field>& layout_choice_second);
+typedef std::vector<Gnome::Gda::Value> type_list_values;
+typedef std::vector< std::pair<Gnome::Gda::Value, type_list_values> > type_list_values_with_second; //TODO: Rename this now that we have more than just 1 extra field.
+type_list_values_with_second get_choice_values_all(const Document* document, const sharedptr<const LayoutItem_Field>& field);
 
-type_list_values_with_second get_choice_values(const Document* document, const sharedptr<const LayoutItem_Field>& field, const Gnome::Gda::Value& foreign_key_value, sharedptr<const LayoutItem_Field>& layout_choice_first, sharedptr<const LayoutItem_Field>& layout_choice_second);
-
-/// Get the full query string suitable for use with std::cout.
-std::string sqlbuilder_get_full_query(
-  const Glib::RefPtr<Gnome::Gda::Connection>& connection,
-  const Glib::ustring& query,
-  const Glib::RefPtr<const Gnome::Gda::Set>& params);
+type_list_values_with_second get_choice_values(const Document* document, const sharedptr<const LayoutItem_Field>& field, const Gnome::Gda::Value& foreign_key_value);
 
 /// Get the full query string suitable for use with std::cout.
 std::string sqlbuilder_get_full_query(
@@ -162,6 +178,7 @@ Glib::ustring string_trim(const Glib::ustring& str, const Glib::ustring& to_remo
 Glib::ustring string_remove_suffix(const Glib::ustring& str, const Glib::ustring& suffix, bool case_sensitive = true);
 
 bool file_exists(const Glib::ustring& uri);
+bool file_exists(const Glib::RefPtr<Gio::File>& file);
 
 /** Delete a directory, if it exists, and its contents.
  * Unlike g_file_delete(), this does not fail if the directory is not empty.
@@ -177,6 +194,54 @@ bool delete_directory(const std::string& uri);
  */
 Glib::ustring get_directory_child_with_suffix(const Glib::ustring& uri_directory, const std::string& suffix, bool recursive);
 
+/** Get a URI with the extension (any extension, not just .glom) removed.
+ */
+Glib::ustring get_file_uri_without_extension(const Glib::ustring& uri);
+
+/** Get a filepath with the extension (any extension, not just .glom) removed.
+ */
+std::string get_file_path_without_extension(const std::string& filepath);
+
+/** Get a string to display to the user, as a representation of a list of layout items.
+ */
+Glib::ustring get_list_of_layout_items_for_display(const LayoutGroup::type_list_items& list_layout_fields);
+
+/** Get a string to display to the user, as a representation of a list of layout items.
+ */
+Glib::ustring get_list_of_layout_items_for_display(const sharedptr<const LayoutGroup>& layout_group);
+
+/** Get a string to display to the user, as a representation of a sort order
+ */
+Glib::ustring get_list_of_sort_fields_for_display(const Formatting::type_list_sort_fields& sort_fields);
+
+/** This returns the provided list of layout items,
+ * plus the primary key, if the primary key is not already present in the list
+ */
+LayoutGroup::type_list_const_items get_layout_items_plus_primary_key(const LayoutGroup::type_list_const_items& items, const Document* document, const Glib::ustring& table_name);
+
+//TODO: Avoid the overload just for constness.
+/** This returns the provided list of layout items,
+ * plus the primary key, if the primary key is not already present in the list
+ */
+LayoutGroup::type_list_items get_layout_items_plus_primary_key(const LayoutGroup::type_list_items& items, const Document* document, const Glib::ustring& table_name);
+
+std::string get_temp_file_path(const std::string& prefix = std::string(), const std::string& extension = std::string());
+Glib::ustring get_temp_file_uri(const std::string& prefix = std::string(), const std::string& extension = std::string());
+
+/** This actually creates the directory.
+ */
+std::string get_temp_directory_path(const std::string& prefix = std::string());
+
+/** This actually creates the directory.
+ */
+Glib::ustring get_temp_directory_uri(const std::string& prefix = std::string());
+
+/** @returns true if the script is OK, or 
+ * false if the script uses pygtk2, which would cause a crash,
+ * because Glom itself uses GTK+ 3.
+ */
+bool script_check_for_pygtk2(const Glib::ustring& script);
+
 } //namespace Utils
 
 } //namespace Glom
