@@ -14,12 +14,14 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #include "config.h"
 
+#include <iostream>
+
 //We use Python for calculated fields.
 //#include <Python.h> //Include it before anything else to avoid "_POSIX_C_SOURCE redefined".
 #include <boost/python.hpp>
@@ -27,10 +29,12 @@
 //#include <gnome.h>
 #include <gtkmm/messagedialog.h>
 #include <glom/libglom/init.h>
-#include <glom/glade_utils.h>
-#include <gtkmm/main.h>
+#include <glom/libglom/connectionpool.h>
+#include <glom/libglom/utils.h>
 
-#include <giomm.h>
+#include <giomm/file.h>
+#include <glibmm/convert.h>
+#include <glibmm/miscutils.h>
 
 // For postgres availability checks:
 #ifdef GLOM_ENABLE_POSTGRESQL
@@ -41,26 +45,22 @@
 #endif //GLOM_ENABLE_POSTGRESQL
 
 // For sanity checks:
-#include <libglom/data_structure/glomconversions.h>
 #include <glom/python_embed/glom_python.h>
 
 #ifndef GLOM_ENABLE_CLIENT_ONLY
 #include <gtksourceviewmm/init.h>
 #include <goocanvasmm/init.h>
 #endif // !GLOM_ENABLE_CLIENT_ONLY
-#include <glibmm/i18n.h>
-
-#ifdef GLOM_ENABLE_MAEMO
-#include <libosso.h>
-#include <hildonmm/init.h>
-#include <hildonmm/note.h>
-#include <hildonmm/program.h>
-#endif
 
 #include <glom/application.h>
-#include <glom/glade_utils.h>
 #include <glom/utils_ui.h>
 
+#include <evince-view.h>
+
+#include <gtkmm/main.h>
+
+#include <glibmm/i18n.h>
+
 #ifdef G_OS_WIN32
 #include <winsock2.h>
 #else
@@ -179,62 +179,6 @@ pgwin32_is_admin(void)
 namespace Glom
 {
 
-class OptionGroup : public Glib::OptionGroup
-{
-public:
-  OptionGroup();
-
-  //These int instances should live as long as the OptionGroup to which they are added,
-  //and as long as the OptionContext to which those OptionGroups are added.
-  std::string m_arg_filename;
-  bool m_arg_version;
-  bool m_arg_restore;
-  bool m_arg_stop_auto_server_shutdown;
-  bool m_arg_debug_sql;
-  bool m_arg_debug_date_check;
-};
-
-OptionGroup::OptionGroup()
-: Glib::OptionGroup("Glom", _("Glom options"), _("Command-line options for glom")),
-  m_arg_version(false),
-  m_arg_restore(false),
-  m_arg_stop_auto_server_shutdown(false),
-  m_arg_debug_sql(false),
-  m_arg_debug_date_check(false)
-{
-  Glib::OptionEntry entry;
-  entry.set_long_name("file");
-  entry.set_short_name('f');
-  entry.set_description(_("The Filename"));
-  add_entry_filename(entry, m_arg_filename);
-
-  Glib::OptionEntry entry_version;
-  entry_version.set_long_name("version");
-  entry_version.set_short_name('V');
-  entry_version.set_description(_("The version of this application."));
-  add_entry(entry_version, m_arg_version);
-
-  Glib::OptionEntry entry_restore;
-  entry_restore.set_long_name("restore");
-  entry_restore.set_description(_("Whether the filename is a .tar.gz backup to be restored."));
-  add_entry(entry_restore, m_arg_restore);
-
-  Glib::OptionEntry entry_stop_auto_server_shutdown;
-  entry_stop_auto_server_shutdown.set_long_name("stop-auto-server-shutdown");
-  entry_stop_auto_server_shutdown.set_description(_("Do not automatically stop the database server if Glom quits. This is helpful for debugging with gdb."));
-  add_entry(entry_stop_auto_server_shutdown, m_arg_stop_auto_server_shutdown);
-
-  Glib::OptionEntry entry_debug_sql;
-  entry_debug_sql.set_long_name("debug_sql");
-  entry_debug_sql.set_description(_("Show the generated SQL queries on stdout, for debugging."));
-  add_entry(entry_debug_sql, m_arg_debug_sql);
-
-  Glib::OptionEntry entry_debug_date_check;
-  entry_debug_date_check.set_long_name("debug-date-check");
-  entry_debug_date_check.set_description(_("Show how Glom outputs a date in this locale, then stop."));
-  add_entry(entry_debug_date_check, m_arg_debug_date_check);
-}
-
 #ifdef GLOM_ENABLE_POSTGRESQL
 bool check_user_is_not_root_with_warning()
 {
@@ -264,14 +208,12 @@ bool check_user_is_not_root_with_warning()
 
   if(!message.empty())
   {
-#ifndef GLOM_ENABLE_MAEMO
+    //Make sure this is on stderr too, in case something goes wrong with the UI:
+    std::cerr << message << std::endl;
+
     Gtk::MessageDialog dialog(Utils::bold_message(_("Running As Root")), true /* use_markup */, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_OK, true /* modal */);
     dialog.set_secondary_text(message);
     dialog.run();
-#else
-    Hildon::Note note(Hildon::NOTE_TYPE_INFORMATION, message);
-    note.run();
-#endif
 
     return false; /* Is root. Bad. */
   }
@@ -296,7 +238,7 @@ bool check_user_is_not_root_with_warning()
  */
 bool check_postgres_is_available_with_warning()
 {
-  const std::string binpath = Glom::ConnectionPoolBackends::PostgresSelfHosted::get_path_to_postgres_executable("postgres");
+  const std::string binpath = Glom::ConnectionPoolBackends::PostgresSelfHosted::get_path_to_postgres_executable("postgres", false /* not quoted */);
 
   // TODO: At least on Windows we should probably also check for initdb and
   // pg_ctl. Perhaps it would also be a good idea to access these files as
@@ -325,8 +267,13 @@ bool check_postgres_is_available_with_warning()
   #else  //DISTRO_SPECIFIC_POSTGRES_INSTALL_IMPLEMENTED
 
   //Show message to the user about the broken installation:
+  const Glib::ustring message = _("Your installation of Glom is not complete, because PostgreSQL is not available on your system. PostgreSQL is needed for self-hosting of Glom databases.\n\nPlease report this bug to your vendor, or your system administrator so it can be corrected.");
+
+  //Make sure this is on stderr too, in case something goes wrong with the UI:
+  std::cerr << message << std::endl;
+
   Gtk::MessageDialog dialog(Utils::bold_message(_("Incomplete Glom Installation")), true /* use_markup */, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_OK, true /* modal */);
-  dialog.set_secondary_text(_("Your installation of Glom is not complete, because PostgreSQL is not available on your system. PostgreSQL is needed for self-hosting of Glom databases.\n\nPlease report this bug to your vendor, or your system administrator so it can be corrected."));
+  dialog.set_secondary_text(message);
   dialog.run();
   return false;
 
@@ -341,17 +288,33 @@ bool check_pyglom_is_available_with_warning()
   if(glom_python_module_is_available())
     return true;
 
-   /* The python module could not be imported by Glom, so warn the user: */
-   const Glib::ustring message = _("Your installation of Glom is not complete, because the Glom Python module is not available on your system.\n\nPlease report this bug to your vendor, or your system administrator so it can be corrected.");
+  /* The python module could not be imported by Glom, so warn the user: */
+  const Glib::ustring message = _("Your installation of Glom is not complete, because the Glom Python module is not available on your system.\n\nPlease report this bug to your vendor, or your system administrator so it can be corrected.");
+
+  //Make sure this is on stderr too, in case something goes wrong with the UI:
+  std::cerr << message << std::endl;
 
-#ifndef GLOM_ENABLE_MAEMO
   Gtk::MessageDialog dialog(Utils::bold_message(_("Glom Python Module Not Installed")), true /* use_markup */, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_OK, true /* modal */);
   dialog.set_secondary_text(message);
   dialog.run();
-#else
-  Hildon::Note note(Hildon::NOTE_TYPE_INFORMATION, message);
-  note.run();
-#endif //GLOM_ENABLE_MAEMO
+
+  return false;
+}
+
+bool check_gir_is_available_with_warning()
+{
+  if(gir_python_module_is_available())
+    return true;
+
+  /* The python module could not be imported by Glom, so warn the user: */
+  const Glib::ustring message = _("Your installation of Glom is not complete, because the gi.repository Python module is not available on your system.\n\nPlease report this bug to your vendor, or your system administrator so it can be corrected.");
+
+  //Make sure this is on stderr too, in case something goes wrong with the UI:
+  std::cerr << message << std::endl;
+
+  Gtk::MessageDialog dialog(Utils::bold_message(_("gi.repository Python Module Not Installed")), true /* use_markup */, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_OK, true /* modal */);
+  dialog.set_secondary_text(message);
+  dialog.run();
 
   return false;
 }
@@ -361,46 +324,46 @@ bool check_pygda_is_available_with_warning()
   if(gda_python_module_is_available())
     return true;
 
-   /* The python module could not be imported by Glom, so warn the user: */
-   const Glib::ustring message = _("Your installation of Glom is not complete, because the gda Python module is not available on your system.\n\nPlease report this bug to your vendor, or your system administrator so it can be corrected.");
+  /* The python module could not be imported by Glom, so warn the user: */
+  const Glib::ustring message = _("Your installation of Glom is not complete, because the gi.repository.Gda python module is not available on your system.\n\nPlease report this bug to your vendor, or your system administrator so it can be corrected.");
 
-#ifndef GLOM_ENABLE_MAEMO
-  Gtk::MessageDialog dialog(Utils::bold_message(_("gda Python Module Not Installed")), true /* use_markup */, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_OK, true /* modal */);
+  //Make sure this is on stderr too, in case something goes wrong with the UI:
+  std::cerr << message << std::endl;
+
+  Gtk::MessageDialog dialog(Utils::bold_message(_("gi.repository.Gda Python Module Not Installed")), true /* use_markup */, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_OK, true /* modal */);
   dialog.set_secondary_text(message);
   dialog.run();
-#else
-  Hildon::Note note(Hildon::NOTE_TYPE_INFORMATION, message);
-  note.run();
-#endif //GLOM_ENABLE_MAEMO
 
   return false;
 }
 
 } //namespace Glom
 
-#ifndef G_OS_WIN32
+#ifdef __linux__
 extern "C" void __libc_freeres(void);
 #endif
 
 int
 main(int argc, char* argv[])
 {
-#ifndef G_OS_WIN32
+#ifdef __linux__
   //Force some cleanup at exit,
   //to help valgrind to detect memory leaks:
   atexit(__libc_freeres);
 #else
+#  ifdef G_OS_WIN32
   WSADATA data;
   int errcode = WSAStartup(MAKEWORD(2, 0), &data);
   if(errcode != 0)
   {
     std::cerr << "Failed to initialize WinSock: " << errcode << std::endl;
-    return -1;
+    return EXIT_FAILURE;
   }
 
   gchar* installation_dir_c = g_win32_get_package_installation_directory_of_module(0);
   const std::string installation_dir(installation_dir_c);
   g_free(installation_dir_c);
+#  endif
 #endif
 
   // TODO: I am not sure why, but this does not work. PYTHONPATH is set
@@ -453,110 +416,33 @@ main(int argc, char* argv[])
     //https://bugzilla.gnome.org/show_bug.cgi?id=619445
     //This should tell us what the problem is:
     std::cerr << G_STRFUNC << ": exception from std::locale::global(std::locale(\"\")): " << ex.what() << std::endl;
+    std::cerr << "  This can happen if the locale is not properly installed or configured." << std::endl;
   }
 
   Glom::libglom_init(); //Also initializes python.
 
-#ifdef GLOM_ENABLE_MAEMO
-  if(!(osso_initialize("org.maemo.glom", PACKAGE_NAME, FALSE /* obsolete */, 0)))
-  {
-    std::cerr << "Glom: Error while initializing libossomm" << std::endl;
-    return 0;
-  }
-  Hildon::init();
-#endif
-
-  Glib::OptionContext context;
-
-  Glom::OptionGroup group;
-  context.set_main_group(group);
-
   //We use python for calculated-fields:
   PySys_SetArgv(argc, argv);
 
-  std::auto_ptr<Gtk::Main> mainInstance;
   try
   {
-    mainInstance = std::auto_ptr<Gtk::Main>( new Gtk::Main(argc, argv, context) );
-  }
-  catch(const Glib::Error& ex)
-  {
-    std::cerr << "Glom: Error while initializing gtkmm: " << ex.what() << std::endl;
-    return 0;
-  }
+    //Create the app here,
+    //so we can use UI, for instance with Gtk::MessageDialog,
+    //even before calling run().
+    Glib::RefPtr<Glom::Application> application = 
+      Glom::Application::create();
 
-  try
-  {
-    context.parse(argc, argv);
-  }
-  catch(const Glib::OptionError& ex)
-  {
-      std::cout << _("Error while parsing command-line options: ") << std::endl << ex.what() << std::endl;
-      std::cout << _("Use --help to see a list of available command-line options.") << std::endl;
-      return 0;
-  }
-  catch(const Glib::Error& ex)
-  {
-    std::cout << "Error: " << ex.what() << std::endl;
-    return 0;
-  }
+    //Call gtk_init() too, because earlier (< 3.9.17) versions
+    //of gtkmm do not do this in the default Gtk::Application constructor.
+    //TODO: Remove this when we can depend on a newer gtkmm:
+    gtk_init(0, 0);
 
-
-  if(group.m_arg_version)
-  {
-    std::cout << PACKAGE_STRING << std::endl;
-    return 0;
-  }
-
-  try
-  {
 #ifndef GLOM_ENABLE_CLIENT_ONLY
-    gtksourceview::init();
-    Goocanvas::init(PACKAGE_NAME, PACKAGE_VERSION, argc, argv);
+    Gsv::init();
+    Goocanvas::init();
 #endif //!GLOM_ENABLE_CLIENT_ONLY
 
-    //Get command-line parameters, if any:
-    Glib::ustring input_uri = group.m_arg_filename;
-
-    // The GOption documentation says that options without names will be returned to the application as "rest arguments".
-    // I guess this means they will be left in the argv. Murray.
-    if(input_uri.empty() && (argc > 1))
-    {
-       const char* pch = argv[1];
-       if(pch)
-         input_uri = pch;
-    }
-
-    if(!input_uri.empty())
-    {
-      //Get a URI (file://something) from the filepath:
-      Glib::RefPtr<Gio::File> file = Gio::File::create_for_commandline_arg(input_uri);
-
-      if(!file->query_exists())
-      {
-        std::cerr << _("Glom: The file does not exist.") << std::endl;
-        std::cerr << "uri: " << input_uri << std::endl;
-
-        std::cerr << std::endl << context.get_help() << std::endl;
-        return -1;
-      }
-
-      const Gio::FileType file_type = file->query_file_type();
-      if(file_type == Gio::FILE_TYPE_DIRECTORY)
-      {
-        std::cerr << _("Glom: The file path is a directory instead of a file.") << std::endl;
-
-        std::cerr << std::endl << context.get_help() << std::endl;
-        return -1;
-      }
-
-      input_uri = file->get_uri();
-
-      //std::cout << "URI = " << input_uri << std::endl;
-    }
-
-    //debugging:
-    //input_uri = "file:///home/murrayc/cvs/gnome212/glom/examples/example_smallbusiness.glom";
+    ev_init();
 
 #ifdef GLOM_ENABLE_POSTGRESQL
 
@@ -567,73 +453,41 @@ main(int argc, char* argv[])
       /* The Postgres provider was not found, so warn the user: */
       const Glib::ustring message = _("Your installation of Glom is not complete, because the PostgreSQL libgda provider is not available on your system. This provider is needed to access Postgres database servers.\n\nPlease report this bug to your vendor, or your system administrator so it can be corrected.");
 
-      #ifndef GLOM_ENABLE_MAEMO
+      //Make sure this is on stderr too, in case something goes wrong with the UI:
+      std::cerr << message << std::endl;
+
       Gtk::MessageDialog dialog(Glom::Utils::bold_message(_("Incomplete Glom Installation")), true /* use_markup */, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_OK, true /* modal */);
       dialog.set_secondary_text(message);
       dialog.run();
-      #else
-      Hildon::Note note(Hildon::NOTE_TYPE_INFORMATION, message);
-      note.run();
-      #endif
 
-      return -1; //There is no point in going further because Glom would not be able to connect to any Postgres servers.
+      return EXIT_FAILURE; //There is no point in going further because Glom would not be able to connect to any Postgres servers.
     }
 
     #ifndef GLOM_ENABLE_CLIENT_ONLY
     //Check that the postgres executable is really available:
     if(!Glom::check_postgres_is_available_with_warning())
-      return -1; //There is no point in going further because the most useful Glom functionality will not work without Postgres. Only a very cut-down Glom client would be useful without self-hosting.
+      return EXIT_FAILURE; //There is no point in going further because the most useful Glom functionality will not work without Postgres. Only a very cut-down Glom client would be useful without self-hosting.
     #endif // !GLOM_ENABLE_CLIENT_ONLY
 
     // Postgres can't be started as root. initdb complains.
     // So just prevent this in general. It is safer anyway.
     if(!Glom::check_user_is_not_root_with_warning())
-      return -1;
+      return EXIT_FAILURE;
 #endif //GLOM_ENABLE_POSTGRESQL
 
-    if(!Glom::check_pyglom_is_available_with_warning())
-      return -1;
+    if(!Glom::check_gir_is_available_with_warning())
+      return EXIT_FAILURE;
 
     if(!Glom::check_pygda_is_available_with_warning())
-      return -1;
+      return EXIT_FAILURE;
+
+    if(!Glom::check_pyglom_is_available_with_warning())
+      return EXIT_FAILURE;
 
 
-    // Some more sanity checking:
-    // These print errors to the stdout if they fail.
-    // In future we might refuse to start if they fail.
-    const bool test1 =
-      Glom::Conversions::sanity_check_date_text_representation_uses_4_digit_years(group.m_arg_debug_date_check);
-    const bool test2 = Glom::Conversions::sanity_check_date_parsing();
-    if(!test1 || !test2)
-    {
-      std::cerr << "Glom: ERROR: Date parsing sanity checks failed. Glom will not display dates correctly or interperet entered dates correctly. This needs attention from a translator. Please file a bug. See http://www.glom.org." << std::endl;
-    }
-
-    if(group.m_arg_debug_date_check)
-      return 0; //This command-line option is documented as stopping afterwards.
-
-    Glom::Application* pApplication = 0;
-    Glom::Utils::get_glade_widget_derived_with_warning(pApplication);
-    g_assert(pApplication);
-
-    pApplication->set_command_line_args(argc, argv);
-    pApplication->set_show_sql_debug(group.m_arg_debug_sql);
-    pApplication->set_stop_auto_server_shutdown(group.m_arg_stop_auto_server_shutdown);
-    Glom::ConnectionPool::get_instance()->set_show_debug_output(group.m_arg_debug_sql);
-
-    const bool test = pApplication->init(input_uri, group.m_arg_restore); //Sets it up and shows it.
-
-    #ifdef GLOM_ENABLE_MAEMO
-    //TODO: What is this really for?
-    Hildon::Program::get_instance()->add_window(*pApplication);
-    #endif
-
-    if(test) //The user could cancel the offer of a new or existing database.
-      Gtk::Main::run(*pApplication); //Quit when the window is closed.
-
-    //Cleanup:
-    delete pApplication;
-    pApplication = 0;
+    const int status = application->run(argc, argv);
+    if(status != EXIT_SUCCESS) //TODO: Is this right?
+      return status;
   }
   catch(const Glib::Exception& ex)
   {
@@ -658,5 +512,5 @@ main(int argc, char* argv[])
   WSACleanup();
 #endif
 
-  return 0;
+  return EXIT_SUCCESS;
 }
