@@ -14,28 +14,27 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
-#include "config.h" // For GLOM_ENABLE_MAEMO
+#include "config.h" // For GLOM_ENABLE_CLIENT_ONLY
 
 #include "box_data.h"
 #include <libglom/data_structure/glomconversions.h>
 #include <glom/utils_ui.h>
 #include <libglom/data_structure/layout/layoutitem_field.h>
+#include <libglom/db_utils.h>
 #include <libglom/privs.h>
 #include <glom/python_embed/glom_python.h>
 #include <glom/python_embed/python_ui_callbacks.h>
-#include <glom/application.h>
+#include <glom/appwindow.h>
+#include <gtkmm/stock.h>
 #include <algorithm> //For std::find()
 #include <libglom/libglom_config.h>
 #include <iostream>
 #include <glibmm/i18n.h>
 
-#ifdef GLOM_ENABLE_MAEMO
-#include <hildonmm/note.h>
-#endif
 
 namespace Glom
 {
@@ -93,6 +92,13 @@ Gnome::Gda::SqlExpr Box_Data::get_find_where_clause() const
   builder->select_add_target(m_table_name);  //This might not be necessary.
   guint where_cond_id = 0;
 
+  Glib::RefPtr<Gnome::Gda::Connection> connection = get_connection();
+  if(!connection)
+  {
+    std::cerr << G_STRFUNC << ": connection was null." << std::endl;
+    return Gnome::Gda::SqlExpr();
+  }
+
   //Look at each field entry and build e.g. 'Name = "Bob"'
   for(type_vecConstLayoutFields::const_iterator iter = m_FieldsShown.begin(); iter != m_FieldsShown.end(); ++iter)
   {
@@ -112,14 +118,14 @@ Gnome::Gda::SqlExpr Box_Data::get_find_where_clause() const
 
         if(use_this_field)
         {
-          const guint cond_id = builder->add_cond(Gnome::Gda::SQL_OPERATOR_TYPE_EQ, //TODO: Use field->sql_find_operator()
+          const guint cond_id = builder->add_cond(field->sql_find_operator(),
             builder->add_field_id(field->get_name(), m_table_name),
-            builder->add_expr(data));
+            builder->add_expr( field->sql_find(data, connection) ));
 
           //And with previous condition, if any:
           if(where_cond_id)
           {
-            where_cond_id = builder->add_cond(Gnome::Gda::SQL_OPERATOR_TYPE_AND, //TODO: Use field->sql_find_operator()
+            where_cond_id = builder->add_cond(Gnome::Gda::SQL_OPERATOR_TYPE_AND,
               where_cond_id,
               cond_id);
           }
@@ -144,22 +150,11 @@ void Box_Data::on_Button_Find()
   //Make sure that the cell is updated:
   //m_AddDel.finish_editing();
 
+  //Call the virtual method to get the find criteria for a details or list view:
   const Gnome::Gda::SqlExpr where_clause = get_find_where_clause();
-  if(where_clause.empty())
-  {
-    const Glib::ustring message = _("You have not entered any find criteria. Try entering information in the fields.");
 
-#ifdef GLOM_ENABLE_MAEMO
-    Hildon::Note dialog(Hildon::NOTE_TYPE_INFORMATION, *get_app_window(), message);
-#else
-    Gtk::MessageDialog dialog(Utils::bold_message(_("No Find Criteria")), true, Gtk::MESSAGE_WARNING );
-    dialog.set_secondary_text(message);
-    dialog.set_transient_for(*get_app_window());
-#endif
-    dialog.run();
-  }
-  else
-    signal_find_criteria.emit(where_clause);
+  //The signal handler then checks and warns if no find criteria were entered.
+  signal_find_criteria.emit(where_clause);
 }
 
 void Box_Data::set_unstored_data(bool bVal)
@@ -177,7 +172,8 @@ void Box_Data::create_layout()
   set_unstored_data(false);
 
   //Cache the table information, for performance:
-  m_TableFields = get_fields_for_table(m_table_name);
+  const Document* document = dynamic_cast<const Document*>(get_document());
+  m_TableFields = DbUtils::get_fields_for_table(document, m_table_name);
 }
 
 bool Box_Data::fill_from_database()
@@ -193,13 +189,8 @@ bool Box_Data::confirm_discard_unstored_data() const
   {
     const Glib::ustring message = _("This data cannot be stored in the database because you have not provided a primary key.\nDo you really want to discard this data?");
     //Ask user to confirm loss of data:
-#ifdef GLOM_ENABLE_MAEMO
-    //Hildon::Note dialog(Hildon::NOTE_TYPE_CONFIRMATION, *get_app_window(), message);
-    Hildon::Note dialog(Hildon::NOTE_TYPE_CONFIRMATION, message);
-#else
     Gtk::MessageDialog dialog(Utils::bold_message(_("No primary key value")), true, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_OK_CANCEL );
     dialog.set_secondary_text(message);
-#endif
     //TODO: It needs a const. I wonder if it should. murrayc. dialog.set_transient_for(*get_app_window());
     const int iButton = dialog.run();
 
@@ -309,7 +300,7 @@ void Box_Data::fill_layout_group_field_info(const sharedptr<LayoutGroup>& group,
         sharedptr<const Relationship> relationship = document->get_relationship(m_table_name, relationship_name);
         if(relationship)
         {
-          sharedptr<Field> field = get_fields_for_table_one_field(relationship->get_to_table(), item->get_name());
+          sharedptr<Field> field = DbUtils::get_fields_for_table_one_field(document, relationship->get_to_table(), item->get_name());
           if(field)
           {
             item_field->set_full_field_details(field);
@@ -324,7 +315,7 @@ void Box_Data::fill_layout_group_field_info(const sharedptr<LayoutGroup>& group,
       else
       {
         //Get the field info:
-        sharedptr<Field> field = get_fields_for_table_one_field(m_table_name, item_field->get_name());
+        sharedptr<Field> field = DbUtils::get_fields_for_table_one_field(document, m_table_name, item_field->get_name());
         if(field)
         {
           item_field->set_full_field_details(field); //TODO_Performance: Just use this as the output arg?
@@ -348,13 +339,9 @@ void Box_Data::fill_layout_group_field_info(const sharedptr<LayoutGroup>& group,
 void Box_Data::print_layout()
 {
   const Glib::ustring message = "Sorry, this feature has not been implemented yet.";
-#ifdef GLOM_ENABLE_MAEMO
-  Hildon::Note dialog(Hildon::NOTE_TYPE_INFORMATION, *get_app_window(), message);
-#else
   Gtk::MessageDialog dialog("<b>Not implemented</b>", true);
   dialog.set_secondary_text(message);
   dialog.set_transient_for(*get_app_window());
-#endif
   dialog.run();
 }
 
@@ -365,27 +352,31 @@ Glib::ustring Box_Data::get_layout_name() const
 
 void Box_Data::execute_button_script(const sharedptr<const LayoutItem_Button>& layout_item, const Gnome::Gda::Value& primary_key_value)
 {
+  const Glib::ustring script = layout_item->get_script();
+  if(!Utils::script_check_for_pygtk2_with_warning(script, get_app_window()))
+    return;
+
   const sharedptr<Field> field_primary_key = get_field_primary_key();
   const type_map_fields field_values = get_record_field_values_for_calculation(m_table_name, field_primary_key, primary_key_value);
 
   //We need the connection when we run the script, so that the script may use it.
   sharedptr<SharedConnection> sharedconnection = connect_to_server(0 /* parent window */);
 
-    //Allow this UI to respond to UI change requests from the Python code:
-    AppPythonUICallbacks callbacks;
+  //Allow this UI to respond to UI change requests from the Python code:
+  AppPythonUICallbacks callbacks;
 
-    Glib::ustring error_message;
-    glom_execute_python_function_implementation(layout_item->get_script(),
-      field_values, //TODO: Maybe use the field's type here.
-      get_document(),
-      get_table_name(), field_primary_key, primary_key_value,
-      sharedconnection->get_gda_connection(),
-      callbacks,
-      error_message);
-    if(!error_message.empty())
-    {
-      std::cerr << "Python Error: " << error_message << std::endl;
-    }
+  Glib::ustring error_message;
+  glom_execute_python_function_implementation(script,
+    field_values, //TODO: Maybe use the field's type here.
+    get_document(),
+    get_table_name(), field_primary_key, primary_key_value,
+    sharedconnection->get_gda_connection(),
+    callbacks,
+    error_message);
+  if(!error_message.empty())
+  {
+    std::cerr << "Python Error: " << error_message << std::endl;
+  }
 }
 
 } //namespace Glom
