@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #ifndef GLOM_MODE_DATA_BOX_DATA_H
@@ -48,7 +48,7 @@ public:
 
   //TODO: Put this in Base_DB_Table_Data?
   ///Create the layout for the database structure, and fill it with data from the database.
-  virtual bool init_db_details(const FoundSet& found_set, const Glib::ustring& layout_platform);
+  bool init_db_details(const FoundSet& found_set, const Glib::ustring& layout_platform);
 
   //Fill the existing layout with data from the database:
   virtual bool refresh_data_from_database_with_where_clause(const FoundSet& found_set);
@@ -108,10 +108,11 @@ protected:
 
   void execute_button_script(const sharedptr<const LayoutItem_Button>& layout_item, const Gnome::Gda::Value& primary_key_value);
 
-  //Signal handlers:
-  virtual void on_Button_Find(); //only used by _Find sub-classes. Should be MI.
-
 private:
+
+  //Signal handlers:
+  void on_Button_Find(); //only used by _Find sub-classes. Should be MI.
+
   //Signal handlers for the PyGlomUI callbacks:
   void on_python_requested_show_table_details(const Glib::ustring& table_name, const Gnome::Gda::Value& primary_key_value);
   void on_python_requested_show_table_list(const Glib::ustring& table_name);
@@ -120,7 +121,6 @@ private:
   void on_python_requested_start_new_record();
 
 protected:
-  static Glib::ustring xslt_process(const xmlpp::Document& xml_document, const std::string& filepath_xslt);
 
 #ifndef GLOM_ENABLE_CLIENT_ONLY
   virtual Dialog_Layout* create_layout_dialog() const = 0;
