@@ -14,16 +14,19 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #include <glom/mode_data/box_data_calendar_related.h>
 #include <glom/mode_design/layout/dialog_layout_calendar_related.h>
 #include <glom/utils_ui.h>
-#include <glom/application.h>
+#include <glom/appwindow.h>
 #include <libglom/data_structure/glomconversions.h>
+#include <libglom/db_utils.h>
 #include <glom/frame_glom.h> //For show_ok_dialog()
+#include <gtkmm/stock.h>
+#include <glom/glade_utils.h>
 #include <glibmm/i18n.h>
 
 namespace Glom
@@ -37,16 +40,16 @@ Box_Data_Calendar_Related::Box_Data_Calendar_Related()
 
   m_Alignment.add(m_calendar);
   m_calendar.show();
-  
+
   //m_calendar.set_show_details();
   m_calendar.set_detail_width_chars(7);
-  m_calendar.set_detail_height_rows(2); 
-  
+  m_calendar.set_detail_height_rows(2);
+
   //Tell the calendar how to get the record details to show:
   m_calendar.set_detail_func( sigc::mem_fun(*this, &Box_Data_Calendar_Related::on_calendar_details) );
-  
+
   m_calendar.signal_month_changed().connect( sigc::mem_fun(*this, &Box_Data_Calendar_Related::on_calendar_month_changed) );
-  
+
   setup_menu();
   //m_calendar.add_events(Gdk::BUTTON_PRESS_MASK); //Allow us to catch button_press_event and button_release_event
   m_calendar.signal_button_press_event().connect_notify( sigc::mem_fun(*this, &Box_Data_Calendar_Related::on_calendar_button_press_event) );
@@ -73,12 +76,12 @@ bool Box_Data_Calendar_Related::init_db_details(const sharedptr<const LayoutItem
 
 bool Box_Data_Calendar_Related::init_db_details(const Glib::ustring& parent_table, bool show_title)
 {
-  //std::cout << "DEBUG: Box_Data_Calendar_Related::init_db_details(): " << parent_table << std::endl;
+  //std::cout << "debug: " << G_STRFUNC << ": " << parent_table << std::endl;
 
   m_parent_table = parent_table;
 
   if(m_portal)
-    LayoutWidgetBase::m_table_name = m_portal->get_table_used(Glib::ustring() /* parent table_name, not used. */); 
+    LayoutWidgetBase::m_table_name = m_portal->get_table_used(Glib::ustring() /* parent table_name, not used. */);
   else
     LayoutWidgetBase::m_table_name = Glib::ustring();
 
@@ -87,16 +90,11 @@ bool Box_Data_Calendar_Related::init_db_details(const Glib::ustring& parent_tabl
   //TODO: This is duplicated in box_data_related_list.cc and box_data_portal.cc. Just use code from the base class?
   if(show_title)
   {
-    Glib::ustring relationship_title;
-    if(m_portal && m_portal->get_has_relationship_name())
-      relationship_title = m_portal->get_title_used(Glib::ustring() /* parent title - not relevant */);
-    else
-    {
-      //Note to translators: This text is shown instead of a table title, when the table has not yet been chosen.
-      relationship_title = _("Undefined Table");
-    }
+    Glib::ustring title;
+    if(m_portal)
+      title = item_get_title(m_portal);
 
-    m_Label.set_markup(Utils::bold_message(relationship_title));
+    m_Label.set_markup(Utils::bold_message(title));
     m_Label.show();
 
     m_Alignment.set_padding(Utils::DEFAULT_SPACING_SMALL /* top */, 0, Utils::DEFAULT_SPACING_LARGE /* left */, 0);
@@ -110,7 +108,11 @@ bool Box_Data_Calendar_Related::init_db_details(const Glib::ustring& parent_tabl
   }
 
   if(m_portal)
-    m_key_field = get_fields_for_table_one_field(LayoutWidgetBase::m_table_name, m_portal->get_to_field_used());
+  {
+    Document* document = get_document();    
+    m_key_field = DbUtils::get_fields_for_table_one_field(document,
+      LayoutWidgetBase::m_table_name, m_portal->get_to_field_used());
+  }
   else
     m_key_field.clear();
 
@@ -121,11 +123,19 @@ bool Box_Data_Calendar_Related::init_db_details(const Glib::ustring& parent_tabl
   return Box_Data::init_db_details(found_set, "" /* layout_platform */); //Calls create_layout() and fill_from_database().
 }
 
+
+void Box_Data_Calendar_Related::create_layout()
+{
+  Box_Data::create_layout();
+
+  m_FieldsShown = get_fields_to_show();
+}
+
 bool Box_Data_Calendar_Related::fill_from_database()
 {
   if(!m_portal)
     return false;
-  
+
   bool result = false;
 
   if(m_key_field && m_found_set.m_where_clause.empty()) //There's a key field, but no value.
@@ -140,24 +150,24 @@ bool Box_Data_Calendar_Related::fill_from_database()
   {
     if(m_query_column_date_field == -1)
       return false; //This is useless without the date in the result.
-       
+
     //Create a date range from the beginning to end of the selected month:
     Glib::Date calendar_date;
     m_calendar.get_date(calendar_date);
     const Glib::Date date_start(1, calendar_date.get_month(), calendar_date.get_year());
     Glib::Date date_end = date_start;
     date_end.add_months(1);
-    
+
     Gnome::Gda::Value date_start_value(date_start);
     Gnome::Gda::Value date_end_value(date_end);
 
     //Add a WHERE clause for this date range:
-    sharedptr<Relationship> relationship = m_portal->get_relationship();
+    sharedptr<const Relationship> relationship = m_portal->get_relationship();
     Glib::ustring where_clause_to_table_name = relationship->get_to_table();
-  
+
     sharedptr<LayoutItem_CalendarPortal> derived_portal = sharedptr<LayoutItem_CalendarPortal>::cast_dynamic(m_portal);
     const Glib::ustring date_field_name = derived_portal->get_date_field()->get_name();
-    
+
     sharedptr<const Relationship> relationship_related = m_portal->get_related_relationship();
     if(relationship_related)
     {
@@ -169,66 +179,64 @@ bool Box_Data_Calendar_Related::fill_from_database()
 
     //Add an AND to the existing where clause, to get only records within these dates, if any:
     sharedptr<const Field> date_field = derived_portal->get_date_field();
-    //TODO: Use a SQL parameter instead of using sql().
-    const Glib::ustring extra_where_clause = "\"" + where_clause_to_table_name + "\".\"" + date_field->get_name() + "\""
-      " BETWEEN DATE " + date_field->sql(date_start_value) + 
-      " AND DATE " + date_field->sql(date_end_value);
-    
-    Glib::ustring where_clause;
+
+    Glib::RefPtr<Gnome::Gda::SqlBuilder> builder =
+      Gnome::Gda::SqlBuilder::create(Gnome::Gda::SQL_STATEMENT_SELECT);
+    const guint cond = builder->add_cond(Gnome::Gda::SQL_OPERATOR_TYPE_BETWEEN,
+       builder->add_field_id(date_field->get_name(), m_found_set.m_table_name),
+       builder->add_expr_as_value(date_start_value),
+       builder->add_expr_as_value(date_end_value));
+    builder->set_where(cond); //Might be unnecessary.
+    const Gnome::Gda::SqlExpr extra_where_clause = builder->export_expression(cond);
+
+    Gnome::Gda::SqlExpr where_clause;
     if(m_found_set.m_where_clause.empty())
+    {
       where_clause = extra_where_clause;
+    }
     else
-      where_clause = "( " + m_found_set.m_where_clause + " ) AND ( " + extra_where_clause + " )";
-    
-    
+    {
+      where_clause = Utils::build_combined_where_expression(
+        m_found_set.m_where_clause, extra_where_clause,
+        Gnome::Gda::SQL_OPERATOR_TYPE_AND);
+    }
+
     //Do one SQL query for the whole month and store the cached values here:
     clear_cached_database_values();
-    
-    const Glib::ustring sql_query = Utils::build_sql_select_with_where_clause(m_found_set.m_table_name, m_FieldsShown, where_clause, m_found_set.m_extra_join, m_found_set.m_sort_clause, m_found_set.m_extra_group_by);
+
+    Glib::RefPtr<Gnome::Gda::SqlBuilder> sql_query = Utils::build_sql_select_with_where_clause(m_found_set.m_table_name, m_FieldsShown, where_clause, m_found_set.m_extra_join, m_found_set.m_sort_clause);
     //std::cout << "DEBUG: sql_query=" << sql_query << std::endl;
-    Glib::RefPtr<Gnome::Gda::DataModel> datamodel = query_execute_select(sql_query);
+    Glib::RefPtr<const Gnome::Gda::DataModel> datamodel = DbUtils::query_execute_select(sql_query);
     if(!(datamodel))
       return true;
-    
+
     const int rows_count = datamodel->get_n_rows();
     if(!(rows_count > 0))
       return true;
-    
+
     //Get the data:
     for(int row_index = 0; row_index < rows_count; ++row_index)
     {
       const int columns_count = datamodel->get_n_columns();
       if(m_query_column_date_field > columns_count)
        continue;
-       
+
       //Get the date value for this row:
-#ifdef GLIBMM_EXCEPTIONS_ENABLED      
-      Gnome::Gda::Value value_date = datamodel->get_value_at(m_query_column_date_field, row_index);
-#else
-      std::auto_ptr<Glib::Error> error;      
-      Gnome::Gda::Value value_date = datamodel->get_value_at(m_query_column_date_field, row_index, error);
-#endif      
+      const Gnome::Gda::Value value_date = datamodel->get_value_at(m_query_column_date_field, row_index);
       const Glib::Date date = value_date.get_date();
-      
+
       //Get all the values for this row:
       type_vector_values* pVector = new type_vector_values(m_FieldsShown.size());
       for(int column_index = 0; column_index < columns_count; ++column_index)
       {
-#ifdef GLIBMM_EXCEPTIONS_ENABLED 
         (*pVector)[column_index] = datamodel->get_value_at(column_index, row_index);
-#else
-        std::auto_ptr<Glib::Error> error;      
-          (*pVector)[column_index] = datamodel->get_value_at(column_index, row_index, error);
-        if (error.get())
-          break;
-#endif 
       }
-      
+
       m_map_values[date].push_back(pVector);
     }
   }
 
- 
+
   return result;
 }
 
@@ -240,12 +248,11 @@ void Box_Data_Calendar_Related::clear_cached_database_values()
     for(type_list_vectors::iterator iter = vec.begin(); iter != vec.end(); ++iter)
     {
       type_vector_values* pValues = *iter;
-      if(pValues)
-        delete pValues;
+      delete pValues;
     }
   }
-  
-  m_map_values.clear(); 
+
+  m_map_values.clear();
 }
 
 //TODO: Make this generic in Box_Data_Portal:
@@ -277,7 +284,7 @@ void Box_Data_Calendar_Related::on_record_added(const Gnome::Gda::Value& primary
   }
   else if(Conversions::value_is_empty(m_key_value))
   {
-    g_warning("Box_Data_Calendar_Related::on_record_added(): m_key_value is NULL.");
+    std::cerr << G_STRFUNC << ": m_key_value is NULL." << std::endl;
   }
   else
   {
@@ -286,13 +293,16 @@ void Box_Data_Calendar_Related::on_record_added(const Gnome::Gda::Value& primary
     //Create the link by setting the foreign key
     if(m_key_field && m_portal)
     {
-      Glib::RefPtr<Gnome::Gda::Set> params = Gnome::Gda::Set::create();
-      params->add_holder(m_key_field->get_holder(m_key_value));
-      params->add_holder(field_primary_key->get_holder(primary_key_value));
-      Glib::ustring strQuery = "UPDATE \"" + m_portal->get_table_used(Glib::ustring() /* not relevant */) + "\"";
-      strQuery += " SET \"" +  /* get_table_name() + "." +*/ m_key_field->get_name() + "\" = " + m_key_field->get_gda_holder_string();
-      strQuery += " WHERE \"" + get_table_name() + "\".\"" + field_primary_key->get_name() + "\" = " + field_primary_key->get_gda_holder_string();
-      const bool test = query_execute(strQuery, params);
+      Glib::RefPtr<Gnome::Gda::SqlBuilder> builder = Gnome::Gda::SqlBuilder::create(Gnome::Gda::SQL_STATEMENT_UPDATE);
+      const Glib::ustring target_table = m_portal->get_table_used(Glib::ustring() /* not relevant */);
+      builder->set_table(target_table);
+      builder->add_field_value_as_value(m_key_field->get_name(), m_key_value);
+      builder->set_where(
+        builder->add_cond(Gnome::Gda::SQL_OPERATOR_TYPE_EQ,
+          builder->add_field_id(field_primary_key->get_name(), target_table),
+          builder->add_expr_as_value(primary_key_value)));
+
+      const bool test = DbUtils::query_execute(builder);
       if(test)
       {
         //Show it on the view, if it's visible:
@@ -307,18 +317,24 @@ void Box_Data_Calendar_Related::on_record_added(const Gnome::Gda::Value& primary
   }
 }
 
-Box_Data_Calendar_Related::type_vecLayoutFields Box_Data_Calendar_Related::get_fields_to_show() const
+Box_Data_Calendar_Related::type_vecConstLayoutFields Box_Data_Calendar_Related::get_fields_to_show() const
 {
-  type_vecLayoutFields layout_fields = Box_Data_Portal::get_fields_to_show();
-  
+  type_vecConstLayoutFields layout_fields = Box_Data_Portal::get_fields_to_show();
+
   sharedptr<LayoutItem_CalendarPortal> derived_portal = sharedptr<LayoutItem_CalendarPortal>::cast_dynamic(m_portal);
   if(!derived_portal)
+  {
+    std::cerr << G_STRFUNC << ": The portal is not a LayoutItem_CalendarPortal." << std::endl;
     return layout_fields;
-  
+  }
+
   sharedptr<const Field> date_field = derived_portal->get_date_field();
   if(!date_field)
+  {
+    std::cerr << G_STRFUNC << ": get_date_field() returned no field." << std::endl;
     return layout_fields;
-  
+  }
+
   //Add it to the list to ensure that we request the date (though it will not really be shown in the calendar):
   sharedptr<LayoutItem_Field> layout_item_date_field = sharedptr<LayoutItem_Field>::create();
   layout_item_date_field->set_full_field_details(date_field);
@@ -331,13 +347,13 @@ Box_Data_Calendar_Related::type_vecLayoutFields Box_Data_Calendar_Related::get_f
 void Box_Data_Calendar_Related::on_dialog_layout_hide()
 {
   Dialog_Layout_Calendar_Related* dialog_related = dynamic_cast<Dialog_Layout_Calendar_Related*>(m_pDialogLayout);
-  g_assert(dialog_related != NULL);
+  g_assert(dialog_related);
   m_portal = dialog_related->get_portal_layout();
 
 
   //Update the UI:
   sharedptr<LayoutItem_CalendarPortal> derived_portal = sharedptr<LayoutItem_CalendarPortal>::cast_dynamic(m_portal);
-  init_db_details(derived_portal); 
+  init_db_details(derived_portal);
 
   Box_Data::on_dialog_layout_hide();
 
@@ -347,7 +363,7 @@ void Box_Data_Calendar_Related::on_dialog_layout_hide()
     sharedptr<LayoutItem_CalendarPortal> derived_portal = sharedptr<LayoutItem_CalendarPortal>::cast_dynamic(m_portal);
     if(derived_portal)
       *pLayoutItem = *derived_portal;
-    
+
     signal_layout_changed().emit(); //TODO: Check whether it has really changed.
   }
 }
@@ -356,32 +372,24 @@ void Box_Data_Calendar_Related::on_dialog_layout_hide()
 #ifndef GLOM_ENABLE_CLIENT_ONLY
 Dialog_Layout* Box_Data_Calendar_Related::create_layout_dialog() const
 {
-  Glib::RefPtr<Gtk::Builder> refXml = Gtk::Builder::create_from_file(
-      GLOM_PKGDATADIR G_DIR_SEPARATOR_S "glade" G_DIR_SEPARATOR_S "glom_developer.glade",
-      "window_data_layout");
-  if(refXml)
-  {
-    Dialog_Layout_Calendar_Related* dialog = 0;
-    refXml->get_widget_derived("window_data_layout", dialog);
-    return dialog;
-  }
-
-  return NULL;
+  Dialog_Layout_Calendar_Related* dialog = 0;
+  Glom::Utils::get_glade_widget_derived_with_warning(dialog);
+  return dialog;
 }
 
 void Box_Data_Calendar_Related::prepare_layout_dialog(Dialog_Layout* dialog)
 {
   Dialog_Layout_Calendar_Related* related_dialog = dynamic_cast<Dialog_Layout_Calendar_Related*>(dialog);
-  g_assert(related_dialog != NULL);
-  
+  g_assert(related_dialog);
+
   sharedptr<LayoutItem_CalendarPortal> derived_portal = sharedptr<LayoutItem_CalendarPortal>::cast_dynamic(m_portal);
   if(derived_portal && derived_portal->get_has_relationship_name())
   {
-    related_dialog->set_document(m_layout_name, m_layout_platform, get_document(), derived_portal);
+    related_dialog->init_with_portal(m_layout_name, m_layout_platform, get_document(), derived_portal);
   }
   else
   {
-    related_dialog->set_document(m_layout_name, m_layout_platform, get_document(), m_parent_table);
+    related_dialog->init_with_tablename(m_layout_name, m_layout_platform, get_document(), m_parent_table);
   }
 }
 #endif // !GLOM_ENABLE_CLIENT_ONLY
@@ -397,31 +405,34 @@ Glib::ustring Box_Data_Calendar_Related::on_calendar_details(guint year, guint m
   sharedptr<LayoutItem_CalendarPortal> derived_portal = sharedptr<LayoutItem_CalendarPortal>::cast_dynamic(m_portal);
   if(!derived_portal)
   {
-    //std::cout << "DEBUG: Box_Data_Calendar_Related::on_calendar_details(): date_field is NULL" << std::endl;
+    //std::cout << "debug: " << G_STRFUNC << ": date_field is NULL" << std::endl;
     return Glib::ustring();
   }
-  
+
   sharedptr<const Field> date_field = derived_portal->get_date_field();
   if(!date_field)
+  {
+    std::cerr << G_STRFUNC << ":  get_date_field() returned no field." << std::endl;
     return Glib::ustring();
-  
+  }
+
   //TODO: month seems to be 143710360 sometimes, which seems to be a GtkCalendar bug:
-  //std::cout << "Box_Data_Calendar_Related::on_calendar_details(): year=" << year << ", month=" << month << " day=" << day << std::endl;
+  //std::cout << "debug: " << G_STRFUNC << ": year=" << year << ", month=" << month << " day=" << day << std::endl;
 
   //Glib::Date is 1-indexed:
   Glib::Date::Month datemonth = (Glib::Date::Month)(month +1);
   if(datemonth > Glib::Date::DECEMBER)
     datemonth = Glib::Date::JANUARY;
   Glib::Date date(day, datemonth, year);
-  
+
   //Examine the cached data:
   type_map_values::const_iterator iter_find = m_map_values.find(date);
   if(iter_find == m_map_values.end())
     return Glib::ustring(); //No data was found for this date.
-  
-  
+
+
   Glib::ustring result;
-  
+
   //Look at each row for this date:
   const type_list_vectors& rows = iter_find->second;
   for(type_list_vectors::const_iterator iter = rows.begin(); iter != rows.end(); ++iter)
@@ -429,58 +440,58 @@ Glib::ustring Box_Data_Calendar_Related::on_calendar_details(guint year, guint m
     type_vector_values* pRow = *iter;
     if(!pRow)
       continue;
-    
+
     //Get the data for each column in the row:
     Glib::ustring row_text;
     int column_index = 0;
-    
+
     //We iterate over the original list of items from the portal,
     //instead of the ones used by the query (m_FieldsShown),
     //because we really don't want to show the extra fields (at the end) to the user:
     LayoutGroup::type_list_items items = m_portal->get_items();
     for(LayoutGroup::type_list_items::const_iterator iter = items.begin(); iter != items.end(); ++iter)
-    {   
+    {
       sharedptr<const LayoutItem> layout_item = *iter;
       if(!layout_item)
         continue;
-            
+
       Glib::ustring text;
-      
+
       //Text for a text item:
       sharedptr<const LayoutItem_Text> layout_item_text = sharedptr<const LayoutItem_Text>::cast_dynamic(layout_item);
       if(layout_item_text)
-        text = layout_item_text->get_text();
+        text = layout_item_text->get_text(AppWindow::get_current_locale());
       else
       {
         //Text for a field:
         sharedptr<const LayoutItem_Field> layout_item_field = sharedptr<const LayoutItem_Field>::cast_dynamic(layout_item);
-        
+
         const Gnome::Gda::Value value = (*pRow)[column_index];
         text = Conversions::get_text_for_gda_value(layout_item_field->get_glom_type(), value, layout_item_field->get_formatting_used().m_numeric_format);
-        
+
         ++column_index;
       }
-      
+
       //Add the field text to the row:
       if(!text.empty())
       {
         if(!row_text.empty())
           row_text += ", "; //TODO: Internationalization?
-      
+
         row_text += text;
       }
     }
-    
+
     //Add the row text to the result:
     if(!row_text.empty())
     {
       if(!result.empty())
-        result += "\n";
-      
+        result += '\n';
+
       result += row_text;
     }
   }
-  
+
   return result;
 }
 
@@ -503,11 +514,11 @@ void Box_Data_Calendar_Related::setup_menu()
     sigc::mem_fun(*this, &Box_Data_Calendar_Related::on_MenuPopup_activate_layout) );
 
   //TODO: This does not work until this widget is in a container in the window:
-  App_Glom* pApp = get_application();
+  AppWindow* pApp = get_appwindow();
   if(pApp)
   {
     pApp->add_developer_action(m_refContextLayout); //So that it can be disabled when not in developer mode.
-    pApp->update_userlevel_ui(); //Update our action's sensitivity. 
+    pApp->update_userlevel_ui(); //Update our action's sensitivity.
   }
 #endif // !GLOM_ENABLE_CLIENT_ONLY
 
@@ -517,11 +528,9 @@ void Box_Data_Calendar_Related::setup_menu()
 
   //TODO: add_accel_group(m_refUIManager->get_accel_group());
 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
   try
   {
-#endif
-    Glib::ustring ui_info = 
+    Glib::ustring ui_info =
         "<ui>"
         "  <popup name='ContextMenu'>"
         "    <menuitem action='ContextEdit'/>"
@@ -531,28 +540,19 @@ void Box_Data_Calendar_Related::setup_menu()
         "  </popup>"
         "</ui>";
 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
     m_refUIManager->add_ui_from_string(ui_info);
   }
   catch(const Glib::Error& ex)
   {
     std::cerr << "building menus failed: " <<  ex.what();
   }
-#else
-  std::auto_ptr<Glib::Error> error;
-  m_refUIManager->add_ui_from_string(ui_info, error);
-  if(error.get() != NULL)
-  {
-    std::cerr << "building menus failed: " << error->what();
-  }
-#endif //GLIBMM_EXCEPTIONS_ENABLED
 
   //Get the menu:
-  m_pMenuPopup = dynamic_cast<Gtk::Menu*>( m_refUIManager->get_widget("/ContextMenu") ); 
+  m_pMenuPopup = dynamic_cast<Gtk::Menu*>( m_refUIManager->get_widget("/ContextMenu") );
   if(!m_pMenuPopup)
     g_warning("menu not found");
 
- 
+
 #ifndef GLOM_ENABLE_CLIENT_ONLY
   if(pApp)
     m_refContextLayout->set_sensitive(pApp->get_userlevel() == AppState::USERLEVEL_DEVELOPER);
@@ -563,17 +563,17 @@ void Box_Data_Calendar_Related::on_calendar_button_press_event(GdkEventButton *e
 {
 #ifndef GLOM_ENABLE_CLIENT_ONLY
   //Enable/Disable items.
-  //We did this earlier, but get_application is more likely to work now:
-  App_Glom* pApp = get_application();
+  //We did this earlier, but get_appwindow is more likely to work now:
+  AppWindow* pApp = get_appwindow();
   if(pApp)
   {
     pApp->add_developer_action(m_refContextLayout); //So that it can be disabled when not in developer mode.
-    pApp->update_userlevel_ui(); //Update our action's sensitivity. 
+    pApp->update_userlevel_ui(); //Update our action's sensitivity.
   }
 #endif
 
   GdkModifierType mods;
-  gdk_window_get_pointer( Gtk::Widget::gobj()->window, 0, 0, &mods );
+  gdk_window_get_device_position( gtk_widget_get_window(Gtk::Widget::gobj()), event->device, 0, 0, &mods );
   if(mods & GDK_BUTTON3_MASK)
   {
     //Give user choices of actions on this item:
@@ -625,8 +625,5 @@ void Box_Data_Calendar_Related::set_primary_key_value(const Gtk::TreeModel::iter
 {
   //TODO
 }
-    
-
-
 
 } //namespace Glom
