@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #include "config.h"
@@ -29,8 +29,11 @@
 #include <glom/glade_utils.h>
 #include <glom/utils_ui.h>
 #include <libglom/privs.h>
-#include <glom/xsl_utils.h>
 #include <glom/python_embed/glom_python.h>
+#include <glom/print_layout/print_layout_utils.h>
+#include <glom/appwindow.h>
+#include <gtkmm/stock.h>
+#include <gtkmm/viewport.h>
 #include <sstream> //For stringstream
 #include <glibmm/i18n.h>
 
@@ -38,16 +41,14 @@ namespace Glom
 {
 
 Box_Data_Details::Box_Data_Details(bool bWithNavButtons /* = true */)
-: m_hbox_content(false, Utils::DEFAULT_SPACING_SMALL),
-#ifndef GLOM_ENABLE_MAEMO
-  m_hbox_buttons(Gtk::BUTTONBOX_END, Utils::DEFAULT_SPACING_SMALL),
+: m_hbox_content(Gtk::ORIENTATION_HORIZONTAL, Utils::DEFAULT_SPACING_SMALL),
+  m_hbox_buttons(Gtk::ORIENTATION_HORIZONTAL),
   m_Button_New(Gtk::Stock::ADD),
   m_Button_Del(Gtk::Stock::DELETE),
   m_Button_Nav_First(Gtk::Stock::GOTO_FIRST),
   m_Button_Nav_Prev(Gtk::Stock::GO_BACK),
   m_Button_Nav_Next(Gtk::Stock::GO_FORWARD),
   m_Button_Nav_Last(Gtk::Stock::GOTO_LAST),
-#endif //GLOM_ENABLE_MAEMO
   m_bDoNotRefreshRelated(false),
   m_ignore_signals(true)
 #ifndef GLOM_ENABLE_CLIENT_ONLY
@@ -56,32 +57,23 @@ Box_Data_Details::Box_Data_Details(bool bWithNavButtons /* = true */)
 {
   m_layout_name = "details";
 
+  m_hbox_buttons.set_layout(Gtk::BUTTONBOX_END);
+  m_hbox_buttons.set_spacing(Utils::DEFAULT_SPACING_SMALL);
+
   add_view(&m_FlowTable); //Allow this to access the document too.
 
   m_FlowTable.set_lines(1); //Sub-groups will have multiple columns (by default, there is one sub-group, with 2 columns).
 
-  #ifndef GLOM_ENABLE_MAEMO
   m_FlowTable.set_horizontal_spacing(Utils::DEFAULT_SPACING_SMALL); //The default anyway.
   m_FlowTable.set_vertical_spacing(Utils::DEFAULT_SPACING_SMALL); //The default anyway.
-  #else
-  m_FlowTable.set_vertical_spacing(0); //The hildon buttons and entries have their own default padding.
-  m_FlowTable.set_horizontal_spacing(HILDON_MARGIN_DOUBLE); //As per the UI specs.
-  #endif
 
   //m_strHint = _("When you change the data in a field the database is updated immediately.\n Click [New] to add a new record.\n Leave automatic ID fields empty - they will be filled for you.");
 
 
   //m_ScrolledWindow.set_border_width(Utils::DEFAULT_SPACING_SMALL);
-#ifdef GLOM_ENABLE_MAEMO
-  // Allow horizontal scrolling in maemo because the screen is rather small and
-  // there might be some database UIs that don't fit horizontally. Such a UI may
-  // be considered non-maemo-friendly, but it can still be fully viewed this way.
-  // TODO: Prevent the need for horizontal scrolling.
-  m_ScrolledWindow.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
-#else
+
   // Allow vertical scrolling, but never scroll horizontally:
   m_ScrolledWindow.set_policy(Gtk::POLICY_NEVER, Gtk::POLICY_AUTOMATIC);
-#endif
   m_ScrolledWindow.set_shadow_type(Gtk::SHADOW_NONE); //SHADOW_IN is Recommended by the GNOME HIG, but looks odd.
 
 #ifndef GLOM_ENABLE_CLIENT_ONLY
@@ -101,6 +93,7 @@ Box_Data_Details::Box_Data_Details(bool bWithNavButtons /* = true */)
   pack_start(m_hbox_content);
 
   m_FlowTable.signal_field_edited().connect( sigc::mem_fun(*this,  &Box_Data_Details::on_flowtable_field_edited) );
+  m_FlowTable.signal_field_choices_changed().connect( sigc::mem_fun(*this,  &Box_Data_Details::on_flowtable_field_choices_changed) );
   m_FlowTable.signal_field_open_details_requested().connect( sigc::mem_fun(*this,  &Box_Data_Details::on_flowtable_field_open_details_requested) );
   show_all();
 
@@ -116,7 +109,6 @@ Box_Data_Details::Box_Data_Details(bool bWithNavButtons /* = true */)
   m_FlowTable.signal_script_button_clicked().connect( sigc::mem_fun(*this, &Box_Data_Details::on_flowtable_script_button_clicked) );
 
 
-#ifndef GLOM_ENABLE_MAEMO
   m_Button_New.set_tooltip_text(_("Create a new record."));
   m_Button_Del.set_tooltip_text(_("Remove this record."));
   m_Button_Nav_First.set_tooltip_text(_("View the first record in the list."));
@@ -151,7 +143,6 @@ Box_Data_Details::Box_Data_Details(bool bWithNavButtons /* = true */)
   m_Button_Nav_Last.signal_clicked().connect(sigc::mem_fun(*this, &Box_Data_Details::on_button_nav_last));
 
   pack_start(m_hbox_buttons, Gtk::PACK_SHRINK);
-#endif //GLOM_ENABLE_MAEMO
 
   m_ignore_signals = false;
 }
@@ -177,7 +168,7 @@ void Box_Data_Details::set_found_set_from_primary_key_value()
   if(!m_field_primary_key)
     return;
 
-  if(!m_primary_key_value.is_null())
+  if(!Conversions::value_is_empty(m_primary_key_value))
   {
     m_found_set.m_where_clause = Utils::build_simple_where_expression(
        m_table_name, m_field_primary_key, m_primary_key_value);
@@ -299,10 +290,8 @@ bool Box_Data_Details::fill_from_database()
       Privileges table_privs = Privs::get_current_privs(m_table_name);
 
       //Enable/Disable record creation and deletion:
-      #ifndef GLOM_ENABLE_MAEMO
       m_Button_New.set_sensitive(table_privs.m_create);
       m_Button_Del.set_sensitive(table_privs.m_delete);
-      #endif //GLOM_ENABLE_MAEMO
 
       if(table_privs.m_view)
       {
@@ -447,7 +436,6 @@ void Box_Data_Details::on_button_new()
   }
 }
 
-#ifndef GLOM_ENABLE_MAEMO
 void Box_Data_Details::on_button_del()
 {
   if( Conversions::value_is_empty(get_primary_key_value_selected()) )
@@ -497,7 +485,6 @@ void Box_Data_Details::on_button_nav_last()
   if(confirm_discard_unstored_data())
     signal_nav_last().emit();
 }
-#endif //GLOM_ENABLE_MAEMO
 
 Gnome::Gda::Value Box_Data_Details::get_entered_field_data(const sharedptr<const LayoutItem_Field>& field) const
 {
@@ -584,7 +571,6 @@ void Box_Data_Details::on_related_record_added(Gnome::Gda::Value /* strKeyValue
   m_bDoNotRefreshRelated = bDoNotRefreshRelated;
 }
 
-#ifndef GLOM_ENABLE_MAEMO
 Box_Data_Details::type_signal_void Box_Data_Details::signal_nav_first()
 {
   return m_signal_nav_first;
@@ -609,7 +595,6 @@ Box_Data_Details::type_signal_record_deleted Box_Data_Details::signal_record_del
 {
   return m_signal_record_deleted;
 }
-#endif //GLOM_ENABLE_MAEMO
 
 Box_Data_Details::type_signal_requested_related_details Box_Data_Details::signal_requested_related_details()
 {
@@ -661,22 +646,28 @@ void Box_Data_Details::on_flowtable_field_open_details_requested(const sharedptr
   if(Conversions::value_is_empty(field_value))
     return; //Ignore empty ID fields.
 
+  //Updating doesn't seem necessary. The field details seem to be full already.
+  //Update the field details from the document:
+  ////sharedptr<LayoutItem_Field> unconst_field = sharedptr<LayoutItem_Field>::cast_const(layout_field); //A hack, because layout_field_should_have_navigation() needs to get full field details.
+  //unconst_field->set_full_field_details(
+  //  document->get_field(field->get_table_used(table_name), field->get_name()) ); //Otherwise get_primary_key() returns false always.
+      
+  sharedptr<Relationship> field_used_in_relationship_to_one;
+  const bool has_open_button = 
+    DbUtils::layout_field_should_have_navigation(m_table_name, layout_field, get_document(), 
+    field_used_in_relationship_to_one);
+         
   //If it's a simple field that is part of a relationship,
   //identifying a related record.
-  sharedptr<const Relationship> relationship = get_document()->get_field_used_in_relationship_to_one(m_table_name, layout_field);
-  if(relationship)
+  if(field_used_in_relationship_to_one)
   {
-    signal_requested_related_details().emit(relationship->get_to_table(), field_value);
+    signal_requested_related_details().emit(field_used_in_relationship_to_one->get_to_table(), field_value);
     return;
   }
 
   //If it is a related field that is a primary key,
   //meaning it identifies a record in another table:
-  sharedptr<const Field> field_info = layout_field->get_full_field_details();
-  const bool field_is_related_primary_key =
-    layout_field->get_has_relationship_name() &&
-    field_info && field_info->get_primary_key();
-  if(field_is_related_primary_key)
+  if(has_open_button)
   {
     signal_requested_related_details().emit(layout_field->get_table_used(m_table_name), field_value);
   }
@@ -684,29 +675,30 @@ void Box_Data_Details::on_flowtable_field_open_details_requested(const sharedptr
 
 void Box_Data_Details::on_flowtable_script_button_clicked(const sharedptr<const LayoutItem_Button>& layout_item)
 {
-  if(layout_item)
+  if(!layout_item)
   {
-    const Gnome::Gda::Value primary_key_value = get_primary_key_value_selected();
-    const Glib::ustring table_name_before = m_table_name;
-    execute_button_script(layout_item, primary_key_value);
+    std::cerr << G_STRFUNC << ": layout_item is null" << std::endl;
+    return;
+  }
+  
+  const Gnome::Gda::Value primary_key_value = get_primary_key_value_selected();
+  const Glib::ustring table_name_before = m_table_name;
+  execute_button_script(layout_item, primary_key_value);
 
-    //Refresh the view, in case the script changed any data,
-    //but not if the script navigated away:
-    if(m_table_name != table_name_before)
-      return;
+  //Refresh the view, in case the script changed any data,
+  //but not if the script navigated away:
+  if(m_table_name != table_name_before)
+    return;
 
-    //(m_primary_key_value seems to be NULL here. We can use primary_key_value instead, but it's a bit strange. murrayc.)
-    if(get_primary_key_is_in_foundset(m_found_set, primary_key_value)) //Check, because maybe the script deleted the current record, or changed something so that it should no longer be shown in the found set.
-    {
-      refresh_data_from_database_with_primary_key(primary_key_value);
-    }
-    else
-    {
-      #ifndef GLOM_ENABLE_MAEMO
-      //Tell the parent to do something appropriate, such as show another record:
-      signal_record_deleted().emit(primary_key_value);
-      #endif
-    }
+  //(m_primary_key_value seems to be NULL here. We can use primary_key_value instead, but it's a bit strange. murrayc.)
+  if(get_primary_key_is_in_foundset(m_found_set, primary_key_value)) //Check, because maybe the script deleted the current record, or changed something so that it should no longer be shown in the found set.
+  {
+    refresh_data_from_database_with_primary_key(primary_key_value);
+  }
+  else
+  {
+    //Tell the parent to do something appropriate, such as show another record:
+    signal_record_deleted().emit(primary_key_value);
   }
 }
 
@@ -723,7 +715,7 @@ void Box_Data_Details::on_flowtable_field_edited(const sharedptr<const LayoutIte
 
   Gnome::Gda::Value primary_key_value = get_primary_key_value_selected();
   //std::cout << "debug: " << G_STRFUNC << ": primary_key_value=" << primary_key_value.to_string() << std::endl;
-  if(!Conversions::value_is_empty(primary_key_value)) //If there is not a primary key value:
+  if(!Conversions::value_is_empty(primary_key_value)) //If there is not a stored primary key value yet:
   {
     Glib::ustring table_name;
     sharedptr<Field> primary_key_field;
@@ -747,7 +739,7 @@ void Box_Data_Details::on_flowtable_field_edited(const sharedptr<const LayoutIte
         table_name = relationship->get_to_table();
         const Glib::ustring to_field_name = relationship->get_to_field();
         //Get the key field in the other table (the table that we will change)
-        primary_key_field = get_fields_for_table_one_field(table_name, to_field_name); //TODO_Performance.
+        primary_key_field = DbUtils::get_fields_for_table_one_field(document, table_name, to_field_name); //TODO_Performance.
         if(primary_key_field)
         {
           //Get the value of the corresponding key in the current table (that identifies the record in the table that we will change)
@@ -769,7 +761,7 @@ void Box_Data_Details::on_flowtable_field_edited(const sharedptr<const LayoutIte
         }
         else
         {
-          g_warning("Box_Data_Details::on_flowtable_field_edited(): key not found for edited related field.");
+          std::cerr << G_STRFUNC << ": key not found for edited related field." << std::endl;
         }
       }
     }
@@ -852,7 +844,7 @@ void Box_Data_Details::on_flowtable_field_edited(const sharedptr<const LayoutIte
   }
   else
   {
-    //There is no current primary key value:
+    //There is no current stored primary key value yet:
 
     if(m_field_primary_key && m_field_primary_key->get_auto_increment()) //If the primary key is an auto-increment:
     {
@@ -876,7 +868,6 @@ void Box_Data_Details::on_flowtable_field_edited(const sharedptr<const LayoutIte
     else
     {
       //It is not auto-generated:
-
       if(m_field_primary_key && strFieldName == m_field_primary_key->get_name()) //if it is the primary key that is being edited.
       {
         if(!check_entered_value_for_uniqueness(m_table_name, layout_field, field_value, window))
@@ -905,13 +896,22 @@ void Box_Data_Details::on_flowtable_field_edited(const sharedptr<const LayoutIte
   } //if(get_primary_key_value_selected().size())
 }
 
+void Box_Data_Details::on_flowtable_field_choices_changed(const sharedptr<const LayoutItem_Field>& layout_field)
+{
+  if(m_ignore_signals)
+    return;
+
+  m_FlowTable.update_choices(layout_field);
+}
+
 void Box_Data_Details::on_userlevel_changed(AppState::userlevels user_level)
 {
 #ifndef GLOM_ENABLE_CLIENT_ONLY
   m_design_mode = ( user_level == AppState::USERLEVEL_DEVELOPER );
   m_FlowTable.set_design_mode(m_design_mode);
+
   // Recreate the layout to correctly set the size of empty flowtables:
-  create_layout();
+  init_db_details(m_found_set, m_layout_platform, m_primary_key_value);
 #endif
 }
 
@@ -920,100 +920,40 @@ sharedptr<Field> Box_Data_Details::get_field_primary_key() const
   return m_field_primary_key;
 }
 
-void Box_Data_Details::print_layout_group(xmlpp::Element* node_parent, const sharedptr<const LayoutGroup>& group)
-{
-  xmlpp::Element* nodeChildGroup = node_parent->add_child("group");
-  nodeChildGroup->set_attribute("title", group->get_title());
-
-  LayoutGroup::type_list_const_items items = group->get_items();
-  for(LayoutGroup::type_list_const_items::const_iterator iter = items.begin(); iter != items.end(); ++iter)
-  {
-    sharedptr<const LayoutItem> layout_item = *iter;
-
-    sharedptr<const LayoutGroup> pLayoutGroup = sharedptr<const LayoutGroup>::cast_dynamic(layout_item);
-    if(pLayoutGroup)
-      print_layout_group(nodeChildGroup, pLayoutGroup); //recurse
-    else
-    {
-      sharedptr<const LayoutItem_Field> pLayoutField = sharedptr<const LayoutItem_Field>::cast_dynamic(layout_item);
-      if(pLayoutField)
-      {
-        xmlpp::Element* nodeField = nodeChildGroup->add_child("field");
-
-        nodeField->set_attribute("title", pLayoutField->get_title_or_name());
-
-        Gnome::Gda::Value value = m_FlowTable.get_field_value(pLayoutField);
-        const Glib::ustring text_representation = Conversions::get_text_for_gda_value(pLayoutField->get_glom_type(), value,
-          pLayoutField->get_formatting_used().m_numeric_format); //In the current locale.
-
-        nodeField->set_attribute("value", text_representation);
-      }
-      else
-      {
-        sharedptr<const LayoutItem_Portal> pLayoutPortal = sharedptr<const LayoutItem_Portal>::cast_dynamic(layout_item);
-        if(pLayoutPortal)
-        {
-          xmlpp::Element* nodePortal = nodeChildGroup->add_child("related_records");
-
-          //Box_Data_List_Related* pPortalWidget = m_FlowTable.get_portals();
-
-          sharedptr<Relationship> relationship = get_document()->get_relationship(m_table_name, pLayoutPortal->get_relationship_name());
-          if(relationship)
-          {
-            nodePortal->set_attribute("title", relationship->get_title_or_name());
-
-            //TODO:
-
-            //TODO: Only print this if the user has access rights.
-          }
-        }
-      }
-    }
-  }
-
-}
-
 void Box_Data_Details::print_layout()
 {
   const Privileges table_privs = Privs::get_current_privs(m_table_name);
 
   //Don't try to print tables that the user can't view.
   if(!table_privs.m_view)
+    return;  //TODO: Warn the user.
+   
+  const Document* document = dynamic_cast<const Document*>(get_document());
+  if(!document)
   {
-    //TODO: Warn the user.
+    std::cerr << G_STRFUNC << ": document was null" << std::endl;
+    return;
   }
-  else
+
+  Glib::RefPtr<Gtk::PageSetup> page_setup = Gtk::PageSetup::create(); //TODO: m_canvas.get_page_setup();
+  if(!page_setup)
   {
-    //Create a DOM Document with the XML:
-    xmlpp::DomParser dom_parser;;
-
-    xmlpp::Document* pDocument = dom_parser.get_document();
-    xmlpp::Element* nodeRoot = pDocument->get_root_node();
-    if(!nodeRoot)
-    {
-      //Add it if it isn't there already:
-      nodeRoot = pDocument->create_root_node("details_print");
-    }
-
-    Glib::ustring table_title = get_document()->get_table_title(m_table_name);
-    if(table_title.empty())
-      table_title = m_table_name;
-
-    nodeRoot->set_attribute("table", table_title);
-
-
-    //The groups:
-    xmlpp::Element* nodeParent = nodeRoot;
-
-    Document::type_list_layout_groups layout_groups = get_data_layout_groups(m_layout_name, m_layout_platform);
-    for(Document::type_list_layout_groups::const_iterator iter = layout_groups.begin(); iter != layout_groups.end(); ++iter)
-    {
-      sharedptr<const LayoutGroup> layout_group = *iter;
-      print_layout_group(nodeParent, layout_group);
-    }
-
-    GlomXslUtils::transform_and_open(*pDocument, "print_details_to_html.xsl", get_app_window());
+    std::cerr << G_STRFUNC << ": page_setup was null" << std::endl;
+    return;
   }
+
+  //Note that we initially create the page layout without spaces for page 
+  //breaks because those spaces would be empty space on the page after
+  //we have moved items down when expanding:
+  //TODO: Squash that space when expanding custom layouts.
+  sharedptr<PrintLayout> print_layout = 
+    PrintLayoutUtils::create_standard(page_setup, m_table_name, document,
+      false /* do not avoid page margins */);
+  
+  //Show the print preview window:
+  AppWindow* app = AppWindow::get_appwindow();
+  PrintLayoutUtils::do_print_layout(print_layout, m_found_set,
+    false /* not preview */, document, true /* avoid page margins */, app);
 }
 
 #ifndef GLOM_ENABLE_CLIENT_ONLY
@@ -1027,7 +967,7 @@ Dialog_Layout* Box_Data_Details::create_layout_dialog() const
 void Box_Data_Details::prepare_layout_dialog(Dialog_Layout* dialog)
 {
   if(dialog)
-    dialog->set_document(m_layout_name, m_layout_platform, get_document(), m_table_name, m_FieldsShown); //TODO: Use m_TableFields?
+    dialog->init(m_layout_name, m_layout_platform, get_document(), m_table_name, m_FieldsShown); //TODO: Use m_TableFields?
 }
 
 void Box_Data_Details::show_layout_toolbar(bool show)
@@ -1045,4 +985,9 @@ void Box_Data_Details::do_new_record()
   on_button_new();
 }
 
+void Box_Data_Details::set_enable_drag_and_drop(bool enabled)
+{
+  m_FlowTable.set_enable_drag_and_drop(enabled);
+}
+
 } //namespace Glom
