@@ -14,28 +14,26 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #include "box_data_list.h"
 #include <libglom/data_structure/glomconversions.h>
 #include <glom/glade_utils.h>
-#include <glom/report_builder.h>
 #include <glom/mode_design/layout/dialog_layout_list.h>
 #include <libglom/privs.h>
 #include <libglom/db_utils.h>
 #include <libglom/utils.h> //For bold_message()).
-//#include <../utility_widgets/db_adddel/glom_db_treemodel.h> //For DbTreeModel.
 #include <sstream> //For stringstream
+#include <glibmm/main.h>
 #include <glibmm/i18n.h>
 
 namespace Glom
 {
 
 Box_Data_List::Box_Data_List()
-: m_has_one_or_more_records(false),
-  m_read_only(false)
+:  m_read_only(false)
 {
   m_layout_name = "list";
 
@@ -49,6 +47,7 @@ Box_Data_List::Box_Data_List()
   m_AddDel.signal_user_requested_edit().connect(sigc::mem_fun(*this, &Box_Data_List::on_adddel_user_requested_edit));
   m_AddDel.signal_script_button_clicked().connect(sigc::mem_fun(*this, &Box_Data_List::on_adddel_script_button_clicked));
   m_AddDel.signal_sort_clause_changed().connect(sigc::mem_fun(*this, &Box_Data_List::on_adddel_user_sort_clause_changed));
+  m_AddDel.signal_record_selection_changed().connect(m_signal_record_selection_changed.make_slot());
 
   //TODO: Re-add this signal if this is really wanted, but make it part of a complete drag-and-drop feature for list views:
   //m_AddDel.signal_user_reordered_columns().connect(sigc::mem_fun(*this, &Box_Data_List::on_adddel_user_reordered_columns));
@@ -94,7 +93,7 @@ void Box_Data_List::refresh_data_from_database_blank()
   found_set.m_where_clause = Gnome::Gda::SqlExpr();
   m_AddDel.set_found_set(found_set);
 
-  std::cout << "debug: Box_Data_List::refresh_data_from_database_blank(): before refresh_from_database_blank()." << std::endl;
+  std::cout << "debug: " << G_STRFUNC << ": before refresh_from_database_blank()." << std::endl;
   m_AddDel.refresh_from_database_blank();
   m_found_set = found_set;
 }
@@ -112,7 +111,6 @@ bool Box_Data_List::fill_from_database()
 
   sharedptr<SharedConnection> sharedconnection;
 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
   try
   {
     sharedconnection = connect_to_server(get_app_window());
@@ -127,15 +125,6 @@ bool Box_Data_List::fill_from_database()
     handle_error(ex);
     result = false;
   }
-#else
-  std::auto_ptr<ExceptionConnection> error;
-  sharedconnection = connect_to_server(get_app_window(), error);
-  if(error.get())
-  {
-    handle_error(*error);
-    result = false;
-  }
-#endif
 
   if(sharedconnection)
   {
@@ -153,14 +142,13 @@ bool Box_Data_List::fill_from_database()
 
     enable_buttons();
 
+    m_AddDel.set_allow_view(table_privs.m_view);
     m_AddDel.set_found_set(m_found_set);
 
     result = m_AddDel.refresh_from_database();
 
     if(table_privs.m_view)
     {
-      //TODO: Don't show it if m_view is false.
-
       //Select first record:
       Glib::RefPtr<Gtk::TreeModel> refModel = m_AddDel.get_model();
       if(refModel)
@@ -288,15 +276,15 @@ void Box_Data_List::on_details_nav_next()
     //Don't go past the last record:
     if( !m_AddDel.get_is_last_row(iter) )
     {
-      std::cout << "DEBUG: Box_Data_List::on_details_nav_next(): The current row was not the last row." << std::endl;
+      //std::cout << "debug: " << G_STRFUNC << ": The current row was not the last row." << std::endl;
 
       iter++;
       m_AddDel.select_item(iter);
 
       signal_user_requested_details().emit(m_AddDel.get_value_key_selected());
     }
-    else
-      std::cout << "DEBUG: Box_Data_List::on_details_nav_next(): Not going past the last row." << std::endl;
+    //else
+    //  std::cout << "debug: " << G_STRFUNC << ": Not going past the last row." << std::endl;
   }
 }
 
@@ -358,7 +346,7 @@ Gnome::Gda::Value Box_Data_List::get_primary_key_value_selected() const
 
 Gnome::Gda::Value Box_Data_List::get_primary_key_value_first() const
 {
-  //std::cout << "Box_Data_List(): get_primary_key_value_first() records_count = " << m_AddDel.get_count() << std::endl;
+  //std::cout << "debug: " << G_STRFUNC << ": get_primary_key_value_first() records_count = " << m_AddDel.get_count() << std::endl;
 
   Glib::RefPtr<Gtk::TreeModel> model = m_AddDel.get_model();
   if(model)
@@ -369,18 +357,18 @@ Gnome::Gda::Value Box_Data_List::get_primary_key_value_first() const
       Gnome::Gda::Value value = get_primary_key_value(iter);
       if(Conversions::value_is_empty(value))
       {
-       //std::cout << "Box_Data_List(): get_primary_key_value_first() iter val is NULL" << std::endl;
+       //std::cout << "debug: " << G_STRFUNC << ": get_primary_key_value_first() iter val is NULL" << std::endl;
         ++iter;
       }
       else
       {
-         //std::cout << "Box_Data_List(): get_primary_key_value_first() returning: " << value.to_string() << std::endl;
+         //std::cout << "debug: " << G_STRFUNC << ": get_primary_key_value_first() returning: " << value.to_string() << std::endl;
         return value;
       }
     }
   }
 
- // std::cout << "Box_Data_List(): get_primary_key_value_first() return NULL" << std::endl;
+ // std::cout << "debug: " << G_STRFUNC << ": get_primary_key_value_first() return NULL" << std::endl;
   return Gnome::Gda::Value();
 }
 
@@ -404,47 +392,6 @@ bool Box_Data_List::get_showing_multiple_records() const
   return m_AddDel.get_count() > 1;
 }
 
-void Box_Data_List::create_layout_add_group(const sharedptr<LayoutGroup>& layout_group)
-{
-  if(!layout_group)
-    return;
-
-  LayoutGroup::type_list_items child_items = layout_group->get_items();
-  for(LayoutGroup::type_list_items::const_iterator iter = child_items.begin(); iter != child_items.end(); ++iter)
-  {
-    sharedptr<LayoutItem> child_item = *iter;
-
-    sharedptr<LayoutGroup> child_group = sharedptr<LayoutGroup>::cast_dynamic(child_item);
-    if(child_group)
-    {
-      //std::cout << "debug: Start Adding child group." << std::endl;
-      create_layout_add_group(child_group);
-      //std::cout << "debug: End Adding child group." << std::endl;
-    }
-    else
-    {
-      if(m_read_only)
-        child_item->set_editable(false);
-
-      //std::cout << "debug: adding column: " << child_item->get_name() << std::endl;
-
-      sharedptr<LayoutItem_Field> child_field = sharedptr<LayoutItem_Field>::cast_dynamic(child_item);
-      if(child_field)
-      {
-        //Check that the field really exists, to avoid SQL errors.
-        //This could probably only happen if we have failed to rename something everywhere, when the user has renamed something.
-        if(!DbUtils::get_field_exists_in_database(child_field->get_table_used(m_table_name), child_field->get_name()))
-        {
-          std::cerr << "debug: Box_Data_List::create_layout_add_group(): Field does not exist in database: table_name=" << child_field->get_table_used(m_table_name) << ", field_name=" << child_field->get_name() << std::endl;
-          continue;
-        }
-      }
-
-      m_AddDel.add_column(child_item);
-    }
-  }
-}
-
 Document::type_list_layout_groups Box_Data_List::create_layout_get_layout()
 {
   //This method is overriden in Box_Data_List_Related.
@@ -457,62 +404,88 @@ void Box_Data_List::create_layout()
   Box_Data::create_layout(); //Fills m_TableFields.
 
   const Document* pDoc = dynamic_cast<const Document*>(get_document());
-  if(pDoc)
+  if(!pDoc)
+    return;
+
+
+  //Field Names:
+  m_AddDel.remove_all_columns();
+  //m_AddDel.set_columns_count(m_Fields.size());
+
+  m_AddDel.set_table_name(m_table_name);
+
+
+  sharedptr<Field> field_primary_key = get_field_primary_key_for_table(m_table_name);
+  if(!field_primary_key)
   {
-    //Field Names:
-    m_AddDel.remove_all_columns();
-    //m_AddDel.set_columns_count(m_Fields.size());
-
-    m_AddDel.set_table_name(m_table_name);
-
-
-    sharedptr<Field> field_primary_key = get_field_primary_key_for_table(m_table_name);
-    if(!field_primary_key)
-    {
-      std::cerr << "Box_Data_List::create_layout(): primary key not found." << std::endl;
-    }
-    else
-    {
-      //std::cout << "DEBUG: Box_Data_List::create_layout(): primary_key=" << field_primary_key->get_name() << std::endl;
-
-      m_AddDel.set_key_field(field_primary_key);
-
-      //This map of layout groups will also contain the field information from the database:
-      Document::type_list_layout_groups layout_groups = create_layout_get_layout();
-
-      //int debug_count = 0;
-      for(Document::type_list_layout_groups::const_iterator iter = layout_groups.begin(); iter != layout_groups.end(); ++iter)
-      {
-        //std::cout << "Box_Data_List::create_layout() group number=" << debug_count;
-        //debug_count++;
-        //iter->second->debug();
-
-        create_layout_add_group(*iter);
-      }
-    }
-
-
-    m_FieldsShown = get_fields_to_show();
-
-    //Add extra possibly-non-visible columns that we need:
-    //TODO: Only add it if it is not already there.
-    if(field_primary_key)
-    {
-      sharedptr<LayoutItem_Field> layout_item = sharedptr<LayoutItem_Field>::create();
-      layout_item->set_hidden();
-      layout_item->set_full_field_details(m_AddDel.get_key_field());
-      m_FieldsShown.push_back(layout_item);
-
-      m_AddDel.add_column(layout_item);
-    }
-
-    m_AddDel.set_found_set(m_found_set);
-
-    //Column-creation happens in fill_database() instead:
-    //otherwise the treeview will be filled twice.
-    //m_AddDel.set_columns_ready();
+    std::cerr << G_STRFUNC << ": primary key not found for table: " << m_table_name << std::endl;
+    return;
   }
 
+   m_AddDel.set_key_field(field_primary_key);
+
+
+
+  LayoutGroup::type_list_items items_to_use;
+
+  //This map of layout groups will also contain the field information from the database:
+  Document::type_list_layout_groups layout_groups = create_layout_get_layout();
+  for(Document::type_list_layout_groups::const_iterator iter = layout_groups.begin(); iter != layout_groups.end(); ++iter)
+  {
+    const sharedptr<LayoutGroup> layout_group = *iter;
+    if(!layout_group)
+      continue;
+
+    const LayoutGroup::type_list_items child_items = layout_group->get_items_recursive();
+    for(LayoutGroup::type_list_items::const_iterator iterItems = child_items.begin(); iterItems != child_items.end(); ++iterItems)
+    {
+      sharedptr<LayoutItem> child_item = *iterItems;
+
+      //TODO: Set the whole thing as read-only instead:
+      if(m_read_only)
+        child_item->set_editable(false);
+
+      sharedptr<const LayoutItem_Field> child_field = sharedptr<const LayoutItem_Field>::cast_dynamic(child_item);
+
+      //This check has already happened in Frame_Glom::update_table_in_document_from_database().
+      //It is inefficient and unnecessary to do it here too.
+      /*
+      if(child_field)
+      {
+        //Check that the field really exists, to avoid SQL errors.
+        //This could probably only happen if we have failed to rename something everywhere, when the user has renamed something.
+        if(!DbUtils::get_field_exists_in_database(child_field->get_table_used(m_table_name), child_field->get_name()))
+        {
+          std::cerr << G_STRFUNC << ": Field does not exist in database: table_name=" << child_field->get_table_used(m_table_name) << ", field_name=" << child_field->get_name() << std::endl;
+          continue;
+        }
+      }
+      */
+
+      items_to_use.push_back(child_item);
+    }
+  }
+
+
+  //Add extra possibly-non-visible columns that we need:
+  //TODO: Only add it if it is not already there.
+  items_to_use = Utils::get_layout_items_plus_primary_key(items_to_use, pDoc, m_table_name);
+  if(field_primary_key)
+  {
+    sharedptr<LayoutItem_Field> layout_item = sharedptr<LayoutItem_Field>::create();
+    layout_item->set_hidden();
+    layout_item->set_full_field_details(m_AddDel.get_key_field());
+
+    m_FieldsShown.push_back(layout_item); //TODO: Do this only if it is not already present.
+  }
+
+  const Privileges table_privs = Privs::get_current_privs(m_found_set.m_table_name);
+  m_AddDel.set_allow_view(table_privs.m_view);
+    
+  m_AddDel.set_found_set(m_found_set);
+  m_AddDel.set_columns(items_to_use); //TODO: Use LayoutGroup::type_list_const_items instead?
+
+  m_FieldsShown = get_fields_to_show();
 }
 
 sharedptr<Field> Box_Data_List::get_field_primary_key() const
@@ -520,37 +493,6 @@ sharedptr<Field> Box_Data_List::get_field_primary_key() const
   return m_AddDel.get_key_field();
 }
 
-void Box_Data_List::print_layout() //TODO: Is this a duplicate of the one in box_data_manyrecords.cc?
-{
-  const Privileges table_privs = Privs::get_current_privs(m_table_name);
-
-  //Don't try to print tables that the user can't view.
-  if(!table_privs.m_view)
-  {
-    //TODO: Warn the user.
-  }
-  else
-  {
-    //Create a simple report on the fly:
-    sharedptr<Report> report_temp(new Report());
-    report_temp->set_name("list");
-    report_temp->set_title(_("List"));
-
-    //Add all the fields from the layout:
-    for(type_vecLayoutFields::const_iterator iter = m_FieldsShown.begin(); iter != m_FieldsShown.end(); ++iter)
-    {
-      report_temp->m_layout_group->add_item(*iter);
-    }
-
-    ReportBuilder report_builder;
-    report_builder.set_document(get_document());
-    report_builder.report_build(m_found_set, report_temp, get_app_window());
-  }
-}
-
-void Box_Data_List::print_layout_group(xmlpp::Element* /* node_parent */, const sharedptr<const LayoutGroup>& /* group */)
-{
-}
 
 void Box_Data_List::set_read_only(bool read_only)
 {
@@ -621,7 +563,7 @@ Dialog_Layout* Box_Data_List::create_layout_dialog() const
 
 void Box_Data_List::prepare_layout_dialog(Dialog_Layout* dialog)
 {
-  dialog->set_document(m_layout_name, m_layout_platform, get_document(), m_table_name, m_FieldsShown); //TODO: Use m_TableFields?
+  dialog->init(m_layout_name, m_layout_platform, get_document(), m_table_name, m_FieldsShown); //TODO: Use m_TableFields?
 }
 #endif // !GLOM_ENABLE_CLIENT_ONLY
 
