@@ -14,24 +14,24 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
-#ifndef BOX_DATA_LIST_H
-#define BOX_DATA_LIST_H
+#ifndef GLOM_MODE_DATA_BOX_DATA_LIST_H
+#define GLOM_MODE_DATA_BOX_DATA_LIST_H
 
 #include "config.h" // GLOM_ENABLE_CLIENT_ONLY
 
 #include "box_data_manyrecords.h"
-#include <glom/utility_widgets/db_adddel/db_adddel_withbuttons.h>
+#include <glom/mode_data/db_adddel/db_adddel_withbuttons.h>
 
 namespace Glom
 {
 
 class Box_Data_List : public Box_Data_ManyRecords
 {
-public: 
+public:
   Box_Data_List();
   virtual ~Box_Data_List();
 
@@ -61,12 +61,11 @@ public:
 
 
   //Signal Handlers:
-  //TODO: Remove virtual?
-  virtual void on_details_nav_first();
-  virtual void on_details_nav_previous();
-  virtual void on_details_nav_next();
-  virtual void on_details_nav_last();
-  virtual void on_details_record_deleted(const Gnome::Gda::Value& primary_key_value);
+  void on_details_nav_first();
+  void on_details_nav_previous();
+  void on_details_nav_next();
+  void on_details_nav_last();
+  void on_details_record_deleted(const Gnome::Gda::Value& primary_key_value);
 
   void get_record_counts(gulong& total, gulong& found) const;
 
@@ -83,8 +82,7 @@ protected:
   //Overrides of functions from Box_Data:
   virtual void create_layout(); //override
   virtual Document::type_list_layout_groups create_layout_get_layout();
-  void create_layout_add_group(const sharedptr<LayoutGroup>& layout_group);
-    
+
   virtual bool fill_from_database(); //override.
   virtual void enable_buttons();
 
@@ -101,10 +99,7 @@ protected:
 #endif // !GLOM_ENABLE_CLIENT_ONLY
 
   void on_adddel_script_button_clicked(const sharedptr<const LayoutItem_Button>& layout_item, const Gtk::TreeModel::iterator& row);
-  virtual bool on_script_button_idle(const Gnome::Gda::Value& primary_key_value);
-
-  virtual void print_layout();
-  virtual void print_layout_group(xmlpp::Element* node_parent, const sharedptr<const LayoutGroup>& group);
+  bool on_script_button_idle(const sharedptr<const LayoutItem_Button>& layout_item, const Gnome::Gda::Value& primary_key);
 
 #ifndef GLOM_ENABLE_CLIENT_ONLY
   virtual Dialog_Layout* create_layout_dialog() const; // override.
@@ -114,10 +109,9 @@ protected:
   //Member widgets:
   mutable DbAddDel_WithButtons m_AddDel; //mutable because its get_ methods aren't const.
 
-  bool m_has_one_or_more_records;
   bool m_read_only;
 };
 
 } //namespace Glom
 
-#endif
+#endif // GLOM_MODE_DATA_BOX_DATA_LIST_H
