@@ -14,15 +14,17 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #include "box_data_manyrecords.h"
 #include <libglom/data_structure/glomconversions.h>
 #include <glom/glade_utils.h>
-#include <glom/report_builder.h>
+#include <glom/appwindow.h>
+#include <libglom/report_builder.h>
 #include <glom/mode_design/layout/dialog_layout_list.h>
+#include <glom/utils_ui.h>
 #include <libglom/privs.h>
 #include <libglom/utils.h> //For bold_message()).
 #include <sstream> //For stringstream
@@ -32,8 +34,7 @@ namespace Glom
 {
 
 Box_Data_ManyRecords::Box_Data_ManyRecords()
-: m_has_one_or_more_records(false),
-  m_read_only(false)
+: m_read_only(false)
 {
   //We do not actually use this,
   //so it is a bug if this appears in the .glom file:
@@ -66,6 +67,11 @@ Box_Data_ManyRecords::type_signal_user_requested_details Box_Data_ManyRecords::s
   return m_signal_user_requested_details;
 }
 
+Box_Data_ManyRecords::type_signal_record_selection_changed Box_Data_ManyRecords::signal_record_selection_changed()
+{
+  return m_signal_record_selection_changed;
+}
+
 void Box_Data_ManyRecords::print_layout()
 {
   const Privileges table_privs = Privs::get_current_privs(m_table_name);
@@ -78,21 +84,14 @@ void Box_Data_ManyRecords::print_layout()
   else
   {
     //Create a simple report on the fly:
-    sharedptr<Report> report_temp(new Report());
-    report_temp->set_name("list");
-    report_temp->set_title(_("List"));
+    Document* document = get_document();
+    sharedptr<Report> report_temp = ReportBuilder::create_standard_list_report(document, m_table_name);
 
-    //Add all the fields from the layout:
-    for(type_vecConstLayoutFields::const_iterator iter = m_FieldsShown.begin(); iter != m_FieldsShown.end(); ++iter)
-    {
-      sharedptr<const LayoutItem> item = *iter;
-      sharedptr<LayoutItem> unconst = sharedptr<LayoutItem>::cast_const(item); //TODO: Avoid this?
-      report_temp->m_layout_group->add_item(unconst);
-    }
-
-    ReportBuilder report_builder;
-    report_builder.set_document(get_document());
-    report_builder.report_build(m_found_set, report_temp, get_app_window());
+    ReportBuilder report_builder(AppWindow::get_current_locale());
+    report_builder.set_document(document);
+    const std::string filepath = 
+      report_builder.report_build_and_save(m_found_set, report_temp);
+    Utils::show_report_in_browser(filepath, get_app_window());
   }
 }
 
