@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #include "config.h"
@@ -23,23 +23,21 @@
 #include "entry.h"
 #include "checkbutton.h"
 #include "label.h"
-#include <glom/mode_data/datawidget/comboentry.h>
 #include <glom/mode_data/datawidget/combo.h>
 #include <glom/mode_data/datawidget/combo_as_radio_buttons.h>
 #include <glom/mode_data/datawidget/textview.h>
 #include <glom/utility_widgets/imageglom.h>
 #include <libglom/data_structure/glomconversions.h>
-#include <glom/application.h>
+#include <glom/appwindow.h>
 #include <glom/mode_design/layout/dialog_choose_field.h>
 #include <glom/mode_data/datawidget/dialog_choose_id.h>
 #include <glom/mode_data/datawidget/dialog_choose_date.h>
+#include <glom/mode_data/datawidget/dialog_new_record.h>
 #include <glom/mode_design/layout/layout_item_dialogs/dialog_field_layout.h>
 #include <glom/utils_ui.h>
 #include <glom/glade_utils.h>
-
-#ifdef GLOM_ENABLE_MAEMO
-#include <hildonmm/button.h>
-#endif //GLOM_ENABLE_MAEMO
+#include <gtkmm/stock.h>
+#include <libglom/db_utils.h>
 
 #include <glibmm/i18n.h>
 
@@ -49,7 +47,7 @@ namespace Glom
 static DataWidgetChildren::ComboChoices* create_combo_widget_for_field(const sharedptr<LayoutItem_Field>& field)
 {
   DataWidgetChildren::ComboChoices* result = 0;
-  bool as_radio_buttons = false; //TODO: Use this.
+  bool as_radio_buttons = false;
   const bool restricted = field->get_formatting_used().get_choices_restricted(as_radio_buttons);
   if(restricted)
   {
@@ -59,7 +57,7 @@ static DataWidgetChildren::ComboChoices* create_combo_widget_for_field(const sha
       result = Gtk::manage(new DataWidgetChildren::ComboGlom());
   }
   else
-    result = Gtk::manage(new DataWidgetChildren::ComboEntry());
+    result = Gtk::manage(new DataWidgetChildren::ComboGlom(true /* has_entry */));
 
   return result;
 }
@@ -70,13 +68,16 @@ DataWidget::DataWidget(const sharedptr<LayoutItem_Field>& field, const Glib::ust
 {
   const Field::glom_field_type glom_type = field->get_glom_type();
   set_layout_item(field, table_name);
+  
+  //The GNOME HIG says that labels should have ":" at the end:
+  //http://library.gnome.org/devel/hig-book/stable/design-text-labels.html.en
+  const Glib::ustring title = Glib::ustring::compose(_("%1:"), item_get_title_or_name(field));
 
   m_child = 0;
-  LayoutWidgetField* pFieldWidget = 0;
-  const Glib::ustring title = field->get_title_or_name();
+  LayoutWidgetField* pFieldWidget = 0;  
   if(glom_type == Field::TYPE_BOOLEAN)
   {
-    DataWidgetChildren::CheckButton* checkbutton = Gtk::manage( new DataWidgetChildren::CheckButton( title ) );
+    DataWidgetChildren::CheckButton* checkbutton = Gtk::manage( new DataWidgetChildren::CheckButton() );
     checkbutton->show();
     checkbutton->signal_toggled().connect( sigc::mem_fun(*this, &DataWidget::on_widget_edited)  );
 
@@ -85,7 +86,9 @@ DataWidget::DataWidget(const sharedptr<LayoutItem_Field>& field, const Glib::ust
     m_child = checkbutton;
     pFieldWidget = checkbutton;
 
-    m_label.set_text( Glib::ustring() ); //It is not used.
+    m_label.set_label(title);
+    m_label.set_alignment(0);
+    m_label.show();
   }
   else if(glom_type == Field::TYPE_IMAGE)
   {
@@ -104,12 +107,9 @@ DataWidget::DataWidget(const sharedptr<LayoutItem_Field>& field, const Glib::ust
     m_label.set_alignment(0);
     m_label.show();
   }
-  // Use hildon widgets for date and time on maemo
   else
   {
-    //The GNOME HIG says that labels should have ":" at the end:
-    //http://library.gnome.org/devel/hig-book/stable/design-text-labels.html.en
-    m_label.set_label(title + ':');
+    m_label.set_label(title);
     m_label.set_alignment(0);
     m_label.show();
 
@@ -120,26 +120,23 @@ DataWidget::DataWidget(const sharedptr<LayoutItem_Field>& field, const Glib::ust
 
       if(field->get_formatting_used().get_has_custom_choices())
       {
-        //set_choices() needs this, for the numeric layout:
+        //set_choices_fixed() needs this, for the numeric layout:
         combo->set_layout_item( get_layout_item(), table_name);
 
-        combo->set_choices( field->get_formatting_used().get_choices_custom() );
+	const Formatting& formatting = field->get_formatting_used();
+	bool as_radio_buttons = false; //Ignored;
+        combo->set_choices_fixed( formatting.get_choices_custom(), formatting.get_choices_restricted(as_radio_buttons));
       }
       else if(field->get_formatting_used().get_has_related_choices())
       {
         combo = create_combo_widget_for_field(field);
         combo->set_layout_item( get_layout_item(), table_name);
 
-        sharedptr<const Relationship> choice_relationship;
-        Glib::ustring choice_field, choice_second;
-        bool choice_show_all = false;
-        field->get_formatting_used().get_choices_related(choice_relationship, choice_field, choice_second, choice_show_all);
-
-        combo->set_choices_related(document, choice_relationship, choice_field, choice_second, choice_show_all);
+        combo->set_choices_related(document, field, Gnome::Gda::Value() /* no ID means show all related records */);
       }
       else
       {
-        g_warning("DataWidget::DataWidget(): Unexpected choice type.");
+        std::cerr << G_STRFUNC << ": Unexpected choice type." << std::endl;
       }
 
       pFieldWidget = combo;
@@ -186,29 +183,21 @@ DataWidget::DataWidget(const sharedptr<LayoutItem_Field>& field, const Glib::ust
 
     bool child_added = false; //Don't use an extra container unless necessary.
 
-    //Check whether the field controls a relationship,
-    //meaning it identifies a record in another table.
-    const bool field_used_in_relationship_to_one = document->get_field_used_in_relationship_to_one(table_name, field);
-    //std::cout << "DEBUG: table_name=" << table_name << ", table_used=" << field->get_table_used(table_name) << ", field=" << field->get_name() << ", field_used_in_relationship_to_one=" << field_used_in_relationship_to_one << std::endl;
+    //Update the field details from the document:
+    field->set_full_field_details(
+      document->get_field(field->get_table_used(table_name), field->get_name()) ); //Otherwise get_primary_key() returns false always.
 
-    //Check whether the field identifies a record in another table
-    //just because it is a primary key in that table:
-    bool field_is_related_primary_key = false;
-    if(document)
-      field->set_full_field_details( document->get_field(field->get_table_used(table_name), field->get_name()) ); //Otherwise get_primary_key() returns false always.
-    sharedptr<const Field> field_info = field->get_full_field_details();
-    field_is_related_primary_key =
-      field->get_has_relationship_name() &&
-      field_info && field_info->get_primary_key();
-    //std::cout <<   "DEBUG: field->get_has_relationship_name()=" << field->get_has_relationship_name() << ", field_info->get_primary_key()=" <<  field_info->get_primary_key() << ", field_is_related_primary_key=" << field_is_related_primary_key << std::endl;
+    sharedptr<Relationship> field_used_in_relationship_to_one;
+    const bool add_open_button = 
+       DbUtils::layout_field_should_have_navigation(table_name, field, document, 
+         field_used_in_relationship_to_one);
 
+    Gtk::Box* hbox_parent = 0; //Only used if there are extra widgets.
 
-    Gtk::HBox* hbox_parent = 0; //Only used if there are extra widgets.
-
-    const bool with_extra_widgets = field_used_in_relationship_to_one || field_is_related_primary_key || (glom_type == Field::TYPE_DATE);
+    const bool with_extra_widgets = field_used_in_relationship_to_one || add_open_button || (glom_type == Field::TYPE_DATE);
     if(with_extra_widgets)
     {
-      hbox_parent = Gtk::manage( new Gtk::HBox() ); //We put the child (and any extra stuff) in this:
+      hbox_parent = Gtk::manage( new Gtk::Box(Gtk::ORIENTATION_HORIZONTAL) ); //We put the child (and any extra stuff) in this:
       hbox_parent->set_spacing(Utils::DEFAULT_SPACING_SMALL);
 
       hbox_parent->pack_start(*m_child);
@@ -221,43 +210,35 @@ DataWidget::DataWidget(const sharedptr<LayoutItem_Field>& field, const Glib::ust
     if(glom_type == Field::TYPE_DATE)
     {
       //Let the user choose a date from a calendar dialog:
-      #ifndef GLOM_ENABLE_MAEMO
       Gtk::Button* button_date = Gtk::manage(new Gtk::Button(_("..."))); //TODO: A better label/icon for "Choose Date".
-      #else
-      Gtk::Button* button_date = Gtk::manage(new Hildon::Button(Gtk::Hildon::SIZE_FINGER_HEIGHT, Hildon::BUTTON_ARRANGEMENT_HORIZONTAL, _("..."), ""));
-      #endif
       button_date->set_tooltip_text(_("Choose a date from an on-screen calendar."));
       button_date->show();
-      hbox_parent->pack_start(*button_date);
+      hbox_parent->pack_start(*button_date, Gtk::PACK_SHRINK);
       button_date->signal_clicked().connect(sigc::mem_fun(*this, &DataWidget::on_button_choose_date));
     }
 
-    if((field_used_in_relationship_to_one || field_is_related_primary_key) && hbox_parent)
+    if(hbox_parent && add_open_button)
     {
       //Add a button for related record navigation:
-      #ifndef GLOM_ENABLE_MAEMO
       m_button_go_to_details = Gtk::manage(new Gtk::Button(Gtk::Stock::OPEN));
-      #else
-      m_button_go_to_details = Gtk::manage(new Hildon::Button(Gtk::Hildon::SIZE_FINGER_HEIGHT, Hildon::BUTTON_ARRANGEMENT_HORIZONTAL, _("Open"), ""));
-      #endif
       m_button_go_to_details->set_tooltip_text(_("Open the record identified by this ID, in the other table."));
-      hbox_parent->pack_start(*m_button_go_to_details);
+      hbox_parent->pack_start(*m_button_go_to_details, Gtk::PACK_SHRINK);
       m_button_go_to_details->signal_clicked().connect(sigc::mem_fun(*this, &DataWidget::on_button_open_details));
 
-      //Add a button to make it easier to choose an ID for this field.
+      //Add an additional button to make it easier to choose an ID for this field.
       //Don't add this for simple related primary key fields, because they
       //can generally not be edited via another table's layout.
       if(field_used_in_relationship_to_one)
       {
-        #ifndef GLOM_ENABLE_MAEMO
         Gtk::Button* button_select = Gtk::manage(new Gtk::Button(Gtk::Stock::FIND));
-        #else
-        Gtk::Button* button_select = Gtk::manage(new Hildon::Button(Gtk::Hildon::SIZE_FINGER_HEIGHT,
-           Hildon::BUTTON_ARRANGEMENT_HORIZONTAL, _("Find"), ""));
-        #endif
         button_select->set_tooltip_text(_("Enter search criteria to identify records in the other table, to choose an ID for this field."));
-        hbox_parent->pack_start(*button_select);
+        hbox_parent->pack_start(*button_select, Gtk::PACK_SHRINK);
         button_select->signal_clicked().connect(sigc::mem_fun(*this, &DataWidget::on_button_select_id));
+
+        Gtk::Button* button_new = Gtk::manage(new Gtk::Button(Gtk::Stock::NEW));
+        button_new->set_tooltip_text(_("Enter details for a new record in the other table, then use its ID for this field."));
+        hbox_parent->pack_start(*button_new, Gtk::PACK_SHRINK);
+        button_new->signal_clicked().connect(sigc::mem_fun(*this, &DataWidget::on_button_new_id));
       }
     }
 
@@ -270,7 +251,7 @@ DataWidget::DataWidget(const sharedptr<LayoutItem_Field>& field, const Glib::ust
 #endif // GLOM_ENABLE_CLIENT_ONLY
 
   set_events(Gdk::BUTTON_PRESS_MASK);
-  signal_style_changed().connect(sigc::mem_fun(*this, &DataWidget::on_self_style_changed));
+  //TODO: signal_style_changed().connect(sigc::mem_fun(*this, &DataWidget::on_self_style_changed));
 }
 
 DataWidget::~DataWidget()
@@ -288,6 +269,10 @@ DataWidget::type_signal_edited DataWidget::signal_edited()
   return m_signal_edited;
 }
 
+DataWidget::type_signal_choices_changed DataWidget::signal_choices_changed()
+{
+  return m_signal_choices_changed;
+}
 
 void DataWidget::set_value(const Gnome::Gda::Value& value)
 {
@@ -296,7 +281,7 @@ void DataWidget::set_value(const Gnome::Gda::Value& value)
   if(generic_field_widget)
   {
     //if(generic_field_widget->get_layout_item())
-    //  std::cout << "DataWidget::set_value(): generic_field_widget->get_layout_item()->get_name()=" << generic_field_widget->get_layout_item()->get_name() << std::endl;
+    //  std::cout << "debug: " << G_STRFUNC << ": generic_field_widget->get_layout_item()->get_name()=" << generic_field_widget->get_layout_item()->get_name() << std::endl;
 
     generic_field_widget->set_value(value);
   }
@@ -368,7 +353,6 @@ void DataWidget::set_child_size_by_field(const sharedptr<const LayoutItem_Field>
     int height = -1; //auto.
     if((glom_type == Field::TYPE_TEXT) && (field->get_formatting_used().get_text_format_multiline()))
     {
-      #ifndef GLOM_ENABLE_MAEMO
       int example_width = 0;
       int example_height = 0;
       Glib::RefPtr<Pango::Layout> refLayout = create_pango_layout("example"); //TODO: Use different text, according to the current locale, or allow the user to choose an example?
@@ -376,13 +360,6 @@ void DataWidget::set_child_size_by_field(const sharedptr<const LayoutItem_Field>
 
       if(example_height > 0)
         height = example_height * field->get_formatting_used().get_text_format_multiline_height_lines();
-      #else
-      //On Maemo, TextView widgets expand automatically.
-      //TODO: Expansion only happens if both are -1, and vertical expansion never happens.
-      //See bug https://bugs.maemo.org/show_bug.cgi?id=5515
-      width = -1;
-      height = -1;
-      #endif //GLOM_ENABLE_MAEMO
     }
 
     m_child->set_size_request(width, height);
@@ -411,21 +388,36 @@ void DataWidget::set_viewable(bool viewable)
 void DataWidget::set_editable(bool editable)
 {
   Gtk::Widget* child = get_data_child_widget();
-  Gtk::Entry* entry = dynamic_cast<Gtk::Entry*>(child);
-  if(entry)
-    entry->set_editable(editable);
-  else
+  Gtk::Editable* gtkeditable = dynamic_cast<Gtk::Editable*>(child);
+  if(gtkeditable)
   {
-    LayoutWidgetBase* base = dynamic_cast<LayoutWidgetBase*>(child);
-    if(base)
-      base->set_read_only(!editable);
-    else
-    {
-      Gtk::CheckButton* checkbutton = dynamic_cast<Gtk::CheckButton*>(child);
-      if(checkbutton)
-        checkbutton->set_sensitive(editable);
-    }
+    gtkeditable->set_editable(editable);
+    return;
   }
+  
+  //GtkTextView does not implement GtkEditable:
+  //See https://bugzilla.gnome.org/show_bug.cgi?id=667008
+  //and our TextView class actually derives from ScrolledView anyway,
+  //and out LayoutWidgetBase::set_read_only() override takes care of it instead.
+  //But let's leave this here just in case:
+  Gtk::TextView* textview = 
+    dynamic_cast<Gtk::TextView*>(child);
+  if(textview)
+  {
+    textview->set_editable(editable);
+    return;
+  }
+  
+  LayoutWidgetBase* base = dynamic_cast<LayoutWidgetBase*>(child);
+  if(base)
+  {
+    base->set_read_only(!editable);
+    return;
+  }
+
+  Gtk::CheckButton* checkbutton = dynamic_cast<Gtk::CheckButton*>(child);
+  if(checkbutton)
+    checkbutton->set_sensitive(editable);
 }
 
 #ifndef GLOM_ENABLE_CLIENT_ONLY
@@ -434,8 +426,8 @@ bool DataWidget::on_button_press_event(GdkEventButton *event)
   //g_warning("DataWidget::on_button_press_event_popup");
 
   //Enable/Disable items.
-  //We did this earlier, but get_application is more likely to work now:
-  Application* pApp = get_application();
+  //We did this earlier, but get_appwindow is more likely to work now:
+  AppWindow* pApp = get_appwindow();
   if(pApp)
   {
     pApp->add_developer_action(m_refContextLayout); //So that it can be disabled when not in developer mode.
@@ -450,7 +442,7 @@ bool DataWidget::on_button_press_event(GdkEventButton *event)
     if(pApp->get_userlevel() == AppState::USERLEVEL_DEVELOPER)
     {
       GdkModifierType mods;
-      gdk_window_get_pointer( gtk_widget_get_window (Gtk::Widget::gobj()), 0, 0, &mods );
+      gdk_window_get_device_position( gtk_widget_get_window (Gtk::Widget::gobj()), event->device, 0, 0, &mods );
       if(mods & GDK_BUTTON3_MASK)
       {
         //Give user choices of actions on this item:
@@ -470,10 +462,10 @@ sharedptr<LayoutItem_Field> DataWidget::offer_field_list(const Glib::ustring& ta
 
 sharedptr<LayoutItem_Field> DataWidget::offer_field_list(const Glib::ustring& table_name, const sharedptr<const LayoutItem_Field>& start_field)
 {
-  return offer_field_list(table_name, start_field, get_document(), get_application());
+  return offer_field_list(table_name, start_field, get_document(), get_appwindow());
 }
 
-sharedptr<LayoutItem_Field> DataWidget::offer_field_list(const Glib::ustring& table_name, const sharedptr<const LayoutItem_Field>& start_field, Document* document, Application* app)
+sharedptr<LayoutItem_Field> DataWidget::offer_field_list(const Glib::ustring& table_name, const sharedptr<const LayoutItem_Field>& start_field, Document* document, AppWindow* app)
 {
   sharedptr<LayoutItem_Field> result;
 
@@ -506,11 +498,13 @@ sharedptr<LayoutItem_Field> DataWidget::offer_field_layout(const sharedptr<const
 
   Dialog_FieldLayout* dialog = 0;
   Utils::get_glade_widget_derived_with_warning(dialog);
+  if(!dialog) //Unlikely and it already warns on stderr.
+    return result;
 
   add_view(dialog); //Give it access to the document.
   dialog->set_field(start_field, m_table_name);
 
-  Gtk::Window* parent = get_application();
+  Gtk::Window* parent = get_appwindow();
   if(parent)
     dialog->set_transient_for(*parent);
 
@@ -573,12 +567,12 @@ void DataWidget::on_child_user_requested_layout()
 }
 #endif // !GLOM_ENABLE_CLIENT_ONLY
 
-Application* DataWidget::get_application()
+AppWindow* DataWidget::get_appwindow() const
 {
-  Gtk::Container* pWindow = get_toplevel();
+  Gtk::Container* pWindow = const_cast<Gtk::Container*>(get_toplevel());
   //TODO: This only works when the child widget is already in its parent.
 
-  return dynamic_cast<Application*>(pWindow);
+  return dynamic_cast<AppWindow*>(pWindow);
 }
 
 #ifndef GLOM_ENABLE_CLIENT_ONLY
@@ -620,6 +614,20 @@ const Gtk::Widget* DataWidget::get_data_child_widget() const
    }
  }
 
+void DataWidget::on_button_new_id()
+{
+  Gnome::Gda::Value chosen_id;
+  const bool chosen = offer_related_record_id_new(chosen_id);
+  if(!chosen)
+    return;
+
+  //Update the choices, if any, to show the new related record:
+  m_signal_choices_changed.emit();
+
+  set_value(chosen_id);
+  m_signal_edited.emit(chosen_id);
+}
+
 void DataWidget::on_button_choose_date()
 {
   DataWidgetChildren::Dialog_ChooseDate* dialog = 0;
@@ -627,7 +635,7 @@ void DataWidget::on_button_choose_date()
 
   if(dialog)
   {
-    Gtk::Window* parent = get_application(); //This doesn't work (and would be wrong) when the widget is in a Field Definitions dialog.
+    Gtk::Window* parent = get_appwindow(); //This doesn't work (and would be wrong) when the widget is in a Field Definitions dialog.
     if(parent)
       dialog->set_transient_for(*parent);
     dialog->set_date_chosen(get_value());
@@ -666,7 +674,7 @@ bool DataWidget::offer_related_record_id_find(Gnome::Gda::Value& chosen_id)
   if(dialog)
   {
     //dialog->set_document(get_document(), table_name, field);
-    Gtk::Window* parent = get_application();
+    Gtk::Window* parent = get_appwindow();
     if(parent)
       dialog->set_transient_for(*parent);
     add_view(dialog);
@@ -701,4 +709,53 @@ bool DataWidget::offer_related_record_id_find(Gnome::Gda::Value& chosen_id)
   return result;
 }
 
+
+bool DataWidget::offer_related_record_id_new(Gnome::Gda::Value& chosen_id)
+{
+  bool result = false;
+
+  //Initialize output variable:
+  chosen_id = Gnome::Gda::Value();
+
+  DataWidgetChildren::Dialog_NewRecord* dialog = 0;
+  Glom::Utils::get_glade_widget_derived_with_warning(dialog);
+
+  if(dialog)
+  {
+    //dialog->set_document(get_document(), table_name, field);
+    Gtk::Window* parent = get_appwindow();
+    if(parent)
+      dialog->set_transient_for(*parent);
+    add_view(dialog);
+
+    //Discover the related table, in the relationship that uses this ID field:
+    Glib::ustring related_table_name;
+    sharedptr<const LayoutItem_Field> layoutField = sharedptr<LayoutItem_Field>::cast_dynamic(get_layout_item());
+    if(layoutField)
+    {
+      sharedptr<const Relationship> relationship = get_document()->get_field_used_in_relationship_to_one(m_table_name, layoutField);
+      if(relationship)
+        related_table_name = relationship->get_to_table();
+    }
+    else
+      g_warning("get_layout_item() was not a LayoutItem_Field");
+
+    dialog->init_db_details(related_table_name, Base_DB::get_active_layout_platform(get_document()));
+
+
+    const int response = dialog->run();
+    dialog->hide();
+    if(response == Gtk::RESPONSE_OK)
+    {
+      //Get the chosen ID:
+      result = dialog->get_id_chosen(chosen_id);
+    }
+
+    remove_view(dialog);
+    delete dialog;
+  }
+
+  return result;
+}
+
 } //namespace Glom
