@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #include "flowtablewithfields.h"
@@ -27,8 +27,7 @@
 #include <glom/utility_widgets/imageglom.h>
 #include <glom/mode_data/datawidget/label.h>
 #include <glom/utility_widgets/dialog_flowtable.h>
-#include <glom/mode_data/placeholder-glom.h>
-#include <glom/application.h>
+#include <glom/appwindow.h>
 #include <gtkmm/checkbutton.h>
 #include <libglom/data_structure/glomconversions.h>
 #include <glom/glade_utils.h>
@@ -54,7 +53,8 @@ FlowTableWithFields::Info::Info()
 FlowTableWithFields::FlowTableWithFields(const Glib::ustring& table_name)
 :
   m_placeholder(0),
-  m_table_name(table_name)
+  m_table_name(table_name),
+  m_find_mode(false)
 {
 }
 
@@ -65,11 +65,15 @@ FlowTableWithFields::~FlowTableWithFields()
   {
     View_Composite_Glom* pViewFirst = dynamic_cast<View_Composite_Glom*>(iter->m_first);
     if(pViewFirst)
+    {
       remove_view(pViewFirst);
+    }
 
-   View_Composite_Glom* pViewSecond = dynamic_cast<View_Composite_Glom*>(iter->m_second);
+    View_Composite_Glom* pViewSecond = dynamic_cast<View_Composite_Glom*>(iter->m_second);
     if(pViewSecond)
+    {
       remove_view(pViewSecond);
+    }
   }
 }
 
@@ -89,17 +93,12 @@ void FlowTableWithFields::set_table(const Glib::ustring& table_name)
 }
 
 void FlowTableWithFields::add_layout_item(const sharedptr<LayoutItem>& item)
-{
-  add_layout_item_at_position(item, m_list_layoutwidgets.end());
-}
-
-void FlowTableWithFields::add_layout_item_at_position(const sharedptr<LayoutItem>& item, const type_list_layoutwidgets::iterator& add_before)
 {
   //Get derived type and do the appropriate thing:
   sharedptr<LayoutItem_Field> field = sharedptr<LayoutItem_Field>::cast_dynamic(item);
   if(field)
   {
-    add_field_at_position(field, m_table_name, add_before);
+    add_field(field, m_table_name);
 
     //Do not allow editing of auto-increment fields:
     sharedptr<const Field> field_details = field->get_full_field_details();
@@ -116,42 +115,35 @@ void FlowTableWithFields::add_layout_item_at_position(const sharedptr<LayoutItem
     sharedptr<LayoutItem_Portal> portal = sharedptr<LayoutItem_Portal>::cast_dynamic(item);
     if(portal)
     {
-      add_layout_portal_at_position(portal, add_before);
+      add_layout_portal(portal);
     }
     else
     {
       sharedptr<LayoutItem_Notebook> notebook = sharedptr<LayoutItem_Notebook>::cast_dynamic(item);
       if(notebook)
       {
-        add_layout_notebook_at_position(notebook, add_before);
+        add_layout_notebook(notebook);
       }
       else
       {
         sharedptr<LayoutGroup> group = sharedptr<LayoutGroup>::cast_dynamic(item);
         if(group)
-          add_layout_group_at_position(group, add_before);
+          add_layout_group(group);
         else
         {
           sharedptr<LayoutItem_Button> layout_button = sharedptr<LayoutItem_Button>::cast_dynamic(item);
           if(layout_button)
-            add_button_at_position(layout_button, m_table_name, add_before);
+            add_button(layout_button, m_table_name);
           else
           {
             sharedptr<LayoutItem_Text> layout_textobject = sharedptr<LayoutItem_Text>::cast_dynamic(item);
             if(layout_textobject)
-              add_textobject_at_position(layout_textobject, m_table_name, add_before);
+              add_textobject(layout_textobject, m_table_name);
             else
             {
               sharedptr<LayoutItem_Image> layout_imageobject = sharedptr<LayoutItem_Image>::cast_dynamic(item);
               if(layout_imageobject)
-                add_imageobject_at_position(layout_imageobject, m_table_name, add_before);
-              else
-              {
-                sharedptr<LayoutItem_Placeholder> layout_placeholder =
-                  sharedptr<LayoutItem_Placeholder>::cast_dynamic(item);
-                if(layout_placeholder)
-                  add_placeholder_at_position(layout_placeholder, m_table_name, add_before);
-              }
+                add_imageobject(layout_imageobject, m_table_name);
             }
           }
         }
@@ -160,12 +152,7 @@ void FlowTableWithFields::add_layout_item_at_position(const sharedptr<LayoutItem
   }
 }
 
-void FlowTableWithFields::add_layout_group(const sharedptr<LayoutGroup>& group)
-{
-  add_layout_group_at_position(group, m_list_layoutwidgets.end());
-}
-
-void FlowTableWithFields::add_layout_group_at_position(const sharedptr<LayoutGroup>& group, const type_list_layoutwidgets::iterator& add_before)
+void FlowTableWithFields::add_layout_group(const sharedptr<LayoutGroup>& group, bool with_indent)
 {
   if(!group)
     return;
@@ -174,10 +161,11 @@ void FlowTableWithFields::add_layout_group_at_position(const sharedptr<LayoutGro
   {
     Gtk::Frame* frame = Gtk::manage( new Gtk::Frame ); //TODO_leak: This is possibly leaked, according to valgrind.
 
-    if(!group->get_title().empty())
+    const Glib::ustring group_title = item_get_title(group);
+    if(!group_title.empty())
     {
       Gtk::Label* label = Gtk::manage( new Gtk::Label ); //TODO: This is maybe leaked, according to valgrind, though it should be managed by GtkFrame.
-      label->set_markup( Utils::bold_message(group->get_title()) );
+      label->set_markup( Utils::bold_message(group_title) );
       label->show();
       frame->set_label_widget(*label);
     }
@@ -187,18 +175,28 @@ void FlowTableWithFields::add_layout_group_at_position(const sharedptr<LayoutGro
 
     Gtk::Alignment* alignment = Gtk::manage( new Gtk::Alignment ); //TODO_leak: This is possibly leaked, according to valgrind.
 
-    if(!group->get_title().empty()) //Don't indent if it has no title, to allow use of groups just for positioning.
+    if(!group_title.empty()) //Don't indent if it has no title, to allow use of groups just for positioning.
     {
-      alignment->set_padding(Glom::Utils::DEFAULT_SPACING_SMALL, 0, 6, 0); //Use left-padding of 6 even on Maemo because indentation is important.
-      #ifdef GLOM_ENABLE_MAEMO
-      std::cerr << "DEBUG: Unexpected group with title causing extra spacing on Maemo." << std::endl;
-      #endif
+      //Add some indenting just to avoid the out-denting caused by this GtkFrame bug:
+      //https://bugzilla.gnome.org/show_bug.cgi?id=644199
+      const int BASE_INDENT = 3;
+      
+      //std::cout << "title= " << group_title << ", with_indent=" << with_indent << std::endl;
+      if(with_indent) 
+      {
+        alignment->set_padding(Glom::Utils::DEFAULT_SPACING_SMALL, 0, Glom::Utils::DEFAULT_SPACING_SMALL + BASE_INDENT, 0);
+      }
+      else
+      {
+        alignment->set_padding(Glom::Utils::DEFAULT_SPACING_SMALL, 0, BASE_INDENT, 0);
+      }
     }
 
     alignment->show();
     frame->add(*alignment);
 
     FlowTableWithFields* flow_table = Gtk::manage( new FlowTableWithFields() );
+    flow_table->set_find_mode(m_find_mode);
     add_view(flow_table); //Allow these sub-flowtables to access the document too.
     flow_table->set_table(m_table_name);
 
@@ -230,17 +228,19 @@ void FlowTableWithFields::add_layout_group_at_position(const sharedptr<LayoutGro
       }
     }
 
-    add(*frame, true /* expand */);
+    add_widgets(*frame, true /* expand */);
 
     m_sub_flow_tables.push_back(flow_table);
     flow_table->set_layout_item(group, m_table_name);
-    add_layoutwidgetbase(flow_table, add_before);
+    add_layoutwidgetbase(flow_table);
+
+    //Connect signals:
+    signal_connect_for_reemit_2args(flow_table->signal_field_edited(), m_signal_field_edited);
+    signal_connect_for_reemit_1arg(flow_table->signal_field_choices_changed(), m_signal_field_choices_changed);
+    signal_connect_for_reemit_2args(flow_table->signal_field_open_details_requested(), m_signal_field_open_details_requested);
+    signal_connect_for_reemit_1arg(flow_table->signal_related_record_changed(), m_signal_related_record_changed);
+    signal_connect_for_reemit_2args(flow_table->signal_requested_related_details(), m_signal_requested_related_details);
 
-    //Connect signal:
-    flow_table->signal_field_edited().connect( sigc::mem_fun(*this, &FlowTableWithFields::on_flowtable_entry_edited) );
-    flow_table->signal_field_open_details_requested().connect( sigc::mem_fun(*this, &FlowTableWithFields::on_flowtable_entry_open_details_requested) );
-    flow_table->signal_related_record_changed().connect( sigc::mem_fun(*this, &FlowTableWithFields::on_flowtable_related_record_changed) );
-    flow_table->signal_requested_related_details().connect( sigc::mem_fun(*this, &FlowTableWithFields::on_flowtable_requested_related_details) );
     flow_table->signal_script_button_clicked().connect( sigc::mem_fun(*this, &FlowTableWithFields::on_script_button_clicked) );
 
     flow_table->align_child_group_labels();
@@ -256,6 +256,7 @@ Box_Data_List_Related* FlowTableWithFields::create_related(const sharedptr<Layou
   if(pDocument)
   {
     Box_Data_List_Related* portal_box = Gtk::manage(new Box_Data_List_Related);
+    portal_box->set_find_mode(m_find_mode);
     add_view(portal_box); //Give it access to the document, needed to get the layout and fields information.
 
     //Create the layout:
@@ -295,6 +296,7 @@ Box_Data_Calendar_Related* FlowTableWithFields::create_related_calendar(const sh
   if(pDocument)
   {
     Box_Data_Calendar_Related* portal_box = Gtk::manage(new Box_Data_Calendar_Related);
+    portal_box->set_find_mode(m_find_mode); //TODO: Implement this in the class
     add_view(portal_box); //Give it access to the document, needed to get the layout and fields information.
 
     //Create the layout:
@@ -325,7 +327,7 @@ Box_Data_Calendar_Related* FlowTableWithFields::create_related_calendar(const sh
   return 0;
 }
 
-void FlowTableWithFields::add_layout_portal_at_position(const sharedptr<LayoutItem_Portal>& portal, const type_list_layoutwidgets::iterator& add_before)
+void FlowTableWithFields::add_layout_portal(const sharedptr<LayoutItem_Portal>& portal)
 {
   Box_Data_Portal* portal_box = 0;
   sharedptr<LayoutItem_CalendarPortal> calendar_portal = sharedptr<LayoutItem_CalendarPortal>::cast_dynamic(portal);
@@ -336,14 +338,14 @@ void FlowTableWithFields::add_layout_portal_at_position(const sharedptr<LayoutIt
 
   if(portal_box)
   {
-    add(*portal_box, true /* expand */);
-    add_layoutwidgetbase(portal_box, add_before);
+    add_widgets(*portal_box, true /* expand */);
+    add_layoutwidgetbase(portal_box);
   }
   else
     std::cerr << G_STRFUNC << ": No portal was created." << std::endl;
 }
 
-void FlowTableWithFields::add_layout_notebook_at_position(const sharedptr<LayoutItem_Notebook>& notebook, const type_list_layoutwidgets::iterator& add_before)
+void FlowTableWithFields::add_layout_notebook(const sharedptr<LayoutItem_Notebook>& notebook)
 {
   if(!notebook)
     return;
@@ -366,29 +368,25 @@ void FlowTableWithFields::add_layout_notebook_at_position(const sharedptr<Layout
       Gtk::Label* tab_label = Gtk::manage(new Gtk::Label());
       tab_label->show();
 #endif
+
+      tab_label->set_label(item_get_title_or_name(group));
+
       sharedptr<LayoutItem_Portal> portal = sharedptr<LayoutItem_Portal>::cast_dynamic(group);
       if(portal)
       {
-        const Glib::ustring tab_title = glom_get_sharedptr_title_or_name(portal->get_relationship());
-        //tab_label->set_markup(Utils::bold_message(tab_title));
-        tab_label->set_label(tab_title);
-
         //Add a Related Records list for this portal:
         Box_Data_List_Related* portal_box = create_related(portal, false /* no label, because it's in the tab instead. */);
         //portal_box->set_border_width(Glom::Utils::DEFAULT_SPACING_SMALL); It has "padding" around the Alignment instead.
         portal_box->show();
         notebook_widget->append_page(*portal_box, *tab_label);
 
-        add_layoutwidgetbase(portal_box, add_before);
+        add_layoutwidgetbase(portal_box);
       }
       else
       {
-        const Glib::ustring tab_title = group->get_title_or_name();
-        //tab_label->set_markup(Utils::bold_message(tab_title));
-        tab_label->set_label(tab_title);
-
         //Add a FlowTable for this group:
         FlowTableWithFields* flow_table = Gtk::manage( new FlowTableWithFields() );
+        flow_table->set_find_mode(m_find_mode);
         add_view(flow_table); //Allow these sub-flowtables to access the document too.
         flow_table->set_table(m_table_name);
 
@@ -429,13 +427,14 @@ void FlowTableWithFields::add_layout_notebook_at_position(const sharedptr<Layout
 
         m_sub_flow_tables.push_back(flow_table);
         flow_table->set_layout_item(group, m_table_name);
-        add_layoutwidgetbase(flow_table, add_before);
+        add_layoutwidgetbase(flow_table);
 
         //Connect signal:
-        flow_table->signal_field_edited().connect( sigc::mem_fun(*this, &FlowTableWithFields::on_flowtable_entry_edited) );
-        flow_table->signal_field_open_details_requested().connect( sigc::mem_fun(*this, &FlowTableWithFields::on_flowtable_entry_open_details_requested) );
-        flow_table->signal_related_record_changed().connect( sigc::mem_fun(*this, &FlowTableWithFields::on_flowtable_related_record_changed) );
-        flow_table->signal_requested_related_details().connect( sigc::mem_fun(*this, &FlowTableWithFields::on_flowtable_requested_related_details) );
+        signal_connect_for_reemit_2args(flow_table->signal_field_edited(), m_signal_field_edited);
+        signal_connect_for_reemit_2args(flow_table->signal_field_open_details_requested(), m_signal_field_open_details_requested);
+        signal_connect_for_reemit_1arg(flow_table->signal_related_record_changed(), signal_related_record_changed());
+        signal_connect_for_reemit_2args(flow_table->signal_requested_related_details(), signal_requested_related_details());
+
 
         flow_table->signal_script_button_clicked().connect( sigc::mem_fun(*this, &FlowTableWithFields::on_script_button_clicked) );
       }
@@ -443,13 +442,9 @@ void FlowTableWithFields::add_layout_notebook_at_position(const sharedptr<Layout
     }
   }
 
-  add_layoutwidgetbase(notebook_widget, add_before);
+  add_layoutwidgetbase(notebook_widget);
   //add_view(button); //So it can get the document.
-  Gtk::Widget* widget = dynamic_cast<Gtk::Widget*>(*add_before);
-  if(widget)
-    insert_before (*notebook_widget, *widget, true /* expand */);
-  else
-    add(*notebook_widget, true /* expand */);
+  add_widgets(*notebook_widget, true /* expand */);
 }
 
 /*
@@ -495,7 +490,7 @@ void FlowTableWithFields::add_group(const Glib::ustring& group_name, const Glib:
       flow_table->add_field(*iter);
     }
 
-    add(*frame);
+    add_widgets(*frame);
 
     m_sub_flow_tables.push_back(flow_table);
 
@@ -506,18 +501,13 @@ void FlowTableWithFields::add_group(const Glib::ustring& group_name, const Glib:
 */
 
 void FlowTableWithFields::add_field(const sharedptr<LayoutItem_Field>& layoutitem_field, const Glib::ustring& table_name)
-{
-  add_field_at_position(layoutitem_field, table_name, m_list_layoutwidgets.end());
-}
-
-void FlowTableWithFields::add_field_at_position(const sharedptr<LayoutItem_Field>& layoutitem_field, const Glib::ustring& table_name, const type_list_layoutwidgets::iterator& add_before)
 {
   Info info;
   info.m_field = layoutitem_field;
 
   //Add the entry or checkbox (handled by the DataWidget)
   DataWidget* pDataWidget = Gtk::manage(new DataWidget(layoutitem_field, table_name, get_document()) ); //TODO_Leak: Possibly leaked, according to valgrind.
-  add_layoutwidgetbase(pDataWidget, add_before);
+  add_layoutwidgetbase(pDataWidget);
   add_view(pDataWidget); //So it can get the document.
 
   info.m_second = pDataWidget;
@@ -555,14 +545,12 @@ void FlowTableWithFields::add_field_at_position(const sharedptr<LayoutItem_Field
   eventbox->set_events(Gdk::ALL_EVENTS_MASK);
   eventbox->show_all();
 
-  Gtk::Widget* widget = dynamic_cast<Gtk::Widget*>(*add_before);
-  if(widget)
-    insert_before(*eventbox, *(info.m_second), *widget, true);
-  else
-    add(*eventbox, *(info.m_second), true);
+  add_widgets(*eventbox, *(info.m_second), true);
 
   info.m_second->signal_edited().connect( sigc::bind(sigc::mem_fun(*this, &FlowTableWithFields::on_entry_edited), layoutitem_field)  ); //TODO:  Is it a good idea to bind the LayoutItem? sigc::bind() probably stores a copy at this point.
 
+  info.m_second->signal_choices_changed().connect( sigc::bind(sigc::mem_fun(*this, &FlowTableWithFields::on_entry_choices_changed), layoutitem_field)  );
+
 #ifndef GLOM_ENABLE_CLIENT_ONLY
   info.m_second->signal_layout_item_added().connect( sigc::bind(
     sigc::mem_fun(*this, &FlowTableWithFields::on_datawidget_layout_item_added), info.m_second) );
@@ -574,11 +562,11 @@ void FlowTableWithFields::add_field_at_position(const sharedptr<LayoutItem_Field
 }
 
 
-void FlowTableWithFields::add_button_at_position(const sharedptr<LayoutItem_Button>& layoutitem_button, const Glib::ustring& table_name, const type_list_layoutwidgets::iterator& add_before)
+void FlowTableWithFields::add_button(const sharedptr<LayoutItem_Button>& layoutitem_button, const Glib::ustring& table_name)
 {
   //Add the widget
   ButtonGlom* button = Gtk::manage(new ButtonGlom());
-  button->set_label(layoutitem_button->get_title_or_name());
+  button->set_label(item_get_title_or_name(layoutitem_button));
   button->set_layout_item(layoutitem_button, table_name);
   button->signal_clicked().connect(
     sigc::bind(
@@ -587,21 +575,21 @@ void FlowTableWithFields::add_button_at_position(const sharedptr<LayoutItem_Butt
 
   button->show();
 
-  add_layoutwidgetbase(button, add_before);
+  add_layoutwidgetbase(button);
   //add_view(button); //So it can get the document.
 
-  const FieldFormatting::HorizontalAlignment alignment =
+  const Formatting::HorizontalAlignment alignment =
     layoutitem_button->get_formatting_used_horizontal_alignment();
   Gtk::Widget* widget_to_add = button;
   bool expand = false;
-  if(alignment != FieldFormatting::HORIZONTAL_ALIGNMENT_LEFT)
+  if(alignment != Formatting::HORIZONTAL_ALIGNMENT_LEFT)
   {
-    //Put the button in a Gtk::HBox so we can have non-default alignment in
+    //Put the button in a Gtk::Box so we can have non-default alignment in
     //its space. Note that we will need a different technique if we ever
     //support center alignment.
-    Gtk::HBox* box_button = Gtk::manage(new Gtk::HBox());
+    Gtk::Box* box_button = Gtk::manage(new Gtk::Box(Gtk::ORIENTATION_HORIZONTAL));
     box_button->show();
-    if(alignment == FieldFormatting::HORIZONTAL_ALIGNMENT_RIGHT)
+    if(alignment == Formatting::HORIZONTAL_ALIGNMENT_RIGHT)
       box_button->pack_end(*button, Gtk::PACK_SHRINK);
     else
       box_button->pack_start(*button, Gtk::PACK_SHRINK);
@@ -610,27 +598,23 @@ void FlowTableWithFields::add_button_at_position(const sharedptr<LayoutItem_Butt
     expand = true;
   }
 
-  Gtk::Widget* widget = dynamic_cast<Gtk::Widget*>(*add_before);
-  if(widget)
-    insert_before(*widget_to_add, *widget, expand);
-  else
-    add(*widget_to_add, expand);
+  add_widgets(*widget_to_add, expand);
 
   apply_formatting(*button, layoutitem_button);
 }
 
-void FlowTableWithFields::add_textobject_at_position(const sharedptr<LayoutItem_Text>& layoutitem_text, const Glib::ustring& table_name , const type_list_layoutwidgets::iterator& add_before)
+void FlowTableWithFields::add_textobject(const sharedptr<LayoutItem_Text>& layoutitem_text, const Glib::ustring& table_name)
 {
   //Add the widget:
 
-  const FieldFormatting::HorizontalAlignment alignment =
+  const Formatting::HorizontalAlignment alignment =
     layoutitem_text->get_formatting_used_horizontal_alignment();
-  const Gtk::Align x_align = (alignment == FieldFormatting::HORIZONTAL_ALIGNMENT_LEFT ? Gtk::ALIGN_START : Gtk::ALIGN_END);
+  const Gtk::Align x_align = (alignment == Formatting::HORIZONTAL_ALIGNMENT_LEFT ? Gtk::ALIGN_START : Gtk::ALIGN_END);
   Gtk::Alignment* alignment_label = Gtk::manage(new Gtk::Alignment());
   alignment_label->set(x_align, Gtk::ALIGN_CENTER);
   alignment_label->show();
 
-  const Glib::ustring text = layoutitem_text->get_text();
+  const Glib::ustring text = layoutitem_text->get_text(AppWindow::get_current_locale());
   DataWidgetChildren::Label* label = Gtk::manage(new DataWidgetChildren::Label(text));
   label->set_layout_item(layoutitem_text, table_name);
   label->show();
@@ -638,16 +622,12 @@ void FlowTableWithFields::add_textobject_at_position(const sharedptr<LayoutItem_
 
   apply_formatting(*label, layoutitem_text);
 
-  add_layoutwidgetbase(label, add_before);
+  add_layoutwidgetbase(label);
 
-  const Glib::ustring title = layoutitem_text->get_title();
+  const Glib::ustring title = item_get_title(layoutitem_text);
   if(title.empty())
   {
-    Gtk::Widget* widget = dynamic_cast<Gtk::Widget*>(*add_before);
-    if(widget)
-      insert_before(*alignment_label, *widget, true /* expand */);
-    else
-      add(*alignment_label, true /* expand */);
+    add_widgets(*alignment_label, true /* expand */);
   }
   else
   {
@@ -659,41 +639,13 @@ void FlowTableWithFields::add_textobject_at_position(const sharedptr<LayoutItem_
     title_label->set_layout_item(layoutitem_text, table_name);
     title_label->show();
     alignment_title->add(*title_label);
-    add_layoutwidgetbase(title_label, add_before);
+    add_layoutwidgetbase(title_label);
 
-    Gtk::Widget* widget = dynamic_cast<Gtk::Widget*>(*add_before);
-    if(widget)
-      insert_before (*alignment_title, *alignment_label, *widget, true /* expand */);
-    else
-      add(*alignment_title, *alignment_label, true /* expand */);
+    add_widgets(*alignment_title, *alignment_label, true /* expand */);
   }
 }
 
-void FlowTableWithFields::add_placeholder_at_position(const sharedptr<LayoutItem_Placeholder>& /* layoutitem_placeholder */, const Glib::ustring& /* table_name */, const type_list_layoutwidgets::iterator& add_before)
-{
-  //Delete any existing placeholder (there can be only one):
-  delete m_placeholder;
-  m_placeholder = 0;
-
-  //Add the widget:
-  m_placeholder = Gtk::manage(new Gtk::Alignment());
-  m_placeholder->set(Gtk::ALIGN_START, Gtk::ALIGN_CENTER);
-  m_placeholder->show();
-
-  PlaceholderGlom* preview = Gtk::manage(new PlaceholderGlom);
-  preview->show();
-
-  m_placeholder->add(*preview);
-
-  m_list_layoutwidgets.insert(add_before, preview);
-  Gtk::Widget* widget = dynamic_cast<Gtk::Widget*>(*add_before);
-  if(widget)
-    insert_before(*m_placeholder, *widget, false /* expand */);
-  else
-    add(*m_placeholder, false);
-}
-
-void FlowTableWithFields::add_imageobject_at_position(const sharedptr<LayoutItem_Image>& layoutitem_image, const Glib::ustring& table_name , const type_list_layoutwidgets::iterator& add_before)
+void FlowTableWithFields::add_imageobject(const sharedptr<LayoutItem_Image>& layoutitem_image, const Glib::ustring& table_name)
 {
   //Add the widget:
   ImageGlom* image = Gtk::manage(new ImageGlom());
@@ -703,17 +655,13 @@ void FlowTableWithFields::add_imageobject_at_position(const sharedptr<LayoutItem
   image->set_layout_item(layoutitem_image, table_name);
   image->show();
 
-  add_layoutwidgetbase(image, add_before);
+  add_layoutwidgetbase(image);
   //add_view(button); //So it can get the document.
 
-  const Glib::ustring title = layoutitem_image->get_title();
+  const Glib::ustring title = item_get_title(layoutitem_image);
   if(title.empty())
   {
-    Gtk::Widget* widget = dynamic_cast<Gtk::Widget*>(*add_before);
-    if(widget)
-      insert_before(*image, *widget, true /* expand */);
-    else
-      add(*image, true /* expand */);
+    add_widgets(*image, true /* expand */);
   }
   else
   {
@@ -724,11 +672,7 @@ void FlowTableWithFields::add_imageobject_at_position(const sharedptr<LayoutItem
     Gtk::Label* title_label = Gtk::manage(new Gtk::Label(title));
     title_label->show();
     alignment_title->add(*title_label);
-    Gtk::Widget* widget = dynamic_cast<Gtk::Widget*>(*add_before);
-    if(widget)
-      insert_before(*alignment_title, *image, *widget, true /* expand */);
-    else
-      add(*alignment_title, *image, true /* expand */);
+    add_widgets(*alignment_title, *image, true /* expand */);
   }
 }
 
@@ -798,7 +742,7 @@ void FlowTableWithFields::set_field_value(const sharedptr<const LayoutItem_Field
     Box_Data_Portal* portal = *iter;
     if(portal)
     {
-      //g_warning("FlowTableWithFields::set_field_value: foreign_key_value=%s", value.to_string().c_str());
+      //std::cerr << G_STRFUNC << ": foreign_key_value=" << value.to_string() << std::endl;
       portal->refresh_data_from_database_with_foreign_key(value /* foreign key value */);
     }
   }
@@ -810,7 +754,7 @@ void FlowTableWithFields::set_field_value(const sharedptr<const LayoutItem_Field
     DataWidgetChildren::ComboChoices* widget = *iter;
     if(widget)
     {
-      //g_warning("FlowTableWithFields::set_field_value: foreign_key_value=%s", value.to_string().c_str());
+      //std::cerr << G_STRFUNC << ": foreign_key_value=" << value.to_string() << std::endl;
       widget->refresh_data_from_database_with_foreign_key(get_document(), value /* foreign key value */);
     }
   }
@@ -824,15 +768,19 @@ void FlowTableWithFields::set_other_field_value(const sharedptr<const LayoutItem
 Gnome::Gda::Value FlowTableWithFields::get_field_value(const sharedptr<const LayoutItem_Field>& field) const
 {
   type_list_const_widgets list_widgets = get_field(field, true);
-  if(!list_widgets.empty())
+  for(type_list_const_widgets::const_iterator iter = list_widgets.begin();
+    iter != list_widgets.end(); ++iter)
   {
-    const DataWidget* datawidget = dynamic_cast<const DataWidget*>(*(list_widgets.begin()));
+    const DataWidget* datawidget = dynamic_cast<const DataWidget*>(*iter);
+    if(!datawidget)
+      continue;
 
-    if(datawidget)
-      return datawidget->get_value();
+    const Gnome::Gda::Value value = datawidget->get_value();
+    if(!Conversions::value_is_empty(value))
+      return value;
   }
 
-  //g_warning("FlowTableWithFields::get_field_value(): returning null");
+  //std::cerr << G_STRFUNC << ": returning null" << std::endl;
   return Gnome::Gda::Value(); //null.
 }
 
@@ -849,6 +797,33 @@ void FlowTableWithFields::set_field_editable(const sharedptr<const LayoutItem_Fi
   }
 }
 
+void FlowTableWithFields::update_choices(const sharedptr<const LayoutItem_Field>& field)
+{
+  type_list_widgets list_widgets = get_field(field, true);
+  for(type_list_widgets::iterator iter = list_widgets.begin(); iter != list_widgets.end(); ++iter)
+  {
+    DataWidget* datawidget = dynamic_cast<DataWidget*>(*iter);
+    if(!datawidget)
+      continue;
+
+    DataWidgetChildren::ComboChoices* combo = 
+      dynamic_cast<DataWidgetChildren::ComboChoices*>(datawidget->get_data_child_widget());
+    if(!combo)
+      continue;
+
+    const sharedptr<const LayoutItem> layout_item = combo->get_layout_item();
+    const sharedptr<const LayoutItem_Field> layout_item_field = 
+      sharedptr<const LayoutItem_Field>::cast_dynamic(layout_item);
+    if(!layout_item_field || !layout_item_field->get_formatting_used().get_has_related_choices())
+      continue;
+
+    //TODO: Handle not-all related choices, too.
+    combo->set_choices_related(get_document(), field, Gnome::Gda::Value() /* no ID means show all related records */);
+  }
+
+  //TODO: See also "Refresh choices widgets which should show the related records for relationships that use this field"
+}
+
 
 FlowTableWithFields::type_portals FlowTableWithFields::get_portals(const sharedptr<const LayoutItem_Field>& from_key)
 {
@@ -857,7 +832,7 @@ FlowTableWithFields::type_portals FlowTableWithFields::get_portals(const sharedp
   const Glib::ustring from_key_name = from_key->get_name();
 
   //Check the single-item widgets:
-   for(type_portals::const_iterator iter = m_portals.begin(); iter != m_portals.end(); ++iter)
+  for(type_portals::const_iterator iter = m_portals.begin(); iter != m_portals.end(); ++iter)
   {
     //*iter is a FlowTableItem.
     Box_Data_Portal* pPortalUI = *iter;
@@ -922,7 +897,7 @@ FlowTableWithFields::type_choice_widgets FlowTableWithFields::get_choice_widgets
     if(!field)
       continue;
 
-    const FieldFormatting& format = field->get_formatting_used();
+    const Formatting& format = field->get_formatting_used();
 
     bool choice_show_all = false;
     const sharedptr<const Relationship> choice_relationship =
@@ -1077,6 +1052,11 @@ FlowTableWithFields::type_signal_field_edited FlowTableWithFields::signal_field_
   return m_signal_field_edited;
 }
 
+FlowTableWithFields::type_signal_field_choices_changed FlowTableWithFields::signal_field_choices_changed()
+{
+  return m_signal_field_choices_changed;
+}
+
 FlowTableWithFields::type_signal_field_open_details_requested FlowTableWithFields::signal_field_open_details_requested()
 {
   return m_signal_field_open_details_requested;
@@ -1107,21 +1087,16 @@ void FlowTableWithFields::on_entry_edited(const Gnome::Gda::Value& value, const
   m_signal_field_edited.emit(field, value);
 }
 
+void FlowTableWithFields::on_entry_choices_changed(const sharedptr<const LayoutItem_Field> field)
+{
+  m_signal_field_choices_changed.emit(field);
+}
+
 void FlowTableWithFields::on_entry_open_details_requested(const Gnome::Gda::Value& value, const sharedptr<const LayoutItem_Field> field)
 {
   m_signal_field_open_details_requested.emit(field, value);
 }
 
-void FlowTableWithFields::on_flowtable_entry_edited(const sharedptr<const LayoutItem_Field>& field, const Gnome::Gda::Value& value)
-{
-  m_signal_field_edited.emit(field, value);
-}
-
-void FlowTableWithFields::on_flowtable_entry_open_details_requested(const sharedptr<const LayoutItem_Field>& field, const Gnome::Gda::Value& value)
-{
-  m_signal_field_open_details_requested.emit(field, value);
-}
-
 void FlowTableWithFields::set_design_mode(bool value)
 {
 #ifndef GLOM_ENABLE_CLIENT_ONLY
@@ -1141,28 +1116,14 @@ void FlowTableWithFields::set_design_mode(bool value)
 
 void FlowTableWithFields::add_layoutwidgetbase(LayoutWidgetBase* layout_widget)
 {
-  add_layoutwidgetbase(layout_widget, m_list_layoutwidgets.end());
-}
-
-void FlowTableWithFields::add_layoutwidgetbase(LayoutWidgetBase* layout_widget, const type_list_layoutwidgets::iterator& add_before)
-{
-  m_list_layoutwidgets.insert(add_before, layout_widget);
+  m_list_layoutwidgets.push_back(layout_widget);
 
   //Handle layout_changed signal:
 #ifndef GLOM_ENABLE_CLIENT_ONLY
-  layout_widget->signal_layout_changed().connect(sigc::mem_fun(*this, &FlowTableWithFields::on_layoutwidget_changed));
+  layout_widget->signal_layout_changed().connect(signal_layout_changed().make_slot());
 #endif // !GLOM_ENABLE_CLIENT_ONLY
 }
 
-#ifndef GLOM_ENABLE_CLIENT_ONLY
-void FlowTableWithFields::on_layoutwidget_changed()
-{
-  //Forward the signal to the container,
-  //so it can make sure that the change is saved in the document.
-  signal_layout_changed().emit();
-}
-#endif // !GLOM_ENABLE_CLIENT_ONLY
-
 #ifndef GLOM_ENABLE_CLIENT_ONLY
 void FlowTableWithFields::on_datawidget_layout_item_added(LayoutWidgetBase::enumType item_type, DataWidget* pDataWidget)
 {
@@ -1202,7 +1163,7 @@ void FlowTableWithFields::on_datawidget_layout_item_added(LayoutWidgetBase::enum
   else if(item_type == LayoutWidgetBase::TYPE_GROUP)
   {
     sharedptr<LayoutGroup> layout_item = sharedptr<LayoutGroup>::create();
-    layout_item->set_title(_("New Group"));
+    layout_item->set_title_original(_("New Group"));
     layout_item_new = layout_item;
   }
   else if(item_type == LayoutWidgetBase::TYPE_NOTEBOOK)
@@ -1217,7 +1178,7 @@ void FlowTableWithFields::on_datawidget_layout_item_added(LayoutWidgetBase::enum
     group_tab->set_name(_("tab1"));
 
     //Note to translators: This is the default label text for a notebook tab.
-    group_tab->set_title(_("Tab One"));
+    group_tab->set_title_original(_("Tab One"));
 
     layout_item->add_item(group_tab);
 
@@ -1231,14 +1192,14 @@ void FlowTableWithFields::on_datawidget_layout_item_added(LayoutWidgetBase::enum
   {
     sharedptr<LayoutItem_Button> layout_item = sharedptr<LayoutItem_Button>::create();
     layout_item->set_name(_("button"));
-    layout_item->set_title(_("New Button"));
+    layout_item->set_title_original(_("New Button"));
     layout_item_new = layout_item;
   }
   else if(item_type == LayoutWidgetBase::TYPE_TEXT)
   {
     sharedptr<LayoutItem_Text> layout_item = sharedptr<LayoutItem_Text>::create();
     layout_item->set_name(_("text"));
-    layout_item->set_text(_("New Text"));
+    layout_item->set_text_original(_("New Text"));
     layout_item_new = layout_item;
   }
 
@@ -1254,14 +1215,7 @@ void FlowTableWithFields::on_datawidget_layout_item_added(LayoutWidgetBase::enum
 }
 #endif // !GLOM_ENABLE_CLIENT_ONLY
 
-void FlowTableWithFields::on_flowtable_related_record_changed(const Glib::ustring& relationship_name)
-{
-  //TODO_DoublyRelated
-
-  //Forward it to the parent:
-  signal_related_record_changed().emit(relationship_name);
-}
-
+//TODO: Use Value by const &
 void FlowTableWithFields::on_portal_user_requested_details(Gnome::Gda::Value primary_key_value, Box_Data_Portal* portal_box)
 {
   sharedptr<const LayoutItem_Portal> portal = portal_box->get_portal();
@@ -1278,13 +1232,6 @@ void FlowTableWithFields::on_portal_user_requested_details(Gnome::Gda::Value pri
     signal_requested_related_details().emit(table_name, primary_key);
 }
 
-
-void FlowTableWithFields::on_flowtable_requested_related_details(const Glib::ustring& table_name, Gnome::Gda::Value primary_key_value)
-{
-  //Forward it to the parent:
-  signal_requested_related_details().emit(table_name, primary_key_value);
-}
-
 void FlowTableWithFields::apply_size_groups_to_labels(const type_vec_sizegroups& size_groups)
 {
   //Remove widgets from any existing size group:
@@ -1391,7 +1338,7 @@ void FlowTableWithFields::on_menu_properties_activate()
   {
     sharedptr<LayoutGroup> group = get_layout_group();
     group->set_columns_count( dialog->get_columns_count() );
-    group->set_title(dialog->get_title());
+    group->set_title(dialog->get_title(), AppWindow::get_current_locale());
     signal_layout_changed().emit();
   }
 
@@ -1402,11 +1349,11 @@ void FlowTableWithFields::on_menu_properties_activate()
 void FlowTableWithFields::on_menu_delete_activate()
 {
   Glib::ustring message;
-  if(!get_layout_item()->get_title().empty())
+  if(!item_get_title(get_layout_item()).empty())
   {
     //TODO: Use a real English sentence here?
     message = Glib::ustring::compose(_("Delete whole group \"%1\"?"),
-                                      get_layout_item()->get_title());
+                                      item_get_title(get_layout_item()));
   }
   else
   {
@@ -1428,11 +1375,11 @@ void FlowTableWithFields::on_menu_delete_activate()
 
 bool FlowTableWithFields::on_button_press_event(GdkEventButton *event)
 {
-  Application* pApp = Application::get_application();
+  AppWindow* pApp = AppWindow::get_appwindow();
   if(pApp && pApp->get_userlevel() == AppState::USERLEVEL_DEVELOPER)
   {
     GdkModifierType mods;
-    gdk_window_get_pointer( gtk_widget_get_window (Gtk::Widget::gobj()), 0, 0, &mods );
+    gdk_window_get_device_position( gtk_widget_get_window (Gtk::Widget::gobj()), event->device, 0, 0, &mods );
     if(mods & GDK_BUTTON3_MASK)
     {
       //Give user choices of actions on this item:
@@ -1473,7 +1420,53 @@ sharedptr<LayoutItem_Portal> FlowTableWithFields::get_portal_relationship()
   return sharedptr<LayoutItem_Portal>();
 }
 
+void FlowTableWithFields::set_find_mode(bool val)
+{
+  m_find_mode = val;
 
+  //Set find mode in all portals:
+  for(type_portals::const_iterator iter = m_portals.begin(); iter != m_portals.end(); ++iter)
+  {
+    //*iter is a FlowTableItem.
+    Box_Data_Portal* portal = *iter;
+    if(portal)
+      portal->set_find_mode(m_find_mode);
+  }
+
+  //Set find mode in all the child flowtables, recursively:
+  for(type_sub_flow_tables::iterator iter = m_sub_flow_tables.begin(); iter != m_sub_flow_tables.end(); ++iter)
+  {
+    FlowTableWithFields* subtable = *iter;
+    if(subtable)
+    {
+      subtable->set_find_mode(m_find_mode);
+    }
+  }
+}
+
+void FlowTableWithFields::set_enable_drag_and_drop(bool enabled)
+{
+  const EggDragEnableMode drag_mode = 
+    (enabled ? EGG_DRAG_FULL : EGG_DRAG_DISABLED);
+
+  //Only enable dragging of the sub-tables.
+  //Otherwise just the whole thing will be dragged,
+  //though there would be nowhere to drop it:
+  set_drag_enabled(EGG_DRAG_DISABLED);
+  set_drop_enabled(enabled);
+  
+  for(type_sub_flow_tables::iterator iter = m_sub_flow_tables.begin(); iter != m_sub_flow_tables.end(); ++iter)
+  {
+    FlowTableWithFields* child = *iter;
+    if(child)
+    {
+      //std::cout << G_STRFUNC << ": child" << std::endl;
+      child->set_drag_enabled(drag_mode);
+      child->set_drop_enabled(enabled);  
+    }
+  }
+}
+  
 #endif // !GLOM_ENABLE_CLIENT_ONLY
 
 } //namespace Glom
