@@ -14,57 +14,62 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #include <glom/mode_data/notebook_data.h>
+#include <glom/signal_reemitter.h>
 #include <libglom/data_structure/glomconversions.h>
+#include <glibmm/main.h>
 #include <glibmm/i18n.h>
 
 #include <iostream>
 
-#ifdef GLOM_ENABLE_MAEMO
-#include <hildonmm/program.h>
-#endif
-
 namespace Glom
 {
 
+static const gchar gNotebookCss[] = "gtkmm__GtkNotebook#glomnotebook { padding: 0 0 0 0; }";
+
+
 Notebook_Data::Notebook_Data()
 :
-  #ifdef GLOM_ENABLE_MAEMO
-  m_window_maemo_details(0),
-  #endif
   m_iPage_Details(0), m_iPage_List(0)
 {
+
+  //Hide the GtkNotebook border:
+  set_name("glomnotebook");
+
+  GtkStyleContext *style_context;
+  GtkCssProvider *provider;
+  GError *error = NULL;
+
+  style_context = gtk_widget_get_style_context(GTK_WIDGET(gobj()));
+  provider = gtk_css_provider_new();
+  if (!gtk_css_provider_load_from_data(provider, gNotebookCss, -1,
+                                        &error)) {
+    g_error("%s", error->message);
+    g_error_free(error);
+    return;
+  }
+  gtk_style_context_add_provider(style_context,
+                                  GTK_STYLE_PROVIDER(provider),
+                                  GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+  g_object_unref(provider);
+   
+   
   //Add Pages:
+  //Translators: This is a noun. It is a notebook tab title.
   append_page(m_Box_List, _("List"));
   m_iPage_List = 0;
 
-  #ifndef GLOM_ENABLE_MAEMO
+  //Translators: This is a noun. It is a notebook tab title.
   append_page(m_Box_Details, _("Details"));
   m_iPage_Details = 1;
-  #else
-  //On Maemo, we add the box to m_window_maemo_details instead:
-  m_window_maemo_details = new Window_BoxHolder(&m_Box_Details, _("Details"));
-
-  //Let this window have the main AppMenu:
-  Hildon::Program::get_instance()->add_window(*m_window_maemo_details);
-
-  Gtk::Window* pWindow = get_app_window();
-  if(pWindow)
-    m_window_maemo_details->set_transient_for(*pWindow);
-
-  //Refresh the list when the details window is closed:
-  m_window_maemo_details->signal_hide().connect(
-    sigc::mem_fun(*this, &Notebook_Data::on_window_maemo_details_closed));
-
-  m_Box_Details.show_all();
-  #endif //GLOM_ENABLE_MAEMO
 
   // Set accessible name for the notebook, to be able to access it via LDTP
 #ifdef GTKMM_ATKMM_ENABLED
+  //Translators: This is a title, not an action.
   get_accessible()->set_name(_("List Or Details View"));
 #endif
 
@@ -72,10 +77,13 @@ Notebook_Data::Notebook_Data()
   //Connect signals:
 
   //Allow List to ask Details to show a record.
-  m_Box_List.signal_user_requested_details().connect(sigc::mem_fun(*this, &Notebook_Data::on_list_user_requested_details));
+  m_Box_List.signal_user_requested_details().connect(sigc::mem_fun(*this,
+    &Notebook_Data::on_list_user_requested_details));
+  
+  //Allow the parent widget to detect list selection changes:
+  m_Box_List.signal_record_selection_changed().connect(m_signal_record_selection_changed.make_slot());
 
   //Allow Details to ask List to ask Details to show a different record:
-  #ifndef GLOM_ENABLE_MAEMO //These navigation buttons are not visible on Maemo.
   m_Box_Details.signal_nav_first().connect(sigc::mem_fun(m_Box_List, &Box_Data_List::on_details_nav_first));
   m_Box_Details.signal_nav_prev().connect(sigc::mem_fun(m_Box_List, &Box_Data_List::on_details_nav_previous));
   m_Box_Details.signal_nav_next().connect(sigc::mem_fun(m_Box_List, &Box_Data_List::on_details_nav_next));
@@ -83,10 +91,10 @@ Notebook_Data::Notebook_Data()
 
   //Allow Details to tell List about record deletion:
   m_Box_Details.signal_record_deleted().connect(sigc::mem_fun(m_Box_List, &Box_Data_List::on_details_record_deleted));
-  #endif //GLOM_ENABLE_MAEMO
 
   //Allow Details to ask to show a different record in a different table:
-  m_Box_Details.signal_requested_related_details().connect(sigc::mem_fun(*this, &Notebook_Data::on_details_user_requested_related_details));
+  signal_connect_for_reemit_2args(m_Box_Details.signal_requested_related_details(), m_signal_record_details_requested);
+  
 
   //Fill composite view:
   add_view(&m_Box_List);
@@ -98,34 +106,14 @@ Notebook_Data::Notebook_Data()
   //This is hidden by default,
   m_Box_Details.show_layout_toolbar(false);
 #endif //GLOM_ENABLE_CLIENT_ONLY
-
-#ifdef GLOM_ENABLE_MAEMO
-  //On Maemo we show the details in a separate window.
-  set_show_tabs(false);
-#endif //GLOM_ENABLE_MAEMO
 }
 
 Notebook_Data::~Notebook_Data()
 {
   remove_view(&m_Box_List);
   remove_view(&m_Box_Details);
-
-#ifdef GLOM_ENABLE_MAEMO
-  if(m_window_maemo_details)
-    delete m_window_maemo_details;
-#endif //GLOM_ENABLE_MAEMO
 }
 
-#ifdef GLOM_ENABLE_MAEMO
-void Notebook_Data::on_window_maemo_details_closed()
-{
-  //Show the changed record in the list:
-  const Gnome::Gda::Value primary_key_selected = m_Box_List.get_primary_key_value_selected();
-  m_Box_List.refresh_data_from_database();
-  m_Box_List.set_primary_key_value_selected(primary_key_selected);
-}
-#endif //GLOM_ENABLE_MAEMO
-
 bool Notebook_Data::init_db_details(const FoundSet& found_set, const Gnome::Gda::Value& primary_key_value_for_details)
 {
   m_table_name = found_set.m_table_name;
@@ -139,13 +127,7 @@ bool Notebook_Data::init_db_details(const FoundSet& found_set, const Gnome::Gda:
   //Performance optimisation:
   //Keep the connection open during all these operations:
   {
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
     sharedptr<SharedConnection> sharedconnection = connect_to_server(get_app_window());
-#else
-    std::auto_ptr<ExceptionConnection> error;
-    sharedptr<SharedConnection> sharedconnection = connect_to_server(get_app_window(), error);
-    // Ignore error, sharedconnection is not used directly within this function
-#endif
 
     result = m_Box_List.init_db_details(found_set, get_active_layout_platform(get_document())); //TODO: Select the last selected record.
 
@@ -189,7 +171,7 @@ bool Notebook_Data::init_db_details(const FoundSet& found_set, const Gnome::Gda:
       m_Box_Details.init_db_details(found_set, get_active_layout_platform(get_document()), primary_key_for_details);
     }
     else
-      std::cerr << "Notebook_Data::init_db_details(): document is NULL" << std::endl;
+      std::cerr << G_STRFUNC << ": document is NULL" << std::endl;
   }
 
 
@@ -244,22 +226,8 @@ void Notebook_Data::show_details(const Gnome::Gda::Value& primary_key_value)
   //std::cout << "DEBUG: Notebook_Data::show_details() primary_key_value=" << primary_key_value.to_string() << std::endl;
   m_Box_Details.refresh_data_from_database_with_primary_key(primary_key_value);
 
-#if GLOM_ENABLE_MAEMO
-  //Details are shown in a separate window on Maemo,
-  //though that window contains the regular m_Box_Details.
-  //TODO: Use the singular form when it's available from the document.
-  Document* document = get_document();
-  g_assert(document);
-  const Glib::ustring title =
-    Glib::ustring::compose(_("%1 Details"),
-      document->get_table_title_singular(m_table_name));
-  m_window_maemo_details->set_title(title);
-
-  m_window_maemo_details->show();
-#else
   if(get_current_view() != DATA_VIEW_Details)
     set_current_view(DATA_VIEW_Details);
-#endif
 
   //Re-enable this handler, so we can respond to notebook page changes:
   if(m_connection_switch_page)
@@ -283,22 +251,41 @@ void Notebook_Data::on_list_user_requested_details(const Gnome::Gda::Value& prim
       primary_key_value));
 }
 
-void Notebook_Data::on_details_user_requested_related_details(const Glib::ustring& table_name, Gnome::Gda::Value primary_key_value)
+FoundSet Notebook_Data::get_found_set_selected() const
 {
-  signal_record_details_requested().emit(table_name, primary_key_value);
+  if(get_current_view() == DATA_VIEW_Details)
+  {
+    return m_Box_Details.get_found_set();
+  }
+  else
+  {
+    //Start with something sensible:
+    FoundSet found_set = m_Box_List.get_found_set();
+    
+    const Gnome::Gda::Value primary_key_value_selected = 
+      m_Box_List.get_primary_key_value_selected();
+    if(Conversions::value_is_empty(primary_key_value_selected))
+    {
+      //Indicate to the caller that no record is selected:
+      found_set.m_where_clause = Gnome::Gda::SqlExpr();
+      return found_set;
+    }
 
-  /*
-  //Show a different table:
-  init_db_details(m_table_name);
-
-  //Show the specific record:
-  on_list_user_requested_details(primary_key_value);
-  */
-}
-
-FoundSet Notebook_Data::get_found_set_details() const
-{
-  return m_Box_Details.get_found_set();
+    const Document* document = get_document();
+    if(!document)
+    {
+      std::cerr << G_STRFUNC << ": document is null" << std::endl;
+      found_set.m_where_clause = Gnome::Gda::SqlExpr();
+      return found_set;
+    }
+    
+    sharedptr<Field> primary_key_field =
+      document->get_field_primary_key(m_table_name);
+    found_set.m_where_clause = Utils::build_simple_where_expression(
+      m_table_name, primary_key_field,
+      primary_key_value_selected);
+    return found_set;
+  }
 }
 
 
@@ -336,16 +323,17 @@ void Notebook_Data::do_menu_developer_layout()
   }
 }
 
-void Notebook_Data::show_layout_toolbar(bool show)
+void Notebook_Data::set_enable_layout_drag_and_drop(bool enable)
 {
-  m_Box_Details.show_layout_toolbar(show);
+  m_Box_Details.show_layout_toolbar(enable);
+  m_Box_Details.set_enable_drag_and_drop(enable);
 }
 
 #endif // !GLOM_ENABLE_CLIENT_ONLY
 
 void Notebook_Data::do_menu_file_print()
 {
-  int iPageCurrent = get_current_page();
+  const int iPageCurrent = get_current_page();
 
   Gtk::Widget* pChild  = get_nth_page(iPageCurrent);
   if(pChild)
@@ -378,7 +366,12 @@ Notebook_Data::type_signal_record_details_requested Notebook_Data::signal_record
   return m_signal_record_details_requested;
 }
 
-void Notebook_Data::on_switch_page_handler(GtkNotebookPage* pPage, guint uiPageNumber)
+Notebook_Data::type_signal_record_selection_changed Notebook_Data::signal_record_selection_changed()
+{
+  return m_signal_record_selection_changed;
+}
+
+void Notebook_Data::on_switch_page_handler(Gtk::Widget* pPage, guint uiPageNumber)
 {
   //Call base class:
   Notebook_Glom::on_switch_page_handler(pPage, uiPageNumber);
