@@ -14,27 +14,23 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
-#ifndef NOTEBOOK_DATA_H
-#define NOTEBOOK_DATA_H
+#ifndef GLOM_MODE_DATA_NOTEBOOK_DATA_H
+#define GLOM_MODE_DATA_NOTEBOOK_DATA_H
 
 #include <glom/notebook_glom.h>
 #include <glom/mode_data/box_data_list.h>
 #include <glom/mode_data/box_data_details.h>
 
-#ifdef GLOM_ENABLE_MAEMO 
-#include <glom/window_boxholder.h>
-#endif
-
 namespace Glom
 {
 
 class Notebook_Data : public Notebook_Glom
 {
-public: 
+public:
   Notebook_Data();
   virtual ~Notebook_Data();
 
@@ -47,9 +43,11 @@ public:
 
   ///Get the existing where clause, previously supplied to init_db_details().
   FoundSet get_found_set() const;
-  
-  ///Get the found set for the currently-visible record in the details tab:
-  FoundSet get_found_set_details() const;
+
+  /** Get the found set for the currently-selected record in the list view,
+   * if the list view is visible, or the currently-visible details tab.
+   */
+  FoundSet get_found_set_selected() const;
 
   ///Show the details for a particular record, without affecting the list view.
   void show_details(const Gnome::Gda::Value& primary_key_value);
@@ -58,15 +56,14 @@ public:
 
 #ifndef GLOM_ENABLE_CLIENT_ONLY
   virtual void do_menu_developer_layout(); //override
-  void show_layout_toolbar(bool show = true);
+
+  void set_enable_layout_drag_and_drop(bool enable = true);
 #endif // !GLOM_ENABLE_CLIENT_ONLY
 
   virtual void do_menu_file_print(); //override
-  
-  #ifdef GLOM_ENABLE_MAEMO
+
   void do_menu_file_add_record();
-  #endif
-  
+
   void get_record_counts(gulong& total, gulong& found);
 
   enum dataview
@@ -80,7 +77,14 @@ public:
 
   typedef sigc::signal<void, const Glib::ustring&, Gnome::Gda::Value> type_signal_record_details_requested;
   type_signal_record_details_requested signal_record_details_requested();
+
+  typedef sigc::signal<void> type_signal_record_selection_changed;
   
+  /** This signal is emitted when the a record is selected, or deselected,
+   * in the list view.
+   */
+  type_signal_record_selection_changed signal_record_selection_changed();
+
 protected:
 
   ///Show the counts of all records and found records.
@@ -88,30 +92,25 @@ protected:
 
   //Signal handlers:
   virtual void on_list_user_requested_details(const Gnome::Gda::Value& primary_key_value);
-  void on_details_user_requested_related_details(const Glib::ustring& table_name, Gnome::Gda::Value primary_key_value);
 
-  virtual void on_switch_page_handler(GtkNotebookPage* pPage, guint uiPageNumber);
+  virtual void on_switch_page_handler(Gtk::Widget* pPage, guint uiPageNumber);
 
 private:
-  #ifdef GLOM_ENABLE_MAEMO
-  void on_window_maemo_details_closed();
-  #endif
-  
+
+  bool on_idle_show_details(const Gnome::Gda::Value& primary_key_value);
+
 protected:
   //Member widgets:
   Box_Data_List m_Box_List;
   Box_Data_Details m_Box_Details;
-  
-  #ifdef GLOM_ENABLE_MAEMO //Details are in a separate window on Maemo.
-  Window_BoxHolder* m_window_maemo_details;
-  #endif
-  
+
   guint m_iPage_Details, m_iPage_List;
   Glib::ustring m_table_name;
-  
+
   type_signal_record_details_requested m_signal_record_details_requested;
+  type_signal_record_selection_changed m_signal_record_selection_changed;
 };
 
 } //namespace Glom
 
-#endif
+#endif // GLOM_MODE_DATA_NOTEBOOK_DATA_H
