@@ -14,16 +14,18 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #include "box_db_table_definition.h"
 #include <glom/frame_glom.h>
 #include <glom/glade_utils.h>
 #include <glom/utils_ui.h> //For bold_message()).
+#include <glom/appwindow.h>
 #include <libglom/libglom_config.h>
 #include <libglom/db_utils.h>
+#include <gtkmm/stock.h>
 #include <glibmm/i18n.h>
 
 namespace Glom
@@ -98,7 +100,7 @@ void Box_DB_Table_Definition::fill_field_row(const Gtk::TreeModel::iterator& ite
 
   m_AddDel.set_value(iter, m_colName, field->get_name());
 
-  const Glib::ustring title = field->get_title();
+  const Glib::ustring title = item_get_title(field);
   m_AddDel.set_value(iter, m_colTitle, title);
 
   //Type:
@@ -124,6 +126,11 @@ void Box_DB_Table_Definition::fill_field_row(const Gtk::TreeModel::iterator& ite
 bool Box_DB_Table_Definition::fill_from_database()
 {
   bool result = Box_DB_Table::fill_from_database();
+  if(!result)
+  {
+    std::cerr << G_STRFUNC << ":  Box_DB_Table::fill_from_database() failed." << std::endl;
+    return false;
+  }
 
   if(!(ConnectionPool::get_instance()->get_ready_to_connect()))
     return false;
@@ -142,8 +149,8 @@ bool Box_DB_Table_Definition::fill_from_database()
       const sharedptr<const Field>& field = *iter;
 
       //Name:
-      Gtk::TreeModel::iterator iter= m_AddDel.add_item(field->get_name());
-      fill_field_row(iter, field);
+      Gtk::TreeModel::iterator tree_iter= m_AddDel.add_item(field->get_name());
+      fill_field_row(tree_iter, field);
     }
 
     result = true;
@@ -169,7 +176,7 @@ void Box_DB_Table_Definition::on_adddel_add(const Gtk::TreeModel::iterator& row)
   {
     sharedptr<Field> field(new Field());
     field->set_name(name);
-    field->set_title( Utils::title_from_string(name) ); //Start with a title that might be useful.
+    field->set_title( Utils::title_from_string(name) , AppWindow::get_current_locale()); //Start with a title that might be useful.
     field->set_glom_type(Field::TYPE_NUMERIC);
 
     Glib::RefPtr<Gnome::Gda::Column> field_info = field->get_field_info();
@@ -239,7 +246,7 @@ void Box_DB_Table_Definition::on_adddel_delete(const Gtk::TreeModel::iterator& r
         get_document()->remove_field(m_table_name, name);
       }
       else
-        std::cerr << "Box_DB_Table_Definition::on_adddel_delete(): field deletion failed." << std::endl;
+        std::cerr << G_STRFUNC << ": field deletion failed." << std::endl;
     }
   }
 
@@ -351,7 +358,7 @@ void Box_DB_Table_Definition::on_adddel_changed(const Gtk::TreeModel::iterator&
     //Get DB field info: (TODO: This might be unnecessary).
     type_vec_fields::const_iterator iterFind = std::find_if( m_vecFields.begin(), m_vecFields.end(), predicate_FieldHasName<Field>(strFieldNameBeingEdited) );
     if(iterFind == m_vecFields.end()) //If it was not found:
-      std::cerr << "Box_DB_Table_Definition::on_adddel_changed(): field not found: " << strFieldNameBeingEdited << std::endl;
+      std::cerr << G_STRFUNC << ": field not found: " << strFieldNameBeingEdited << std::endl;
     else
     {
       sharedptr<const Field> constfield = *iterFind;
@@ -431,7 +438,8 @@ sharedptr<Field> Box_DB_Table_Definition::get_field_definition(const Gtk::TreeMo
 
   //Start with original definitions, so that we preserve things like UNSIGNED.
   //TODO maybe use document's fieldinfo instead of m_vecFields.
-  sharedptr<const Field> field_temp = get_fields_for_table_one_field(m_table_name, strFieldNameBeforeEdit);
+  sharedptr<const Field> field_temp = 
+    DbUtils::get_fields_for_table_one_field(pDoc, m_table_name, strFieldNameBeforeEdit);
   if(field_temp)
   {
     Glib::RefPtr<Gnome::Gda::Column> fieldInfo = field_temp->get_field_info()->copy();
@@ -442,7 +450,7 @@ sharedptr<Field> Box_DB_Table_Definition::get_field_definition(const Gtk::TreeMo
 
     //Title:
     const Glib::ustring title = m_AddDel.get_value(row, m_colTitle);
-    fieldResult->set_title(title);
+    fieldResult->set_title(title, AppWindow::get_current_locale());
 
     //Type:
     const Glib::ustring& strType = m_AddDel.get_value(row, m_colType);
@@ -617,7 +625,7 @@ void Box_DB_Table_Definition::fill_fields()
   //std::cout << "DEBUG: Box_DB_Table_Definition::fill_fields()" << std::endl;
 
   //Update the fields (also checking the actual database):
-  m_vecFields = get_fields_for_table(m_table_name);
+  m_vecFields = DbUtils::get_fields_for_table(get_document(), m_table_name);
 }
 
 bool Box_DB_Table_Definition::field_has_null_values(const sharedptr<const Field>& field)
@@ -644,7 +652,7 @@ bool Box_DB_Table_Definition::field_has_null_values(const sharedptr<const Field>
   }
   else
   {
-    std::cerr << "Box_DB_Table_Definition::field_has_null_values(): query failed." << std::endl;
+    std::cerr << G_STRFUNC << ": query failed." << std::endl;
   }
 
   return null_count > 0; 
@@ -669,7 +677,7 @@ bool Box_DB_Table_Definition::field_has_non_unique_values(const sharedptr<const
   }
   else
   {
-    g_warning("Box_DB_Table_Definition::field_has_non_unique_values(): SELECT COUNT() failed.");
+    std::cerr << G_STRFUNC << ": SELECT COUNT() failed." << std::endl;
   }
 
   //Count all rows, to compare. TODO_performance: Is there a more efficient way to do this? Maybe count(*), which apparently doesn't ignore NULL rows like count(somefield) would.
@@ -686,7 +694,7 @@ bool Box_DB_Table_Definition::field_has_non_unique_values(const sharedptr<const
   }
   else
   {
-    g_warning("Box_DB_Table_Definition::field_has_non_unique_values(): SELECT COUNT() failed.");
+    std::cerr << G_STRFUNC << ": SELECT COUNT() failed." << std::endl;
   }
 
 
