@@ -14,12 +14,12 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
-#ifndef BOX_DB_TABLE_DEFINITION_H
-#define BOX_DB_TABLE_DEFINITION_H
+#ifndef GLOM_MODE_DESIGN_BOX_DB_TABLE_DEFINITION_H
+#define GLOM_MODE_DESIGN_BOX_DB_TABLE_DEFINITION_H
 
 #include "../../box_db_table.h"
 #include "dialog_fielddefinition.h"
@@ -31,7 +31,7 @@ class Box_DB_Table_Definition : public Box_DB_Table
 {
 public: 
   Box_DB_Table_Definition();
-  Box_DB_Table_Definition(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade);
+  Box_DB_Table_Definition(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& builder);
   virtual ~Box_DB_Table_Definition();
 
 private:
@@ -64,9 +64,10 @@ private:
 
   Dialog_FieldDefinition* m_pDialog;
   sharedptr<const Field> m_Field_BeingEdited; //TODO_FieldShared
-  type_vecFields m_vecFields;
+  type_vec_fields m_vecFields;
 };
 
 } //namespace Glom
 
-#endif
+#endif // GLOM_MODE_DESIGN_BOX_DB_TABLE_DEFINITION_H
+
