@@ -14,15 +14,17 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #include "dialog_fielddefinition.h"
 #include "dialog_fieldcalculation.h"
 #include <glom/glade_utils.h>
 #include <glom/utils_ui.h>
+#include <glom/appwindow.h>
 #include "../../box_db_table.h"
+#include <libglom/db_utils.h>
 //#include <libgnome/gnome-i18n.h>
 #include <glibmm/i18n.h>
 
@@ -57,7 +59,7 @@ Dialog_FieldDefinition::Dialog_FieldDefinition(BaseObjectType* cobject, const Gl
   builder->get_widget("checkbutton_lookup",  m_pCheck_Lookup);
   builder->get_widget("table_lookup",  m_pTable_Lookup);
   builder->get_widget_derived("combobox_lookup_relationship",  m_pCombo_LookupRelationship);
-  builder->get_widget_derived("combobox_lookup_field",  m_pCombo_LookupField);
+  builder->get_widget("combobox_lookup_field",  m_pCombo_LookupField);
 
   builder->get_widget("radiobutton_calculate",  m_pRadio_Calculate);
   builder->get_widget("alignment_calculate",  m_pAlignment_Calculate);
@@ -87,7 +89,7 @@ Dialog_FieldDefinition::Dialog_FieldDefinition(BaseObjectType* cobject, const Gl
   builder->get_widget("box_formatting_placeholder", m_box_formatting_placeholder);
 
   //Get the formatting stuff:
-  Utils::get_glade_widget_derived_with_warning(m_box_formatting);
+  Utils::get_glade_child_widget_derived_with_warning(m_box_formatting);
 
   if(m_box_formatting) ////Unlikely to fail and it already warns on stderr.
     m_box_formatting_placeholder->pack_start(*m_box_formatting);
@@ -99,12 +101,14 @@ Dialog_FieldDefinition::Dialog_FieldDefinition(BaseObjectType* cobject, const Gl
   on_foreach_connect(*m_pBox_DefaultValueSimple);
   on_foreach_connect(*m_pBox_ValueTab);
 
-  if(m_box_formatting) ////Unlikely to fail and it already warns on stderr.
+  if(m_box_formatting)
+  {
     on_foreach_connect(*m_box_formatting);
 
-  //Plus an extra signal for the related extra show-also fields:
-  m_box_formatting->signal_modified().connect(
-   sigc::mem_fun(*this, &Dialog_FieldDefinition::on_anything_changed));
+    //Plus an extra signal for the related extra show-also fields:
+    m_box_formatting->signal_modified().connect(
+     sigc::mem_fun(*this, &Dialog_FieldDefinition::on_anything_changed));
+  }
 
   Dialog_Properties::set_modified(false);
 
@@ -154,7 +158,7 @@ void Dialog_FieldDefinition::set_field(const sharedptr<const Field>& field, cons
   sharedptr<LayoutItem_Field> layout_item = sharedptr<LayoutItem_Field>::create();
   sharedptr<Field> field_default_value = glom_sharedptr_clone(m_Field);
   field_default_value->set_name("glom_temp_default_value");
-  field_default_value->set_title(_("Default Value"));
+  field_default_value->set_title_original(_("Default Value"));
   layout_item->set_full_field_details(field_default_value);
   m_pDataWidget_DefaultValueSimple = Gtk::manage( new DataWidget(layout_item, "", get_document()) );
   if(!m_pDataWidget_DefaultValueSimple->get_data_child_widget())
@@ -209,7 +213,7 @@ void Dialog_FieldDefinition::set_field(const sharedptr<const Field>& field, cons
   m_pTextView_Calculation->get_buffer()->set_text(calculation);
   //std::cout << "  debug: dialog_fielddefinition.c:: after get_buffer()" << std::endl;
 
-  m_pEntry_Title->set_text(field->get_title());
+  m_pEntry_Title->set_text(item_get_title(field));
 
   //Formatting:
   m_box_formatting->set_formatting_for_field(field->m_default_formatting, m_table_name, field);
@@ -269,7 +273,7 @@ sharedptr<Field> Dialog_FieldDefinition::get_field() const
   field->set_unique_key(m_pCheck_Unique->get_active());
   field->set_primary_key(m_pCheck_PrimaryKey->get_active());
 
-  field->set_title(m_pEntry_Title->get_text());
+  field->set_title(m_pEntry_Title->get_text(), AppWindow::get_current_locale());
 
   //Formatting:
   m_box_formatting->get_formatting(field->m_default_formatting);
@@ -362,7 +366,7 @@ void Dialog_FieldDefinition::on_combo_lookup_relationship_changed()
       if(!to_table.empty())
       {
         //Get the fields in the other table, and add them to the combo:
-        const type_vec_fields fields_in_to_table = get_fields_for_table(to_table);
+        const type_vec_fields fields_in_to_table = DbUtils::get_fields_for_table(document, to_table);
         for(type_vec_fields::const_iterator iter = fields_in_to_table.begin(); iter != fields_in_to_table.end(); ++iter)
         {
           m_pCombo_LookupField->append((*iter)->get_name());
