@@ -14,15 +14,18 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #ifndef GLOM_MODE_DESIGN_DIALOG_FIELDDEFINITION_H
 #define GLOM_MODE_DESIGN_DIALOG_FIELDDEFINITION_H
 
-#include <gtkmm.h>
-#include "../../utility_widgets/combo_textglade.h"
+#include <gtkmm/dialog.h>
+#include <gtkmm/checkbutton.h>
+#include <gtkmm/comboboxtext.h>
+#include <gtkmm/alignment.h>
+#include <gtkmm/entry.h>
 #include <glom/mode_design/layout/combobox_relationship.h>
 #include "combo_fieldtype.h"
 //#include "../../utility_widgets/entry_numerical.h"
@@ -31,7 +34,7 @@
 #include <libglom/data_structure/field.h>
 #include <glom/mode_design/layout/layout_item_dialogs/box_formatting.h>
 #include <glom/base_db.h>
-#include <gtksourceviewmm/sourceview.h>
+#include <gtksourceviewmm/view.h>
 
 namespace Glom
 {
@@ -69,29 +72,29 @@ private:
   Combo_FieldType* m_pCombo_Type;
   Gtk::CheckButton* m_pCheck_Unique;
   Gtk::CheckButton* m_pCheck_NotNull;
-  Gtk::HBox* m_pBox_DefaultValueSimple;
+  Gtk::Box* m_pBox_DefaultValueSimple;
   Gtk::CheckButton* m_pCheck_PrimaryKey;
   Gtk::CheckButton* m_pCheck_AutoIncrement;
 
-  Gtk::VBox* m_pBox_ValueTab;
+  Gtk::Box* m_pBox_ValueTab;
 
   Gtk::RadioButton* m_pRadio_UserEntry;
   Gtk::Alignment* m_pAlignment_UserEntry;
   Gtk::CheckButton* m_pCheck_Lookup;
-  Gtk::Table* m_pTable_Lookup;
+  Gtk::Widget* m_pTable_Lookup; //So we can make it insensitive.
   ComboBox_Relationship* m_pCombo_LookupRelationship;
-  Combo_TextGlade* m_pCombo_LookupField;
+  Gtk::ComboBoxText* m_pCombo_LookupField;
 
   Gtk::RadioButton* m_pRadio_Calculate;
   Gtk::Alignment* m_pAlignment_Calculate;
-  Gsv::SourceView* m_pTextView_Calculation;
+  Gsv::View* m_pTextView_Calculation;
   Gtk::Button* m_pButton_EditCalculation;
 
   Gtk::Entry* m_pEntry_Title;
 
   DataWidget* m_pDataWidget_DefaultValueSimple;
 
-  Gtk::VBox* m_box_formatting_placeholder;
+  Gtk::Box* m_box_formatting_placeholder;
   Box_Formatting* m_box_formatting;
 
   sharedptr<Field> m_Field;
