@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #include <glom/mode_design/layout/dialog_layout_details.h>
@@ -26,12 +26,16 @@
 #include <glom/frame_glom.h> //For show_ok_dialog()
 //#include <libgnome/gnome-i18n.h>
 #include <glom/utils_ui.h> //For bold_message()).
+#include <glom/appwindow.h>
 #include <glibmm/i18n.h>
 #include <sstream> //For stringstream
 
 namespace Glom
 {
 
+const char* Dialog_Layout_Details::glade_id("window_data_layout");
+const bool Dialog_Layout_Details::glade_developer(true);
+
 Dialog_Layout_Details::Dialog_Layout_Details(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& builder)
 : Dialog_Layout(cobject, builder),
   m_treeview_fields(0),
@@ -54,7 +58,10 @@ Dialog_Layout_Details::Dialog_Layout_Details(BaseObjectType* cobject, const Glib
   m_button_field_delete(0),
   m_button_formatting(0),
   m_button_edit(0),
-  m_label_table_name(0)
+  m_label_table_name(0),
+  m_hbox_rows_count(0),
+  m_spinbutton_rows_count_min(0),
+  m_spinbutton_rows_count_max(0)
 {
   // Get the alternate sets of widgets, only one of which should be shown:
   // Derived classes will hide one and show the other:
@@ -62,15 +69,23 @@ Dialog_Layout_Details::Dialog_Layout_Details(BaseObjectType* cobject, const Glib
   m_box_table_widgets->show();
   builder->get_widget("hbox_related_table_widgets", m_box_related_table_widgets);
   m_box_related_table_widgets->hide();
-  builder->get_widget("frame_related_table_navigation", m_box_related_navigation); 
+  builder->get_widget("frame_related_table_navigation", m_box_related_navigation);
   m_box_related_navigation->hide();
+  builder->get_widget("frame_lines", m_box_frame_lines);
+  m_box_frame_lines->hide();
 
   Gtk::Frame* box_calendar = 0;
-  builder->get_widget("frame_calendar", box_calendar); 
+  builder->get_widget("frame_calendar", box_calendar);
   box_calendar->hide();
 
   builder->get_widget("label_table_name", m_label_table_name);
-
+  
+  //This is only shown in Dialog_Layout_List_Related:
+  builder->get_widget("hbox_rows_count", m_hbox_rows_count);
+  builder->get_widget("spinbutton_rows_count_min", m_spinbutton_rows_count_min);
+  builder->get_widget("spinbutton_rows_count_max", m_spinbutton_rows_count_max);
+  m_hbox_rows_count->hide();
+  
   builder->get_widget("treeview_fields", m_treeview_fields);
   if(m_treeview_fields)
   {
@@ -226,12 +241,12 @@ void Dialog_Layout_Details::fill_group(const Gtk::TreeModel::iterator& iter, sha
       sharedptr<LayoutItem_Portal> layout_portal = sharedptr<LayoutItem_Portal>::cast_dynamic(layout_item);
       if(layout_portal)
       {
-        //std::cout << "fill_group(): adding portal." << std::endl;
+        //std::cout << "debug: " << G_STRFUNC << ": adding portal." << std::endl;
         group->add_item(glom_sharedptr_clone(layout_portal));
       }
       else
       {
-        //std::cout << "fill_group(): adding group." << std::endl;
+        //std::cout << "debug: " << G_STRFUNC << ": adding group." << std::endl;
         sharedptr<LayoutGroup> layout_group = sharedptr<LayoutGroup>::cast_dynamic(layout_item);
         sharedptr<LayoutItem_Portal> layout_portal = sharedptr<LayoutItem_Portal>::cast_dynamic(layout_group);
         if(layout_group && !layout_portal)
@@ -243,7 +258,7 @@ void Dialog_Layout_Details::fill_group(const Gtk::TreeModel::iterator& iter, sha
         }
         else if(layout_item)
         {
-          //std::cout << "fill_group(): adding item." << std::endl;
+          //std::cout << "debug: " << G_STRFUNC << ": adding item." << std::endl;
 
           //Add field or button:
           sharedptr<LayoutItem> item = glom_sharedptr_clone(layout_item);
@@ -316,18 +331,18 @@ void Dialog_Layout_Details::add_group(const Gtk::TreeModel::iterator& parent, co
   }
 }
 
-void Dialog_Layout_Details::set_document(const Glib::ustring& layout_name, const Glib::ustring& layout_platform, Document* document, const Glib::ustring& table_name, const type_vecLayoutFields& table_fields)
+void Dialog_Layout_Details::init(const Glib::ustring& layout_name, const Glib::ustring& layout_platform, Document* document, const Glib::ustring& table_name, const type_vecConstLayoutFields& table_fields)
 {
   m_modified = false;
 
-  Dialog_Layout::set_document(layout_name, layout_platform, document, table_name, table_fields);
+  Dialog_Layout::init(layout_name, layout_platform, document, table_name, table_fields);
 
   //Update the tree models from the document
   if(document)
   {
     //Set the table name and title:
     m_label_table_name->set_text(table_name);
-    m_entry_table_title->set_text( document->get_table_title(table_name) );
+    m_entry_table_title->set_text( document->get_table_title(table_name, AppWindow::get_current_locale()) );
 
     Document::type_list_layout_groups list_groups = document->get_data_layout_groups_plus_new_fields(m_layout_name, m_table_name, m_layout_platform);
     document->fill_layout_field_details(m_table_name, list_groups); //Update with full field information.
@@ -341,7 +356,7 @@ void Dialog_Layout_Details::set_document(const Glib::ustring& layout_name, const
       group->set_columns_count(1);
 
       list_groups.push_back(group);
-    }      
+    }
 
     //Show the field layout
     //typedef std::list< Glib::ustring > type_listStrings;
@@ -516,7 +531,7 @@ void Dialog_Layout_Details::on_button_down()
 void Dialog_Layout_Details::on_button_add_field()
 {
   type_list_field_items fields_list = offer_field_list(m_table_name, this);
-  for(type_list_field_items::iterator iter_chosen = fields_list.begin(); iter_chosen != fields_list.end(); ++iter_chosen) 
+  for(type_list_field_items::iterator iter_chosen = fields_list.begin(); iter_chosen != fields_list.end(); ++iter_chosen)
   {
     sharedptr<LayoutItem_Field> layout_item = *iter_chosen;
     if(!layout_item)
@@ -548,72 +563,26 @@ sharedptr<LayoutItem_Button> Dialog_Layout_Details::offer_button_script_edit(con
 {
   sharedptr<LayoutItem_Button> result;
 
-  try
+  Dialog_ButtonScript* dialog = 0;
+  Glom::Utils::get_glade_widget_derived_with_warning(dialog);
+  if(!dialog) //Unlikely and it already warns on stderr.
+    return result;
+
+  dialog->set_script(button, m_table_name);
+  dialog->set_transient_for(*this);
+  const int response = Glom::Utils::dialog_run_with_help(dialog);
+  dialog->hide();
+  if(response == Gtk::RESPONSE_OK)
   {
-    Glib::RefPtr<Gtk::Builder> refXml = Gtk::Builder::create_from_file(Utils::get_glade_file_path("glom_developer.glade"), "window_button_script");
-
-    Dialog_ButtonScript* dialog = 0;
-    refXml->get_widget_derived("window_button_script", dialog);
-
-    if(dialog)
-    {
-      dialog->set_script(button, m_table_name);
-      dialog->set_transient_for(*this);
-      const int response = Glom::Utils::dialog_run_with_help(dialog, "window_button_script");
-      dialog->hide();
-      if(response == Gtk::RESPONSE_OK)
-      {
-        //Get the chosen relationship:
-        result = dialog->get_script();
-      }
-
-      delete dialog;
-    }
-  }
-  catch(const Gtk::BuilderError& ex)
-  {
-    std::cerr << ex.what() << std::endl;
+    //Get the chosen relationship:
+     result = dialog->get_script();
   }
 
+  delete dialog;
+
   return result;
 }
 
-/*
-sharedptr<LayoutItem_Text> Dialog_Layout_Details::offer_textobject_edit(const sharedptr<const LayoutItem_Text>& textobject)
-{
-  sharedptr<LayoutItem_Text> result;
-
-  try
-  {
-    Glib::RefPtr<Gtk::Builder> refXml = Gtk::Builder::create_from_file(Utils::get_glade_file_path("glom_developer.glade"), "window_textobject");
-
-    Dialog_TextObject* dialog = 0;
-    refXml->get_widget_derived("window_textobject", dialog);
-
-    if(dialog)
-    {
-      dialog->set_textobject(textobject, m_table_name);
-      dialog->set_transient_for(*this);
-      const int response = dialog->run();
-      dialog->hide();
-      if(response == Gtk::RESPONSE_OK)
-      {
-        //Get the chosen relationship:
-        result = dialog->get_textobject();
-      }
-
-      delete dialog;
-    }
-  }
-  catch(const Gtk::BuilderError& ex)
-  {
-    std::cerr << ex.what() << std::endl;
-  }
-
-  return result;
-}
-*/
-
 sharedptr<Relationship> Dialog_Layout_Details::offer_relationship_list()
 {
   return offer_relationship_list(sharedptr<Relationship>());
@@ -623,34 +592,24 @@ sharedptr<Relationship> Dialog_Layout_Details::offer_relationship_list(const sha
 {
   sharedptr<Relationship> result = glom_sharedptr_clone(item);
 
-  try
+  Dialog_ChooseRelationship* dialog = 0;
+  Utils::get_glade_widget_derived_with_warning(dialog);
+  if(!dialog) //Unlikely and it already warns on stderr.
+    return result;
+
+  dialog->set_document(get_document(), m_table_name);
+  dialog->select_item(item);
+  dialog->set_transient_for(*this);
+  const int response = dialog->run();
+  dialog->hide();
+  if(response == Gtk::RESPONSE_OK)
   {
-    Glib::RefPtr<Gtk::Builder> refXml = Gtk::Builder::create_from_file(Utils::get_glade_file_path("glom_developer.glade"), "dialog_choose_relationship");
-
-    Dialog_ChooseRelationship* dialog = 0;
-    refXml->get_widget_derived("dialog_choose_relationship", dialog);
-
-    if(dialog)
-    {
-      dialog->set_document(get_document(), m_table_name);
-      dialog->select_item(item);
-      dialog->set_transient_for(*this);
-      const int response = dialog->run();
-      dialog->hide();
-      if(response == Gtk::RESPONSE_OK)
-      {
-        //Get the chosen relationship:
-        result = dialog->get_relationship_chosen();
-      }
-
-      delete dialog;
-    }
-  }
-  catch(const Gtk::BuilderError& ex)
-  {
-    std::cerr << ex.what() << std::endl;
+    //Get the chosen relationship:
+    result = dialog->get_relationship_chosen();
   }
 
+  delete dialog;
+
   return result;
 }
 
@@ -700,7 +659,7 @@ void Dialog_Layout_Details::on_button_add_button()
 
     //Add a new button:
     sharedptr<LayoutItem_Button> button = sharedptr<LayoutItem_Button>::create();
-    button->set_title(_("New Button")); //Give the button a default title, so it is big enough, and so people see that they should change it.
+    button->set_title_original(_("New Button")); //Give the button a default title, so it is big enough, and so people see that they should change it.
     row[m_model_items->m_columns.m_col_layout_item] = button;
 
     //Scroll to, and select, the new row:
@@ -725,7 +684,7 @@ void Dialog_Layout_Details::on_button_add_text()
 
     //Add a new button:
     sharedptr<LayoutItem_Text> textobject = sharedptr<LayoutItem_Text>::create();
-    textobject->set_title(_("Text Title")); //Give the button a default title, so it is big enough, and so people see that they should change it.
+    textobject->set_title_original(_("Text Title")); //Give the button a default title, so it is big enough, and so people see that they should change it.
     row[m_model_items->m_columns.m_col_layout_item] = textobject;
 
     //Scroll to, and select, the new row:
@@ -750,7 +709,7 @@ void Dialog_Layout_Details::on_button_add_image()
 
     //Add a new button:
     sharedptr<LayoutItem_Image> imageobject = sharedptr<LayoutItem_Image>::create();
-    imageobject->set_title(_("Image Title")); //Give the item a default title, so it is big enough, and so people see that they should change it.
+    imageobject->set_title_original(_("Image Title")); //Give the item a default title, so it is big enough, and so people see that they should change it.
     row[m_model_items->m_columns.m_col_layout_item] = imageobject;
 
     //Scroll to, and select, the new row:
@@ -768,8 +727,6 @@ void Dialog_Layout_Details::on_button_add_image()
 
 void Dialog_Layout_Details::on_button_add_notebook()
 {
-  Gtk::TreeModel::iterator parent = get_selected_group_parent();
-
   Gtk::TreeModel::iterator iter = append_appropriate_row();
   if(iter)
   {
@@ -794,8 +751,6 @@ void Dialog_Layout_Details::on_button_add_notebook()
 
 void Dialog_Layout_Details::on_button_add_related()
 {
-  Gtk::TreeModel::iterator parent = get_selected_group_parent();
-
   /* We don't need to ask this because the portal layout dialog can now handle an empty portal:
   sharedptr<Relationship> relationship = offer_relationship_list();
   if(relationship)
@@ -828,8 +783,6 @@ void Dialog_Layout_Details::on_button_add_related()
 
 void Dialog_Layout_Details::on_button_add_related_calendar()
 {
-  Gtk::TreeModel::iterator parent = get_selected_group_parent();
-
   /* We don't need to ask this because the portal layout dialog can now handle an empty portal:
   sharedptr<Relationship> relationship = offer_relationship_list();
   if(relationship)
@@ -945,7 +898,7 @@ void Dialog_Layout_Details::on_button_formatting()
       if(field)
       {
         //Handle field formatting, which includes more than the generic formatting stuff:
-        sharedptr<LayoutItem_Field> chosenitem = offer_field_formatting(field, get_fields_table(), this);
+        sharedptr<LayoutItem_Field> chosenitem = offer_field_formatting(field, get_fields_table(), this, m_editable_layout);
         if(chosenitem)
         {
           *field = *chosenitem; //TODO_Performance.
@@ -959,7 +912,7 @@ void Dialog_Layout_Details::on_button_formatting()
         sharedptr<LayoutItem_WithFormatting> withformatting = sharedptr<LayoutItem_WithFormatting>::cast_dynamic(layout_item);
         if(withformatting)
         {
-          const bool changed = offer_item_formatting(withformatting, this);
+          const bool changed = offer_non_field_item_formatting(withformatting, this);
           if(changed)
             m_modified = true;
         }
@@ -991,7 +944,7 @@ void Dialog_Layout_Details::on_button_edit()
         if(relationship)
         {
           layout_portal->set_relationship(relationship);
- 
+
           //This is unnecessary and seems to cause a crash.
           //row[m_model_items->m_columns.m_col_layout_item] = layout_portal;
 
@@ -1101,7 +1054,7 @@ void Dialog_Layout_Details::save_to_document()
     //Set the table name and title:
     Document* document = get_document();
     if(document)
-      document->set_table_title( m_table_name, m_entry_table_title->get_text() );
+      document->set_table_title( m_table_name, m_entry_table_title->get_text(), AppWindow::get_current_locale());
 
     //Get the data from the TreeView and store it in the document:
 
@@ -1166,9 +1119,9 @@ void Dialog_Layout_Details::on_cell_data_name(Gtk::CellRenderer* renderer, const
       {
         sharedptr<LayoutItem_CalendarPortal> layout_calendar = sharedptr<LayoutItem_CalendarPortal>::cast_dynamic(layout_portal);
         if(layout_calendar)
-          markup = _("Related Calendar: ") + layout_portal->get_relationship_name();
+          markup = Glib::ustring::compose(_("Related Calendar: %1"), layout_portal->get_relationship_name());
         else
-          markup = _("Related List: ") + layout_portal->get_relationship_name();
+          markup = Glib::ustring::compose(_("Related List: %1"), layout_portal->get_relationship_name());
       }
       else
       {
@@ -1185,7 +1138,7 @@ void Dialog_Layout_Details::on_cell_data_name(Gtk::CellRenderer* renderer, const
           sharedptr<LayoutItem_Field> layout_item_field = sharedptr<LayoutItem_Field>::cast_dynamic(layout_item);
           if(layout_item_field)
           {
-            markup = _("Field: ") + layout_item_field->get_layout_display_name(); //TODO: Put this all in a sprintf-style string so it can be translated properly.
+            markup = Glib::ustring::compose(_("Field: %1"), layout_item_field->get_layout_display_name());
 
             //Just for debugging:
             //if(!row[m_model_items->m_columns.m_col_editable])
@@ -1246,7 +1199,7 @@ void Dialog_Layout_Details::on_cell_data_title(Gtk::CellRenderer* renderer, cons
       if(layout_notebook)
         renderer_text->property_text() = _("(Notebook)");
       else if(layout_item)
-        renderer_text->property_text() = layout_item->get_title();
+        renderer_text->property_text() = item_get_title(layout_item);
       else
         renderer_text->property_text() = Glib::ustring();
 
@@ -1279,11 +1232,11 @@ void Dialog_Layout_Details::on_cell_data_column_width(Gtk::CellRenderer* rendere
         sharedptr<LayoutItem_Field> layout_field = sharedptr<LayoutItem_Field>::cast_dynamic(layout_item);
         const bool editable = (layout_field || layout_button || layout_text); //Only these have column widths that can be edited.
         renderer_text->property_editable() = editable;
-      
+
         column_width = layout_item->get_display_width();
       }
 
-      Glib::ustring text; 
+      Glib::ustring text;
       if(column_width) //Show nothing if no width has been specified, meaning that it's automatic.
         text = Utils::string_from_decimal(column_width);
 
@@ -1340,8 +1293,7 @@ void Dialog_Layout_Details::on_treeview_cell_edited_title(const Glib::ustring& p
       if(layout_item)
       {
         //Store the user's new text in the model:
-        Gtk::TreeRow row = *iter;
-        layout_item->set_title(new_text);
+        layout_item->set_title(new_text, AppWindow::get_current_locale());
 
         m_modified = true;
       }
@@ -1365,7 +1317,6 @@ void Dialog_Layout_Details::on_treeview_cell_edited_name(const Glib::ustring& pa
       if(layout_item)
       {
         //Store the user's new text in the model:
-        Gtk::TreeRow row = *iter;
         layout_item->set_name(new_text);
 
         m_modified = true;
@@ -1393,7 +1344,6 @@ void Dialog_Layout_Details::on_treeview_cell_edited_column_width(const Glib::ust
         const guint new_value = static_cast<guint>( strtod(new_text.c_str(), &pchEnd) );
 
         //Store the user's new value in the model:
-        Gtk::TreeRow row = *iter;
         layout_item->set_display_width(new_value);
 
         m_modified = true;
@@ -1447,12 +1397,3 @@ Glib::ustring Dialog_Layout_Details::get_fields_table() const
 }
 
 } //namespace Glom
-
-
-
-
-
-
-
-
-
