@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #include <glom/mode_design/layout/dialog_layout_export.h>
@@ -135,9 +135,7 @@ void Dialog_Layout_Export::set_layout_groups(Document::type_list_layout_groups&
     }
     */
 
-    //Show the field layout
-    typedef std::list< Glib::ustring > type_listStrings;
-
+    //Show the field layout:
     m_model_fields->clear();
 
     guint field_sequence = 1; //0 means no sequence
@@ -322,7 +320,7 @@ void Dialog_Layout_Export::on_cell_data_name(Gtk::CellRenderer* renderer, const
       sharedptr<LayoutItem_Field> item = row[m_ColumnsFields.m_col_layout_item];
 
       //Names can never be edited.
-      g_object_set(renderer_text->gobj(), "markup", item->get_layout_display_name().c_str(), "editable", FALSE, (gpointer)0);
+      renderer_text->property_markup() = item->get_layout_display_name();
     }
   }
 }
