@@ -14,12 +14,12 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
-#ifndef GLOM_MODE_DATA_DIALOG_LAYOUT_LIST_H
-#define GLOM_MODE_DATA_DIALOG_LAYOUT_LIST_H
+#ifndef GLOM_MODE_DESIGN_DIALOG_LAYOUT_LIST_H
+#define GLOM_MODE_DESIGN_DIALOG_LAYOUT_LIST_H
 
 #include <glom/mode_design/layout/dialog_layout_details.h>
 
@@ -29,10 +29,13 @@ namespace Glom
 class Dialog_Layout_List : public Dialog_Layout_Details
 {
 public:
+  static const char* glade_id;
+  static const bool glade_developer;
+  
   Dialog_Layout_List(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& builder);
   virtual ~Dialog_Layout_List();
 };
 
 } //namespace Glom
 
-#endif //GLOM_MODE_DATA_DIALOG_LAYOUT_LIST_H
+#endif // GLOM_MODE_DESIGN_DIALOG_LAYOUT_LIST_H
