@@ -14,14 +14,15 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #include <glom/mode_design/layout/dialog_layout_list_related.h>
 #include <glom/mode_design/layout/dialog_choose_field.h>
 #include <glom/mode_design/layout/layout_item_dialogs/dialog_field_layout.h>
 #include <libglom/utils.h> //For bold_message().
+#include <libglom/db_utils.h>
 #include <glom/utils_ui.h> //For show_ok_dialog().
 
 //#include <libgnome/gnome-i18n.h>
@@ -66,11 +67,16 @@ Dialog_Layout_List_Related::Dialog_Layout_List_Related(BaseObjectType* cobject,
     sigc::mem_fun(*this, &Dialog_Layout_List_Related::on_spinbutton_changed));
 
   builder->get_widget_derived("combo_relationship_name", m_combo_relationship);
-  m_combo_relationship->signal_changed().connect(sigc::mem_fun(*this, &Dialog_Layout_List_Related::on_combo_relationship_changed));
+  if(m_combo_relationship)
+  {
+    m_combo_relationship->signal_changed().connect(sigc::mem_fun(*this, &Dialog_Layout_List_Related::on_combo_relationship_changed));
+  }
 
   builder->get_widget("checkbutton_show_child_relationships", m_checkbutton_show_child_relationships);
-  m_checkbutton_show_child_relationships->signal_toggled().connect(sigc::mem_fun(*this, &Dialog_Layout_List_Related::on_checkbutton_show_child_relationships));
-
+  if(m_checkbutton_show_child_relationships)
+  {
+    m_checkbutton_show_child_relationships->signal_toggled().connect(sigc::mem_fun(*this, &Dialog_Layout_List_Related::on_checkbutton_show_child_relationships));
+  }
 
 
   builder->get_widget("radiobutton_navigation_automatic", m_radio_navigation_automatic);
@@ -81,18 +87,32 @@ Dialog_Layout_List_Related::Dialog_Layout_List_Related(BaseObjectType* cobject,
 
   builder->get_widget("radiobutton_navigation_specify", m_radio_navigation_specify);
   builder->get_widget_derived("combobox_navigation_specify", m_combo_navigation_specify);
-  make_sensitivity_depend_on_toggle_button(*m_radio_navigation_specify, *m_combo_navigation_specify);
-  m_combo_navigation_specify->signal_changed().connect(sigc::mem_fun(*this, &Dialog_Layout_List_Related::on_combo_navigation_specific_changed));
+  if(m_radio_navigation_specify && m_combo_navigation_specify)
+  {
+    make_sensitivity_depend_on_toggle_button(*m_radio_navigation_specify, *m_combo_navigation_specify);
+    m_combo_navigation_specify->signal_changed().connect(sigc::mem_fun(*this, &Dialog_Layout_List_Related::on_combo_navigation_specific_changed));
+  }
 
   builder->get_widget("spinbutton_row_line_width", m_spinbutton_row_line_width);
-  m_spinbutton_row_line_width->signal_value_changed().connect(
-    sigc::mem_fun(*this, &Dialog_Layout_List_Related::on_spinbutton_changed));
+  if(m_spinbutton_row_line_width)
+  {
+    m_spinbutton_row_line_width->signal_value_changed().connect(
+      sigc::mem_fun(*this, &Dialog_Layout_List_Related::on_spinbutton_changed));
+  }
+
   builder->get_widget("spinbutton_column_line_width", m_spinbutton_column_line_width);
-  m_spinbutton_column_line_width->signal_value_changed().connect(
-    sigc::mem_fun(*this, &Dialog_Layout_List_Related::on_spinbutton_changed));
+  if(m_spinbutton_column_line_width)
+  {
+    m_spinbutton_column_line_width->signal_value_changed().connect(
+      sigc::mem_fun(*this, &Dialog_Layout_List_Related::on_spinbutton_changed));
+  }
+
   builder->get_widget("colorbutton_line", m_colorbutton_line);
-  m_colorbutton_line->signal_color_set().connect(
-    sigc::mem_fun(*this, &Dialog_Layout_List_Related::on_spinbutton_changed));
+  if(m_colorbutton_line)
+  {
+    m_colorbutton_line->signal_color_set().connect(
+      sigc::mem_fun(*this, &Dialog_Layout_List_Related::on_spinbutton_changed));
+  }
 
 
   m_modified = false;
@@ -113,7 +133,7 @@ Dialog_Layout_List_Related::~Dialog_Layout_List_Related()
 }
 
 
-void Dialog_Layout_List_Related::set_document(const Glib::ustring& layout_name, const Glib::ustring& layout_platform, Document* document, const sharedptr<const LayoutItem_Portal>& portal, const Glib::ustring& from_table, bool for_print_layout)
+void Dialog_Layout_List_Related::init_with_portal(const Glib::ustring& layout_name, const Glib::ustring& layout_platform, Document* document, const sharedptr<const LayoutItem_Portal>& portal, const Glib::ustring& from_table, bool for_print_layout)
 {
   m_for_print_layout = for_print_layout;
 
@@ -141,7 +161,7 @@ void Dialog_Layout_List_Related::set_document(const Glib::ustring& layout_name,
   m_spinbutton_rows_count_max->set_value(rows_count_max);
 
   type_vecConstLayoutFields empty_fields; //Just to satisfy the base class.
-  Dialog_Layout::set_document(layout_name, layout_platform, document, actual_from_table, empty_fields);
+  Dialog_Layout::init(layout_name, layout_platform, document, actual_from_table, empty_fields);
   //m_table_name is now actually the parent_table_name.
 
   //Hide unwanted widgets and show extra ones:
@@ -397,8 +417,8 @@ void Dialog_Layout_List_Related::on_combo_relationship_changed()
   //The relationship's to field may not be a unique field, because that would
   //prevent the portal from having multiple records.
   sharedptr<Field> to_key_field =
-    get_fields_for_table_one_field(relationship->get_to_table(),
-      relationship->get_to_field());
+    DbUtils::get_fields_for_table_one_field(get_document(), 
+      relationship->get_to_table(), relationship->get_to_field());
   bool relationship_invalid = false;
   if(!to_key_field)
   {
