@@ -14,14 +14,16 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #ifndef GLOM_MODE_DESIGN_DIALOG_TEXTOBJECT_H
 #define GLOM_MODE_DESIGN_DIALOG_TEXTOBJECT_H
 
-#include <gtkmm.h>
+#include <gtkmm/dialog.h>
+#include <gtkmm/entry.h>
+#include <gtkmm/textview.h>
 #include <gtkmm/builder.h>
 #include <libglom/data_structure/layout/layoutitem_text.h>
 #include <glom/base_db.h>
@@ -46,7 +48,7 @@ public:
   void get_textobject(sharedptr<LayoutItem_Text>& textobject) const;
 
 private:
-  Gtk::VBox* m_box_title;
+  Gtk::Box* m_box_title;
   Gtk::Entry* m_entry_title;
   Gtk::TextView* m_text_view;
 
