@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #include <glom/mode_design/report_layout/dialog_layout_report.h>
@@ -34,6 +34,7 @@
 #include <glom/mode_design/layout/layout_item_dialogs/dialog_group_by.h>
 #include <glom/mode_design/layout/layout_item_dialogs/dialog_field_summary.h>
 #include <glom/mode_design/layout/dialog_choose_relationship.h>
+#include <glom/appwindow.h>
 //#include <libgnome/gnome-i18n.h>
 #include <libglom/utils.h> //For bold_message()).
 #include <glibmm/i18n.h>
@@ -42,6 +43,9 @@
 namespace Glom
 {
 
+const char* Dialog_Layout_Report::glade_id("window_report_layout");
+const bool Dialog_Layout_Report::glade_developer(true);
+
 Dialog_Layout_Report::Dialog_Layout_Report(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& builder)
 : Dialog_Layout(cobject, builder, false /* No table title */),
   m_notebook_parts(0),
@@ -175,11 +179,15 @@ Dialog_Layout_Report::Dialog_Layout_Report(BaseObjectType* cobject, const Glib::
 
   show_all_children();
 
-  m_notebook_parts->signal_switch_page().connect(sigc::mem_fun(*this, &Dialog_Layout_Report::on_notebook_switch_page));
+  //We save the connection, so we can disconnect it in the destructor,
+  //because, for some reason, this signal handler is still called _after_ the destructor.
+  //TODO: Fix that problem in GTK+ or gtkmm?
+  m_signal_connection = m_notebook_parts->signal_switch_page().connect(sigc::mem_fun(*this, &Dialog_Layout_Report::on_notebook_switch_page));
 }
 
 Dialog_Layout_Report::~Dialog_Layout_Report()
 {
+  m_signal_connection.disconnect();
 }
 
 void Dialog_Layout_Report::setup_model(Gtk::TreeView& treeview, Glib::RefPtr<type_model>& model)
@@ -200,7 +208,7 @@ void Dialog_Layout_Report::setup_model(Gtk::TreeView& treeview, Glib::RefPtr<typ
 
   Gtk::CellRendererText* renderer_part = Gtk::manage(new Gtk::CellRendererText);
   column_part->pack_start(*renderer_part);
-  column_part->set_cell_data_func(*renderer_part, 
+  column_part->set_cell_data_func(*renderer_part,
     sigc::bind( sigc::mem_fun(*this, &Dialog_Layout_Report::on_cell_data_part), model));
 
   //Details column:
@@ -209,7 +217,7 @@ void Dialog_Layout_Report::setup_model(Gtk::TreeView& treeview, Glib::RefPtr<typ
 
   Gtk::CellRendererText* renderer_details = Gtk::manage(new Gtk::CellRendererText);
   column_details->pack_start(*renderer_details);
-  column_details->set_cell_data_func(*renderer_details, 
+  column_details->set_cell_data_func(*renderer_details,
     sigc::bind( sigc::mem_fun(*this, &Dialog_Layout_Report::on_cell_data_details), model));
 
 
@@ -264,7 +272,7 @@ void Dialog_Layout_Report::fill_group_children(const sharedptr<LayoutGroup>& gro
       if(child_group)
         fill_group_children(child_group, iterChild, model);
 
-      //std::cout << "fill_report_parts(): Adding group child: parent part type=" << group->get_part_type_name() << ", child part type=" << item->get_part_type_name() << std::endl;
+      //std::cout << "debug: " << G_STRFUNC << ": Adding group child: parent part type=" << group->get_part_type_name() << ", child part type=" << item->get_part_type_name() << std::endl;
       group->add_item(item);
     }
 
@@ -342,8 +350,8 @@ void Dialog_Layout_Report::set_report(const Glib::ustring& table_name, const sha
   //Set the table name and title:
   m_label_table_name->set_text(table_name);
 
-  m_entry_name->set_text(report->get_name()); 
-  m_entry_title->set_text(report->get_title());
+  m_entry_name->set_text(report->get_name());
+  m_entry_title->set_text(item_get_title(report));
   m_checkbutton_table_title->set_active(report->get_show_table_title());
 
   //Update the tree models from the document
@@ -352,7 +360,7 @@ void Dialog_Layout_Report::set_report(const Glib::ustring& table_name, const sha
   {
 
 
-    //m_entry_table_title->set_text( document->get_table_title(table_name) );
+    //m_entry_table_title->set_text( document->get_table_title(table_name, AppWindow::get_current_locale()) );
 
     //document->fill_layout_field_details(m_table_name, mapGroups); //Update with full field information.
 
@@ -362,7 +370,7 @@ void Dialog_Layout_Report::set_report(const Glib::ustring& table_name, const sha
     m_model_parts_footer->clear();
 
     //Add most parts to main, adding any found header or footer chidlren to those other models:
-    add_group_children(m_model_parts_main, Gtk::TreeModel::iterator() /* null == top-level */, report->m_layout_group);
+    add_group_children(m_model_parts_main, Gtk::TreeModel::iterator() /* null == top-level */, report->get_layout_group());
 
     //treeview_fill_sequences(m_model_parts_main, m_model_parts_main->m_columns.m_col_sequence); //The document should have checked this already, but it does not hurt to check again.
   }
@@ -442,7 +450,7 @@ void Dialog_Layout_Report::enable_buttons()
       bool enable_formatting = false;
       sharedptr<LayoutItem_Field> item_field = sharedptr<LayoutItem_Field>::cast_dynamic(layout_item_parent);
       if(item_field)
-        enable_formatting = true; 
+        enable_formatting = true;
 
       m_button_formatting->set_sensitive(enable_formatting);
 
@@ -689,33 +697,23 @@ sharedptr<Relationship> Dialog_Layout_Report::offer_relationship_list()
 {
   sharedptr<Relationship> result;
 
-  try
+  Dialog_ChooseRelationship* dialog = 0;
+  Utils::get_glade_widget_derived_with_warning(dialog);
+  if(!dialog) //Unlikely and it already warns on stderr.
+    return result;
+
+  dialog->set_document(get_document(), m_table_name);
+  dialog->set_transient_for(*this);
+  const int response = dialog->run();
+  dialog->hide();
+  if(response == Gtk::RESPONSE_OK)
   {
-    Glib::RefPtr<Gtk::Builder> refXml = Gtk::Builder::create_from_file(Utils::get_glade_file_path("glom_developer.glade"), "dialog_choose_relationship");
-
-    Dialog_ChooseRelationship* dialog = 0;
-    refXml->get_widget_derived("dialog_choose_relationship", dialog);
-
-    if(dialog)
-    {
-      dialog->set_document(get_document(), m_table_name);
-      dialog->set_transient_for(*this);
-      int response = dialog->run();
-      dialog->hide();
-      if(response == Gtk::RESPONSE_OK)
-      {
-        //Get the chosen relationship:
-        result = dialog->get_relationship_chosen();
-      }
-
-      delete dialog;
-    }
-  }
-  catch(const Gtk::BuilderError& ex)
-  {
-    std::cerr << ex.what() << std::endl;
+    //Get the chosen relationship:
+    result = dialog->get_relationship_chosen();
   }
 
+  delete dialog;
+
   return result;
 }
 
@@ -793,7 +791,7 @@ void Dialog_Layout_Report::on_button_formatting()
       sharedptr<LayoutItem_Field> field = sharedptr<LayoutItem_Field>::cast_dynamic(item);
       if(field)
       {
-        sharedptr<LayoutItem_Field> field_chosen = offer_field_formatting(field, m_table_name, this);
+        sharedptr<LayoutItem_Field> field_chosen = offer_field_formatting(field, m_table_name, this, false /* no editing options */);
         if(field_chosen)
         {
           *field = *field_chosen;
@@ -826,43 +824,30 @@ void Dialog_Layout_Report::on_button_edit()
       sharedptr<LayoutItem_FieldSummary> fieldsummary = sharedptr<LayoutItem_FieldSummary>::cast_dynamic(item);
       if(fieldsummary)
       {
-        try
+        Dialog_FieldSummary* dialog = 0;
+        Utils::get_glade_widget_derived_with_warning(dialog);
+        add_view(dialog);
+        dialog->set_item(fieldsummary, m_table_name);
+        dialog->set_transient_for(*this);
+
+        const int response = dialog->run();
+        dialog->hide();
+
+        if(response == Gtk::RESPONSE_OK)
         {
-          Glib::RefPtr<Gtk::Builder> refXml = Gtk::Builder::create_from_file(Utils::get_glade_file_path("glom_developer.glade"), "dialog_field_summary");
-
-          Dialog_FieldSummary* dialog = 0;
-          refXml->get_widget_derived("dialog_field_summary", dialog);
-
-          if(dialog)
+          //Get the chosen relationship:
+          sharedptr<LayoutItem_FieldSummary> chosenitem = dialog->get_item();
+          if(chosenitem)
           {
-            add_view(dialog);
-            dialog->set_item(fieldsummary, m_table_name);
-            dialog->set_transient_for(*this);
+            *fieldsummary = *chosenitem; //TODO_Performance.
 
-            const int response = dialog->run();
-            dialog->hide();
-
-            if(response == Gtk::RESPONSE_OK)
-            {
-              //Get the chosen relationship:
-              sharedptr<LayoutItem_FieldSummary> chosenitem = dialog->get_item();
-              if(chosenitem)
-              {
-                *fieldsummary = *chosenitem; //TODO_Performance.
-
-                model->row_changed(Gtk::TreeModel::Path(iter), iter); //TODO: Add row_changed(iter) to gtkmm?
-                m_modified = true;
-              }
-            }
-
-            remove_view(dialog);
-            delete dialog;
+            model->row_changed(Gtk::TreeModel::Path(iter), iter); //TODO: Add row_changed(iter) to gtkmm?
+            m_modified = true;
           }
         }
-        catch(const Gtk::BuilderError& ex)
-        {
-          std::cerr << ex.what() << std::endl;
-        }
+
+        remove_view(dialog);
+        delete dialog;
       }
       else
       {
@@ -908,42 +893,32 @@ void Dialog_Layout_Report::on_button_edit()
               sharedptr<LayoutItem_GroupBy> group_by = sharedptr<LayoutItem_GroupBy>::cast_dynamic(item);
               if(group_by)
               {
-                try
+                Dialog_GroupBy* dialog = 0;
+                Utils::get_glade_widget_derived_with_warning(dialog);
+                if(!dialog)
+                  return;
+
+                add_view(dialog);
+                dialog->set_item(group_by, m_table_name);
+                dialog->set_transient_for(*this);
+
+                const int response = dialog->run();
+                dialog->hide();
+
+                if(response == Gtk::RESPONSE_OK)
                 {
-                  Glib::RefPtr<Gtk::Builder> refXml = Gtk::Builder::create_from_file(Utils::get_glade_file_path("glom_developer.glade"), "dialog_group_by");
-  
-                  Dialog_GroupBy* dialog = 0;
-                  refXml->get_widget_derived("dialog_group_by", dialog);
-  
-                  if(dialog)
+                  //Get the chosen relationship:
+                  sharedptr<LayoutItem_GroupBy> chosenitem = dialog->get_item();
+                  if(chosenitem)
                   {
-                    add_view(dialog);
-                    dialog->set_item(group_by, m_table_name);
-                    dialog->set_transient_for(*this);
-  
-                    const int response = dialog->run();
-                    dialog->hide();
-  
-                    if(response == Gtk::RESPONSE_OK)
-                    {
-                      //Get the chosen relationship:
-                      sharedptr<LayoutItem_GroupBy> chosenitem = dialog->get_item();
-                      if(chosenitem)
-                      {
-                        *group_by = *chosenitem;
-                        model->row_changed(Gtk::TreeModel::Path(iter), iter); //TODO: Add row_changed(iter) to gtkmm?
-                        m_modified = true;
-                      }
-                    }
-  
-                    remove_view(dialog);
-                    delete dialog;
+                    *group_by = *chosenitem;
+                    model->row_changed(Gtk::TreeModel::Path(iter), iter); //TODO: Add row_changed(iter) to gtkmm?
+                    m_modified = true;
                   }
                 }
-                catch(const Gtk::BuilderError& ex)
-                {
-                  std::cerr << ex.what() << std::endl;
-                }
+
+                remove_view(dialog);
+                delete dialog;
               }
             }
           }
@@ -1037,27 +1012,26 @@ Glib::ustring Dialog_Layout_Report::get_original_report_name() const
 sharedptr<Report> Dialog_Layout_Report::get_report()
 {
   m_report->set_name( m_entry_name->get_text() );
-  m_report->set_title( m_entry_title->get_text() );
+  m_report->set_title( m_entry_title->get_text() , AppWindow::get_current_locale());
   m_report->set_show_table_title( m_checkbutton_table_title->get_active() );
 
-  m_report->m_layout_group->remove_all_items();
-
-  m_report->m_layout_group->remove_all_items();
+  sharedptr<LayoutGroup> group = m_report->get_layout_group();
+  group->remove_all_items();
 
   //The Header and Footer parts are implicit (they are the whole header or footer treeview)
   sharedptr<LayoutItem_Header> header = sharedptr<LayoutItem_Header>::create();
   sharedptr<LayoutGroup> group_temp = header;
   fill_report_parts(group_temp, m_model_parts_header);
   if(header->get_items_count())
-    m_report->m_layout_group->add_item(header);
+    group->add_item(header);
 
-  fill_report_parts(m_report->m_layout_group, m_model_parts_main);
+  fill_report_parts(group, m_model_parts_main);
 
   sharedptr<LayoutItem_Footer> footer = sharedptr<LayoutItem_Footer>::create();
   group_temp = footer;
   fill_report_parts(group_temp, m_model_parts_footer);
   if(footer->get_items_count())
-    m_report->m_layout_group->add_item(footer);
+    group->add_item(footer);
 
   return m_report;
 }
@@ -1084,7 +1058,7 @@ void Dialog_Layout_Report::fill_report_parts(sharedptr<LayoutGroup>& group, cons
   }
 }
 
-void Dialog_Layout_Report::on_notebook_switch_page(GtkNotebookPage*, guint page_number)
+void Dialog_Layout_Report::on_notebook_switch_page(Gtk::Widget*, guint page_number)
 {
   //Change the list of available parts, depending on the destination treeview:
   Glib::RefPtr<type_model> model_available_parts;
