@@ -14,24 +14,28 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #include "dialog_copy_translation.h"
 #include <libglom/utils.h> //For bold_message()).
+#include <glom/appwindow.h>
 #include <glibmm/i18n.h>
 
 namespace Glom
 {
 
+const char* Dialog_CopyTranslation::glade_id("dialog_translation_copy");
+const bool Dialog_CopyTranslation::glade_developer(true);
+
 Dialog_CopyTranslation::Dialog_CopyTranslation(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& builder)
 : Gtk::Dialog(cobject),
   m_combo_locale(0)
 {
   builder->get_widget_derived("combobox_locale", m_combo_locale);
-
-  m_combo_locale->set_selected_locale(TranslatableItem::get_current_locale());
+  if(m_combo_locale)
+    m_combo_locale->set_selected_locale(AppWindow::get_current_locale());
 }
 
 Dialog_CopyTranslation::~Dialog_CopyTranslation()
