@@ -14,25 +14,30 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #include "dialog_groups_list.h"
 #include "dialog_user.h"
 #include "dialog_users_list.h"
 #include "dialog_new_group.h"
-#include <glom/libglom/standard_table_prefs_fields.h>
-#include <glom/libglom/glade_utils.h>
-#include <glom/glom_privs.h>
+#include <libglom/standard_table_prefs_fields.h>
+#include <glom/glade_utils.h>
+#include <glom/appwindow.h>
+#include <libglom/privs.h>
 //#include <libgnome/gnome-i18n.h>
-#include <bakery/App/App_Gtk.h> //For util_bold_message().
+#include <glom/utils_ui.h> //For bold_message()).
+#include <libglom/db_utils.h>
 #include <glibmm/i18n.h>
 
 namespace Glom
 {
 
-Dialog_GroupsList::Dialog_GroupsList(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade)
+const char* Dialog_GroupsList::glade_id("window_groups");
+const bool Dialog_GroupsList::glade_developer(true);
+
+Dialog_GroupsList::Dialog_GroupsList(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& builder)
 : Gtk::Dialog(cobject),
   m_treeview_groups(0),
   m_button_group_new(0),
@@ -41,13 +46,14 @@ Dialog_GroupsList::Dialog_GroupsList(BaseObjectType* cobject, const Glib::RefPtr
 {
   //set_default_size(600, -1);
 
-  refGlade->get_widget("treeview_groups", m_treeview_groups);
-  refGlade->get_widget("treeview_tables", m_treeview_tables);
+  builder->get_widget("treeview_groups", m_treeview_groups);
+  builder->get_widget("treeview_tables", m_treeview_tables);
 
   m_model_groups = Gtk::ListStore::create(m_model_columns_groups);
   m_model_tables = Gtk::ListStore::create(m_model_columns_tables);
 
-  fill_group_list();
+  //Do this only in load_from_document(), to avoid ever doing it too early, before there is a connection:
+  //fill_group_list();
   //fill_table_list();
 
   m_treeview_groups->set_model(m_model_groups);
@@ -92,13 +98,13 @@ Dialog_GroupsList::Dialog_GroupsList(BaseObjectType* cobject, const Glib::RefPtr
     refSelection->signal_changed().connect( sigc::mem_fun(*this, &Dialog_GroupsList::on_treeview_tables_selection_changed) );
   }
 
-  refGlade->get_widget("button_group_new", m_button_group_new);
+  builder->get_widget("button_group_new", m_button_group_new);
   m_button_group_new->signal_clicked().connect( sigc::mem_fun(*this, &Dialog_GroupsList::on_button_group_new) );
 
-  refGlade->get_widget("button_group_delete", m_button_group_delete);
+  builder->get_widget("button_group_delete", m_button_group_delete);
   m_button_group_delete->signal_clicked().connect( sigc::mem_fun(*this, &Dialog_GroupsList::on_button_group_delete) );
 
-  refGlade->get_widget("button_group_users", m_button_group_users);
+  builder->get_widget("button_group_users", m_button_group_users);
   m_button_group_users->signal_clicked().connect( sigc::mem_fun(*this, &Dialog_GroupsList::on_button_group_users) );
 
   enable_buttons();
@@ -111,7 +117,7 @@ Dialog_GroupsList::~Dialog_GroupsList()
 }
 
 /*
-void Dialog_GroupsList::set_document(const Glib::ustring& layout, Document_Glom* document, const Glib::ustring& table_name, const type_vecLayoutFields& table_fields)
+void Dialog_GroupsList::set_document(const Glib::ustring& layout, Document* document, const Glib::ustring& table_name, const type_vecLayoutFields& table_fields)
 {
   m_modified = false;
 
@@ -123,9 +129,9 @@ void Dialog_GroupsList::set_document(const Glib::ustring& layout, Document_Glom*
   {
     //Set the table name and title:
     m_label_table_name->set_text(table_name);
-    m_entry_table_title->set_text( document->get_table_title(table_name) );
+    m_entry_table_title->set_text( document->get_table_title(table_name, AppWindow::get_current_locale()) );
 
-    Document_Glom::type_list_layout_groups mapGroups = document->get_data_layout_groups_plus_new_fields(layout, m_table_name, m_layout_platform);
+    Document::type_list_layout_groups mapGroups = document->get_data_layout_groups_plus_new_fields(layout, m_table_name, m_layout_platform);
 
     //If no information is stored in the document, then start with something:
 
@@ -153,7 +159,7 @@ void Dialog_GroupsList::set_document(const Glib::ustring& layout, Document_Glom*
 
     m_model_groups->clear();
 
-    for(Document_Glom::type_list_layout_groups::const_iterator iter = mapGroups.begin(); iter != mapGroups.end(); ++iter)
+    for(Document::type_list_layout_groups::const_iterator iter = mapGroups.begin(); iter != mapGroups.end(); ++iter)
     {
       sharedptr<const LayoutGroup> group = iter->second;
 
@@ -209,7 +215,7 @@ void Dialog_GroupsList::on_button_group_delete()
       if(!group.empty())
       {
         //TODO: Prevent deletion of standard groups
-        Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("Delete Group")), true, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_OK_CANCEL);
+        Gtk::MessageDialog dialog(Utils::bold_message(_("Delete Group")), true, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_OK_CANCEL);
         dialog.set_secondary_text(_("Are your sure that you wish to delete this group?"));
         dialog.set_transient_for(*this);
 
@@ -218,10 +224,10 @@ void Dialog_GroupsList::on_button_group_delete()
 
         if(response == Gtk::RESPONSE_OK)
         {
-          Glib::ustring strQuery = "DROP GROUP \"" + group + "\"";
-          const bool test = query_execute(strQuery);
+          const Glib::ustring strQuery = "DROP GROUP " + DbUtils::escape_sql_id(group);
+          const bool test = DbUtils::query_execute_string(strQuery);
           if(!test)
-            std::cerr << "Box_DB_Table_Definition::on_adddel_delete(): DROP GROUP failed." << std::endl;
+            std::cerr << G_STRFUNC << ": DROP GROUP failed." << std::endl;
 
           fill_group_list();
         }
@@ -235,19 +241,12 @@ void Dialog_GroupsList::on_button_group_delete()
 void Dialog_GroupsList::on_button_group_new()
 {
   Dialog_NewGroup* dialog = 0;
-  try
-  {
-    Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(Utils::get_glade_file_path("glom_developer.glade"), "dialog_new_group");
-
-    refXml->get_widget_derived("dialog_new_group", dialog);
-  }
-  catch(const Gnome::Glade::XmlError& ex)
-  {
-    std::cerr << ex.what() << std::endl;
-  }
-
+  Utils::get_glade_widget_derived_with_warning(dialog);
+  if(!dialog) //Unlikely and it already warns on stderr.
+    return;
+    
   dialog->set_transient_for(*this);
-  int response = Glom::Utils::dialog_run_with_help(dialog, "dialog_new_group");
+  const int response = Glom::Utils::dialog_run_with_help(dialog);
 
   const Glib::ustring group_name = dialog->m_entry_name->get_text();
 
@@ -256,30 +255,18 @@ void Dialog_GroupsList::on_button_group_new()
   if(response != Gtk::RESPONSE_OK)
     return;
 
-  if(!group_name.empty())
+  if(group_name.empty())
   {
-    const Glib::ustring strQuery = "CREATE GROUP \"" + group_name + "\"";
-    const bool test = query_execute(strQuery);
-    if(!test)
-      std::cout << "Dialog_GroupsList::on_button_group_new(): CREATE GROUP failed." << std::endl;
-
-    //Give the new group some sensible default privileges:
-    Privileges priv;
-    priv.m_view = true;
-    priv.m_edit = true;
-
-    Document_Glom::type_listTableInfo table_list = get_document()->get_tables(true /* plus system prefs */);
-
-    for(Document_Glom::type_listTableInfo::const_iterator iter = table_list.begin(); iter != table_list.end(); ++iter)
-    {
-      Privs::set_table_privileges(group_name, (*iter)->get_name(), priv);
-    }
-
-    //Let them edit the autoincrements too:
-    Privs::set_table_privileges(group_name, GLOM_STANDARD_TABLE_AUTOINCREMENTS_TABLE_NAME, priv);
-
-    fill_group_list();
+    std::cerr << ": group_name is empty" << std::endl;
+    return;
   }
+
+  if(!DbUtils::add_group(get_document(), group_name))
+  {
+    std::cerr << ": DbUtils::add_group() failed." << std::endl;
+  }
+
+  fill_group_list();
 }
 
 void Dialog_GroupsList::on_button_group_users()
@@ -296,23 +283,17 @@ void Dialog_GroupsList::on_button_group_users()
       const Glib::ustring group_name = row[m_model_columns_groups.m_col_name];
 
       Dialog_UsersList* dialog = 0;
-      try
-      {
-        Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(Utils::get_glade_file_path("glom_developer.glade"), "window_users");
-
-        refXml->get_widget_derived("window_users", dialog);
-      }
-      catch(const Gnome::Glade::XmlError& ex)
-      {
-        std::cerr << ex.what() << std::endl;
-      }
-
+      Utils::get_glade_widget_derived_with_warning(dialog);
+      if(!dialog) //Unlikely and it already warns on stderr.
+        return;
+    
       dialog->set_transient_for(*this);
       add_view(dialog); //Give it access to the document.
 
+      dialog->fill_list();
       dialog->set_group(group_name);
 
-      Glom::Utils::dialog_run_with_help(dialog, "window_groups");
+      Glom::Utils::dialog_run_with_help(dialog);
 
       remove_view(dialog);
       delete dialog;
@@ -356,7 +337,9 @@ Glib::ustring Dialog_GroupsList::get_selected_group() const
 void Dialog_GroupsList::on_treeview_groups_selection_changed()
 {
   //Update the tables list for the currently-selected group:
-  fill_table_list( get_selected_group() );
+  const Glib::ustring group_name = get_selected_group();
+  if(!group_name.empty()) //This can happen when clearing the list, just before filling it, so something real can be selected.
+    fill_table_list(group_name);
 
   enable_buttons();
 }
@@ -366,8 +349,8 @@ void Dialog_GroupsList::fill_group_list()
   //Fill the model rows:
   m_model_groups->clear();
 
-  type_vecStrings group_list = Privs::get_database_groups();
-  for(type_vecStrings::const_iterator iter = group_list.begin(); iter != group_list.end(); ++iter)
+  type_vec_strings group_list = Privs::get_database_groups();
+  for(type_vec_strings::const_iterator iter = group_list.begin(); iter != group_list.end(); ++iter)
   {
     Gtk::TreeModel::iterator iterTree = m_model_groups->append();
     Gtk::TreeModel::Row row = *iterTree;
@@ -395,10 +378,15 @@ void Dialog_GroupsList::fill_group_list()
 
 void Dialog_GroupsList::fill_table_list(const Glib::ustring& group_name)
 {
+  if(group_name.empty())
+  {
+    std::cerr << G_STRFUNC << ": group_name is empty." << std::endl;
+  }
+
   //Fill the model rows:
   m_model_tables->clear();
 
-  Document_Glom* pDocument = get_document();
+  Document* pDocument = get_document();
   if(pDocument)
   {
     // Make sure that these are in the document,
@@ -406,9 +394,9 @@ void Dialog_GroupsList::fill_table_list(const Glib::ustring& group_name)
     GroupInfo group_info;
     group_info.set_name(group_name);
 
-    Document_Glom::type_listTableInfo table_list = pDocument->get_tables(true /* plus system prefs */);
+    Document::type_listTableInfo table_list = pDocument->get_tables(true /* plus system prefs */);
 
-    for(Document_Glom::type_listTableInfo::const_iterator iter = table_list.begin(); iter != table_list.end(); ++iter)
+    for(Document::type_listTableInfo::const_iterator iter = table_list.begin(); iter != table_list.end(); ++iter)
     {
       Gtk::TreeModel::iterator iterTree = m_model_tables->append();
       Gtk::TreeModel::Row row = *iterTree;
@@ -416,7 +404,7 @@ void Dialog_GroupsList::fill_table_list(const Glib::ustring& group_name)
       const Glib::ustring table_name = (*iter)->get_name();
 
       row[m_model_columns_tables.m_col_name] = table_name;
-      row[m_model_columns_tables.m_col_title] = (*iter)->get_title_or_name();
+      row[m_model_columns_tables.m_col_title] = item_get_title_or_name(*iter);
 
       const Privileges privs = Privs::get_table_privileges(group_name, table_name);
       row[m_model_columns_tables.m_col_view] = privs.m_view;
@@ -435,7 +423,7 @@ void Dialog_GroupsList::fill_table_list(const Glib::ustring& group_name)
 void Dialog_GroupsList::load_from_document()
 {
   //Ensure that the glom_developer group exists.
-  add_standard_groups();
+  DbUtils::add_standard_groups(get_document());
 
   fill_group_list();
   //fill_table_list();
@@ -456,6 +444,7 @@ void Dialog_GroupsList::treeview_append_bool_column(Gtk::TreeView& treeview, con
   pCellRenderer->signal_toggled().connect( slot_toggled );
 }
 
+//TODO: Use Privs::set_table_privileges() instead?
 bool Dialog_GroupsList::set_table_privilege(const Glib::ustring& table_name, const Glib::ustring& group_name, bool grant, enumPriv priv)
 {
   if(group_name.empty() || table_name.empty())
@@ -486,7 +475,7 @@ bool Dialog_GroupsList::set_table_privilege(const Glib::ustring& table_name, con
   else if(priv == PRIV_DELETE)
     strPrivilege = "DELETE";
 
-  strQuery += " " + strPrivilege + " ON \"" + table_name + "\" ";
+  strQuery += " " + strPrivilege + " ON " + DbUtils::escape_sql_id(table_name) + " ";
 
   //This must match the Grant or Revoke:
   if(grant)
@@ -494,11 +483,11 @@ bool Dialog_GroupsList::set_table_privilege(const Glib::ustring& table_name, con
   else
     strQuery += "FROM";
 
-  strQuery += " GROUP \"" + group_name + "\"";
+  strQuery += " GROUP " + DbUtils::escape_sql_id(group_name);
 
-  const bool test = query_execute(strQuery); //TODO: Handle errors.
+  const bool test = DbUtils::query_execute_string(strQuery); //TODO: Handle errors.
   if(!test)
-    std::cerr << "Dialog_GroupsList::set_table_privilege(): GRANT/REVOKE failed." << std::endl;
+    std::cerr << G_STRFUNC << ": GRANT/REVOKE failed." << std::endl;
 
   return test;
 }
@@ -632,5 +621,3 @@ void Dialog_GroupsList::on_cell_data_group_name(Gtk::CellRenderer* renderer, con
 }
 
 } //namespace Glom
-
-
