@@ -14,15 +14,17 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #ifndef GLOM_MODE_DESIGN_DIALOG_GROUPS_LIST_H
 #define GLOM_MODE_DESIGN_DIALOG_GROUPS_LIST_H
 
-#include <gtkmm.h>
+#include <gtkmm/dialog.h>
+#include <gtkmm/treeview.h>
 #include <gtkmm/builder.h>
+#include <gtkmm/liststore.h>
 #include <glom/base_db.h>
 
 namespace Glom
