@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #include "dialog_users_list.h"
@@ -44,7 +44,7 @@ Dialog_UsersList::Dialog_UsersList(BaseObjectType* cobject, const Glib::RefPtr<G
   m_button_user_delete(0),
   m_button_user_edit(0)
 {
-  builder->get_widget_derived("combobox_group", m_combo_group);
+  builder->get_widget("combobox_group", m_combo_group);
   m_combo_group->signal_changed().connect(sigc::mem_fun(*this, &Dialog_UsersList::on_combo_group_changed));
 
 
@@ -59,8 +59,6 @@ Dialog_UsersList::Dialog_UsersList(BaseObjectType* cobject, const Glib::RefPtr<G
 
     m_model_users = Gtk::ListStore::create(m_model_columns_users);
 
-    fill_list();
-
     m_treeview_users->set_model(m_model_users);
 
 
@@ -133,7 +131,7 @@ void Dialog_UsersList::on_button_user_remove()
         const Glib::ustring user = row[m_model_columns_users.m_col_name];
         if(!user.empty())
         {
-          remove_user_from_group(user, m_combo_group->get_active_text());
+          DbUtils::remove_user_from_group(user, m_combo_group->get_active_text());
 
           fill_list();
         }
@@ -169,7 +167,7 @@ void Dialog_UsersList::on_button_user_delete()
 
           if(response == Gtk::RESPONSE_OK)
           {
-            remove_user(user);
+            DbUtils::remove_user(user); //TODO: Warn about failure when this returns false?
             fill_list();
           }
         }
@@ -182,13 +180,23 @@ void Dialog_UsersList::on_button_user_delete()
 
 void Dialog_UsersList::on_button_user_add()
 {
+  //Fill it with the list of users:
+  const Privs::type_vec_strings users = Privs::get_database_users();
+  if(users.empty())
+  {
+    Utils::show_ok_dialog(_("Error Retrieving the List of Users"),
+      _("Glom could not get the list of users from the database server. You probably do not have enough permissions. You should be a superuser."), *this, Gtk::MESSAGE_ERROR);
+    return;
+  }
+
   Dialog_ChooseUser* dialog = 0;
   Utils::get_glade_widget_derived_with_warning(dialog);
+  if(!dialog) //Unlikely and it already warns on stderr.
+    return;
 
   dialog->set_transient_for(*this);
 
-  //Fill it with the list of users:
-  dialog->set_user_list( Privs::get_database_users() );
+  dialog->set_user_list(users);
 
   const int response = Glom::Utils::dialog_run_with_help(dialog);
 
@@ -202,20 +210,21 @@ void Dialog_UsersList::on_button_user_add()
   if(!user.empty())
   {
     //Add it to the group:
-    const Glib::ustring strQuery = "ALTER GROUP \"" + m_combo_group->get_active_text() + "\" ADD USER \"" + user + "\"";
+    const Glib::ustring strQuery = DbUtils::build_query_add_user_to_group(m_combo_group->get_active_text(), user);
     const bool test = DbUtils::query_execute_string(strQuery);
     if(!test)
-      std::cerr << "Dialog_UsersList::on_button_user_add(): ALTER GROUP failed." << std::endl;
+      std::cerr << G_STRFUNC << ": ALTER GROUP failed." << std::endl;
 
     //Remove any user rights, so that all rights come from the user's presence in the group:
     Document::type_listTableInfo table_list = get_document()->get_tables();
 
     for(Document::type_listTableInfo::const_iterator iter = table_list.begin(); iter != table_list.end(); ++iter)
     {
-      const Glib::ustring strQuery = "REVOKE ALL PRIVILEGES ON \"" + (*iter)->get_name() + "\" FROM \"" + user + "\"";
+      const Glib::ustring table_name = (*iter)->get_name();
+      const Glib::ustring strQuery = "REVOKE ALL PRIVILEGES ON " + DbUtils::escape_sql_id(table_name) + " FROM " + DbUtils::escape_sql_id(user);
       const bool test = DbUtils::query_execute_string(strQuery);
       if(!test)
-        std::cerr << "Dialog_UsersList::on_button_user_add(): REVOKE failed." << std::endl;
+        std::cerr << G_STRFUNC << ": REVOKE failed." << std::endl;
     }
 
     fill_list();
@@ -226,6 +235,8 @@ void Dialog_UsersList::on_button_user_new()
 {
   Dialog_User* dialog = 0;
   Utils::get_glade_widget_derived_with_warning(dialog);
+  if(!dialog) //Unlikely and it already warns on stderr.
+    return;
 
   dialog->set_transient_for(*this);
   dialog->m_combo_group->set_sensitive(false); //It is being added to the current group, so don't offer a different group.
@@ -255,7 +266,7 @@ void Dialog_UsersList::on_button_user_new()
   if(response != Gtk::RESPONSE_OK)
     return;
 
-  add_user(user, password, m_combo_group->get_active_text() /* group */);
+  DbUtils::add_user(get_document(), user, password, m_combo_group->get_active_text() /* group */);
   fill_list();
 }
 
@@ -272,6 +283,8 @@ void Dialog_UsersList::on_button_user_edit()
 
       Dialog_User* dialog = 0;
       Utils::get_glade_widget_derived_with_warning(dialog);
+      if(!dialog) //Unlikely and it already warns on stderr.
+        return;
 
       dialog->set_transient_for(*this);
 
@@ -280,12 +293,12 @@ void Dialog_UsersList::on_button_user_edit()
       dialog->m_entry_user->set_sensitive(false); //They can edit the password, but not the name. TODO: Allow editing of name?
 
       //Fill groups:
-      dialog->m_combo_group->clear_items();
+      dialog->m_combo_group->remove_all();
 
       type_vec_strings group_list = Privs::get_database_groups();
       for(type_vec_strings::const_iterator iter = group_list.begin(); iter != group_list.end(); ++iter)
       {
-         dialog->m_combo_group->append_text(*iter);
+         dialog->m_combo_group->append(*iter);
       }
 
       dialog->m_combo_group->set_active_text(m_combo_group->get_active_text());
@@ -319,10 +332,10 @@ void Dialog_UsersList::on_button_user_edit()
 
       if(!user.empty() && !password.empty())
       {
-        const Glib::ustring strQuery = "ALTER USER \"" + user + "\" PASSWORD '" + password + "'" ; //TODO: Escape the password.
+        const Glib::ustring strQuery = "ALTER USER " + DbUtils::escape_sql_id(user) + " PASSWORD '" + password + "'" ; //TODO: Escape the password.
         const bool test = DbUtils::query_execute_string(strQuery);
         if(!test)
-          std::cerr << "Dialog_UsersList::on_button_user_edit(): ALTER USER failed." << std::endl;
+          std::cerr << G_STRFUNC << ": ALTER USER failed." << std::endl;
 
         //Change the password in the current connection, if this is the current user.
         ConnectionPool* connection_pool = ConnectionPool::get_instance();
@@ -373,12 +386,12 @@ void Dialog_UsersList::fill_list()
 void Dialog_UsersList::set_group(const Glib::ustring& group_name)
 {
   //Fill the list of groups:
-  m_combo_group->clear_items();
+  m_combo_group->remove_all();
 
   type_vec_strings group_list = Privs::get_database_groups();
   for(type_vec_strings::const_iterator iter = group_list.begin(); iter != group_list.end(); ++iter)
   {
-    m_combo_group->append_text(*iter);
+    m_combo_group->append(*iter);
   }
 
   m_combo_group->set_active_text(group_name);
