@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #include <libglom/libglom_config.h> // For GLOM_ENABLE_CLIENT_ONLY
@@ -30,17 +30,18 @@ Box_Data_Details_Find::Box_Data_Details_Find()
 : Box_Data_Details(false)
 {
   //Instead of nav buttons:
-  #ifndef GLOM_ENABLE_MAEMO //TODO_Maemo: Really have this buttons somewhere?
   m_hbox_buttons.pack_end(m_Button_Find, Gtk::PACK_SHRINK);
-  #endif
 
-  g_object_set(m_Button_Find.gobj(), "can-default", TRUE, (gpointer)0); //TODO: Make this a real method in gtkmm?
+  //A signal handler is connected in the Box_Data base class.
+  m_Button_Find.set_can_default();
 
 #ifndef GLOM_ENABLE_CLIENT_ONLY
   //Hide this because it is useless for Find mode:
   show_layout_toolbar(false);
 #endif
 
+  m_FlowTable.set_find_mode();
+
   show_all_children();
 }
 
@@ -76,6 +77,7 @@ bool Box_Data_Details_Find::fill_from_database()
   return result;
 }
 
+//TODO: Remove this?
 void Box_Data_Details_Find::fill_related()
 {
   //Clear existing pages:
@@ -85,7 +87,7 @@ void Box_Data_Details_Find::fill_related()
   Document::type_vec_relationships vecRelationships = get_document()->get_relationships(m_table_name);
 
   //Add the relationships:
-  for(Document::type_vec_relationships::iterator iter = vecRelationships.begin(); iter != vecRelationships.end(); iter++)
+  for(Document::type_vec_relationships::iterator iter = vecRelationships.begin(); iter != vecRelationships.end(); ++iter)
   {
      /*
      const Relationship& relationship = *iter;
