@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #ifndef GLOM_BOX_DATA_DETAILS_FIND_H
@@ -32,7 +32,7 @@ public:
   Box_Data_Details_Find();
   virtual ~Box_Data_Details_Find();
 
-  virtual bool init_db_details(const Glib::ustring& table_name, const Glib::ustring& layout_platform);
+  bool init_db_details(const Glib::ustring& table_name, const Glib::ustring& layout_platform);
 
   virtual Gtk::Widget* get_default_button(); //override
 
