@@ -14,25 +14,27 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
-#include "box_data_list_find.h"
+#include <glom/mode_find/box_data_list_find.h>
+#include <glom/utils_ui.h>
 #include <glibmm/i18n.h>
 
 namespace Glom
 {
 
 Box_Data_List_Find::Box_Data_List_Find()
-: m_HBox(false, Glom::Utils::DEFAULT_SPACING_SMALL)
+: m_HBox(Gtk::ORIENTATION_HORIZONTAL, Glom::Utils::DEFAULT_SPACING_SMALL)
 {
   //m_strHint = _("Enter the search criteria and click [Find]\n Glom will then change to Data mode to display the results.");
 
   m_HBox.pack_end(m_Button_Find, Gtk::PACK_SHRINK);
   pack_start(m_HBox, Gtk::PACK_SHRINK);
 
-  g_object_set(m_Button_Find.gobj(), "can-default", TRUE, (gpointer)NULL); //TODO: Make this a real method in gtkmm?
+  //A signal handler is connected in the Box_Data base class.
+  m_Button_Find.set_can_default();
 
   //Prevent the widget from trying to add or change records:
   m_AddDel.set_find_mode();
@@ -51,7 +53,7 @@ void Box_Data_List_Find::create_layout()
 
 bool Box_Data_List_Find::fill_from_database()
 {
-  Bakery::BusyCursor busy_cursor(get_app_window());
+  BusyCursor busy_cursor(get_app_window());
 
   const bool result = Base_DB_Table_Data::fill_from_database();
   if(!result)
@@ -62,9 +64,6 @@ bool Box_Data_List_Find::fill_from_database()
 
   m_FieldsShown = get_fields_to_show();
 
-  m_AddDel.set_columns_ready(); //Actually create the columns.
-  //m_AddDel.add_item(Glib::ValueBase("find"));
-
   return result;
 }
 
