@@ -14,21 +14,27 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #include <glom/navigation/box_tables.h>
 #include <glom/utils_ui.h> //For bold_message()).
-#include <glom/application.h>
+#include <libglom/db_utils.h>
+#include <glom/appwindow.h>
+#include <gtkmm/stock.h>
 #include <glibmm/i18n.h>
 
+#include <iostream>
+
 namespace Glom
 {
 
+const char* Box_Tables::glade_id("box_navigation_tables");
+const bool Box_Tables::glade_developer(false);
+
 Box_Tables::Box_Tables(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& builder)
 : Box_WithButtons(cobject, builder),
-  m_pLabelFrameTitle(0),
   m_pCheckButtonShowHidden(0),
   m_colTableName(0),
   m_colHidden(0),
@@ -38,7 +44,7 @@ Box_Tables::Box_Tables(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>
 {
   //Get the Glade-instantiated widgets, and connect signal handlers:
   Gtk::Button* pButtonCancel = 0;
-  builder->get_widget("button_cancel", pButtonCancel);
+  builder->get_widget("button_cancel_tables", pButtonCancel);
   set_button_cancel(*pButtonCancel);
 
   // Set a name for the AddDel TreeView, so it can be accessed by LDTP
@@ -48,8 +54,6 @@ Box_Tables::Box_Tables(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>
   builder->get_widget("alignment_placeholder_adddel", pAligmentPlaceholder);
   pAligmentPlaceholder->add(m_AddDel);
 
-  builder->get_widget("label_frame_title", m_pLabelFrameTitle);
-
   builder->get_widget("checkbutton_show_hidden", m_pCheckButtonShowHidden);
   m_pCheckButtonShowHidden->signal_toggled().connect(sigc::mem_fun(*this, &Box_Tables::on_show_hidden_toggled));
 
@@ -71,7 +75,7 @@ void Box_Tables::fill_table_row(const Gtk::TreeModel::iterator& iter, const shar
 {
   if(!table_info)
   {
-    std::cerr << "Box_Tables::fill_table_row(): table_info was null." << std::endl;
+    std::cerr << G_STRFUNC << ": table_info was null." << std::endl;
     return;
   }
   
@@ -81,27 +85,27 @@ void Box_Tables::fill_table_row(const Gtk::TreeModel::iterator& iter, const shar
 
     m_AddDel.set_value_key(iter, table_info->get_name());
     m_AddDel.set_value(iter, m_colTableName, table_info->get_name());
-    m_AddDel.set_value(iter, m_colHidden, table_info->m_hidden);
+    m_AddDel.set_value(iter, m_colHidden, table_info->get_hidden());
 
     if(developer_mode)
     {
-      //std::cout << "Box_Tables::fill_table_row(): dev title=" << table_info->get_title() << std::endl;
-      m_AddDel.set_value(iter, m_colTitle, table_info->get_title());
-      m_AddDel.set_value(iter, m_colTitleSingular, table_info->get_title_singular());
+      //std::cout << "debug: " << G_STRFUNC << ": dev title=" << table_info->get_title(AppWindow::get_current_locale()) << std::endl;
+      m_AddDel.set_value(iter, m_colTitle, item_get_title(table_info));
+      m_AddDel.set_value(iter, m_colTitleSingular, table_info->get_title_singular(AppWindow::get_current_locale()));
     }
     else
     {
-      //std::cout << "Box_Tables::fill_table_row(): op get_title_or_name=" << table_info->get_title_or_name() << std::endl;
-      m_AddDel.set_value(iter, m_colTitle, table_info->get_title_or_name());
+      //std::cout << "debug: " << G_STRFUNC << ": op get_title_or_name=" << table_info->get_title_or_name(AppWindow::get_current_locale()) << std::endl;
+      m_AddDel.set_value(iter, m_colTitle, item_get_title_or_name(table_info));
     }
 
-    m_AddDel.set_value(iter, m_colDefault, table_info->m_default);
+    m_AddDel.set_value(iter, m_colDefault, table_info->get_default());
   }
 }
 
 bool Box_Tables::fill_from_database()
 {
-  BusyCursor busy_cursor(App_Glom::get_application());
+  BusyCursor busy_cursor(AppWindow::get_appwindow());
 
   bool result = Base_DB::fill_from_database();
 
@@ -125,7 +129,7 @@ bool Box_Tables::fill_from_database()
 
   const bool editable = developer_mode;
   const bool visible_extras = developer_mode;
-  m_colTableName = m_AddDel.add_column(_("Tables"), AddDelColumnInfo::STYLE_Text, editable, visible_extras);
+  m_colTableName = m_AddDel.add_column(_("Table"), AddDelColumnInfo::STYLE_Text, editable, visible_extras);
   m_AddDel.prevent_duplicates(m_colTableName); //Prevent two tables with the same name from being added.
   m_AddDel.set_prevent_duplicates_warning(_("This table already exists. Please choose a different table name"));
 
@@ -138,10 +142,6 @@ bool Box_Tables::fill_from_database()
   if(developer_mode)
     m_colTitleSingular = m_AddDel.add_column(_("Title (Singular Form)"), AddDelColumnInfo::STYLE_Text, editable, true);
 
-  //_("Server: ") +  m_strServerName + ", " + 
-  //Glib::ustring strTitle = Glib::ustring("<b>") + _("Tables from Database: ") + get_database_name() + "");
-  //m_pLabelFrameTitle->set_markup(strTitle);
-
   //Get the list of hidden tables:
 
   Document::type_listTableInfo listTablesDocument;
@@ -151,25 +151,19 @@ bool Box_Tables::fill_from_database()
     listTablesDocument = document->get_tables();
   }
   else
-    g_warning("Box_Tables::fill_from_database(): document is null");
+    std::cerr << G_STRFUNC << ": document is null" << std::endl;
 
   //Get the list of tables in the database, from the server:
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
-  sharedptr<SharedConnection> sharedconnection = connect_to_server(App_Glom::get_application());
-#else
-  std::auto_ptr<ExceptionConnection> error;
-  sharedptr<SharedConnection> sharedconnection = connect_to_server(App_Glom::get_application(), error);
-  // Ignore error because sharedconnection presence is checked below.
-#endif
+  sharedptr<SharedConnection> sharedconnection = connect_to_server(AppWindow::get_appwindow());
 
   if(sharedconnection)
   {
     m_AddDel.remove_all();
     Glib::RefPtr<Gnome::Gda::Connection> connection = sharedconnection->get_gda_connection();
 
-    const type_vec_strings vecTables = get_table_names_from_database();
+    const type_vec_strings vecTables = DbUtils::get_table_names_from_database();
 
-    for(type_vec_strings::const_iterator iter = vecTables.begin(); iter != vecTables.end(); iter++)
+    for(type_vec_strings::const_iterator iter = vecTables.begin(); iter != vecTables.end(); ++iter)
     {
       const Glib::ustring strName = *iter;
 
@@ -181,7 +175,7 @@ bool Box_Tables::fill_from_database()
       {
         table_info = *iterFind;
 
-        //std::cout << "fill_from_database(): name=" << (*iterFind)->get_name() << ", table_info->get_title()=" << (*iterFind)->get_title() << std::endl;
+        //std::cout << "debug: " << G_STRFUNC << ": name=" << (*iterFind)->get_name() << ", item_get_title(table_info)=" << item_get_title(*iterFind) << std::endl;
       }
       else
       {
@@ -189,10 +183,10 @@ bool Box_Tables::fill_from_database()
         //Show it as hidden:
         table_info = sharedptr<TableInfo>(new TableInfo());
         table_info->set_name(strName);
-        table_info->m_hidden = true;
+        table_info->set_hidden(true);
       }
 
-      const bool hidden = table_info->m_hidden;
+      const bool hidden = table_info->get_hidden();
 
       bool bAddIt = true;
       if(hidden && !developer_mode) //Don't add hidden tables unless we are in developer mode:
@@ -235,13 +229,13 @@ void Box_Tables::on_adddel_Add(const Gtk::TreeModel::iterator& row)
   bool created = false; 
 
   //Check whether it exists already. (Maybe it is somehow in the database but not in the document. That shouldn't happen.)
-  const bool exists_in_db = get_table_exists_in_database(table_name);
+  const bool exists_in_db = DbUtils::get_table_exists_in_database(table_name);
   if(exists_in_db)
   {
     //Ask the user if they want us to try to cope with this:
     Gtk::MessageDialog dialog(Utils::bold_message(_("Table Already Exists")), true, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_OK_CANCEL);
     dialog.set_secondary_text(_("This table already exists on the database server, though it is not mentioned in the .glom file. This should not happen. Would you like Glom to attempt to use the existing table?"));
-    dialog.set_transient_for(*App_Glom::get_application());
+    dialog.set_transient_for(*AppWindow::get_appwindow());
 
     const int response = dialog.run();
     dialog.hide();
@@ -254,7 +248,7 @@ void Box_Tables::on_adddel_Add(const Gtk::TreeModel::iterator& row)
   }
   else
   {
-    created = create_table_with_default_fields(table_name);
+    created = DbUtils::create_table_with_default_fields(get_document(), table_name);
   }
   
   if(created)
@@ -289,7 +283,7 @@ void Box_Tables::on_adddel_Delete(const Gtk::TreeModel::iterator& rowStart, cons
   bool something_changed = false;
   for(Gtk::TreeModel::iterator iter = rowStart; iter != iterAfter; ++iter)
   {
-    Glib::ustring table_name = m_AddDel.get_value_key(iter);
+    const Glib::ustring table_name = m_AddDel.get_value_key(iter);
 
     if(!table_name.empty())
     {
@@ -301,29 +295,53 @@ void Box_Tables::on_adddel_Delete(const Gtk::TreeModel::iterator& rowStart, cons
         {
            //TODO: Do not show tables that are not in the document.
            Gtk::MessageDialog dialog(_("You cannot delete this table, because there is no information about this table in the document."));
-           dialog.set_transient_for(*App_Glom::get_application());
+           dialog.set_transient_for(*AppWindow::get_appwindow());
            dialog.run();
         }
         else
         {
           //Ask the user to confirm:
-          Glib::ustring strMsg = _("Are you sure that you want to delete this table?\nTable name: ") + table_name;
+          const Glib::ustring strMsg = Glib::ustring::compose(_("Are you sure that you want to delete this table?\nTable name: %1"), table_name);
           Gtk::MessageDialog dialog(Utils::bold_message(_("Delete Table")), true);
           dialog.set_secondary_text(strMsg);
-          dialog.set_transient_for(*App_Glom::get_application());
-          int iButtonClicked = dialog.run();
+          dialog.set_transient_for(*AppWindow::get_appwindow());
+          const int iButtonClicked = dialog.run();
+          
+          //Get a list of autoincrementing fields in the table:
+          const Glom::Document::type_vec_fields fields = document->get_table_fields(table_name);
+   
 
           //Delete the table:
           if(iButtonClicked == Gtk::RESPONSE_OK)
           {
-            const bool test = query_execute( "DROP TABLE \"" + table_name + "\"");
+            const bool test = DbUtils::drop_table(table_name);
             if(!test)
-              std::cerr << "Box_Tables::on_adddel_Delete(): DROP TABLE failed." << std::endl;
+              std::cerr << G_STRFUNC << ": DROP TABLE failed." << std::endl;
             else
             {
-              get_document()->remove_table(table_name); //Forget about it in the document too.
+              //Forget about it in the document too.
+              get_document()->remove_table(table_name);
+              
               something_changed = true;
             }
+            
+            //Remove the auto-increment rows.
+            //Otherwise it would not start at 0 if a table with the same name, and same field, is added again later.
+            for(Glom::Document::type_vec_fields::const_iterator iter = fields.begin(); iter != fields.end(); ++iter)
+            {
+              const Glom::sharedptr<const Glom::Field> field = *iter;
+              if(!field || !field->get_auto_increment())
+                continue;
+                
+              const Glib::ustring field_name = field->get_name();
+            
+              if(!field_name.empty())
+                DbUtils::remove_auto_increment(table_name, field_name);
+              else
+              {
+                std::cerr << G_STRFUNC << ": field_name is empty" << std::endl;
+              }
+            }
           }
         }
       }
@@ -389,7 +407,7 @@ void Box_Tables::on_adddel_changed(const Gtk::TreeModel::iterator& row, guint co
         //Rename the table:
         if(iButtonClicked == Gtk::RESPONSE_OK)
         {
-          const bool test = query_execute( "ALTER TABLE \"" + table_name + "\" RENAME TO \"" + table_name_new + "\"");
+          const bool test = DbUtils::rename_table(table_name, table_name_new);
           if(test)
           {
             //Change the AddDel item's key:
@@ -424,7 +442,7 @@ void Box_Tables::on_adddel_Edit(const Gtk::TreeModel::iterator& row)
     {
        Gtk::MessageDialog dialog(Utils::bold_message(_("Unknown Table")), true);
        dialog.set_secondary_text(_("You cannot open this table, because there is no information about this table in the document."));
-       dialog.set_transient_for(*App_Glom::get_application());
+       dialog.set_transient_for(*AppWindow::get_appwindow());
        dialog.run();
     }
     else
@@ -459,11 +477,11 @@ void Box_Tables::save_to_document()
 
         if(!table_info->get_name().empty())
         {
-          table_info->m_hidden = m_AddDel.get_value_as_bool(iter, m_colHidden);
-          table_info->set_title( m_AddDel.get_value(iter, m_colTitle) ); //TODO_Translations: Store the TableInfo in the TreeView.
-          table_info->set_title_singular( m_AddDel.get_value(iter, m_colTitleSingular) ); //TODO_Translations: Store the TableInfo in the TreeView.
-          //std::cout << "save_to_document(): title=" << table_info->get_title() << std::endl;
-          table_info->m_default = m_AddDel.get_value_as_bool(iter, m_colDefault);
+          table_info->set_hidden( m_AddDel.get_value_as_bool(iter, m_colHidden) );
+          table_info->set_title( m_AddDel.get_value(iter, m_colTitle) , AppWindow::get_current_locale()); //TODO_Translations: Store the TableInfo in the TreeView.
+          table_info->set_title_singular( m_AddDel.get_value(iter, m_colTitleSingular), AppWindow::get_current_locale()); //TODO_Translations: Store the TableInfo in the TreeView.
+          //std::cout << "debug: " << G_STRFUNC << ": title=" << item_get_title(table_info) << std::endl;
+          table_info->set_default( m_AddDel.get_value_as_bool(iter, m_colDefault) );
 
           listTables.push_back(table_info);
         }
