@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #ifndef BOX_TABLES_H
@@ -25,6 +25,7 @@
 #include <glom/base_db.h>
 #include <glom/utility_widgets/adddel/adddel_withbuttons.h>
 #include <gtkmm/builder.h>
+#include <gtkmm/checkbutton.h>
 
 namespace Glom
 {
@@ -38,6 +39,9 @@ class Box_Tables
   public Base_DB
 {
 public:
+  static const char* glade_id;
+  static const bool glade_developer;
+
   Box_Tables(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& builder);
   virtual ~Box_Tables();
 
@@ -60,12 +64,12 @@ private:
 
   virtual void on_userlevel_changed(AppState::userlevels userlevel);
       
-  Gtk::Label* m_pLabelFrameTitle;
   Gtk::CheckButton* m_pCheckButtonShowHidden;
   guint m_colTableName;
   guint m_colHidden;
   guint m_colTitle;
   guint m_colDefault;
+  guint m_colTitleSingular;
 
   mutable AddDel_WithButtons m_AddDel; //mutable because its get_ methods aren't const.
 };
