@@ -14,13 +14,14 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #ifndef GLOM_NOTEBOOK_GLOM_H
 #define GLOM_NOTEBOOK_GLOM_H
 
+#include <glom/utility_widgets/notebook_noframe.h>
 #include <glom/box_withbuttons.h>
 #include <libglom/document/document.h>
 
@@ -30,7 +31,7 @@ namespace Glom
 /** Notebook with document methods.
   */
 class Notebook_Glom :
-  public Gtk::Notebook,
+  public NotebookNoFrame,
   public Base_DB
 {
 public: 
