@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #include <glom/utils_ui.h>
@@ -26,6 +26,7 @@
 #include <glom/utility_widgets/canvas/canvas_line_movable.h>
 #include <glom/utility_widgets/canvas/canvas_group_movable.h>
 #include <glom/utility_widgets/canvas/canvas_table_movable.h>
+#include <glom/appwindow.h>
 #include <libglom/data_structure/layout/layoutitem_button.h>
 #include <libglom/data_structure/layout/layoutitem_text.h>
 #include <libglom/data_structure/layout/layoutitem_image.h>
@@ -48,14 +49,6 @@ CanvasLayoutItem::CanvasLayoutItem()
   signal_resized().connect( sigc::mem_fun(*this, &CanvasLayoutItem::on_resized) );
 }
 
-CanvasLayoutItem::CanvasLayoutItem(const sharedptr<LayoutItem>& layout_item)
-{
-  set_layout_item(layout_item);
-
-  //Rescale images when the canvas item is resized:
-  signal_resized().connect( sigc::mem_fun(*this, &CanvasLayoutItem::on_resized) );
-}
-
 CanvasLayoutItem::~CanvasLayoutItem()
 {
 }
@@ -65,11 +58,6 @@ Glib::RefPtr<CanvasLayoutItem> CanvasLayoutItem::create()
   return Glib::RefPtr<CanvasLayoutItem>(new CanvasLayoutItem());
 }
 
-Glib::RefPtr<CanvasLayoutItem> CanvasLayoutItem::create(const sharedptr<LayoutItem>& layout_item)
-{
-  return Glib::RefPtr<CanvasLayoutItem>(new CanvasLayoutItem(layout_item));
-}
-
 sharedptr<LayoutItem> CanvasLayoutItem::get_layout_item()
 {
   return m_layout_item;
@@ -84,25 +72,18 @@ void CanvasLayoutItem::apply_formatting(const Glib::RefPtr<CanvasTextMovable>& c
     return;
 
   //Horizontal alignment:
-  const FieldFormatting::HorizontalAlignment alignment =
+  const Formatting::HorizontalAlignment alignment =
     layout_item->get_formatting_used_horizontal_alignment();
-  const Pango::Alignment x_align = (alignment == FieldFormatting::HORIZONTAL_ALIGNMENT_LEFT ? Pango::ALIGN_LEFT : Pango::ALIGN_RIGHT);
-#ifdef GLIBMM_PROPERTIES_ENABLED
+  const Pango::Alignment x_align = (alignment == Formatting::HORIZONTAL_ALIGNMENT_LEFT ? Pango::ALIGN_LEFT : Pango::ALIGN_RIGHT);
   canvas_item->property_alignment() = x_align;
-#else
-  canvas_item->set_property("alignment", alignment);
-#endif
 
-  const FieldFormatting& formatting = layout_item->get_formatting_used();
+  const Formatting& formatting = layout_item->get_formatting_used();
 
   Glib::ustring font = formatting.get_text_format_font();
   if(font.empty())
   {
+    //Just a sanity-check default that should never actually be used:
     font = "Serif 9";
-
-    //Set it in the input parameter,
-    //so that this is the default:
-    //TODO? formatting.set_text_format_font(font);
   }
 
   canvas_item->set_font_points(font);
@@ -114,21 +95,14 @@ void CanvasLayoutItem::apply_formatting(const Glib::RefPtr<CanvasTextMovable>& c
   const Glib::ustring fg = formatting.get_text_format_color_foreground();
   if(!fg.empty())
   {
-    #ifdef GLIBMM_PROPERTIES_ENABLED
-    canvas_item->property_stroke_color() = fg;
-    #else
-    canvas_item->set_property("stroke-color", fg);
-    #endif
+    //GooCanvasText uses fill-color for the text foreground color.
+    //Presumably stroke-color would be an outline, if we had a line-width of >0 width.
+    canvas_item->property_fill_color() = fg;
   }
-
   const Glib::ustring bg = formatting.get_text_format_color_background();
   if(!bg.empty())
   {
-    #ifdef GLIBMM_PROPERTIES_ENABLED
-    canvas_item->property_fill_color() = bg;
-    #else
-    canvas_item->set_property("fill-color", bg);
-    #endif
+    //TODO: Add a filled rectangle.
   }
 }
 
@@ -141,11 +115,17 @@ void CanvasLayoutItem::on_resized()
 
 void CanvasLayoutItem::set_layout_item(const sharedptr<LayoutItem>& layout_item)
 {
+  //TODO: If we can ever avoid this the also update the CanvasLayoutItem class documentation.
+  if(!get_canvas())
+  {
+    std::cerr << G_STRFUNC << ": get_canvas() returned null. This should not be called before the CanvasLayoutItem is in a canvas due to goocanvas bug https://bugzilla.gnome.org/show_bug.cgi?id=657592#c16 ." << std::endl;
+  }
+  
   //Add the new child:
   m_layout_item = layout_item;
 
   if(!m_layout_item)
-    std::cerr << "CanvasLayoutItem::set_layout_item(): item was NULL." << std::endl;
+    std::cerr << G_STRFUNC << ": item was NULL." << std::endl;
 
   Glib::RefPtr<CanvasItemMovable> child_item = create_canvas_item_for_layout_item(m_layout_item);
 
@@ -163,7 +143,7 @@ void CanvasLayoutItem::set_layout_item(const sharedptr<LayoutItem>& layout_item)
 
     set_xy(x, y);
     set_width_height(width, height);
-    //std::cout << "CanvasLayoutItem::set_layout_item(): item x=" << x << std::endl;
+    //std::cout << "debug: " << G_STRFUNC << ": item x=" << x << std::endl;
 
     set_child(child_item);
   }
@@ -177,6 +157,7 @@ void CanvasLayoutItem::set_layout_item(const sharedptr<LayoutItem>& layout_item)
   }
 }
 
+//TODO: Remove this?
 int CanvasLayoutItem::get_rows_count_for_portal(const sharedptr<const LayoutItem_Portal>& portal, double& row_height)
 {
   if(!portal)
@@ -196,29 +177,24 @@ int CanvasLayoutItem::get_rows_count_for_portal(const sharedptr<const LayoutItem
   const double max_rows_fraction = total_height / row_height;
   double max_rows = 0;
   modf(max_rows_fraction, &max_rows);
+  std::cout << "debug: max_rows=" << max_rows << ", for total_height=" << total_height << ", row_height=" << row_height << std::endl;
 
   return max_rows;
 }
 
 Glib::RefPtr<CanvasItemMovable> CanvasLayoutItem::create_canvas_item_for_layout_item(const sharedptr<LayoutItem>& layout_item)
 {
-  sharedptr<LayoutItem_Line> line;
-
   Glib::RefPtr<CanvasItemMovable> child;
   Glib::RefPtr<Goocanvas::Item> child_item;
   sharedptr<LayoutItem_Text> text = sharedptr<LayoutItem_Text>::cast_dynamic(layout_item);
   if(text)
   {
     Glib::RefPtr<CanvasTextMovable> canvas_item = CanvasTextMovable::create();
-    #ifdef GLIBMM_PROPERTIES_ENABLED
     canvas_item->property_line_width() = 0;
-    #else
-    canvas_item->set_property("line-width", 0);
-    #endif
 
     apply_formatting(canvas_item, text);
 
-    canvas_item->set_text(text->get_text());
+    canvas_item->set_text(text->get_text(AppWindow::get_current_locale()));
     child = canvas_item;
     child_item = canvas_item;
   }
@@ -234,11 +210,8 @@ Glib::RefPtr<CanvasItemMovable> CanvasLayoutItem::create_canvas_item_for_layout_
       else
         canvas_item->set_image_empty(); //show a no-image picture.
 
-      #ifdef GLIBMM_PROPERTIES_ENABLED
-      canvas_item->property_fill_color() = "white"; //This makes the whole area clickable, not just the outline stroke.
-      #else
-      canvas_item->set_property("fill-color", Glib::ustring("white")); //This makes the whole area clickable, not just the outline stroke.
-      #endif
+      //canvas_item->property_fill_color() = "white"; //This makes the whole area clickable, not just the outline stroke.
+      //canvas_item->property_fill_color_rgba() = 0xFFFFFF00;
 
       child = canvas_item;
       child_item = canvas_item;
@@ -255,22 +228,13 @@ Glib::RefPtr<CanvasItemMovable> CanvasLayoutItem::create_canvas_item_for_layout_
         line->get_coordinates(start_x, start_y, end_x, end_y);
 
         Glib::RefPtr<CanvasLineMovable> canvas_item = CanvasLineMovable::create();
-        #ifdef GLIBMM_PROPERTIES_ENABLED
-        canvas_item->property_line_width() = 1;
-        canvas_item->property_stroke_color() = "black";
-        #else
-        canvas_item->set_property("line-width", 1);
-        canvas_item->set_property("stroke-color", Glib::ustring("black"));
-        #endif
+        canvas_item->property_line_width() = line->get_line_width();
+        canvas_item->property_stroke_color() = line->get_line_color();
 
         Goocanvas::Points points(2);
         points.set_coordinate(0, start_x, start_y);
         points.set_coordinate(0, end_x, end_y);
-        #ifdef GLIBMM_PROPERTIES_ENABLED
         canvas_item->property_points() = points;
-        #else
-        canvas_item->set_property("points", points);
-        #endif
         child = canvas_item;
         child_item = canvas_item;
       }
@@ -291,12 +255,7 @@ Glib::RefPtr<CanvasItemMovable> CanvasLayoutItem::create_canvas_item_for_layout_
           else //text, numbers, date, time, boolean:
           {
             Glib::RefPtr<CanvasTextMovable> canvas_item = CanvasTextMovable::create();
-            #ifdef GLIBMM_PROPERTIES_ENABLED
             canvas_item->property_line_width() = 0;
-            #else
-            canvas_item->set_property("line-width", 0);
-            #endif
-
             apply_formatting(canvas_item, field);
 
             Glib::ustring name = field->get_name();
@@ -315,67 +274,27 @@ Glib::RefPtr<CanvasItemMovable> CanvasLayoutItem::create_canvas_item_for_layout_
           if(portal)
           {
             Glib::RefPtr<CanvasTableMovable> canvas_item = CanvasTableMovable::create();
-            #ifdef GLIBMM_PROPERTIES_ENABLED
-            canvas_item->property_vert_grid_line_width() = 1;
-            canvas_item->property_horz_grid_line_width() = 1;
-            canvas_item->property_stroke_color() = "black";
-            #else
-            canvas_item->set_property("vert-grid-line-width", 1);
-            canvas_item->set_property("horz-grid-line-width", 1);
-            canvas_item->set_property("stroke-color", Glib::ustring("black"));
-            #endif
 
-            //Show as many rows as can fit in the height.
-            double row_height = 0;
-            const int max_rows = get_rows_count_for_portal(portal, row_height);
+            canvas_item->set_lines_details(
+              portal->get_print_layout_row_line_width(),
+              portal->get_print_layout_column_line_width(),
+              portal->get_print_layout_line_color());
 
-            const LayoutGroup::type_list_items child_items = portal->get_items();
-
-            for(guint row = 0; row < (guint)max_rows; ++row)
-            {
-              guint col = 0;
-              for(LayoutGroup::type_list_items::const_iterator iter = child_items.begin(); iter != child_items.end(); ++iter)
-              {
-                sharedptr<LayoutItem> layout_item = *iter;
-
-                //We use create_canvas_item_for_layout_item() instead of just
-                //creating another CanvasLayoutItem, because that would be a group,
-                //but goocanvas cannot yet support Groups inside Tables. murrayc.
-                //TODO: Bug number.
-                Glib::RefPtr<CanvasItemMovable> cell = create_canvas_item_for_layout_item(layout_item);
-                if(cell)
-                {
-                  //Make sure that the width is sensible:
-                  guint width = layout_item->get_display_width();
-                  width = std::max(width, (guint)10);
-                  cell->set_width_height(width, row_height);
-                  std::cout << "DEBUG: width=" << width << std::endl;
-
-                  //TODO: Add/Remove rows when resizing, instead of resizing the rows:
-                  Glib::RefPtr<Goocanvas::Item> cell_as_item = CanvasItemMovable::cast_to_item(cell);
-                  if(cell_as_item)
-                  {
-                    canvas_item->attach(cell_as_item,
-                      col /* left_attach */, col+1 /* right_attach */,
-                      row /* top_attach */, row + 1 /* right_attach */,
-                      Gtk::FILL, (Gtk::AttachOptions)Gtk::FILL | Gtk::EXPAND);
-                  }
-                }
-
-                ++col;
-              }
-            }
+            gulong rows_count_min = 0;
+            gulong rows_count_max = 0;
+            portal->get_rows_count(rows_count_min, rows_count_max);
+            add_portal_rows_if_necessary(canvas_item, portal, rows_count_min);
 
             child = canvas_item;
             child_item = canvas_item;
           }
           else if(layout_item)
           {
-            std::cerr << "CanvasLayoutItem::set_layout_item(): Unhandled LayoutItem type. part type=" << layout_item->get_part_type_name() << std::endl;
+            std::cerr << G_STRFUNC << ": Unhandled LayoutItem type. part type=" << layout_item->get_part_type_name() << std::endl;
           }
           else
           {
-            std::cerr << "CanvasLayoutItem::set_layout_item(): NULL LayoutItem type." << std::endl;
+            std::cerr << G_STRFUNC << ": NULL LayoutItem type." << std::endl;
           }
         }
       }
@@ -383,33 +302,144 @@ Glib::RefPtr<CanvasItemMovable> CanvasLayoutItem::create_canvas_item_for_layout_
   }
 
   return child;
+}
 
-  if(child && child_item)
+
+Glib::RefPtr<Goocanvas::Item> CanvasLayoutItem::get_canvas_table_cell_child(const Glib::RefPtr<Goocanvas::Table>& table, int row, int col)
+{
+  Glib::RefPtr<Goocanvas::Item> result;
+
+  if(!table)
+    return result;
+
+  const int count = table->get_n_children();
+  for(int i = 0; i < count; ++i)
   {
-    //child_item->property_pointer_events() =
-    //  (Goocanvas::PointerEvents)(Goocanvas::EVENTS_VISIBLE_FILL & GOO_CANVAS_EVENTS_VISIBLE_STROKE);
+    Glib::RefPtr<Goocanvas::Item> child = table->get_child(i);
+    if(!child)
+      continue;
 
-    //Set the position and dimensions of this group to match the child:
-    double x = 0;
-    double y = 0;
-    double width = 0;
-    double height = 0;
-    layout_item->get_print_layout_position(x, y, width, height);
-    child->set_width_height(width, height);
-    //std::cout << "CanvasLayoutItem::set_layout_item(): item x=" << x << std::endl;
+    int column_value = 0;
+    table->get_child_property(child, "column", column_value);
+    int row_value = 0;
+    table->get_child_property(child, "row", row_value);
+
+    //This assumes that all items occupy only one cell:
+    if( (column_value == col) &&
+        (row_value == row) )
+    {
+      return child;
+    }
   }
 
-  //Scale images.
-  //This can only be done after setting the size:
-  Glib::RefPtr<CanvasImageMovable> canvas_image = Glib::RefPtr<CanvasImageMovable>::cast_dynamic(child);
-  if(canvas_image)
-  {
-    canvas_image->scale_to_size();
+  return result;
+}
 
-    //It will also be rescaled when this canvas item is resized - see on_resized().
+void CanvasLayoutItem::add_portal_rows_if_necessary(guint rows_count)
+{
+  Glib::RefPtr<CanvasItemMovable> child = get_child();
+  Glib::RefPtr<CanvasTableMovable> canvas_table = 
+    Glib::RefPtr<CanvasTableMovable>::cast_dynamic(child);
+  if(!canvas_table)
+    return;
+
+  sharedptr<LayoutItem_Portal> portal = 
+    sharedptr<LayoutItem_Portal>::cast_dynamic(get_layout_item());
+  if(!portal)
+  {
+    std::cerr << G_STRFUNC << ": The layout item was not a portal." << std::endl;
+    return;
   }
 
-  return child;
+  add_portal_rows_if_necessary(canvas_table, portal, rows_count);
+}
+
+void CanvasLayoutItem::add_portal_rows_if_necessary(const Glib::RefPtr<CanvasTableMovable>& canvas_table, const sharedptr<LayoutItem_Portal>& portal, guint rows_count)
+{
+  const double row_height = portal->get_print_layout_row_height();
+  const LayoutGroup::type_list_items child_items = portal->get_items();
+
+  for(guint row = 0; row < rows_count; ++row)
+  {
+    guint col = 0;
+    const guint num_cols = child_items.size();
+    bool something_expanded = false;
+    for(LayoutGroup::type_list_items::const_iterator iter = child_items.begin(); iter != child_items.end(); ++iter)
+    {
+      //std::cout << "  row=" << row << ", col=" << col << std::endl;
+      sharedptr<LayoutItem> layout_item = *iter;
+
+      //Check if a child already exists:
+      Glib::RefPtr<Goocanvas::Item> existing_child = 
+        get_canvas_table_cell_child(canvas_table, row, col);
+      if(existing_child)
+      {
+        //std::cout << "    existing child" << std::endl;
+        col++;
+        continue;
+      }
+
+      //We use create_canvas_item_for_layout_item() instead of just
+      //creating another CanvasLayoutItem, because that would be a group,
+      //but goocanvas cannot yet support Groups inside Tables. murrayc.
+      //TODO: Bug number.
+      Glib::RefPtr<CanvasItemMovable> cell = create_canvas_item_for_layout_item(layout_item);
+      Glib::RefPtr<Goocanvas::Item> cell_as_item = CanvasItemMovable::cast_to_item(cell);
+
+      if(cell && cell_as_item)
+      {
+        const guint width = layout_item->get_display_width();
+        bool expand = (width == 0);
+        
+        //If this is the last item, and no other item has expanded,
+        //let this one expand,
+        //Otherwise, we could allocate less space than the table has left.
+        //TODO: Prevent the user from allocating _more_ space than the table has.
+        if(!something_expanded && (col == (num_cols - 1)))
+        {
+          expand = true;
+        }
+        
+        if(expand)
+        {
+            //Let the table allocate the width automatically:
+
+            cell->set_width_height(-1, row_height);
+            canvas_table->attach(cell_as_item,
+              col /* left_attach */, col + 1 /* right_attach */,
+              row /* top_attach */, row + 1 /* right_attach */,
+              (Gtk::AttachOptions)(Gtk::FILL | Gtk::EXPAND), (Gtk::AttachOptions)(Gtk::FILL | Gtk::EXPAND));
+  
+            something_expanded = true;
+        }
+        else
+        {
+            //Enforce a width:
+
+            //TODO: Add/Remove rows when resizing, instead of resizing the rows:
+            cell->set_width_height(width, row_height);
+
+            canvas_table->attach(cell_as_item,
+              col /* left_attach */, col + 1 /* right_attach */,
+              row /* top_attach */, row + 1 /* right_attach */,
+              (Gtk::FILL), (Gtk::AttachOptions)(Gtk::FILL | Gtk::EXPAND));
+  
+            //Add a second item (an invisible rect) to make sure that the size is really used:
+            Glib::RefPtr<Goocanvas::Rect> rect = 
+              Goocanvas::Rect::create(0, 0, width, row_height);
+            //TODO: Find out why this doesn't work: rect->property_stroke_pattern() = Cairo::RefPtr<Cairo::Pattern>();
+            g_object_set(rect->gobj(), "stroke-pattern", (void*)0, (void*)0);
+ 
+            canvas_table->attach(rect,
+              col /* left_attach */, col + 1 /* right_attach */,
+              row /* top_attach */, row + 1 /* right_attach */,
+              Gtk::SHRINK, Gtk::SHRINK);
+        }
+      }
+
+      ++col;
+    }
+  }
 }
 
 void CanvasLayoutItem::set_db_data(const Gnome::Gda::Value& value)
@@ -441,7 +471,7 @@ void CanvasLayoutItem::set_db_data(const Gnome::Gda::Value& value)
       if(text_value.empty() && sharedptr<const LayoutItem_FieldSummary>::cast_dynamic(field) && (field_type == Field::TYPE_NUMERIC))
       {
         //Use get_text_for_gda_value() instead of "0" so we get the correct numerical formatting:
-        Gnome::Gda::Value value = Conversions::parse_value(0);
+        const Gnome::Gda::Value value = Conversions::parse_value(0);
         text_value = Conversions::get_text_for_gda_value(field_type, value, field->get_formatting_used().m_numeric_format);
       }
 
@@ -461,13 +491,15 @@ void CanvasLayoutItem::set_db_data(const Gnome::Gda::Value& value)
       canvas_item->get_width_height(width, height);
 
       Glib::RefPtr<Gdk::Pixbuf> pixbuf = Utils::get_pixbuf_for_gda_value(value);
-      if(pixbuf) //TODO: Remove this if() check when goocanvas has my patch to avoid crashes when this is NULL.
+      if(pixbuf)
         canvas_item->set_image(pixbuf);
+      else
+        canvas_item->set_image_empty();
 
       break;
     }
     default:
-      std::cerr << "CanvasLayoutItem::set_db_data(): unhandled field type." << std::endl;
+      std::cerr << G_STRFUNC << ": unhandled field type." << std::endl;
   }
 
 }
@@ -482,13 +514,28 @@ void CanvasLayoutItem::remove_empty_indicators()
     if(canvas_image->get_image_empty())
     {
       Glib::RefPtr<Gdk::Pixbuf> really_empty;
-      #ifdef GLIBMM_PROPERTIES_ENABLED
       canvas_image->property_pixbuf() = really_empty;
-      #else
-      canvas_image->set_property("pixbuf", really_empty);
-      #endif
     }
   }
 }
 
+void CanvasLayoutItem::update_layout_position_from_canvas()
+{
+  sharedptr<LayoutItem> layout_item = get_layout_item();
+  if(!layout_item)
+    return;
+
+  //Get the actual position:
+  double x = 0;
+  double y = 0;
+  get_xy(x, y);
+  //std::cout << "debug: " << G_STRFUNC << ": x=" << x << std::endl;
+
+  double width = 0;
+  double height = 0;
+  get_width_height(width, height);
+
+  layout_item->set_print_layout_position(x, y, width, height);
+}
+
 } //namespace Glom
