@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #ifndef GLOM_MODE_DESIGN_PRINT_LAYOUTS_CANVAS_LAYOUT_ITEM_H
@@ -24,21 +24,28 @@
 #include <glom/utility_widgets/canvas/canvas_group_resizable.h>
 #include <libglom/data_structure/layout/layoutitem_withformatting.h>
 #include <libgdamm/value.h>
+#include <goocanvasmm/table.h>
 
 namespace Glom
 {
 
 class CanvasTextMovable;
-class FieldFormatting;
+class CanvasTableMovable;
+class Formatting;
 class LayoutItem_Portal;
 
 /** This has the appropriate child canvas item, depending on the type of the child LayoutItem.
+ * You should call set_layout_item() after instantiating a CanvasLayoutItem via create(),
+ * and after adding the CanvasLayoutItem to a parent CanvasItem that is already in a Goocanvas::Canvas.
+ *
+ * If the CanvasLayoutItem is not already (indirectly) in a GooCanvas::Canvas then
+ * Goocanvas::Image items will show over-scaled images, due to goocanvas bug:
+ * https://bugzilla.gnome.org/show_bug.cgi?id=657592#c16 
  */
 class CanvasLayoutItem : public CanvasGroupResizable
 {
 private:
   CanvasLayoutItem();
-  CanvasLayoutItem(const sharedptr<LayoutItem>& layout_item);
   virtual ~CanvasLayoutItem();
 
 public:
@@ -63,12 +70,25 @@ public:
 
   static int get_rows_count_for_portal(const sharedptr<const LayoutItem_Portal>& portal, double& row_height);
 
+  /** Make sure that the LayoutItem has the same position info as the CanvasItem that represents it.
+   */
+  void update_layout_position_from_canvas();
+
+  /* Add table child items if any are missing,
+   * for instance if the table has been made bigger.
+   */
+  void add_portal_rows_if_necessary(guint rows_count);
+
+  static Glib::RefPtr<Goocanvas::Item> get_canvas_table_cell_child(const Glib::RefPtr<Goocanvas::Table>& table, int row, int col); //TODO: Add this to Goocanvas::Table.
+
 private:
   /// Create the appropriate inner canvas item to represent the layout item.
   static Glib::RefPtr<CanvasItemMovable> create_canvas_item_for_layout_item(const sharedptr<LayoutItem>& layout_item);
 
   static void apply_formatting(const Glib::RefPtr<CanvasTextMovable>& canvas_item, const sharedptr<const LayoutItem_WithFormatting>& layout_item);
   
+  static void add_portal_rows_if_necessary(const Glib::RefPtr<CanvasTableMovable>& canvas_table, const sharedptr<LayoutItem_Portal>& portal, guint rows_count);
+
   void on_resized();
 
   sharedptr<LayoutItem> m_layout_item;
