@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #ifndef GLOM_PYTHON_H
@@ -34,6 +34,11 @@ namespace Glom
  */
 bool glom_python_module_is_available();
 
+/** Check that Python can really import the gi.repository module,
+ * as a runtime sanity check.
+ */
+bool gir_python_module_is_available();
+
 /** Check that Python can really import the gda module,
  * as a runtime sanity check.
  */
