@@ -14,28 +14,25 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
-#include "config.h" // For GLOM_ENABLE_MAEMO
+#include "config.h" // For GLOM_ENABLE_CLIENT_ONLY
 
 #include <glom/utility_widgets/adddel/adddel.h>
 #include <algorithm> //For std::find.
 #include <glibmm/i18n.h>
-#include <glom/utility_widgets/cellrendererlist/cellrendererlist.h>
+#include <glom/utility_widgets/cellrendererlist.h>
 #include <glom/utility_widgets/adddel/treeviewcolumn_glom.h>
 #include <libglom/data_structure/glomconversions.h>
 #include <glom/dialog_invalid_data.h>
 #include <glom/utils_ui.h>
+#include <gtkmm/stock.h>
 #include <libglom/utils.h>
 //#include <glom/bakery/app_gtk.h>
 #include <iostream> //For debug output.
 
-#ifdef GLOM_ENABLE_MAEMO
-#include <hildonmm/note.h>
-#endif // GLOM_ENABLE_MAEMO
-
 namespace Glom
 {
 
@@ -75,7 +72,8 @@ AddDelColumnInfo& AddDelColumnInfo::operator=(const AddDelColumnInfo& src)
 }
 
 AddDel::AddDel()
-: m_col_key(0),
+: Gtk::Box(Gtk::ORIENTATION_VERTICAL),
+  m_col_key(0),
   m_pMenuPopup(0),
   m_auto_add(true),
   m_allow_add(true),
@@ -86,7 +84,7 @@ AddDel::AddDel()
 
 
 AddDel::AddDel(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& /* builder */)
-: Gtk::VBox(cobject),
+: Gtk::Box(cobject),
   m_col_key(0),
   m_pMenuPopup(0),
   m_auto_add(true),
@@ -148,7 +146,7 @@ void AddDel::set_treeview_accessible_name(const Glib::ustring& name)
 {
 #ifdef GTKMM_ATKMM_ENABLED
   m_TreeView.get_accessible()->set_name(name);
-#endif  
+#endif
 }
 
 void AddDel::warn_about_duplicate()
@@ -160,12 +158,9 @@ void AddDel::warn_about_duplicate()
   else
     message = m_prevent_duplicates_warning; //Something more specific and helpful.
 
-#ifdef GLOM_ENABLE_MAEMO
-  Hildon::Note dialog(Hildon::NOTE_TYPE_INFORMATION, message);
-#else
   Gtk::MessageDialog dialog(Utils::bold_message(_("Duplicate")), true, Gtk::MESSAGE_WARNING, Gtk::BUTTONS_OK);
   dialog.set_secondary_text(message);
-#endif
+
   //TODO: dialog.set_transient_for(get_parent_window());
 
   dialog.run();
@@ -254,11 +249,9 @@ void AddDel::setup_menu()
 
   //TODO: add_accel_group(m_refUIManager->get_accel_group());
 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
   try
   {
-#endif
-    Glib::ustring ui_info = 
+    Glib::ustring ui_info =
         "<ui>"
         "  <popup name='ContextMenu'>"
         "    <menuitem action='ContextEdit'/>"
@@ -266,24 +259,15 @@ void AddDel::setup_menu()
         "  </popup>"
         "</ui>";
 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
     m_refUIManager->add_ui_from_string(ui_info);
   }
   catch(const Glib::Error& ex)
   {
     std::cerr << "building menus failed: " <<  ex.what();
   }
-#else
-  std::auto_ptr<Glib::Error> error;
-  m_refUIManager->add_ui_from_string(ui_info, error);
-  if(error.get())
-  {
-    std::cerr << "building menus failed: " << error->what();
-  }
-#endif //GLIBMM_EXCEPTIONS_ENABLED
 
   //Get the menu:
-  m_pMenuPopup = dynamic_cast<Gtk::Menu*>( m_refUIManager->get_widget("/ContextMenu") ); 
+  m_pMenuPopup = dynamic_cast<Gtk::Menu*>( m_refUIManager->get_widget("/ContextMenu") );
   if(!m_pMenuPopup)
     g_warning("menu not found");
 }
@@ -291,7 +275,7 @@ void AddDel::setup_menu()
 bool AddDel::on_button_press_event_Popup(GdkEventButton *event)
 {
   GdkModifierType mods;
-  gdk_window_get_pointer( Gtk::Widget::gobj()->window, 0, 0, &mods );
+  gdk_window_get_device_position( gtk_widget_get_window (Gtk::Widget::gobj()), event->device, 0, 0, &mods );
   if(mods & GDK_BUTTON3_MASK)
   {
     //Give user choices of actions on this item:
@@ -322,7 +306,7 @@ Gtk::TreeModel::iterator AddDel::get_item_placeholder()
     return add_item_placeholder();
   }
 }
-  
+
 Gtk::TreeModel::iterator AddDel::add_item_placeholder()
 {
   Gtk::TreeModel::iterator iter = m_refListStore->append();
@@ -404,7 +388,7 @@ bool AddDel::get_value_as_bool(const Gtk::TreeModel::iterator& iter, guint col)
 {
 //TODO: I doubt that this works. It should really get the value from the treeview as a bool. murrayc
   Glib::ustring strValue = get_value(iter, col);
-  return (strValue == "true");   
+  return (strValue == "true");
 }
 
 Glib::ustring AddDel::get_value_key_selected()
@@ -526,7 +510,7 @@ void AddDel::add_blank()
 
   if(get_allow_user_actions()) //The extra blank line is only used if the user may add items:
   {
-    Gtk::TreeModel::iterator iter = get_last_row();        
+    Gtk::TreeModel::iterator iter = get_last_row();
     if(get_is_placeholder_row(iter))
     {
         bAddNewBlank  = false; //One already exists.
@@ -627,7 +611,7 @@ void AddDel::construct_specified_columns()
   m_TreeView.set_model(m_refListStore);
 
   //Remove all View columns:
-  m_TreeView.remove_all_columns();
+  Utils::treeview_delete_all_columns(&m_TreeView);
 
   //Add new View Colums:
   int model_column_index = 0;
@@ -641,7 +625,6 @@ void AddDel::construct_specified_columns()
       const Glib::ustring column_name = m_ColumnTypes[model_column_index].m_name;
       const Glib::ustring column_id = m_ColumnTypes[model_column_index].m_id;
 
-      int cols_count = 0;
       switch(m_ColumnTypes[model_column_index].m_style)
       {
         case(AddDelColumnInfo::STYLE_Choices):
@@ -659,7 +642,7 @@ void AddDel::construct_specified_columns()
           // Append the View column.
           // We use a derived Gtk::TreeViewColumn so that we can store extra information in it.
           // This means that we must reimplement the code from the convenience template methods from gtkmm.
-          cols_count = treeview_append_column( Utils::string_escape_underscores(column_name), *pCellRenderer,  *pModelColumn, column_id);
+          treeview_append_column( Utils::string_escape_underscores(column_name), *pCellRenderer,  *pModelColumn, column_id);
 
           break;
         }
@@ -672,7 +655,7 @@ void AddDel::construct_specified_columns()
 
           Gtk::TreeModelColumn<bool>* pModelColumnDerived = static_cast< Gtk::TreeModelColumn<bool>* >(pModelColumn);
           if(pModelColumnDerived)
-            cols_count = treeview_append_column(Utils::string_escape_underscores(column_name), *pModelColumnDerived, column_id);
+            treeview_append_column(Utils::string_escape_underscores(column_name), *pModelColumnDerived, column_id);
 
           break;
         }
@@ -684,7 +667,7 @@ void AddDel::construct_specified_columns()
           //to use the correct specialization:
           Gtk::TreeModelColumn<Glib::ustring>* pModelColumnDerived = static_cast< Gtk::TreeModelColumn<Glib::ustring>* >(pModelColumn);
           if(pModelColumnDerived)
-            cols_count = treeview_append_column(Utils::string_escape_underscores(column_name), *pModelColumnDerived, column_id);
+            treeview_append_column(Utils::string_escape_underscores(column_name), *pModelColumnDerived, column_id);
 
           break;
         }
@@ -699,7 +682,7 @@ void AddDel::construct_specified_columns()
           if(pCellRenderer)
           {
             //Make it editable:
-            g_object_set(pCellRenderer->gobj(), "editable", TRUE, (gpointer)0);
+            pCellRenderer->property_editable() = true;
 
             //Connect to its signal:
             pCellRenderer->signal_edited().connect(
@@ -711,7 +694,7 @@ void AddDel::construct_specified_columns()
            Gtk::CellRendererToggle* pCellRenderer = dynamic_cast<Gtk::CellRendererToggle*>(m_TreeView.get_column_cell_renderer(view_column_index));
            if(pCellRenderer)
            {
-             g_object_set(pCellRenderer->gobj(), "activatable", TRUE, (gpointer)0);
+             pCellRenderer->property_activatable() = true;
 
              //Connect to its signal:
              pCellRenderer->signal_toggled().connect(
@@ -769,9 +752,9 @@ void AddDel::construct_specified_columns()
     }
     else
     {
-      std::cerr << "debug: AddDel::construct_specified_columns(): Leaking a Gtk::TreeModelColumn<>." << std::endl;
+      std::cerr << G_STRFUNC << ": Leaking a Gtk::TreeModelColumn<>." << std::endl;
     }
-    
+
     ++model_column_index;
   }
 
@@ -786,7 +769,7 @@ void AddDel::set_value(const Gtk::TreeModel::iterator& iter, guint col, const Gn
   {
     case(AddDelColumnInfo::STYLE_Boolean):
     {
-      g_warning("AddDel::set_value(): boolean column being set as bool.");
+      std::cerr << G_STRFUNC << ": boolean column being set as bool." << std::endl;
       set_value(iter, col, value.get_bool());
       break;
     }
@@ -804,7 +787,7 @@ void AddDel::set_value(const Gtk::TreeModel::iterator& iter, guint col, const Gl
   InnerIgnore innerIgnore(this);
 
   if(!m_refListStore)
-    g_warning("AddDel::set_value: No model.");
+    std::cerr << G_STRFUNC << ": No model." << std::endl;
   else
   {
     Gtk::TreeModel::Row treerow = *iter;
@@ -854,7 +837,7 @@ void AddDel::set_value(const Gtk::TreeModel::iterator& iter, guint col, bool bVa
   InnerIgnore innerIgnore(this);
 
   if(!m_refListStore)
-    g_warning("AddDel::set_value: No model.");
+    std::cerr << G_STRFUNC << ": No model." << std::endl;
   else
   {
     Gtk::TreeModel::Row treerow = *iter;
@@ -911,7 +894,7 @@ guint AddDel::add_column(const AddDelColumnInfo& column_info)
 
   return m_ColumnTypes.size()-1;
 }
- 
+
 guint AddDel::add_column(const Glib::ustring& strTitle, AddDelColumnInfo::enumStyles style, bool editable, bool visible)
 {
   //Use the title as the column_id:
@@ -988,8 +971,8 @@ void AddDel::set_allow_delete(bool val)
 {
   m_allow_delete= val;
 }
-  
-  
+
+
 void AddDel::set_allow_user_actions(bool bVal)
 {
   m_bAllowUserActions = bVal;
@@ -1087,7 +1070,7 @@ AddDel::InnerIgnore::~InnerIgnore()
     m_pOuter->set_ignore_treeview_signals(m_bIgnoreSheetSignals);
   }
 
-  m_pOuter = false;
+  m_pOuter = 0;
 }
 
 Glib::ustring AddDel::treeview_get_key(const Gtk::TreeModel::iterator& row)
@@ -1096,7 +1079,7 @@ Glib::ustring AddDel::treeview_get_key(const Gtk::TreeModel::iterator& row)
 
   if(m_refListStore)
   {
-    Gtk::TreeModel::Row treerow = m_refListStore->children()[row];
+    Gtk::TreeModel::Row treerow = *row;
 
     if(treerow)
       treerow.get_value(m_col_key, value);
@@ -1126,7 +1109,7 @@ void AddDel::on_treeview_cell_edited_bool(const Glib::ustring& path_string, int
     row.set_value(model_column_index, value_new);
 
     //TODO: Did it really change?
-    
+
     //Is this an add or a change?:
 
     bool bIsAdd = false;
@@ -1165,7 +1148,7 @@ void AddDel::on_treeview_cell_edited_bool(const Glib::ustring& path_string, int
     {
       //Change it back, so that we ignore it:
       row.set_value(model_column_index, value_old);
-         
+
       //Signal that a new key was added:
       //We will ignore editing of bool values in the blank row. It seems like a bad way to start a new record.
       //m_signal_user_added.emit(row);
@@ -1176,7 +1159,7 @@ void AddDel::on_treeview_cell_edited_bool(const Glib::ustring& path_string, int
 
       m_signal_user_changed.emit(row, model_column_index);
     }
-    
+
   }
 }
 
@@ -1238,7 +1221,7 @@ void AddDel::on_treeview_cell_edited(const Glib::ustring& path_string, const Gli
     //Fire appropriate signal:
     if(bIsAdd)
     {
-        //Signal that a new key was added"        
+        //Signal that a new key was added"
         m_signal_user_added.emit(row);
     }
     else if(bIsChange)
@@ -1521,7 +1504,7 @@ void AddDel::set_value_key(const Gtk::TreeModel::iterator& iter, const Glib::ust
     //This is not a placeholder anymore, if it every was:
     iter->set_value(m_col_placeholder, false);
   }
-  
+
   iter->set_value(m_col_key, strValue);
 }
 
@@ -1551,7 +1534,7 @@ bool AddDel::get_model_column_index(guint view_column_index, guint& model_column
   const guint count = m_ColumnTypes.size();
   if(hidden > count)
     return false; //This should never happen.
-  
+
   if(view_column_index >=  (count - hidden))
     return false;
 
