@@ -14,14 +14,15 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
-#ifndef ADDDEL_WITHBUTTONS_H
-#define ADDDEL_WITHBUTTONS_H
+#ifndef GLOM_UTILITY_WIDGETS_ADDDEL_WITHBUTTONS_H
+#define GLOM_UTILITY_WIDGETS_ADDDEL_WITHBUTTONS_H
 
 #include "adddel.h"
+#include <gtkmm/buttonbox.h>
 
 namespace Glom
 {
@@ -46,7 +47,7 @@ private:
   void on_button_edit();
 
   //member widgets:
-  Gtk::HButtonBox m_ButtonBox;
+  Gtk::ButtonBox m_ButtonBox;
   Gtk::Button m_Button_Add;
   Gtk::Button m_Button_Del;
   Gtk::Button m_Button_Edit;
@@ -54,4 +55,4 @@ private:
 
 } //namespace Glom
 
-#endif //ADDDEL_WITHBUTTONS_H
+#endif // GLOM_UTLITY_WIDGETS_ADDDEL_WITHBUTTONS_H
