@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #ifndef GLOM_UTILITY_WIDGETS_CANVAS_ITEM_MOVABLE_H
@@ -58,9 +58,17 @@ public:
   virtual void set_width_height(double width, double height) = 0;
 
   void set_drag_cursor(Gdk::CursorType cursor);
-  void set_drag_cursor(const Gdk::Cursor& cursor);
+  void set_drag_cursor(const Glib::RefPtr<Gdk::Cursor>& cursor);
 
-  typedef sigc::signal<void> type_signal_moved;
+  /** For instance,
+   *
+   *   @param item The item (this item) that was moved, for convenience.
+   *   @param x_offset How much the item has moved in the x dimension.
+   *   @param y_offset How much the item has moved in the y dimension.
+   *
+   *   void on_moved(bool group_select, double x_offset, double y_offset);
+   */
+  typedef sigc::signal<void, const Glib::RefPtr<CanvasItemMovable>&, double, double> type_signal_moved;
 
   /// This signal is emitted when the canvas item is moved by the user.
   type_signal_moved signal_moved();
@@ -70,6 +78,21 @@ public:
   typedef sigc::signal<void, guint, guint32> type_signal_show_context;
   type_signal_show_context signal_show_context();
 
+  /** For instance,
+   *
+   *   @param item The item (this item) that was selected/deseleted, for convenience.
+   *   @param group_select Whether the user selected this while pressing Shift to select multiple items.
+   *
+   *   void on_selected(bool group_select);
+   */
+  typedef sigc::signal<void, const Glib::RefPtr<CanvasItemMovable>&, bool> type_signal_selected;
+
+  /** This signal is emitted if the user causes the item 
+   * to be selected or deselected. See get_selected().
+   */
+  type_signal_selected signal_selected();
+
+
   /** Provide information about a grid or rules,
    * to which the item should snap when moving:
    *
@@ -91,11 +114,25 @@ public:
 
   virtual void snap_position(double& x, double& y) const;
 
+  /** Mark the item as selected,
+   * meaning that its boundaries will be visible,
+   * and this can be queried later, for instance to move several items together.
+   */
+  void set_selected(bool selected = true);
+
+  bool get_selected() const;
+
 private:
 
+  /** Show some visual cue that the item is selected,
+   * depending on the value of get_selected(),
+   * hiding that visual cue if it is not selected.
+   */
+  virtual void show_selected();
+
   virtual Goocanvas::Canvas* get_parent_canvas_widget() = 0;
 
-  void set_cursor(const Gdk::Cursor& cursor);
+  void set_cursor(const Glib::RefPtr<Gdk::Cursor>& cursor);
   void unset_cursor();
 
 public:
@@ -111,16 +148,20 @@ private:
   bool m_dragging_vertical_only, m_dragging_horizontal_only; //Set by using Ctrl while dragging.
   double m_drag_start_cursor_x, m_drag_start_cursor_y;
   double m_drag_start_position_x, m_drag_start_position_y;
-  Gdk::Cursor m_drag_cursor;
+  double m_drag_latest_position_x, m_drag_latest_position_y; //To discover how much the latest motion_event has moved the item.
+  Glib::RefPtr<Gdk::Cursor> m_drag_cursor;
 
 protected:
   Glib::RefPtr<const CanvasGroupGrid> m_grid;
 
 private:
   bool m_allow_vertical_movement, m_allow_horizontal_movement;
+  bool m_selected;
+  bool m_shift_click;
 
   type_signal_moved m_signal_moved;
   type_signal_show_context m_signal_show_context;
+  type_signal_selected m_signal_selected;
 };
 
 } //namespace Glom
