@@ -14,12 +14,13 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #include "canvas_text_movable.h"
 #include <goocanvasmm/canvas.h>
+#include <glibmm/markup.h>
 #include <goocanvastext.h>
 #include <iostream>
 
@@ -27,7 +28,7 @@ namespace Glom
 {
 
 
-CanvasTextMovable::CanvasTextMovable(const Glib::ustring& text, double x, double y, double width, Gtk::AnchorType anchor)
+CanvasTextMovable::CanvasTextMovable(const Glib::ustring& text, double x, double y, double width, Goocanvas::AnchorType anchor)
 : Goocanvas::Text(text, x, y, width, anchor), 
   m_snap_corner(CORNER_TOP_LEFT) //arbitrary default.
 {
@@ -48,7 +49,7 @@ void CanvasTextMovable::init()
   signal_leave_notify_event().connect(sigc::mem_fun(*this, &CanvasItemMovable::on_leave_notify_event));
 }
 
-Glib::RefPtr<CanvasTextMovable> CanvasTextMovable::create(const Glib::ustring& string, double x, double y, double width, Gtk::AnchorType anchor)
+Glib::RefPtr<CanvasTextMovable> CanvasTextMovable::create(const Glib::ustring& string, double x, double y, double width, Goocanvas::AnchorType anchor)
 {
   return Glib::RefPtr<CanvasTextMovable>(new CanvasTextMovable(string, x, y, width, anchor));
 }
@@ -176,15 +177,19 @@ void CanvasTextMovable::reconstruct_markup()
   char* markup = 0;
   if(!m_text.empty())
   {
+    //We will use the text as markup, so remove anything that could be 
+    //interpreted as pango markup.
+    //This is not really pango-specific, but it might just work:
+    const Glib::ustring text_escaped = Glib::Markup::escape_text(m_text);
+    
     //We add px (meaning absolute points size).
     //Otherwise both GooCanvas and GTK+ scale the font up, making it too large.
     //This really seems like a bug in GooCanvas.
     //TODO: This might not be robust - it assumes that the font size is at the end of the font_desc 
     //provided by GtkFontButton.
-    markup = g_strdup_printf("<span font_desc=\"%s\">%s</span>", m_font.c_str(), m_text.c_str());
+    markup = g_strdup_printf("<span font_desc=\"%s\">%s</span>", m_font.c_str(), text_escaped.c_str());
     //std::cout << "DEBUG: markup=" << markup << std::endl;
   }
-  
   property_use_markup() = true;
 
 
