@@ -14,14 +14,16 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
-#ifndef DIALOG_FLOWTABLE_H
-#define DIALOG_FLOWTABLE_H
+#ifndef GLOM_DIALOG_FLOWTABLE_H
+#define GLOM_DIALOG_FLOWTABLE_H
 
-#include <gtkmm.h>
+#include <gtkmm/dialog.h>
+#include <gtkmm/spinbutton.h>
+#include <gtkmm/entry.h>
 #include <gtkmm/builder.h>
 #include "../mode_data/flowtablewithfields.h"
 #include <glom/base_db.h>
@@ -29,19 +31,23 @@
 namespace Glom
 {
 
+//TODO: Is this used?
 class Dialog_FlowTable
  : public Gtk::Dialog,
    public Base_DB //Give this class access to the current document, and to some utility methods.
 
 {
 public:
+  static const char* glade_id;
+  static const bool glade_developer;
+	
   Dialog_FlowTable(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& builder);
   virtual ~Dialog_FlowTable();
 
   void set_flowtable(FlowTableWithFields* flowtable);
   
-  gint get_columns_count();
-  Glib::ustring get_title();
+  gint get_columns_count(); //TODO: Make const?
+  Glib::ustring get_title(); //TODO: Isn't this the same as Widget::get_title()?
     
 private:
   Gtk::Entry* m_entry_title;
@@ -53,4 +59,4 @@ private:
 
 } //namespace Glom
 
-#endif //DIALOG_TEXTOBJECT_H
+#endif // GLOM_DIALOG_FLOWTABLE_H
