@@ -14,12 +14,14 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
-#include "dialog_properties.h"
 #include "glom/utility_widgets/adddel/adddel.h"
+#include "dialog_properties.h"
+#include <gtkmm/togglebutton.h>
+#include <gtkmm/textview.h>
 
 namespace Glom
 {
@@ -34,7 +36,7 @@ Dialog_Properties::Dialog_Properties(BaseObjectType* cobject, const Glib::RefPtr
 
   //In general, we don't want to allow changes to windows underneath while editing properties.
   //Also, if we don't set this then seconday windows (from a modal dialog) will be on top but unusable.
-  set_modal(); 
+  set_modal();
 
   //Connect signal handlers:
   m_pButton_Cancel->signal_clicked().connect( sigc::mem_fun(*this, &Dialog_Properties::on_button_cancel) );
@@ -158,5 +160,3 @@ void Dialog_Properties::enforce_constraints()
 }
 
 } //namespace Glom
-
-
