@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #include "config.h"
@@ -29,25 +29,29 @@ namespace Glom
 {
 
 FileChooserDialog_SaveExtras::FileChooserDialog_SaveExtras(const Glib::ustring& title, Gtk::FileChooserAction action, const Glib::ustring& backend)
-: Gtk::FileChooserDialog(title, action, backend)
+: Gtk::FileChooserDialog(title, action, backend),
+  m_extra_widget(Gtk::ORIENTATION_VERTICAL)
 {
   create_child_widgets();
 }
 
 FileChooserDialog_SaveExtras::FileChooserDialog_SaveExtras(Gtk::Window& parent, const Glib::ustring& title, Gtk::FileChooserAction action, const Glib::ustring& backend)
-: Gtk::FileChooserDialog(parent, title, action, backend)
+: Gtk::FileChooserDialog(parent, title, action, backend),
+  m_extra_widget(Gtk::ORIENTATION_VERTICAL)
 {
   create_child_widgets();
 }
 
-FileChooserDialog_SaveExtras::FileChooserDialog_SaveExtras (const Glib::ustring& title, Gtk::FileChooserAction action)
-: Gtk::FileChooserDialog(title, action)
+FileChooserDialog_SaveExtras::FileChooserDialog_SaveExtras(const Glib::ustring& title, Gtk::FileChooserAction action)
+: Gtk::FileChooserDialog(title, action),
+  m_extra_widget(Gtk::ORIENTATION_VERTICAL)
 {
   create_child_widgets();
 }
 
-FileChooserDialog_SaveExtras::FileChooserDialog_SaveExtras (Gtk::Window& parent, const Glib::ustring& title, Gtk::FileChooserAction action)
-: Gtk::FileChooserDialog(parent, title, action)
+FileChooserDialog_SaveExtras::FileChooserDialog_SaveExtras(Gtk::Window& parent, const Glib::ustring& title, Gtk::FileChooserAction action)
+: Gtk::FileChooserDialog(parent, title, action),
+  m_extra_widget(Gtk::ORIENTATION_VERTICAL)
 {
   create_child_widgets();
 }
@@ -60,13 +64,18 @@ FileChooserDialog_SaveExtras::~FileChooserDialog_SaveExtras()
 void FileChooserDialog_SaveExtras::set_extra_message(const Glib::ustring& message)
 {
   m_label_extra_message.set_text(message);
+
+  if (!message.empty()) {
+    m_label_extra_message.show();
+  } else {
+    m_label_extra_message.hide();
+  }
 }
 
 void FileChooserDialog_SaveExtras::create_child_widgets()
 {
   //m_extra_widget.pack_start(m_label_extra_message);
   m_label_extra_message.set_alignment(0.0f, 0.5f);
-  m_label_extra_message.show();
 
   Gtk::Frame* frame = Gtk::manage(new Gtk::Frame());
   Gtk::Label* frame_label = Gtk::manage(new Gtk::Label());
@@ -76,12 +85,12 @@ void FileChooserDialog_SaveExtras::create_child_widgets()
   frame->set_shadow_type(Gtk::SHADOW_NONE);
 
   Gtk::Alignment* alignment = Gtk::manage(new Gtk::Alignment());
-  alignment->set_padding(0, 0, Utils::DEFAULT_SPACING_LARGE, 0); //Add padding at the left.
+  alignment->set_padding(Utils::DEFAULT_SPACING_SMALL, 0, Utils::DEFAULT_SPACING_LARGE, 0); //Add padding at the top and left.
   alignment->show();
   frame->add(*alignment);
   frame->show();
 
-  Gtk::VBox* vbox = Gtk::manage(new Gtk::VBox(false, Utils::DEFAULT_SPACING_SMALL));
+  Gtk::Box* vbox = Gtk::manage(new Gtk::Box(Gtk::ORIENTATION_VERTICAL, Utils::DEFAULT_SPACING_SMALL));
   alignment->add(*vbox);
   vbox->show();
 
@@ -89,10 +98,11 @@ void FileChooserDialog_SaveExtras::create_child_widgets()
 
   Gtk::Label* label_newdb = Gtk::manage(new Gtk::Label(_("Please choose a human-readable title for the new database. You can change this later in the database properties. It may contain any characters.")));
   vbox->pack_start(*label_newdb);
+  label_newdb->set_alignment(0.0f, 0.5f);
   label_newdb->show();
 
-  Gtk::HBox* box_label = Gtk::manage(new Gtk::HBox(false, Utils::DEFAULT_SPACING_SMALL));
-  Gtk::Label* label_title = Gtk::manage(new Gtk::Label(_("Title")));
+  Gtk::Box* box_label = Gtk::manage(new Gtk::Box(Gtk::ORIENTATION_HORIZONTAL, Utils::DEFAULT_SPACING_LARGE));
+  Gtk::Label* label_title = Gtk::manage(new Gtk::Label(_("_Title:"), true));
   box_label->pack_start(*label_title, Gtk::PACK_SHRINK);
   label_title->show();
   box_label->pack_start(m_entry_title);
