@@ -14,15 +14,16 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #include "layoutwidgetbase.h"
 #include <glibmm/i18n.h>
-#include <glom/application.h>
-#include <glom/utility_widgets/textviewglom.h>
-#include <glom/utility_widgets/labelglom.h>
+#include <glom/appwindow.h>
+#include <glom/mode_data/datawidget/textview.h>
+#include <glom/mode_data/datawidget/label.h>
+#include <iostream>
 
 namespace Glom
 {
@@ -55,7 +56,7 @@ sharedptr<LayoutItem> LayoutWidgetBase::get_layout_item()
   return m_pLayoutItem;
 }
 
-Application* LayoutWidgetBase::get_application() const
+AppWindow* LayoutWidgetBase::get_appwindow() const
 {
   return 0; //override to implement.
 }
@@ -91,25 +92,31 @@ void LayoutWidgetBase::apply_formatting(Gtk::Widget& widget, const sharedptr<con
   Gtk::Widget* widget_to_change = &widget;
 
   Gtk::Button* button = dynamic_cast<Gtk::Button*>(&widget);
-  LabelGlom* labelglom = dynamic_cast<LabelGlom*>(&widget);
+  DataWidgetChildren::Label* labelglom = dynamic_cast<DataWidgetChildren::Label*>(&widget);
   if(button)
     widget_to_change = button->get_child();
   else
   {
-    TextViewGlom* textview = dynamic_cast<TextViewGlom*>(&widget);
+    DataWidgetChildren::TextView* textview = dynamic_cast<DataWidgetChildren::TextView*>(&widget);
     if(textview)
       widget_to_change = textview->get_textview();
     else if(labelglom)
       widget_to_change = labelglom->get_label();
   }
 
+  if(!widget_to_change)
+  {
+    std::cerr << G_STRFUNC << ": widget_to_change is null." << std::endl;
+    return;
+  }
+
   if(!layout_item)
     return;
 
   //Horizontal alignment:
-  const FieldFormatting::HorizontalAlignment alignment =
-    layout_item->get_formatting_used_horizontal_alignment();
-  const float x_align = (alignment == FieldFormatting::HORIZONTAL_ALIGNMENT_LEFT ? 0.0 : 1.0);
+  const Formatting::HorizontalAlignment alignment =
+    layout_item->get_formatting_used_horizontal_alignment(true /* for details view */);
+  const float x_align = (alignment == Formatting::HORIZONTAL_ALIGNMENT_LEFT ? 0.0 : 1.0);
   Gtk::Misc* misc = dynamic_cast<Gtk::Misc*>(widget_to_change);
   if(misc)
     misc->set_alignment(x_align);
@@ -121,18 +128,18 @@ void LayoutWidgetBase::apply_formatting(Gtk::Widget& widget, const sharedptr<con
     Gtk::Label* label = dynamic_cast<Gtk::Label*>(widget_to_change);
     if(label)
     {    
-      const Gtk::Justification justification = (alignment == FieldFormatting::HORIZONTAL_ALIGNMENT_LEFT ? Gtk::JUSTIFY_LEFT : Gtk::JUSTIFY_RIGHT);
+      const Gtk::Justification justification = (alignment == Formatting::HORIZONTAL_ALIGNMENT_LEFT ? Gtk::JUSTIFY_LEFT : Gtk::JUSTIFY_RIGHT);
       label->set_justify(justification);
     }
   }
 
-  const FieldFormatting& formatting = layout_item->get_formatting_used();
+  const Formatting& formatting = layout_item->get_formatting_used();
 
   //Use the text formatting:
   const Glib::ustring font_desc = formatting.get_text_format_font();
   if(!font_desc.empty())
   {
-    widget_to_change->modify_font( Pango::FontDescription(font_desc) );
+    widget_to_change->override_font( Pango::FontDescription(font_desc) );
   }
 
   
@@ -140,11 +147,11 @@ void LayoutWidgetBase::apply_formatting(Gtk::Widget& widget, const sharedptr<con
   if(!fg.empty())
   {
     // "text" is the text color. (Works for GtkEntry and GtkTextView, 
-    // for which modify_fg() doesn't seem to have any effect.
-    widget_to_change->modify_text(Gtk::STATE_NORMAL, Gdk::Color(fg));
+    // for which override_color() doesn't seem to have any effect.
+    widget_to_change->override_color(Gdk::RGBA(fg));
     
-    // This works for GtkLabel, for which modify_text() does not.
-    widget_to_change->modify_fg(Gtk::STATE_NORMAL, Gdk::Color(fg));
+    // This works for GtkLabel, for which override_color() does not.
+    widget_to_change->override_color(Gdk::RGBA(fg));
   }
 
 
@@ -154,20 +161,20 @@ void LayoutWidgetBase::apply_formatting(Gtk::Widget& widget, const sharedptr<con
     if(!labelglom && !button)
     {
       // "base" is the background color for GtkEntry. "bg" seems to change the border:
-      widget_to_change->modify_base(Gtk::STATE_NORMAL, Gdk::Color(bg));
+      widget_to_change->override_background_color(Gdk::RGBA(bg));
     }
-    //According to the gtk_widget_modify_base() documentation, 
+    //According to the gtk_widget_override_background_color() documentation, 
     //a GtkLabel can only have a background color by, for instance, placing it 
-    //in a GtkEventBox. Luckily LabelGlom is actually derived from EventBox.
+    //in a GtkEventBox. Luckily Label is actually derived from EventBox.
     else if(labelglom)
     {
-      //label->modify_base(Gtk::STATE_NORMAL, Gdk::Color("bg"));
-      labelglom->modify_bg(Gtk::STATE_NORMAL, Gdk::Color(bg));
+      //label->override_background_color(Gdk::RGBA("bg"));
+      labelglom->override_background_color(Gdk::RGBA(bg));
     }
     else if(button)
     {
-      //button->modify_base(Gtk::STATE_NORMAL, Gdk::Color("bg"));
-      button->modify_bg(Gtk::STATE_NORMAL, Gdk::Color(bg));
+      //button->override_background_color(Gdk::RGBA("bg"));
+      button->override_background_color(Gdk::RGBA(bg));
     }
   }
 }
