@@ -14,20 +14,20 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #include "notebookglom.h"
 #include <gtkmm/messagedialog.h>
-#include "../application.h"
+#include <glom/appwindow.h>
 #include <glibmm/i18n.h>
 //#include <sstream> //For stringstream
 
 namespace Glom
 {
 
-NotebookGlom::NotebookGlom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& /* refGlade */)
+NotebookGlom::NotebookGlom(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& /* builder */)
 : Gtk::Notebook(cobject)
 {
 #ifndef GLOM_ENABLE_CLIENT_ONLY
@@ -59,12 +59,12 @@ void NotebookGlom::init()
 
 }
 
-App_Glom* NotebookGlom::get_application()
+AppWindow* NotebookGlom::get_appwindow() const
 {
-  Gtk::Container* pWindow = get_toplevel();
+  Gtk::Container* pWindow = const_cast<Gtk::Container*>(get_toplevel());
   //TODO: This only works when the child widget is already in its parent.
 
-  return dynamic_cast<App_Glom*>(pWindow);
+  return dynamic_cast<AppWindow*>(pWindow);
 }
 
 void NotebookGlom::delete_from_layout()
