@@ -14,13 +14,14 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
-#include "config.h" // For GLOM_ENABLE_MAEMO
+#include "config.h" // For GLOM_ENABLE_CLIENT_ONLY
 
 #include <glom/utils_ui.h>
+#include <glom/appwindow.h>
 #include <libglom/connectionpool.h>
 #include <libglom/data_structure/layout/report_parts/layoutitem_fieldsummary.h>
 #include <libglom/data_structure/glomconversions.h>
@@ -31,13 +32,12 @@
 
 #include <gtkmm/messagedialog.h>
 #include <gtkmm/stock.h>
+#include <glibmm/convert.h>
+#include <glibmm/miscutils.h>
+#include <glibmm/fileutils.h>
 
-#include <giomm.h>
-
-#ifdef GLOM_ENABLE_MAEMO
-#include <hildonmm/note.h>
-#endif
-
+#include <giomm/file.h>
+#include <glibmm/main.h>
 #include <glibmm/i18n.h>
 
 #include <string.h> // for strchr
@@ -53,6 +53,11 @@
 
 #include <stack>
 
+// For ShellExecute:
+#ifdef G_OS_WIN32
+# include <windows.h>
+#endif
+
 namespace
 {
 
@@ -97,15 +102,11 @@ int Utils::dialog_run_with_help(Gtk::Dialog* dialog, const Glib::ustring& id)
 {
   int result = dialog->run();
   
-  //Maemo has no help system since Maemo 5, 
-  //so we hide the buttons in anyway.
-  #ifndef GLOM_ENABLE_MAEMO
   while (result == Gtk::RESPONSE_HELP)
   {
     show_help(id);
     result = dialog->run();
   }
-  #endif //GLOM_ENABLE_MAEMO
 
   dialog->hide();
   return result;
@@ -117,9 +118,6 @@ int Utils::dialog_run_with_help(Gtk::Dialog* dialog, const Glib::ustring& id)
  * Launch a help browser with the glom help and load the given id if given
  * If the help cannot be found an error dialog will be shown
  */
-
-// Maemo has no help system since Maemo 5 (Fremantle).
-#ifndef GLOM_ENABLE_MAEMO
 void Utils::show_help(const Glib::ustring& id)
 {
   GError* err = 0;
@@ -159,25 +157,18 @@ void Utils::show_help(const Glib::ustring& id)
   }
   catch(const std::exception& ex)
   {
-    const Glib::ustring message = _("Could not display help: ") + Glib::ustring(ex.what());
+    const Glib::ustring message = Glib::ustring::compose(_("Could not display help: %1"), Glib::ustring(ex.what()));
     Gtk::MessageDialog dialog(message, false, Gtk::MESSAGE_ERROR);
     dialog.run();
   }
 }
-#endif //GLOM_ENABLE_MAEMO
 
 void Utils::show_ok_dialog(const Glib::ustring& title, const Glib::ustring& message, Gtk::Window* parent, Gtk::MessageType message_type)
 {
-#undef GLOM_ENABLE_MAEMO
-#ifdef GLOM_ENABLE_MAEMO
-  // TODO_maemo: Map message_type to a sensible stock_id?
-  Hildon::Note dialog(Hildon::NOTE_TYPE_INFORMATION, parent, message);
-#else
   Gtk::MessageDialog dialog("<b>" + title + "</b>", true /* markup */, message_type, Gtk::BUTTONS_OK);
   dialog.set_secondary_text(message);
   if(parent)
     dialog.set_transient_for(*parent);
-#endif
 
   dialog.run();
 }
@@ -245,7 +236,7 @@ Glib::RefPtr<Gdk::Pixbuf> Utils::get_pixbuf_for_gda_value(const Gnome::Gda::Valu
       {
         buffer_binary_length = 0;
         buffer_binary = 0;
-        g_warning("Conversions::get_pixbuf_for_gda_value(): Failed to read BLOB data");
+        std::cerr << G_STRFUNC << ": Failed to read BLOB data" << std::endl;
       }
     }
     else
@@ -272,15 +263,12 @@ Glib::RefPtr<Gdk::Pixbuf> Utils::get_pixbuf_for_gda_value(const Gnome::Gda::Valu
       catch(const Gdk::PixbufError& ex)
       {
         refPixbufLoader.reset();
-#ifdef GLIBMM_EXCEPTIONS_ENABLED        
         std::cerr << "PixbufLoader::create failed: " << ex.what() << std::endl;
-#endif        
       }
 
       if(refPixbufLoader)
       {
         guint8* puiData = (guint8*)buffer_binary;
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
         try
         {
           refPixbufLoader->write(puiData, static_cast<gsize>(buffer_binary_length));
@@ -288,25 +276,10 @@ Glib::RefPtr<Gdk::Pixbuf> Utils::get_pixbuf_for_gda_value(const Gnome::Gda::Valu
 
           refPixbufLoader->close(); //This throws if write() threw, so it must be inside the try block.
         }
-#else
-        std::auto_ptr<Glib::Error> error;
-        refPixbufLoader->write(puiData, static_cast<gsize>(buffer_binary_length), error);
-        if(!error.get())
-        {
-          result = refPixbufLoader->get_pixbuf();
-          refPixbufLoader->close(error);
-        }
-#endif
 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
         catch(const Glib::Exception& ex)
         {
-#else
-        if(error.get())
-        {
-          const Glib::Exception& ex = *error.get();        
-#endif
-          g_warning("Conversions::get_pixbuf_for_gda_value(): PixbufLoader::write() failed: %s", ex.what().c_str());
+          std::cerr << G_STRFUNC << ": PixbufLoader::write() failed: " << ex.what() << std::endl;
         }
       }
 
@@ -334,7 +307,7 @@ static int get_width_for_text(Gtk::Widget& widget, const Glib::ustring& text)
   return result;
 }
 
-int Utils::get_suitable_field_width_for_widget(Gtk::Widget& widget, const sharedptr<const LayoutItem_Field>& field_layout, bool or_title)
+int Utils::get_suitable_field_width_for_widget(Gtk::Widget& widget, const sharedptr<const LayoutItem_Field>& field_layout, bool or_title, bool for_treeview)
 {
   int result = 150; //Suitable default.
 
@@ -345,7 +318,7 @@ int Utils::get_suitable_field_width_for_widget(Gtk::Widget& widget, const shared
   {
     case(Field::TYPE_DATE):
     {
-      Glib::Date date(31, Glib::Date::Month(12), 2000);
+      const Glib::Date date(31, Glib::Date::Month(12), 2000);
       example_text = Conversions::get_text_for_gda_value(field_type, Gnome::Gda::Value(date));
       break;
     }
@@ -360,24 +333,21 @@ int Utils::get_suitable_field_width_for_widget(Gtk::Widget& widget, const shared
     }
     case(Field::TYPE_NUMERIC):
     {
-#ifdef GLOM_ENABLE_MAEMO
-      //Maemo's screen is not so big, so don't be so generous:
-      example_text = "EUR 9999999";
-#else
-      example_text = "EUR 9999999999";
-#endif
+      if(for_treeview)
+        example_text = "EUR 999.99";
+      else
+        example_text = "EUR 9999999999";
+        
       break;
     }
     case(Field::TYPE_TEXT):
     case(Field::TYPE_IMAGE): //Give images the same width as text fields, so they will often line up.
     {
-      //if(!field_layout->get_text_format_multiline()) //Use the full width for multi-line text.
-#ifdef GLOM_ENABLE_MAEMO
-        //Maemo's screen is not so big, so don't be so generous:
-        example_text = "AAAAAAAAAAAAAAAA";
-#else
+      if(for_treeview)
+        example_text = "AAAAAAAAAAAA";
+      else
         example_text = "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAA";
-#endif
+
       break;
     }
     default:
@@ -386,7 +356,6 @@ int Utils::get_suitable_field_width_for_widget(Gtk::Widget& widget, const shared
     }
   }
 
-
   if(!example_text.empty())
   {
     //Get the width required for this string in the current font:
@@ -396,7 +365,7 @@ int Utils::get_suitable_field_width_for_widget(Gtk::Widget& widget, const shared
   if(or_title)
   {
     //Make sure that there's enough space for the title too.
-    const int title_width = get_width_for_text(widget, field_layout->get_title());
+    const int title_width = get_width_for_text(widget, item_get_title(field_layout));
     if(title_width > result)
       result = title_width;
   }
@@ -446,6 +415,7 @@ Glib::RefPtr<Gdk::Pixbuf> Utils::image_scale_keeping_ratio(const Glib::RefPtr<Gd
   {
     SCALE_WIDTH,
     SCALE_HEIGHT,
+    SCALE_BOTH,
     SCALE_NONE
   };
 
@@ -458,11 +428,7 @@ Glib::RefPtr<Gdk::Pixbuf> Utils::image_scale_keeping_ratio(const Glib::RefPtr<Gd
   {
     if(pixbuf_width > target_width)
     {
-      //Both are bigger than the target, so find the biggest one:
-      if(pixbuf_width > pixbuf_height)
-        scale_mode = SCALE_WIDTH;
-      else
-        scale_mode = SCALE_HEIGHT;
+      scale_mode = SCALE_BOTH;
     }
     else
     {
@@ -488,6 +454,14 @@ Glib::RefPtr<Gdk::Pixbuf> Utils::image_scale_keeping_ratio(const Glib::RefPtr<Gd
     const float ratio = (float)target_width / (float) pixbuf_width;
     target_height = (int)((float)pixbuf_height * ratio);
   }
+  else if(scale_mode == SCALE_BOTH)
+  {
+    const float ratio = std::min(
+      (float)target_width / (float) pixbuf_width,
+      (float)target_height / (float) pixbuf_height);
+    target_width = (int)((float)pixbuf_width * ratio);
+    target_height = (int)((float)pixbuf_height * ratio);
+  }
 
  if( (target_height == 0) || (target_width == 0) )
  {
@@ -501,13 +475,9 @@ bool Utils::show_warning_no_records_found(Gtk::Window& transient_for)
 {
   const Glib::ustring message = _("Your find criteria did not match any records in the table.");
 
-#ifdef GLOM_ENABLE_MAEMO
-  Hildon::Note dialog(Hildon::NOTE_TYPE_CONFIRMATION_BUTTON, transient_for, message);
-#else
   Gtk::MessageDialog dialog(Utils::bold_message(_("No Records Found")), true, Gtk::MESSAGE_WARNING, Gtk::BUTTONS_NONE);
   dialog.set_secondary_text(message);
   dialog.set_transient_for(transient_for);
-#endif
 
   dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
   dialog.add_button(_("New Find"), Gtk::RESPONSE_OK);
@@ -517,4 +487,104 @@ bool Utils::show_warning_no_records_found(Gtk::Window& transient_for)
 }
 
 
+void Utils::show_report_in_browser(const std::string& filepath, Gtk::Window* parent_window)
+{
+  //Give the user a clue, in case the web browser opens in the background, for instance in a new tab:
+  if(parent_window)
+    show_ok_dialog(_("Report Finished"), _("The report will now be opened in your web browser."), *parent_window, Gtk::MESSAGE_INFO);
+
+#ifdef G_OS_WIN32
+  // gtk_show_uri doesn't seem to work on Win32, at least not for local files
+  // We use Windows API instead.
+  // TODO: Check it again and file a bug if necessary.
+  ShellExecute(0, "open", filepath.c_str(), 0, 0, SW_SHOW);
+#else
+
+  Glib::ustring uri;
+  try
+  {
+    uri = Glib::filename_to_uri(filepath);
+  }
+  catch(const Glib::ConvertError& ex)
+  {
+    std::cerr << G_STRFUNC << ": Could not convert filepath to URI: " << filepath << std::endl;
+    return;
+  }
+
+  //Use the GNOME browser:
+  GError* gerror = 0;
+  if(!gtk_show_uri(0 /* screen */, uri.c_str(), GDK_CURRENT_TIME, &gerror))
+  {
+    std::cerr << G_STRFUNC << ": " << gerror->message << std::endl;
+    g_error_free(gerror);
+  }
+#endif //G_OS_WIN32
+}
+
+std::string Utils::get_icon_path(const Glib::ustring& filename)
+{
+#ifdef G_OS_WIN32
+  gchar* basepath = g_win32_get_package_installation_directory_of_module(0);
+  const std::string result = Glib::build_filename(Glib::build_filename(basepath,
+      "share" G_DIR_SEPARATOR_S "glom" G_DIR_SEPARATOR_S "pixmaps"), filename);
+  g_free(basepath);
+  return result;
+#else
+  return Glib::build_filename(GLOM_PKGDATADIR G_DIR_SEPARATOR_S "pixmaps", filename);
+#endif
+}
+
+bool Utils::script_check_for_pygtk2_with_warning(const Glib::ustring& script, Gtk::Window* parent_window)
+{
+  if(!Utils::script_check_for_pygtk2(script))
+  {
+    Utils::show_ok_dialog(_("Script Uses PyGTK 2"),
+      _("Glom cannot run this script because it uses pygtk 2, but Glom uses GTK+ 3, and attempting to use pygtk 2 would cause Glom to crash."), parent_window, Gtk::MESSAGE_ERROR);
+    return false;
+  }
+
+  return true;
+}
+
+void Utils::treeview_delete_all_columns(Gtk::TreeView* treeview)
+{
+  if(!treeview)
+    return;
+
+  //We use this instead of just Gtk::TreeView::remove_all_columns()
+  //because that deletes columns as a side-effect of unreferencing them,
+  //and that behaviour might be fixed in gtkmm sometime,
+  //and whether they should be deleted by that would depend on whether we used Gtk::manage().
+  //Deleting them explicitly is safer and clearer. murrayc.
+
+  //Remove all View columns:
+  typedef std::vector<Gtk::TreeView::Column*> type_vec_columns;
+  type_vec_columns vecViewColumns (treeview->get_columns());
+
+  for (type_vec_columns::iterator iter (vecViewColumns.begin ()), columns_end (vecViewColumns.end ());
+    iter != columns_end;
+    ++iter)
+  {
+    Gtk::TreeView::Column* pViewColumn (*iter);
+    if(!pViewColumn)
+      continue;
+
+    GtkTreeViewColumn* weak_ptr = 0;
+    g_object_add_weak_pointer (G_OBJECT (pViewColumn->gobj()), (gpointer*)&weak_ptr);
+
+    //Keep the object alive, instead of letting gtk_tree_view_remove_column() delete it by reducing its reference to 0,
+    //so we can explicitly delete it.
+    //This feels safer, considering some strange crashes I've seen when using Gtk::TreeView::remove_all_columns(),
+    //though that might have been just because we didn't reset m_treeviewcolumn_button. murrayc.
+    pViewColumn->reference();
+    treeview->remove_column(*pViewColumn);
+    delete pViewColumn; //This should cause it to be removed.
+
+    if(weak_ptr)
+    {
+      std::cerr << G_STRFUNC << ": The GtkTreeViewColumn was not destroyed as expected." << std::endl;
+    }
+  }
+}
+
 } //namespace Glom
