@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #ifndef GLOM_WINDOW_BOXHOLDER_H
@@ -25,22 +25,13 @@
 #include <gtkmm/window.h>
 #include <gtkmm/button.h>
 
-#ifdef GLOM_ENABLE_MAEMO
-#include <gtkmm/alignment.h>
-#include <hildonmm/stackable-window.h>
-#endif
-
 namespace Glom
 {
 
 /** A window that can hold a Box_WithButtons.
  */
 class Window_BoxHolder :
-#ifndef GLOM_ENABLE_MAEMO
   public Gtk::Window
-#else
-  public Hildon::StackableWindow
-#endif
 {
 public: 
   explicit Window_BoxHolder(Box_WithButtons* pBox, const Glib::ustring& title = Glib::ustring());
@@ -48,10 +39,6 @@ public:
 
 private:
 
-  #ifdef GLOM_ENABLE_MAEMO
-  Gtk::Alignment m_alignment;
-  #endif
-
   //Signal handlers:
   void on_box_cancelled();
 };
