@@ -1,4 +1,8 @@
 #include <tests/import/utils.h>
+#include <libglom/utils.h>
+#include <glibmm/convert.h>
+#include <glibmm/fileutils.h>
+#include <glibmm/main.h>
 
 namespace ImportTests
 {
@@ -14,21 +18,18 @@ bool check(const std::string& name, bool test, std::stringstream& report)
   return test;
 }
 
-// Returns the file name of the temporary created file, which will contain the buffer's contents.
-static std::string create_file_from_buffer(const char* input, guint input_size)
+// Returns the file URI of the temporary created file, which will contain the buffer's contents.
+static Glib::ustring create_file_from_buffer(const char* input, guint input_size)
 {
-  // Use Glib's file utilities to get a unique temporary filename:
-  std::string tmp_filename;
-  const int tmp_file_handle = Glib::file_open_tmp(tmp_filename, "glom_testdata");
-  if(-1 < tmp_file_handle)
-    close(tmp_file_handle);
+  const std::string file_uri = Glom::Utils::get_temp_file_uri("glom_import_testdata");
+  if(file_uri.empty())
+  {
+    std::cerr << G_STRFUNC << ": file_uri was empty." << std::endl;
+    return std::string();
+  }
 
-  std::string file_uri;
-  //TODO: Catch exception.
-  file_uri = Glib::filename_to_uri(tmp_filename);
-  
   Glib::RefPtr<Gio::File> file = Gio::File::create_for_uri(file_uri);
-  
+
   gssize result = 0;
   //TODO: Catch exception.
   result = file->append_to()->write(input, input_size);
@@ -37,28 +38,28 @@ static std::string create_file_from_buffer(const char* input, guint input_size)
   return file_uri;
 }
 
-static void on_mainloop_killed_by_watchdog(MainLoopRp mainloop)
+static void on_mainloop_killed_by_watchdog(const Glib::RefPtr<Glib::MainLoop>& mainloop)
 {
   finished_parsing = false;
   //Quit the mainloop that we ran because the parser uses an idle handler.
   mainloop->quit();
 }
 
-static void on_parser_encoding_error(MainLoopRp mainloop)
+static void on_parser_encoding_error(const Glib::RefPtr<Glib::MainLoop>& mainloop)
 {
   finished_parsing = true;
   //Quit the mainloop that we ran because the parser uses an idle handler.
   mainloop->quit();
 }
 
-static void on_parser_finished(MainLoopRp mainloop)
+static void on_parser_finished(const Glib::RefPtr<Glib::MainLoop>& mainloop)
 {
   finished_parsing = true;
   //Quit the mainloop that we ran because the parser uses an idle handler.
   mainloop->quit();
 }
 
-static void on_file_read_error(const std::string& /*unused*/, MainLoopRp mainloop)
+static void on_file_read_error(const std::string& /*unused*/, const Glib::RefPtr<Glib::MainLoop>& mainloop)
 {
   finished_parsing = true;
   //Quit the mainloop that we ran because the parser uses an idle handler.
@@ -76,7 +77,7 @@ bool run_parser_from_buffer(const FuncConnectParserSignals& connect_parser_signa
 
   //Start a mainloop because the parser uses an idle handler.
   //TODO: Stop the parser from doing that.
-  MainLoopRp mainloop = Glib::MainLoop::create();
+  Glib::RefPtr<Glib::MainLoop> mainloop = Glib::MainLoop::create();
   Glom::CsvParser parser("UTF-8");
 
   parser.signal_encoding_error().connect(sigc::bind(&on_parser_encoding_error, mainloop));
@@ -89,7 +90,7 @@ bool run_parser_from_buffer(const FuncConnectParserSignals& connect_parser_signa
 
   connect_parser_signals(parser);
 
-  const std::string file_uri = create_file_from_buffer(input, input_size);
+  const Glib::ustring file_uri = create_file_from_buffer(input, input_size);
   parser.set_file_and_start_parsing(file_uri);
   if (Glom::CsvParser::STATE_PARSING != parser.get_state())
     return false;
@@ -111,7 +112,7 @@ bool run_parser_on_file(const FuncConnectParserSignals& connect_parser_signals,
 
   //Start a mainloop because the parser uses an idle handler.
   //TODO: Stop the parser from doing that.
-  MainLoopRp mainloop = Glib::MainLoop::create();
+  Glib::RefPtr<Glib::MainLoop> mainloop = Glib::MainLoop::create();
   Glom::CsvParser parser("UTF-8");
 
   parser.signal_encoding_error().connect(sigc::bind(&on_parser_encoding_error, mainloop));
